/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALACTIONDISPATCHER_H
#define BLOKKALACTIONDISPATCHER_H

#include <QtCore/QObject>

class BlokkalMainWindow;

class KMenu;

class QAction;

namespace Blokkal {
class Account;
class Blog;
}

/**
 * @short This class handles account and blog context menus
 *
 * This class handles account and blog context menus
 * @author Martin Mueller <orvio@orvio.de>
*/
class BlokkalActionDispatcher : public QObject {
Q_OBJECT
public:
	/**
	 * Constructor
	 */
	BlokkalActionDispatcher( BlokkalMainWindow * mainWindow );
	~BlokkalActionDispatcher( void );

	enum MenuFlags {
		IncludeBlogMenus = 0x01
	};

	/**
	 * @return the account menu in it's current state
	 */
	KMenu * accountMenu( void );

	/**
	 * Configures the account context menu for the current account.
	 * The current account is determined with Blokkal::ActionTargetRelay
	 * The returned value should never be 0, but it is recommended
	 * to check the returned value to avoid crashes due to
	 * missing xml files.
	 */
	KMenu * configureAccountMenu( unsigned int flags = 0 );

	/**
	 * @return the blog menu in it's current state
	 */
	KMenu * blogMenu( void );

	/**
	 * Configure the blog context menu for the current blog
	 */
	KMenu * configureBlogMenu( unsigned int flags = 0 );
	
private Q_SLOTS:
	void connectAccount( void );

	void startNewEntry( void );
	void browseEntries( void );
	void openBlog( void );

	void saveHoverAction( QAction * action );
	void configureDynamicMenues( void );

	void blogMenuTitleDestroyed( void );
	void accountMenuTitleDestroyed( void );

	void slotCurrentBlogChanged( Blokkal::Blog * blog );

private:
	class Private;
	Private * const d;
};

#endif
