/**************************************************************************
*   Copyright (C) 2006 - 2008 by Martin Mueller                           *
*   orvio@orvio.de                                                        *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include "blokkalapplication.h"
#include "blokkalapplication.moc"

#include "blokkalmainwindow.h"
#include "blokkalsystemtray.h"
#include "blokkalactiondispatcher.h"

#include <blokkalaccountmanager.h>
#include <blokkalaccount.h>
#include <blokkalentry.h>
#include <blokkalpostentryqueue.h>
#include <blokkalbehavioursettings.h>
#include <blokkalfileengine.h>
#include <blokkalui/globalsettings.h>
#include <blokkalui/view.h>
#include <blokkalui/stdaction.h>

#include <kdebug.h>
#include <kcmdlineargs.h>
#include <kmessagebox.h>
#include <klocale.h>
#include <kstartupinfo.h>

#include <QTimer>

class BlokkalApplication::Private {
public:
	Private( void ):
	mainWindow( 0 ),
	actionDispatcher( 0 ),
	fileEngineHandler( 0 ),
	firstInstance( TRUE ),
	isShuttingDown( FALSE ){}

	BlokkalMainWindow * mainWindow;
	BlokkalActionDispatcher * actionDispatcher;
	Blokkal::FileEngineHandler * fileEngineHandler;
	bool firstInstance;
	bool isShuttingDown;
};

BlokkalApplication::BlokkalApplication( void ) :
KUniqueApplication( TRUE, TRUE ),
d( new Private() )
{
	d->fileEngineHandler = new Blokkal::FileEngineHandler();
	
	d->mainWindow = new BlokkalMainWindow();
	Blokkal::Ui::GlobalSettings::self()->setMainWidget( d->mainWindow );
	d->actionDispatcher = new BlokkalActionDispatcher( d->mainWindow );
	d->mainWindow->setupGUI( KXmlGuiWindow::StatusBar|KXmlGuiWindow::Keys|\
	                         KXmlGuiWindow::Save|KXmlGuiWindow::Create|\
	                         KXmlGuiWindow::StatusBar,
	                         "blokkalmainwindowui.rc" );

	applyConfigurationChanges();
	connect( Blokkal::BehaviourSettings::self(),
	         SIGNAL( configChanged( void ) ),
	         SLOT( applyConfigurationChanges( void ) ) );	

	
	Blokkal::AccountManager::self()->loadAccounts();
	Blokkal::AccountList accounts = Blokkal::AccountManager::self()->accounts();
	for( Blokkal::AccountList::Iterator it = accounts.begin();
	     it != accounts.end();
	     ++it )
	{
		(*it)->restoreBlogs();
		if( Blokkal::BehaviourSettings::self()->autoConnect() && (*it)->autoConnect() ) {
			(*it)->connectAccount();
		}
	}

	d->mainWindow->restoreBlogView();

	if( !accounts.isEmpty() ) {
		QTimer::singleShot( 0, Blokkal::PostEntryQueue::self(), SLOT( restoreState( void ) ) );
	}
}


int BlokkalApplication::newInstance( void )
{
	bool raiseMainWindow = TRUE;
	
	if( !isSessionRestored() || !d->firstInstance ) {
		KCmdLineArgs * args = KCmdLineArgs::parsedArgs();

		for( int i = 0; i < args->count(); ++i ) {
			bool canceled = FALSE;
			Blokkal::Entry * entry = Blokkal::Entry::open( args->url( i ).path(), &canceled );

			if( entry ) {
				entry->view()->showView();
			}
			else if( !canceled ) {
				KMessageBox::error( mainWindow(), i18n( "Error opening the file %1.",
				                                        args->url( i ).path() ) );
			}

			raiseMainWindow = FALSE;
		}

		args->clear();
	}

	if( isSessionRestored() ) {
		if( sessionConfig() ) {
			kDebug() << "session config name " << sessionConfig()->name() << endl;
		}
		d->mainWindow->restore( 1, FALSE );
		raiseMainWindow = FALSE;
		KCmdLineArgs * args = KCmdLineArgs::parsedArgs();
		args->clear();
	}

	d->firstInstance = FALSE;
	
	if( raiseMainWindow ) {
		d->mainWindow->show();
		KStartupInfo::setNewStartupId( d->mainWindow, kapp->startupId());
	}

	return 0;
}

BlokkalApplication * BlokkalApplication::self( void )
{
	return static_cast<BlokkalApplication *>( kapp );
}

BlokkalApplication::~BlokkalApplication( void )
{
	delete d->fileEngineHandler;
	delete d->mainWindow;
	d->mainWindow = 0;
	delete d;
}

BlokkalMainWindow * BlokkalApplication::mainWindow( void ) const
{
	return d->mainWindow;
}

void BlokkalApplication::quitBlokkal( void )
{
	d->isShuttingDown = TRUE;
	
	for( int i = 0; i < KMainWindow::memberList().size(); ++i ) {
		KMainWindow * closeWindow = KMainWindow::memberList()[i];
		if( closeWindow == d->mainWindow ) {
			continue;
		}
		
		if( !closeWindow->close() ) {
			d->isShuttingDown = FALSE;
			return;
		}
   }

	if( isShuttingDown() ) {
		d->isShuttingDown =  d->mainWindow->close();
	}

	if( isShuttingDown() ) {
		QTimer::singleShot( 0, this, SLOT( quit( void ) ) );
	}
}

void BlokkalApplication::applyConfigurationChanges( void )
{
	setQuitOnLastWindowClosed( !Blokkal::BehaviourSettings::showSystemTray() );
}

void BlokkalApplication::commitData( QSessionManager & manager )
{
	d->isShuttingDown = TRUE;
	KUniqueApplication::commitData( manager );
}

bool BlokkalApplication::isShuttingDown( void ) const
{
	return d->isShuttingDown;
}

BlokkalActionDispatcher * BlokkalApplication::actionDispatcher( void ) const
{
	return d->actionDispatcher;
}
