/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkalbev.h"
#include "blokkalbev.moc"

#include "browsetabpages.h"

#include <blokkalentry.h>
#include <blokkalblog.h>
#include <blokkalaccount.h>
#include <blokkalui/htmlentrytextedit.h>
#include <blokkalui/view.h>
#include <blokkalio/jobs.h>
#include <blokkalio/blogbrowsingdriver.h>

#include <klocale.h>
#include <kglobalsettings.h>
#include <kguiitem.h>
#include <kstandardguiitem.h>
#include <kprogressdialog.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <ktabwidget.h>
#include <kpushbutton.h>

#include <QLayout>
#include <QTimer>
#include <QSplitter>

class BlokkalBEV::BlokkalBEV::Private
{
public:
	Private( void ) :
	deleteEntryJob( 0 ),
	currentEntry( 0 ),
	progressDialog( 0 ),
	hasSeenCalendar( FALSE ),
	hasSeenHistory( FALSE ){}

	Blokkal::Io::BlogBrowsingDriver * driver;
	Blokkal::Io::EntryJob * deleteEntryJob;

	Blokkal::Entry * currentEntry;

	QSplitter * mainWidget;
	KTabWidget * tabWidget;
	BrowseTabPage * calendarTab;
	BrowseTabPage * historyTab;
	QByteArray calendarSplitterState;
	QByteArray historySplitterState;

	KTextEdit * entryPreview;

	KProgressDialog * progressDialog;

	bool hasSeenCalendar;
	bool hasSeenHistory;
};

BlokkalBEV::BlokkalBEV( Blokkal::Blog * blog ) :
BrowseEntryDialog(),
d( new Private() )
{
	//reload = User1
	//delete = User2
	//edit/link = User3
	setModal( FALSE );
	setCaption( i18n( "Browse Entries in \"%1\" (%2)",
	                  blog->title().isEmpty() ? blog->id() : blog->title(),
	                  blog->account()->id() ) );
	connect( blog,
	         SIGNAL( blogDestroyed( Blokkal::Blog * ) ),
	         SLOT( close( void ) ) );
	d->driver = blog->blogBrowsingDriver();

	QSplitter * mainWidget = new QSplitter( Qt::Vertical, this );
	d->mainWidget = mainWidget;

	d->tabWidget = new KTabWidget();
	mainWidget->addWidget( d->tabWidget );
	//-------------
	//Calendar view
	//-------------
	d->calendarTab = new CalendarTabPage( d->driver );//, d->tabWidget );
	connect( d->calendarTab,
	         SIGNAL( jobReady( KJob *, const QString & ) ),
	         SLOT( startJob( KJob *, const QString & ) ) );
	connect(  d->calendarTab,
	         SIGNAL( entrySelected( Blokkal::Entry * ) ),
	         SLOT( slotEntrySelected( Blokkal::Entry * ) ) );
	d->tabWidget->addTab( d->calendarTab, KIcon( "view-pim-calendar" ), i18n( "Calendar" ) );
	d->tabWidget->setTabEnabled( d->tabWidget->indexOf( d->calendarTab ),
	                             d->driver->supportedOperations() & Blokkal::Io::BlogBrowsingDriver::LoadEntriesByDate );

	//------------
	//History view
	//------------
	d->historyTab = new HistoryTabPage( d->driver ); //, d->tabWidget );
	connect( d->historyTab,
	         SIGNAL( jobReady( KJob *, const QString & ) ),
	         SLOT( startJob( KJob *, const QString & ) ) );
	connect(  d->historyTab,
	         SIGNAL( entrySelected( Blokkal::Entry * ) ),
	         SLOT( slotEntrySelected( Blokkal::Entry * ) ) );	
	d->tabWidget->addTab( d->historyTab, KIcon( "view-history" ), i18n( "History" ) );
	d->tabWidget->setTabEnabled( d->tabWidget->indexOf( d->historyTab ),
	                             d->driver->supportedOperations() & Blokkal::Io::BlogBrowsingDriver::LoadEntryById
	                             || d->driver->supportedOperations() & Blokkal::Io::BlogBrowsingDriver::LoadRecentEntries );

	//-----------------
	//select active tab
	//-----------------
	if( d->driver->supportedOperations() & Blokkal::Io::BlogBrowsingDriver::LoadIndex
	    && d->tabWidget->isTabEnabled( d->tabWidget->indexOf( d->calendarTab ) ) )
	{
		//prefer calendar view if LoadIndex is available
		d->tabWidget->setCurrentWidget( d->calendarTab );
	}
	else if( ( d->driver->supportedOperations() & Blokkal::Io::BlogBrowsingDriver::LoadEntryById
	           || d->driver->supportedOperations() & Blokkal::Io::BlogBrowsingDriver::LoadRecentEntries )
	         && d->tabWidget->isTabEnabled( d->tabWidget->indexOf( d->historyTab ) ) )
	{
		//use history mode if possible
		d->tabWidget->setCurrentWidget( d->historyTab );
	}
	else if( d->tabWidget->isTabEnabled( d->tabWidget->indexOf( d->calendarTab ) ) ) {
		//no history mode available use calendar view anyway
		d->tabWidget->setCurrentWidget( d->calendarTab );
	}
	else if( !d->driver->supportedOperations() ){
		d->tabWidget->setEnabled( FALSE );
		KMessageBox::sorry( this, i18n( "This protocol does not provide any means of loading old entries." ) );
	}
	connect( d->tabWidget,
	         SIGNAL( currentChanged ( int ) ),
	         SLOT( slotTabChanged( int ) ) );

	d->entryPreview = new Blokkal::Ui::HtmlEntryTextEdit( );
	mainWidget->addWidget( d->entryPreview );
	d->entryPreview->setReadOnly( TRUE );
	mainWidget->setStretchFactor( 1, 2 );

	setMainWidget( mainWidget );

	
	connect( this,
	         SIGNAL( user1Clicked( void ) ),
	         SLOT( refresh( void ) ) );

	connect( this,
	         SIGNAL( user3Clicked( void ) ),
	         SLOT( openEntry( void ) ) );
	connect( this,
	         SIGNAL( user2Clicked( void ) ),
	         SLOT( deleteEntry( void ) ) );

	KConfigGroup group = KGlobal::config()->group( "BlokkalBEV" );
	restoreDialogSize( group );
	//d->mainWidget->restoreState( group.readEntry( "preview-splitter-state", QByteArray() ) );

	d->calendarSplitterState = group.readEntry( "preview-splitter-state-calendar", QByteArray() );
	d->historySplitterState = group.readEntry( "preview-splitter-state-history", QByteArray() );
	
	if( d->tabWidget->currentWidget() == d->calendarTab ) {
		d->hasSeenCalendar = TRUE;
		d->mainWidget->restoreState( d->calendarSplitterState );
	}
	else if( d->tabWidget->currentWidget() == d->historyTab ) {
		d->hasSeenHistory = TRUE;
		d->mainWidget->restoreState( d->historySplitterState );
	}
	QTimer::singleShot( 0, this, SLOT( refresh( void ) ) );
}

BlokkalBEV::~BlokkalBEV( void )
{
	KConfigGroup group = KGlobal::config()->group( "BlokkalBEV" );
	saveDialogSize( group );
	//group.writeEntry( "preview-splitter-state", d->mainWidget->saveState() );
	group.writeEntry( "preview-splitter-state-calendar", d->calendarSplitterState );
	group.writeEntry( "preview-splitter-state-history", d->historySplitterState );

	delete d;
}

void BlokkalBEV::applyViewMode( void )
{
	switch( mode() ) {
	case Blokkal::Io::BlogBrowsingDriver::BrowseEdit:
		setButtons( Close|User1|User2|User3 );
		setButtonGuiItem( User3, KGuiItem( i18n( "Edit" ), "accessories-text-editor" )  );
		setButtonGuiItem( User2, KStandardGuiItem::del() );
		break;
	case Blokkal::Io::BlogBrowsingDriver::BrowseLink:
		setButtons( Ok|Cancel|User1 );
		break;
	}

	//all buttons are reset on setButtons()
	setButtonGuiItem( User1, KGuiItem( i18n( "Reload"), "view-refresh"  ) );
	setDefaultButton( NoDefault );

	if( button( User2 ) ) { //delete
		button( User2 )->setEnabled( d->currentEntry );
	}	
	if( button( User3 ) ) { //edit
		button( User3 )->setEnabled( d->currentEntry );
	}
	if( button( Ok ) ) {
		button( Ok )->setEnabled( d->currentEntry && d->currentEntry->url().isValid() );
	}
}

void BlokkalBEV::refresh( void )
{
	d->entryPreview->clear();
	
	if( d->tabWidget->currentWidget() == d->calendarTab ) {
		d->calendarTab->refresh();
	}
	else if( d->tabWidget->currentWidget() == d->historyTab ) {
		d->historyTab->refresh();
	}
}

void BlokkalBEV::setProgress( KJob * job, unsigned long percent )
{
	if( !job ) {
		return;
	}
		
	d->progressDialog->progressBar()->setValue( percent );
}

void BlokkalBEV::slotJobFinished( KJob * job )
{
	if( job == d->deleteEntryJob ) {
		Blokkal::Entry * entry = d->deleteEntryJob->entry();
		d->deleteEntryJob = 0;
		if( !job->error() ) {
			entry->deleteLater();
			slotEntrySelected( 0 );
		}
	}

	
	destroyProgressDialog();
	
	if( job->error() ) {
		QString message;
		if( job->errorString().isEmpty() ) {
			message = i18n( "An error occured." );
		}
		else {
			message = i18n( "An error occured.\nThe underlying protocol reported the following:\n\"%1\"",
			                job->errorString() );
		}
		
		KMessageBox::error( this, message );
	}
}

void BlokkalBEV::initProgressDialog( const QString & message )
{
	destroyProgressDialog();
	d->progressDialog = new KProgressDialog( this, QString::null, QString::null );
	d->progressDialog->setModal( TRUE );
	d->progressDialog->progressBar()->setMinimum( 0 );
	d->progressDialog->progressBar()->setMaximum( 100 );
	d->progressDialog->progressBar()->setValue( 0 );
	d->progressDialog->setAutoClose( FALSE );
	d->progressDialog->setAllowCancel( TRUE );
	d->progressDialog->setMinimumDuration( 100 );
	d->progressDialog->setLabelText( message );
	connect( d->progressDialog,
	         SIGNAL( cancelClicked( void ) ),
	         SLOT( cancelJobs( void ) ) );
}

void BlokkalBEV::destroyProgressDialog( void )
{
	if( d->progressDialog ) {
		d->progressDialog->hide();
		d->progressDialog->deleteLater();
		d->progressDialog = 0;
	}
}

void BlokkalBEV::slotTabChanged( int index )
{
	if( d->tabWidget->widget( index ) == d->calendarTab ) {
		d->historySplitterState = d->mainWidget->saveState();
		d->mainWidget->restoreState( d->calendarSplitterState );
		slotEntrySelected( d->calendarTab->entry() );
	}
	else if( d->tabWidget->widget( index ) == d->historyTab ) {
		d->calendarSplitterState = d->mainWidget->saveState();
		d->mainWidget->restoreState( d->historySplitterState );
		slotEntrySelected( d->historyTab->entry() );
	}

	if( d->currentEntry ) {
		return;
	}
	
	if( d->tabWidget->currentWidget() == d->calendarTab && !d->hasSeenCalendar ) {
		d->hasSeenCalendar = TRUE;
		QTimer::singleShot( 0, this, SLOT( refresh() ) );
	}
	else if( d->tabWidget->currentWidget() == d->historyTab && !d->hasSeenHistory ) {
		d->hasSeenHistory = TRUE;
		QTimer::singleShot( 0, this, SLOT( refresh() ) );
	}
}

void BlokkalBEV::slotEntrySelected( Blokkal::Entry * entry )
{
	d->currentEntry = entry;

	if( d->currentEntry ) {
		d->entryPreview->setPlainText( d->currentEntry->text() );
	}
	else {
		d->entryPreview->clear();
	}

	if( button( User2 ) ) {
		button( User2 )->setEnabled( d->currentEntry );
	}
	if( button( User3 ) ) {
		button( User3 )->setEnabled( d->currentEntry );
	}
	if( button( Ok ) ) {
		button( Ok )->setEnabled( d->currentEntry && d->currentEntry->url().isValid() );
	}	
}

void BlokkalBEV::openEntry( void )
{
	if( d->currentEntry ) {
		d->currentEntry->view()->showView();
		//closeView();
	}
}

void BlokkalBEV::deleteEntry( void )
{
	if( !d->currentEntry ) {
		return;
	}
	
	QString warning = i18n( "Are you sure you want to delete the entry\n\"%1\"?",
	                        d->currentEntry->preview() );

	if( KMessageBox::warningYesNo( this, warning ) != KMessageBox::Yes ) {
		return;
	}
	

	d->deleteEntryJob = d->currentEntry->createJob( Blokkal::Entry::DeleteJob );
	if( d->deleteEntryJob ) {
		startJob( d->deleteEntryJob, i18n( "Deleting entry." ) );
	}
}

void BlokkalBEV::cancelJobs( void )
{
	if( !d->progressDialog ) {
		return;
	}

	d->progressDialog->deleteLater();
	d->progressDialog = 0;
	
	if( d->deleteEntryJob ) {
		d->deleteEntryJob->kill();
		d->deleteEntryJob = 0;
	}
	
	if( d->tabWidget->currentWidget() == d->calendarTab ) {
		d->calendarTab->cancelJobs();
	}
	else if( d->tabWidget->currentWidget() == d->historyTab ) {
		d->historyTab->cancelJobs();
	}
}

KUrl BlokkalBEV::entryUrl( void ) const
{
	if( d->currentEntry ) {
		return d->currentEntry->url();
	}

	return KUrl();
}

void BlokkalBEV::startJob( KJob * job, const QString & message )
{
	initProgressDialog( message );

	if( button( User3 ) ) {
		button( User3 )->setEnabled( FALSE );
	}
	if( button( Ok ) ) {
		button( Ok )->setEnabled( FALSE );
	}
	if( button( User2 ) ) {
		button( User2 )->setEnabled( FALSE );
	}
	
	connect( job,
	         SIGNAL( percent( KJob *, unsigned long ) ),
	         SLOT( setProgress( KJob *, unsigned long  ) ) );
	connect( job,
	         SIGNAL( result( KJob * ) ),
	         SLOT( slotJobFinished( KJob * ) ) );
	job->start();
}
