/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALBEV_H
#define BLOKKALBEV_H

#include <blokkalui/browseentryviewfactory.h>

class KJob;

namespace Blokkal {
class Blog;
class Entry;
}

/**
 * @short A view for browsing entries.
 *
 * This is a dialog suitable for browsing entries in a blog.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BlokkalBEV : public Blokkal::Ui::BrowseEntryDialog
{
Q_OBJECT
public:
	/**
	 * @param blog browse the entries of this blog
	 */
	BlokkalBEV( Blokkal::Blog * blog );
	virtual ~BlokkalBEV( void );

	/**
	 * @return the url of the currently selected entry or KUrl() if no entry has been selected
	 */
	virtual KUrl entryUrl( void ) const;	

public slots:
	/**
	 * Refreshes the data in the dialog
	 */
	void refresh( void );

protected:
	/**
	 * Updates the view as appropriate for the current mode.
	 */
	virtual void applyViewMode( void );
	
private slots:
	void setProgress( KJob * job, unsigned long percent );
	void slotJobFinished( KJob * job );
	void cancelJobs( void );

	void slotEntrySelected( Blokkal::Entry * entry );
	void slotTabChanged( int index );
	void openEntry( void );
	void deleteEntry( void );
	void startJob( KJob * job, const QString & message );

private:
	void initProgressDialog( const QString & message = QString::null );
	void destroyProgressDialog( void );
	
	class Private;
	Private * const d;
	
};

#endif
