/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "browseoptionwidgets.h"
#include "browseoptionwidgets.moc"

#include <blokkalio/blogbrowsingdriver.h>

#include <knuminput.h>
#include <klineedit.h>
#include <klocale.h>
#include <kdebug.h>

#include <QLayout>
#include <QLabel>

//----------------
//LoadRecentWidget
//----------------
class LoadRecentWidget::Private {
public:
	Private( void ) {}

	KIntNumInput * countInput;
};

LoadRecentWidget::LoadRecentWidget( Blokkal::Io::BlogBrowsingDriver * driver, QWidget * parent ):
Blokkal::Ui::BrowseOptionWidget( driver, parent ),
d( new Private() )
{
	d->countInput = new KIntNumInput( this );
	d->countInput->setLabel( i18n( "Fetch recent posts" ), Qt::AlignVCenter );
	d->countInput->setRange( 1,
	                         driver->recentLimit() ? driver->recentLimit() : 200 );
	d->countInput->setSteps( 20, 100 );
	d->countInput->setSliderEnabled( TRUE );
	d->countInput->setValue( 5 );
	d->countInput->setSizePolicy( QSizePolicy::Expanding,
	                              d->countInput->sizePolicy().verticalPolicy() );

	layout()->addWidget( d->countInput );
}

LoadRecentWidget::~LoadRecentWidget( void )
{
	delete d;
}

Blokkal::Io::LoadEntryJob * LoadRecentWidget::createLoadEntryJob( void )
{
	return driver()->createLoadEntryJob( d->countInput->value() );
}

//--------------
//LoadByIdWidget
//--------------
class LoadByIdWidget::Private {
public:
	Private( void ) {}

	KLineEdit * idLineEdit;
};

LoadByIdWidget::LoadByIdWidget( Blokkal::Io::BlogBrowsingDriver * driver, QWidget * parent ):
Blokkal::Ui::BrowseOptionWidget( driver, parent ),
d( new Private() )
{
	QLabel * label = new QLabel( i18n( "Fetch post by id" ), this );
	d->idLineEdit = new KLineEdit( this );
	d->idLineEdit->setTrapReturnKey( TRUE );
	label->setBuddy( d->idLineEdit );

	layout()->addWidget( label );
	layout()->addWidget( d->idLineEdit );
	connect( d->idLineEdit,
	         SIGNAL( returnPressed( void ) ),
	         SLOT( emitReloadRequested( void ) ) );

}

LoadByIdWidget::~LoadByIdWidget( void )
{
	delete d;
}

Blokkal::Io::LoadEntryJob * LoadByIdWidget::createLoadEntryJob( void )
{
	QStringList idList;
	idList.append( d->idLineEdit->text() );
	return driver()->createLoadEntryJob( idList );
}
