/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BROWSETABPAGES_H
#define BROWSETABPAGES_H

#include <QtGui/QWidget>
#include <QtCore/QDate>

class KJob;

class EntryList;

namespace Blokkal {
class Entry;
namespace Io {
class LoadEntryJob;
class BlogBrowsingDriver;
};
};

/**
 * @short Base class for browser tab pages
 *
 * Base class for browser tab pages
 * @author Martin Mueller <orvio@orvio.de>
 */
class BrowseTabPage : public QWidget
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param driver browsing driver to use
	 * @param parent parent widget
	 */
	BrowseTabPage( Blokkal::Io::BlogBrowsingDriver * driver,
	                 QWidget * parent = 0 );
	virtual ~BrowseTabPage( void );

	/**
	 * @return the current entry in the widget
	 */
	virtual Blokkal::Entry * entry( void ) const = 0;

public Q_SLOTS:
	/**
	 * Reloads the content of the widget
	 */
	virtual void refresh( void ) = 0;

	virtual void cancelJobs( void ) = 0;

Q_SIGNALS:
	/**
	 * This signal is emitted when some kind of io job has been started.
	 * @see announceJob()
	 */
	void jobReady( KJob * job, const QString & message );

	void entrySelected( Blokkal::Entry * entry = 0 );
	
protected:
	/**
	 * @return the driver passed to the constructor
	 */
	Blokkal::Io::BlogBrowsingDriver * driver( void ) const;

	/**
	 * Sets the load entry job.
	 * @param job the LoadEntryJob
	 */
	void setLoadEntryJob( Blokkal::Io::LoadEntryJob * job );

	/**
	 * Returns the currently set LoadEntryJob.
	 * @return the currently set LoadEntryJob
	 */
	Blokkal::Io::LoadEntryJob * loadEntryJob( void ) const;

	void setEntryList( EntryList * entryList );

	EntryList * entryList( void ) const;

	/**
	 * Prepares and announces a job @a job.
	 */
	void announceJob( KJob * job, const QString & message );

protected Q_SLOTS:
	virtual void handleFinishedJob( KJob * job ) = 0;

private Q_SLOTS:
	void emitEntrySelected( Blokkal::Entry * entry );

private:
	class Private;
	Private * const d;
};

/**
 * @short Page for browsing entries like a history
 *
 * Page for browsing entries like a history
 * @author Martin Mueller <orvio@orvio.de>
 */
class HistoryTabPage : public BrowseTabPage
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param driver browsing driver to use
	 * @param parent parent widget
	 */
	HistoryTabPage( Blokkal::Io::BlogBrowsingDriver * driver,
	                 QWidget * parent = 0 );
	virtual ~HistoryTabPage( void );

	/**
	 * @return the current entry in the widget
	 */
	virtual Blokkal::Entry * entry( void ) const;
	
public Q_SLOTS:
	/**
	 * Reloads the content of the widget
	 */
	virtual void refresh( void );

	virtual void cancelJobs( void );

protected Q_SLOTS:
	virtual void handleFinishedJob( KJob * job );	
	
private:
	class Private;
	Private * const d;
};

/**
 * @short Page for browsing entries like a calendar
 *
 * Page for browsing entries like a calendar
 * @author Martin Mueller <orvio@orvio.de>
 */
class CalendarTabPage : public BrowseTabPage
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param driver browsing driver to use
	 * @param parent parent widget
	 */
	CalendarTabPage( Blokkal::Io::BlogBrowsingDriver * driver,
	                 QWidget * parent = 0 );
	virtual ~CalendarTabPage( void );

	/**
	 * @return the current entry in the widget
	 */
	virtual Blokkal::Entry * entry( void ) const;

public Q_SLOTS:
	/**
	 * Reloads the content of the widget
	 */
	virtual void refresh( void );

	virtual void cancelJobs( void );

protected Q_SLOTS:
	virtual void handleFinishedJob( KJob * job );

private Q_SLOTS:
	void slotDateChanged( QDate date );	
	bool loadIndex( void );
	void loadCurrentDay( void );
	
private:
	class Private;
	Private * const d;
};

#endif
