/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ENTRYLIST_H
#define ENTRYLIST_H

#include <QtGui/QTreeView>
#include <QtCore/QAbstractItemModel>

namespace Blokkal {
class Entry;
}

class EntryListModel : public QAbstractItemModel {
Q_OBJECT
public:
	enum GroupingMode {
		NoGroups = 1,
		DateGroups = 2
	};

	enum CustomRoles {
		DisplayTextRole = Qt::UserRole,
		SortRole = Qt::UserRole + 1
	};
	
	/**
	 * Constructor
	 * @param parent parent object
	 */
	EntryListModel( GroupingMode mode, QObject * parent  );
	virtual ~EntryListModel( void );

	virtual QModelIndex index( int row,
	                           int column,
	                           const QModelIndex & parent = QModelIndex() ) const;
	virtual QModelIndex parent( const QModelIndex & child ) const;
	virtual int rowCount( const QModelIndex & parent = QModelIndex() ) const;
	virtual int columnCount( const QModelIndex & parent = QModelIndex() ) const;
	virtual QVariant data( const QModelIndex & index,
	                       int role = Qt::DisplayRole) const;

	/**
	 * Sets the entries
	 */
	void setEntries( const QList<Blokkal::Entry*> & entries );

	/**
	 * clears the model
	 */
	void clear( void );
	
private Q_SLOTS:
	void slotEntryDestroyed( Blokkal::Entry * entry );

private:
	class Private;
	Private * const d;
};

/**
 * @short List of entries.
 *
 * List of entries.
 * @author Martin Mueller <orvio@orvio.de>
*/
class EntryList : public QTreeView
{
Q_OBJECT
public:
	/**
	 * Creates a new EntryList.
	 * @param parent parent widget
	 */
	EntryList( EntryListModel::GroupingMode mode, QWidget * parent = 0 );
   virtual ~EntryList( void );

	/**
	 * Sets the entries in in this view
	 */
	void setEntries( const QList<Blokkal::Entry*> & entries );

	/**
	 * Returns the currently selected entry
	 */
	Blokkal::Entry * selectedEntry( void ) const;

	/**
	 * Clears the view
	 */
	void clear( void );

Q_SIGNALS:
	/**
	 * This signal is emitted whenever the selection changes.
	 * If @a entry is 0, no entry is selected.
	 */
	void entrySelected( Blokkal::Entry * entry );

private:
	friend class EntrySelectionModel;
	class Private;
	Private * const d;
};

#endif
