/***************************************************************************
 *   Copyright (C) 2007 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkalblogview.h"
#include "blokkalblogview.moc"

#include "blokkalactiondispatcher.h"
#include "blokkalapplication.h"

#include <blokkalblogmodel.h>
#include <blokkalaccount.h>
#include <blokkalblog.h>
#include <blokkalactiontargetrelay.h>

#include <kmenu.h>
#include <kdebug.h>
#include <kglobal.h>
#include <kconfiggroup.h>
#include <kconfig.h>
#include <ksharedconfig.h>
#include <klocale.h>

#include <QContextMenuEvent>
#include <QSortFilterProxyModel>

//----------------------
//BlogViewSelectionModel
//----------------------
class BlogViewSelectionModel : public QItemSelectionModel {
public:
	BlogViewSelectionModel( QAbstractItemModel * model, QObject * parent );
	virtual ~BlogViewSelectionModel( void );

public Q_SLOTS:
	virtual void select( const QModelIndex & index, QItemSelectionModel::SelectionFlags command );
	virtual void select( const QItemSelection & selection, QItemSelectionModel::SelectionFlags command );
};

BlogViewSelectionModel::BlogViewSelectionModel( QAbstractItemModel * model, QObject * parent ):
QItemSelectionModel( model, parent )
{
}

BlogViewSelectionModel::~BlogViewSelectionModel( void )
{}

void BlogViewSelectionModel::select( const QModelIndex & index, QItemSelectionModel::SelectionFlags command )
{
	if( !index.isValid() ) {
		return;
	}
	
	QItemSelectionModel::select( index, command );
}

void BlogViewSelectionModel::select( const QItemSelection & selection, QItemSelectionModel::SelectionFlags command )
{
	if( selection.indexes().size() != 1 ) {
		return;
	}

	QModelIndex index = selection.indexes().first();
	if( !index.isValid() ) {
		return;
	}	
	
	QItemSelectionModel::select( selection, command );
	/*if( command.testFlag( NoUpdate ) ) {
		return;
	}*/
	
	if( command.testFlag( Select ) ) {
		Blokkal::ActionTargetRelay::setCurrentAccount( dynamic_cast<Blokkal::Account *>( (QObject*) index.data().toULongLong() ) );
		Blokkal::ActionTargetRelay::setCurrentBlog( dynamic_cast<Blokkal::Blog *>( (QObject*) index.data().toULongLong() ) );
	}
	/*else if( command != NoUpdate ) {
		kDebug() << "unhandled command: " << command << endl;
	}*/
}

//--------
//BlogView
//--------
class BlogView::Private {
public:
	Private( void ):
	viewConfig( KConfigGroup( KGlobal::config(), "BlogView Settings" ) ){}

	KConfigGroup viewConfig;
	BlogViewSelectionModel * selectionModel;
};

BlogView::BlogView( QWidget * parent, unsigned int displayOptions ):
Blokkal::Ui::BlogView( parent, displayOptions ),
d( new Private(  ) )
{
	QSortFilterProxyModel *  const sortModel = new QSortFilterProxyModel( this );
	sortModel->setSourceModel( new Blokkal::BlogModel( this ) );
	sortModel->setSortRole( Blokkal::BlogModel::SortRole );
	setModel( sortModel );
	model()->sort( 0 );
	d->selectionModel = new BlogViewSelectionModel( sortModel, this );
	setSelectionModel( d->selectionModel );
	connect( this,
	         SIGNAL( activated( const QModelIndex & ) ),
	         SLOT( slotActivated( const QModelIndex & ) ) );
	connect( sortModel->sourceModel(),
	         SIGNAL( dataChanged( const QModelIndex &, const QModelIndex & ) ),
	         SLOT( resort( void ) ) );
}

BlogView::~BlogView( void )
{
	delete d;
}

void BlogView::contextMenuEvent( QContextMenuEvent * event )
{
	const QModelIndex index = indexAt( event->pos() );
	if( !index.isValid() ) {
		return;
	}

	Blokkal::Account * const account = dynamic_cast<Blokkal::Account*>( (QObject*) index.data().toULongLong() );
	if( account ) {
		Blokkal::ActionTargetRelay::setCurrentAccount( account );
		KMenu * const popup = BlokkalApplication::self()->actionDispatcher()->configureAccountMenu();
		if( popup ) {
			popup->popup( mapToGlobal( event->pos() ) );
		}
		return;
	}

	Blokkal::Blog * const blog = dynamic_cast<Blokkal::Blog*>( (QObject*) index.data().toULongLong() );	
	if( blog ) {
		Blokkal::ActionTargetRelay::setCurrentBlog( blog );
		KMenu * const popup = BlokkalApplication::self()->actionDispatcher()->configureBlogMenu();
		if( popup ) {
			popup->popup( mapToGlobal( event->pos() ) );
		}
		return;
	}
}

void BlogView::slotActivated( const QModelIndex & index )
{
	Blokkal::Blog * const blog = dynamic_cast<Blokkal::Blog*>( (QObject*) index.data().toULongLong() );
	if( blog ) {
		blog->startNewEntry();
	}
}

void BlogView::saveTreeState( void )
{
	const QModelIndex rootIndex = this->rootIndex();
	QStringList expandedAccountList;
	for( int i = 0; i < model()->rowCount( rootIndex ); ++i ) {
		QModelIndex accountIndex = model()->index( i, 0, rootIndex );
		if( isExpanded( accountIndex ) ) {
			Blokkal::Account * const account = dynamic_cast<Blokkal::Account*>( (QObject*) accountIndex.data().toULongLong() );
			if ( !account ) {
				kWarning() << "encountered invalid data type in blogview" << endl;
				continue;
			}
			
			expandedAccountList.append( account->id() );
		}
	}

	d->viewConfig.writeEntry( "expanded-accounts", expandedAccountList );
}

void BlogView::restoreTreeState( void )
{
	const QStringList expandedAccountList = d->viewConfig.readEntry( "expanded-accounts", QStringList() );
	if( expandedAccountList.isEmpty() ) {
		return;
	}
	
	const QModelIndex rootIndex = this->rootIndex();
	for( int i = 0; i < model()->rowCount( rootIndex ); ++i ) {
		QModelIndex accountIndex = model()->index( i, 0, rootIndex );
		Blokkal::Account * const account = dynamic_cast<Blokkal::Account*>( (QObject*) accountIndex.data().toULongLong() );
		if ( !account ) {
			kWarning() << "encountered invalid data type in blogview" << endl;
			continue;
		}

		setExpanded( accountIndex, expandedAccountList.contains( account->id() ) );
	}
}

void BlogView::resort( void )
{
	model()->sort( 0 );
}
