/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkaleev.h"
#include "blokkaleev.moc"

#include "crosspostselector.h"
#include "dialogs.h"

#include <blokkalentry.h>
#include <blokkalblog.h>
#include <blokkalaccount.h>
#include <blokkalpostentryqueue.h>
#include <blokkalui/editentrywidget.h>
#include <blokkalui/globalsettings.h>
#include <blokkalui/stdaction.h>
#include <blokkalui/formattingbackend.h>

#include <kstdaction.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kshortcut.h>
#include <kaction.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <kmenubar.h>
#include <kfiledialog.h>
#include <kactioncollection.h>
#include <ktoggleaction.h>
#include <kxmlguifactory.h>
#include <kurl.h>
#include <kmenu.h>
#include <kpushbutton.h>
#include <kedittoolbar.h>

#include <QStackedWidget>
#include <QMap>
#include <QSplitter>
#include <QDropEvent>

class BlokkalEEV::Private {
public:
	Private( void ):
	entryQueued( FALSE ),
	entryModified( FALSE ),
	sameAccount( TRUE ),
	crossPostSelector( 0 ),
	setupImageLinkDialog( 0 ),
	setupWebLinkDialog( 0 ),
	droppedURL( 0 ),
	dropPopupMenu( 0 ),
	previewDialog( 0 ){}

	bool entryQueued;
	bool entryModified;
	bool sameAccount;

	CrossPostSelector * crossPostSelector;
	SetupImageLinkDialog * setupImageLinkDialog;
	SetupWebLinkDialog * setupWebLinkDialog;
	
	KUrl * droppedURL;
	KMenu * dropPopupMenu;

	PreviewDialog * previewDialog;
	
	QSplitter * mainWidget;
	QStackedWidget * widgetStack;
	Blokkal::Blog * currentBlog;
	QMap<Blokkal::Blog*,Blokkal::Ui::EditEntryWidget*> blogWidgetMap;
	KToggleAction * crossPostAction;

	/**
	 * Merges the content of the entry in the widget belonging to
	 * @a blog into all other entries
	 */
	void mergeEntries( Blokkal::Blog * blog ) {
		Blokkal::Entry * entry = blogWidgetMap[blog]->apply();
		//kdDebug() << "read text: " << entry->text() << endl;
		
		for( QMap<Blokkal::Blog*, Blokkal::Ui::EditEntryWidget*>::ConstIterator it = blogWidgetMap.begin();
		     it != blogWidgetMap.end();
		     ++it )
		{
			if( it.key() != blog ) {
				//kdDebug() << "merging into: " << it.key()->id() << endl;
				it.value()->entry()->merge( entry );
				it.value()->rollBack();
			}
		}		
	}
};

BlokkalEEV::BlokkalEEV( Blokkal::Entry * entry ) :
MainWindowView( 0 ),
d( new Private() )
{
	setObjectName( "BlokkalEEV#" );
	setAttribute( Qt::WA_GroupLeader );
	d->mainWidget = new QSplitter( this );
	d->widgetStack = new QStackedWidget( d->mainWidget );
	setCentralWidget( d->mainWidget );
	
	KStandardAction::close( this, SLOT( close( void ) ), actionCollection() );
	setStandardToolBarMenuEnabled( TRUE );
	KStandardAction::configureToolbars( this, SLOT( configureToolbar( void ) ), actionCollection() );
	
	KAction * action;
	action = Blokkal::Ui::StdAction::post( this, SLOT( postEntry( void ) ), actionCollection() );
	action->setStatusTip( i18n( "Posts the entry" ) );
	actionCollection()->addAction( "file_postentry", action );
	//preview
	action = new KAction( KIcon( "view-preview" ),
	                      i18n( "Preview" ),
	                      actionCollection() );
	action->setStatusTip( i18n( "Displays a preview of this entry" ) );	
	actionCollection()->addAction( "file_previewentry", action );
	connect( action, SIGNAL( triggered( void ) ), SLOT( showPreview( void ) ) );


	action = KStandardAction::save( this, SLOT( saveEntry( void ) ), actionCollection() );
	action->setEnabled( !entry->fileName( TRUE ).isEmpty() );
	action = KStandardAction::saveAs( this, SLOT( saveEntryAs( void ) ), actionCollection() );
	d->crossPostAction = new KToggleAction( KIcon( "crosspost" ),
	                                        i18n( "Crosspost" ),
	                                        actionCollection() );
	actionCollection()->addAction( "edit_crosspost", d->crossPostAction );
	d->crossPostAction->setEnabled( entry->id().isNull() );
	connect( d->crossPostAction,
	         SIGNAL( toggled( bool ) ),
	         SLOT( toggleCrossposting( bool ) ) );
	Blokkal::Ui::StdAction::preferences( actionCollection() );

	KToggleAction * menubarAction = KStandardAction::showMenubar( 0, 0, actionCollection() );
	connect( menubarAction, SIGNAL( toggled( bool ) ),
	         menuBar(), SLOT( setHidden( bool ) ) );
	KStandardAction::keyBindings( guiFactory(), SLOT( configureShortcuts( void ) ), actionCollection() );
	KStandardAction::configureToolbars( this, SLOT( configureToolbars( void ) ), actionCollection() );

	//editing actions
	Blokkal::Ui::StdAction::bold( this, SLOT( makeTextBold( void ) ), actionCollection(), "edit_bold" );
	Blokkal::Ui::StdAction::italic( this, SLOT( makeTextItalic( void ) ), actionCollection(), "edit_italic" );
	Blokkal::Ui::StdAction::underline( this, SLOT( makeTextUnderline( void ) ), actionCollection(), "edit_strikeout" );
	Blokkal::Ui::StdAction::strike( this, SLOT( makeTextStrikeout( void ) ), actionCollection(), "edit_underline" );
	Blokkal::Ui::StdAction::image( this, SLOT( insertImage( void ) ), actionCollection(),  "edit_insertimage" );
	Blokkal::Ui::StdAction::link( this, SLOT( insertLink( void ) ), actionCollection(), "edit_insertlink" );

	//setup the widget
	d->currentBlog = entry->blog();
	d->blogWidgetMap.insert( d->currentBlog, d->currentBlog->createEditEntryWidget( entry, d->widgetStack ) );

	if( !d->blogWidgetMap[d->currentBlog] ) {
		setCaption( 0 );
		kError() << "0 widget created!" << endl;
		return;
	}

	d->widgetStack->addWidget( d->blogWidgetMap[d->currentBlog] );
	d->widgetStack->setCurrentWidget( d->blogWidgetMap[d->currentBlog] );
	setCaption( baseCaption() );
	
	createGUI( "blokkal_eevpui.rc" );
	connect( d->blogWidgetMap[d->currentBlog],
	         SIGNAL( contentsChanged( void ) ),
	         SLOT( slotContentsChanged( void ) ) );
	connect( d->blogWidgetMap[d->currentBlog],
	         SIGNAL( captionChanged( const QString & ) ),
	         SLOT( updateCaption( void ) ) );
	actionCollection()->action( "file_postentry" )->setEnabled( d->blogWidgetMap[d->currentBlog]->isContentLegal() );
	
	factory()->addClient( d->blogWidgetMap[d->currentBlog]->formattingBackend() );
	entryTextEdit(d->blogWidgetMap[d->currentBlog])->setDropProxy( this );
	setAutoSaveSettings( QString::fromLatin1( "BlokkalEEV" ) );

	if( entry->isDirty() ) {
		slotContentsChanged();
	}
}

BlokkalEEV::~BlokkalEEV( void )
{
	//delete the entries, because they will not be destroyed with the EditEntryWidgets
	//the only reason for an entry to be alive after this view has been closed is when it
	//has been queued for posting
	//Or: it is an old entry and is living inside the old entry
	//browser as well -> check id()
	if( !d->entryQueued ) {
		for( QMap<Blokkal::Blog*, Blokkal::Ui::EditEntryWidget*>::ConstIterator it = d->blogWidgetMap.begin();
				it != d->blogWidgetMap.end();
				++it )
		{
			if( it.value()->entry()->id().isNull() ) {
				delete it.value()->entry();
			}
		}
	}

	delete d->droppedURL;
	delete d;
}

void BlokkalEEV::slotContentsChanged( void )
{
	setCaption( i18n( "%1 [modified]", baseCaption() ) );
	actionCollection()->action( "file_postentry" )->setEnabled( d->blogWidgetMap[d->currentBlog] && d->blogWidgetMap[d->currentBlog]->isContentLegal() );
	d->entryModified = TRUE;
}

QString BlokkalEEV::baseCaption( void )
{
	if( !d->blogWidgetMap[d->currentBlog] ) {
		return QString::null;
	}

	QString blog;
	if( d->blogWidgetMap.count() <= 1 ) {
		blog = d->currentBlog->title();
		if( blog.isEmpty() ) {
			blog = d->currentBlog->id();
		}
	}
	
	QString caption = d->blogWidgetMap[d->currentBlog]->caption();
	if( caption.isEmpty() ) {
		caption = i18n( "Entry" );
	}
	
	if( d->blogWidgetMap.count() <= 1 ) {
		return i18n( "%1 in %2 (%3)", caption, blog, d->currentBlog->type() );
	}

	return caption;
}

void BlokkalEEV::postEntry( void )
{
	if( d->blogWidgetMap[d->currentBlog] ) {
		d->mergeEntries( d->currentBlog );
		for( QMap<Blokkal::Blog*, Blokkal::Ui::EditEntryWidget*>::ConstIterator it = d->blogWidgetMap.begin();
		     it != d->blogWidgetMap.end();
		     ++it )
		{
			Blokkal::Entry * entry = it.value()->apply();
			if( d->crossPostAction->isChecked() ) {
				entry->setCategories( d->crossPostSelector->categories( it.key() ) );
			}
			entry->setDirty( TRUE );
			Blokkal::PostEntryQueue::self()->queue( entry );
		}
		
		d->entryQueued = TRUE;
	}
	close();
}

bool BlokkalEEV::queryClose( void )
{
	if( !(d->blogWidgetMap[d->currentBlog] && d->blogWidgetMap[d->currentBlog]->entry() ) ) {
		return TRUE;
	}
	
	if( d->entryQueued ) {
		return TRUE;
		//don't delete
	}
	
	if( d->entryModified ) {
		int result = KMessageBox::warningYesNo( this,
		                                        i18n( "The entry has been modified. Do you want to discard the changes?" ) );
		if( result == KMessageBox::No ) {
			kDebug() << "returning FALSE" << endl;
			return FALSE;
		}
	}

	return TRUE;
}

void BlokkalEEV::saveEntry( void )
{
	if( !(d->blogWidgetMap[d->currentBlog] && d->blogWidgetMap[d->currentBlog]->entry() ) ) {
		return;
	}
	
	QString fileName = d->blogWidgetMap[d->currentBlog]->entry()->fileName();
	if( fileName.isEmpty() ) {
		kWarning() << "entry has no file name!" << endl;
		return;
	}
	
	if( !d->blogWidgetMap[d->currentBlog]->apply()->saveAs( fileName ) ) {
		KMessageBox::error( this, i18n( "An error occured while saving the entry!" ) );
	}
	else {
		d->entryModified = FALSE;
		setCaption( baseCaption() );
	}
}

void BlokkalEEV::saveEntryAs( void )
{
	if( !(d->blogWidgetMap[d->currentBlog] && d->blogWidgetMap[d->currentBlog]->entry() ) ) {
		return;
	}
	
	QString fileName = KFileDialog::getSaveFileName( KUrl("kfiledialog:///entryfiles"), //startdir
	                                                 "application/x-blokkal",
	                                                 this );
	if( !fileName.isNull() ) {
		bool fileExists = QFile::exists( fileName );
		if( fileExists ) {
			QString question = i18n( "A file with this name already exists: %1\nDo you want to replace it?", fileName );
			if( KMessageBox::warningYesNo( this,  question ) == KMessageBox::Yes )  {
				if( !d->blogWidgetMap[d->currentBlog]->apply()->saveAs( fileName ) ) {
					KMessageBox::error( this, i18n( "An error occured while saving the entry!" ) );
				}
				else {
					d->entryModified = FALSE;
					setCaption( baseCaption() );
					actionCollection()->action( KStandardAction::name( KStandardAction::Save ) )->setEnabled( TRUE );
				}
			}
		}
		else {
			if( !d->blogWidgetMap[d->currentBlog]->apply()->saveAs( fileName ) ) {
				KMessageBox::error( this, i18n( "An error occured while saving the entry!" ) );
			}
			else {
				d->entryModified = FALSE;
				setCaption( baseCaption() );
				actionCollection()->action( KStandardAction::name( KStandardAction::Save ) )->setEnabled( TRUE );
			}
		}
	}
}

void BlokkalEEV::updateCaption( void )
{
	if( d->entryModified ) {
		slotContentsChanged();
	}
	else {
		setCaption( baseCaption() );
	}
}

void BlokkalEEV::toggleCrossposting( bool on )
{
	if( on ) {
		if( !d->crossPostSelector ) {
			d->crossPostSelector = new CrossPostSelector( );
			d->mainWidget->addWidget( d->crossPostSelector );
			d->crossPostSelector->selectBlog( d->currentBlog );
			connect( d->crossPostSelector,
			         SIGNAL( blogSelected( Blokkal::Blog * ) ),
			         SLOT( slotBlogSelected( Blokkal::Blog * ) ) );
			connect( d->crossPostSelector,
			         SIGNAL( blogChecked( Blokkal::Blog * ) ),
			         SLOT( slotBlogChecked( Blokkal::Blog * ) ) );
			connect( d->crossPostSelector,
			         SIGNAL( blogUnchecked( Blokkal::Blog * ) ),
			         SLOT( slotBlogUnchecked( Blokkal::Blog * ) ) );
		}

		d->crossPostSelector->setCategories( d->currentBlog, d->blogWidgetMap[d->currentBlog]->apply()->categories() );
		d->crossPostSelector->show();
		d->blogWidgetMap[d->currentBlog]->hideFeature( Blokkal::Blog::FeatureCategory );
	}
	else {
		Blokkal::Entry * entry = d->blogWidgetMap[d->currentBlog]->apply();
		entry->setCategories( d->crossPostSelector->categories( d->currentBlog ) );
		d->blogWidgetMap[d->currentBlog]->rollBack();
		d->blogWidgetMap[d->currentBlog]->hideFeature( 0 );
		d->crossPostSelector->hide();

		QList<Blokkal::Blog*> removeList;
		for( QMap<Blokkal::Blog*, Blokkal::Ui::EditEntryWidget*>::ConstIterator it = d->blogWidgetMap.begin();
			it != d->blogWidgetMap.end();
			++it )
		{
			if( it.key() != d->currentBlog ) {
				removeList.append( it.key() );
			}
		}

		for( QList<Blokkal::Blog*>::ConstIterator it = removeList.begin();
		     it != removeList.end();
		     ++it )
		{
			d->crossPostSelector->uncheckBlog( *it );
		}
	}
}

void BlokkalEEV::slotBlogSelected( Blokkal::Blog * blog )
{
	if( !blog ) {
		return;
	}

	if( blog == d->currentBlog ) {
		return;
	}
	
	if( d->blogWidgetMap.contains( blog ) ) {
		d->mergeEntries( d->currentBlog );
		d->widgetStack->setCurrentWidget( d->blogWidgetMap[blog] );
		factory()->removeClient( d->blogWidgetMap[d->currentBlog]->formattingBackend() );

		if( d->sameAccount ) {
			factory()->addClient( d->blogWidgetMap[blog]->formattingBackend() );
		}

		d->currentBlog = blog;
		updateCaption();
	}
}

void BlokkalEEV::slotBlogChecked( Blokkal::Blog * blog )
{
	if( !blog ) {
		return;
	}

	if( !d->blogWidgetMap.contains( blog ) ) {
		d->blogWidgetMap.insert( blog,
		                         blog->createEditEntryWidget( blog->createEntry(), d->widgetStack ) );
		d->widgetStack->addWidget( d->blogWidgetMap[blog] );
		d->blogWidgetMap[blog]->hideFeature( Blokkal::Blog::FeatureCategory );
		entryTextEdit(d->blogWidgetMap[blog])->setDropProxy( this );
		connect( d->blogWidgetMap[blog],
		         SIGNAL( contentsChanged( void ) ),
		         SLOT( slotContentsChanged( void ) ) );
		connect( d->blogWidgetMap[blog],
		         SIGNAL( captionChanged( const QString & ) ),
		         SLOT( updateCaption( void ) ) );

		if( d->sameAccount ) {
			Blokkal::Account * account = d->currentBlog->account();
			for( QMap<Blokkal::Blog*, Blokkal::Ui::EditEntryWidget*>::ConstIterator it = d->blogWidgetMap.begin();
				it != d->blogWidgetMap.end();
				++it )
			{
				if( it.key()->account() != account ) {
					d->sameAccount = FALSE;
					factory()->removeClient( d->blogWidgetMap[d->currentBlog]->formattingBackend() );
					break;
				}
			}
		}
	}
}

void BlokkalEEV::slotBlogUnchecked( Blokkal::Blog * blog )
{
	if( !blog ) {
		return;
	}

	if( d->blogWidgetMap.contains( blog ) ) {
		if( d->currentBlog == blog ) {
			for( QMap<Blokkal::Blog*, Blokkal::Ui::EditEntryWidget*>::ConstIterator it = d->blogWidgetMap.begin();
		     it != d->blogWidgetMap.end();
		     ++it )
			{
				if( it.key() != blog ) {
					d->crossPostSelector->selectBlog( it.key() );
					break;
				}
			}
		}

		Blokkal::Entry * entry = d->blogWidgetMap[blog]->entry();
		delete d->blogWidgetMap[blog];
		delete entry;
		d->blogWidgetMap.remove( blog );
		updateCaption();

		d->sameAccount = TRUE;
		Blokkal::Account * account = d->currentBlog->account();
		for( QMap<Blokkal::Blog*, Blokkal::Ui::EditEntryWidget*>::ConstIterator it = d->blogWidgetMap.begin();
			it != d->blogWidgetMap.end();
			++it )
		{
			if( it.key()->account() != account ) {
				d->sameAccount = FALSE;
				break;
			}
		}

		if( d->sameAccount
		    && !factory()->clients().contains( d->blogWidgetMap[d->currentBlog]->formattingBackend() ) )
		{
			factory()->addClient( d->blogWidgetMap[d->currentBlog]->formattingBackend() );
		}
	}
}

void BlokkalEEV::insertImage( const QString & url )
{
	if( !d->setupImageLinkDialog ) {
		d->setupImageLinkDialog = new SetupImageLinkDialog( this );
	}

	if( !url.isNull() ) {
		d->setupImageLinkDialog->setURL( url );
	}

	if( d->setupImageLinkDialog->exec()  ) {
		entryTextEdit(d->blogWidgetMap[d->currentBlog])->insertTextAtCursor( d->setupImageLinkDialog->link() );
	}	
}

void BlokkalEEV::insertLink( const QString & url )
{
	if( !d->setupWebLinkDialog ) {
		d->setupWebLinkDialog = new SetupWebLinkDialog( this, d->currentBlog->blogBrowsingDriver() );
	}

	if( !url.isNull() ) {
		d->setupWebLinkDialog->setURL( url );
	}

	if( d->setupWebLinkDialog->exec()  ) {
		entryTextEdit(d->blogWidgetMap[d->currentBlog])->insertTextAtCursor( d->setupWebLinkDialog->link() );
	}	
}

void BlokkalEEV::makeTextBold( void )
{
	entryTextEdit(d->blogWidgetMap[d->currentBlog])->frameSelection( QString::fromLatin1( "<b>" ), QString::fromLatin1( "</b>" ) );	
}

void BlokkalEEV::makeTextItalic( void )
{
	entryTextEdit(d->blogWidgetMap[d->currentBlog])->frameSelection( QString::fromLatin1( "<i>" ), QString::fromLatin1( "</i>" ) );	
}

void BlokkalEEV::makeTextStrikeout( void )
{
	entryTextEdit(d->blogWidgetMap[d->currentBlog])->frameSelection( QString::fromLatin1( "<s>" ), QString::fromLatin1( "</s>" ) );	
}

void BlokkalEEV::makeTextUnderline( void )
{
	entryTextEdit(d->blogWidgetMap[d->currentBlog])->frameSelection( QString::fromLatin1( "<u>" ), QString::fromLatin1( "</u>" ) );	
}

bool BlokkalEEV::handleDrop( QDropEvent * event, QWidget * widget )
{
	if( widget != entryTextEdit(d->blogWidgetMap[d->currentBlog]) ) {
		return FALSE;
	}

	KUrl::List urls = KUrl::List::fromMimeData( event->mimeData() );
	if ( urls.isEmpty() ) {
		return FALSE;
	}
	
	d->droppedURL = new KUrl( urls.first() );
	if( !d->dropPopupMenu ) {
		d->dropPopupMenu = new KMenu( this );
		KAction * action = 0;

		action = Blokkal::Ui::StdAction::link( this, SLOT( dropAsLink( void ) ) );
		action->setParent( d->dropPopupMenu );
		action->setShortcut( 0 );
		action->setText(i18n( "Insert as &link" ) );
		d->dropPopupMenu->addAction( action );

		action = Blokkal::Ui::StdAction::image( this, SLOT( dropAsImage( void ) ) );
		action->setParent( d->dropPopupMenu );
		action->setShortcut( 0 );
		action->setText( i18n( "Insert as &image" ) );
		d->dropPopupMenu->addAction( action );

		action = new KAction( KIcon( "insert-text" ), i18n("Insert as &text"), d->dropPopupMenu );
		connect( action, SIGNAL( triggered( void ) ), SLOT( dropAsText( void ) ) );
		d->dropPopupMenu->addAction( action );

		d->dropPopupMenu->addSeparator();

		KGuiItem guiItem = KStandardGuiItem::cancel();
		action = new KAction( guiItem.icon(), guiItem.text(), d->dropPopupMenu );
		connect( action, SIGNAL( triggered( void ) ), SLOT( clearDroppedURL( void ) ) );
		d->dropPopupMenu->addAction( action );
	}

	d->dropPopupMenu->popup( entryTextEdit(d->blogWidgetMap[d->currentBlog])->mapToGlobal( event->pos() ) );
	return TRUE;
}

void BlokkalEEV::dropAsLink( void )
{
	if( !d->droppedURL ) {
		return;
	}

	insertLink( d->droppedURL->url() );
	
	clearDroppedURL();	
}

void BlokkalEEV::dropAsImage( void )
{
	if( !d->droppedURL ) {
		return;
	}

	insertImage( d->droppedURL->url() );

	clearDroppedURL();	
}

void BlokkalEEV::dropAsText( void )
{
	if( !d->droppedURL ) {
		return;
	}

	entryTextEdit(d->blogWidgetMap[d->currentBlog])->insertTextAtCursor( d->droppedURL->prettyUrl() );
	
	clearDroppedURL();
}

void BlokkalEEV::clearDroppedURL( void )
{
	delete d->droppedURL;
	d->droppedURL = 0;
}

void BlokkalEEV::showPreview( void )
{
	if( !d->previewDialog ) {
		d->previewDialog = new PreviewDialog( this );
		d->previewDialog->setButtons( KDialog::Close|KDialog::User1 );
		d->previewDialog->setButtonGuiItem( KDialog::User1, KGuiItem( i18n( "Post Entry"), "blokkal"  ) );
		d->previewDialog->setDefaultButton( KDialog::Close );
		connect( d->previewDialog,
		         SIGNAL( user1Clicked( void) ),
		         SLOT( postEntry( void ) ) );
	}

	d->previewDialog->button( KDialog::User1 )->setEnabled(\
		d->blogWidgetMap[d->currentBlog]&&\
		d->blogWidgetMap[d->currentBlog]->isContentLegal()\
	);

	d->previewDialog->clearPreviews();
	d->previewDialog->setContent( entryTextEdit(d->blogWidgetMap[d->currentBlog])->toPlainText() );
	for( QMap<Blokkal::Blog*, Blokkal::Ui::EditEntryWidget*>::ConstIterator it = d->blogWidgetMap.begin();
	     it != d->blogWidgetMap.end();
	     ++it )
	{
		d->previewDialog->addBlog( it.key(), it.value()->formattingBackend() );
	}
	d->previewDialog->setCurrentBlog( d->currentBlog );

	if( !d->previewDialog->isVisible() ) {
		d->previewDialog->show();
	}
	else {
		d->previewDialog->raise();
	}
}

void BlokkalEEV::configureToolbar( void )
{
	KConfigGroup configGroup( KGlobal::config(), "BlokkalEEV" );
	saveMainWindowSettings( configGroup );
 	KEditToolBar * const editDialog = new KEditToolBar( factory() );
	connect( editDialog,
				SIGNAL( newToolbarConfig( void ) ),
	         SLOT( updateToolbar( void ) ) );
	connect( editDialog,
	         SIGNAL( finished( void ) ),
	         editDialog,
	         SLOT( delayedDestruct( void ) ) );
	editDialog->show();	
}

void BlokkalEEV::updateToolbar( void )
{
	applyMainWindowSettings( KGlobal::config()->group( "BlokkalEEV") );
}
