/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALEEV_H
#define BLOKKALEEV_H

#include <blokkalui/mainwindowview.h>
#include <blokkalui/htmlentrytextedit.h>

namespace Blokkal {
class Entry;
class Blog;
namespace Ui {
class EditEntryWidget;
}
}

/**
 * @short Standard view for editing entries.
 *
 * The GUI of the Blokkal::Ui::EditEntryWidget will be merged with
 * this window's GUI.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BlokkalEEV : public Blokkal::Ui::MainWindowView, public Blokkal::Ui::DropProxy
{
Q_OBJECT
public:
	/**
	 * Creates a new BlokkalEEV.
	 * @param entry edit this entry
	 */
	BlokkalEEV( Blokkal::Entry * entry );

	~BlokkalEEV( void );

	/**
	 * This method is called when a drop event needs to be handled
	 * @param event the event to handle
	 * @param widget the widget that received the drop event
	 */
	virtual bool handleDrop( QDropEvent * event, QWidget * widget );

public Q_SLOTS:
	/**
	 * Opens a dialog asking for the required information and
	 * inserts a link to the image at the current cursor position.
	 * @param url the url to set up
	 */
	void insertImage( const QString & url = QString::null );

	/**
	 * Opens a dialog asking for the required information and
	 * inserts a weblink at the current cursor position.
	 * @param url the url to set up
	 */
	void insertLink( const QString & url = QString::null );

	/**
	 * Frames the selected text with "<b>" and "</b>"
	 * or inserts the tags at the current cursor position if
	 * nothing is selected.
	 */
	void makeTextBold( void );

	/**
	 * Frames the selected text with "<i>" and "</i>"
	 * or inserts the tags at the current cursor position if
	 * nothing is selected.
	 */
	void makeTextItalic( void );

	/**
	 * Frames the selected text with "<s>" and "</s>"
	 * or inserts the tags at the current cursor position if
	 * nothing is selected.
	 */
	void makeTextStrikeout( void );

	/**
	 * Frames the selected text with "<u>" and "</u>"
	 * or inserts the tags at the current cursor position if
	 * nothing is selected.
	 */
	void makeTextUnderline( void );
	
protected:
	/**
	 * Overridden for internal purposes.
	 */
	virtual bool queryClose( void );

private Q_SLOTS:
	void slotContentsChanged( void );
	void updateCaption( void );
	void postEntry( void );
	void saveEntry( void );
	void saveEntryAs( void );

	void toggleCrossposting( bool on );

	void slotBlogSelected( Blokkal::Blog * blog );
	void slotBlogChecked( Blokkal::Blog * blog );
	void slotBlogUnchecked( Blokkal::Blog * blog );

	/**
	 * Called by the drop popup menu.
	 * Asks for a name for the dropped URL and inserts it as weblink.
	 */
	void dropAsLink( void );

	/**
	 * Called by the drop popup menu.
	 * Opens a setup image dialog and inserts the dropped URL as image link.
	 */
	void dropAsImage( void );

	/**
	 * Called by the drop popup menu.
	 * Inserts the dropped URL as text.
	 */
	void dropAsText( void );

	/**
	 * Called by the drop popup menu.
	 * Deletes the dropped URL.
	 */
	void clearDroppedURL( void );

	void showPreview( void );

	void configureToolbar( void );
	void updateToolbar( void );	

private:
	QString baseCaption( void );

	class Private;
	Private * const d;
};

#endif
