/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "crosspostselector.h"
#include "crosspostselector.moc"

#include <blokkalaccountmanager.h>
#include <blokkalaccount.h>
#include <blokkalblog.h>
#include <blokkalio/categorymanager.h>
#include <blokkalblogmodel.h>

#include <kdebug.h>


#include <QSortFilterProxyModel>

//-----------------
//CrossPostDelegate
//-----------------
class CrossPostDelegate::Private {
public:
	Private( void ):
	signalProxy( 0 ) {}

	CrossPostSelector * signalProxy;
};

CrossPostDelegate::CrossPostDelegate( void ) :
Blokkal::Ui::BlogDelegate( Blokkal::Ui::BlogDelegate::DisplayAccountIcon
                           |Blokkal::Ui::BlogDelegate::DisplayBlogIcon
                           |Blokkal::Ui::BlogDelegate::DisplayCheckBoxes),
d( new Private() )
{
}

CrossPostDelegate::~CrossPostDelegate( void )
{
	delete d;
}

void CrossPostDelegate::stateChanged( QAbstractItemModel * model,
                                      const QModelIndex & index,
                                      Qt::CheckState newState )
{
	if( !index.isValid() || !index.parent().isValid() ) {
		return;
	}
	//a blog
	if( !index.parent().parent().isValid() ) {
		Blokkal::Blog * const blog = dynamic_cast<Blokkal::Blog *>( (QObject*) index.data().toULongLong() );
		if( blog ) {
			if( newState == Qt::Checked ) {
				if( d->signalProxy ) {
					emit d->signalProxy->blogChecked( blog );
				}
			}
			else {
				const int accountCount = model->rowCount();
				for( int i = 0; i < accountCount; ++i ) {
					const QModelIndex accountIndex = model->index( i, 0 );
					const int blogCount = model->rowCount( accountIndex );
					for( int j = 0; j < blogCount; ++j ) {
						const QModelIndex blogIndex = model->index( j, 0, accountIndex );
						const Qt::CheckState blogState = static_cast<Qt::CheckState>(blogIndex.data( Qt::CheckStateRole ).toInt() );						
						if( blogIndex != index
						    && blogState == Qt::Checked )
						{
							//we have found another checked blog
							//->continue unchecking
							if( d->signalProxy ) {
								emit d->signalProxy->blogUnchecked( blog );
							}
							const int rowCount = model->rowCount( index );
							for( int i = 0; i < rowCount; ++i ) {
								model->setData( model->index( i, 0, index ),
								                Qt::Unchecked,
								                Qt::CheckStateRole );
							}
							return;
						}
					}
				}

				//if we get here, then this is the last checked blog
				//->check it again and don't emit any signals
				model->setData( index, Qt::Checked, Qt::CheckStateRole );
			}
		}
		else {
			kError() << "data could not be casted to Blokkal::Blog" << endl;
		}
	}
	else { //is a category
		if( newState == Qt::Checked ) {
			Blokkal::Blog * const blog = dynamic_cast<Blokkal::Blog *>( (QObject*) index.parent().data().toULongLong() );
			const Qt::CheckState blogState = static_cast<Qt::CheckState>(index.parent().data( Qt::CheckStateRole ).toInt() );
			if( blogState != Qt::Checked
			    && model->setData( index.parent(), Qt::Checked, Qt::CheckStateRole )
			    && d->signalProxy )
			{
				if( blog ) {
					emit d->signalProxy->blogChecked( blog );
				}
				else {
					kError() << "data could not be casted to Blokkal::Blog" << endl;
				}
			}
			
			//force single selection if blog does not support multiple categories
			if( blog && !( blog->supportedFeatures()
			               & Blokkal::Blog::FeatureMultipleCategories
			               & ~Blokkal::Blog::FeatureCategory ) )
			{
				const int rowCount = model->rowCount( index.parent() );
				for( int i = 0; i < rowCount; ++i ) {
					QModelIndex currentIndex = model->index( i, 0, index.parent() );
					if( currentIndex != index ) {
						model->setData( currentIndex,
					                Qt::Unchecked,
					                Qt::CheckStateRole );
					}
				}
			}
		}
	}
}

//-----------------------
//CrossPostSelectionModel
//-----------------------
class CrossPostSelectionModel::Private {
public:
	Private( void ):
	signalProxy( 0 ) {}

	CrossPostSelector * signalProxy;
};
CrossPostSelectionModel::CrossPostSelectionModel( QAbstractItemModel * model, QObject * parent ):
QItemSelectionModel( model, parent ),
d( new Private() )
{
}

CrossPostSelectionModel::~CrossPostSelectionModel( void )
{
	delete d;
}

void CrossPostSelectionModel::select( const QModelIndex & index, QItemSelectionModel::SelectionFlags command )
{
	if( !index.isValid() || !index.parent().isValid() ) {
		return;
	}
	
	if( !index.parent().parent().isValid() ) {
		QItemSelectionModel::select( index, command );
	}
}

void CrossPostSelectionModel::select( const QItemSelection & selection, QItemSelectionModel::SelectionFlags command )
{
	if( selection.indexes().size() != 1 ) {
		return;
	}

	QModelIndex index = selection.indexes().first();
	if( !index.isValid() || !index.parent().isValid() ) {
		return;
	}	
	
	if( !index.parent().parent().isValid() ) {
		const Qt::CheckState blogState = static_cast<Qt::CheckState>(index.data( Qt::CheckStateRole ).toInt() );
		if( blogState == Qt::Checked ) {
			QItemSelectionModel::select( selection, command );
			if( d->signalProxy && command & QItemSelectionModel::Select ) {
				Blokkal::Blog * const blog = dynamic_cast<Blokkal::Blog *>( (QObject*) index.data().toULongLong() );
				emit d->signalProxy->blogSelected( blog );
			}
		}
	}
}

//-----------------
//CrossPostSelector
//-----------------
class CrossPostSelector::Private {
public:
	Private( QSortFilterProxyModel * sortModel ):
	delegate( 0 ),
	sortModel( sortModel ) {}

	CrossPostDelegate * delegate;
	QSortFilterProxyModel * const sortModel;
};

CrossPostSelector::CrossPostSelector( QWidget * parent ):
Blokkal::Ui::CheckBoxView( new CrossPostDelegate(), parent ),
d( new Private( new QSortFilterProxyModel( this ) ) )
{
	Blokkal::BlogCategoryModel * model = new Blokkal::BlogCategoryModel( this );
	d->sortModel->setSourceModel( model );
	d->sortModel->setSortRole( Blokkal::BlogModel::SortRole );
	setModel( d->sortModel );
	d->delegate = dynamic_cast<CrossPostDelegate*>(itemDelegate());
	d->delegate->d->signalProxy = this;
	CrossPostSelectionModel * selectionModel = new CrossPostSelectionModel( d->sortModel, this );
	selectionModel->d->signalProxy = this;
	setSelectionModel( selectionModel );
	
	setRootIsDecorated( TRUE );
	
	connect( model,
		 SIGNAL( dataChanged( const QModelIndex &, const QModelIndex & ) ),
		 SLOT( resort( void ) ) );
}

CrossPostSelector::~CrossPostSelector( void )
{
	delete d;
}

void CrossPostSelector::selectBlog( const Blokkal::Blog * blog )
{
	Blokkal::Account * const account = blog->account();
	const int accountCount = model()->rowCount();
	for( int i = 0; i < accountCount; ++i ) {
		QModelIndex accountIndex = model()->index( i, 0 );
		if( (Blokkal::Account*)model()->data( accountIndex ).toULongLong() == account ) {
			const int blogCount = model()->rowCount( accountIndex );
			for( int j = 0; j < blogCount; ++j ) {
				QModelIndex blogIndex = model()->index( j, 0, accountIndex );
				if( (Blokkal::Blog*)model()->data( blogIndex ).toULongLong() == blog ) {
					model()->setData( blogIndex, Qt::Checked, Qt::CheckStateRole );
					selectionModel()->select( blogIndex, QItemSelectionModel::SelectCurrent );
					expand( blogIndex );
					expand( accountIndex );
					return;
				}
			}
		}
	}
}

void CrossPostSelector::uncheckBlog( const Blokkal::Blog * blog )
{
	Blokkal::Account * const account = blog->account();
	const int accountCount = model()->rowCount();
	for( int i = 0; i < accountCount; ++i ) {
		QModelIndex accountIndex = model()->index( i, 0 );
		if( (Blokkal::Account*)model()->data( accountIndex ).toULongLong() == account ) {
			const int blogCount = model()->rowCount( accountIndex );
			for( int j = 0; j < blogCount; ++j ) {
				QModelIndex blogIndex = model()->index( j, 0, accountIndex );
				if( (Blokkal::Blog*)model()->data( blogIndex ).toULongLong() == blog ) {
					model()->setData( blogIndex, Qt::Unchecked, Qt::CheckStateRole );
					d->delegate->stateChanged( model(), blogIndex, Qt::Unchecked );
					return;
				}
			}
		}
	}	
}

QStringList CrossPostSelector::categories( const Blokkal::Blog * blog ) const
{
	QStringList categoryIds;
	Blokkal::Account * const account = blog->account();
	const int accountCount = model()->rowCount();
	for( int i = 0; i < accountCount; ++i ) {
		QModelIndex accountIndex = model()->index( i, 0 );
		if( (Blokkal::Account*)model()->data( accountIndex ).toULongLong() == account ) {
			const int blogCount = model()->rowCount( accountIndex );
			for( int j = 0; j < blogCount; ++j ) {
				QModelIndex blogIndex = model()->index( j, 0, accountIndex );
				if( (Blokkal::Blog*)model()->data( blogIndex ).toULongLong() == blog ) {
					const int categoryCount = model()->rowCount( blogIndex );
					for( int k = 0; k < categoryCount; ++k ) {
						QModelIndex categoryIndex = model()->index( k, 0, blogIndex);
						Blokkal::Io::Category * category = dynamic_cast<Blokkal::Io::Category *>( (QObject*) model()->data( categoryIndex ).toULongLong() );
						Qt::CheckState state = static_cast<Qt::CheckState>(categoryIndex.data( Qt::CheckStateRole ).toInt() );
						if( category && state == Qt::Checked ) {
							categoryIds.append( category->id() );
						}
					}
					return categoryIds;
				}
			}
		}
	}
	
	return categoryIds;
}

void CrossPostSelector::setCategories( const Blokkal::Blog * blog, const QStringList & categories )
{
	Blokkal::Account * const account = blog->account();
	const int accountCount = model()->rowCount();
	for( int i = 0; i < accountCount; ++i ) {
		QModelIndex accountIndex = model()->index( i, 0 );
		if( (Blokkal::Account*)model()->data( accountIndex ).toULongLong() == account ) {
			const int blogCount = model()->rowCount( accountIndex );
			for( int j = 0; j < blogCount; ++j ) {
				QModelIndex blogIndex = model()->index( j, 0, accountIndex );
				if( (Blokkal::Blog*)model()->data( blogIndex ).toULongLong() == blog ) {
					const int categoryCount = model()->rowCount( blogIndex );
					for( int k = 0; k < categoryCount; ++k ) {
						QModelIndex categoryIndex = model()->index( k, 0, blogIndex);
						Blokkal::Io::Category * category = dynamic_cast<Blokkal::Io::Category *>( (QObject*) model()->data( categoryIndex ).toULongLong() );
						if( category && categories.contains( category->id() )  ) {
							model()->setData( categoryIndex,
							                  Qt::Checked,
							                  Qt::CheckStateRole );
						}
					}
					return;
				}
			}
		}
	}	
}

void CrossPostSelector::resort(void )
{
	model()->sort(0);
}
