/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef CROSSPOSTSELECTOR_H
#define CROSSPOSTSELECTOR_H

#include <blokkalui/checkboxview.h>
#include <blokkalui/blogdelegate.h>

namespace Blokkal {
class BlogCategoryModel;
class Blog;
}

class CrossPostSelector;
class CrossPostDelegate;
class CrossPostSelectionModel;

class CrossPostSelector : public Blokkal::Ui::CheckBoxView
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param parent parent widget
	 */
	CrossPostSelector( QWidget * parent = 0 );
	virtual ~CrossPostSelector( void );

	void selectBlog( const Blokkal::Blog * blog );
	void uncheckBlog( const Blokkal::Blog * blog );
	QStringList categories( const Blokkal::Blog * blog ) const;
	void setCategories( const Blokkal::Blog * blog, const QStringList & categories );
	
public Q_SLOTS:
	void resort( void );

Q_SIGNALS:
	void blogChecked( Blokkal::Blog * blog );
	void blogUnchecked( Blokkal::Blog * blog );
	void blogSelected( Blokkal::Blog * blog );

private:
	friend class CrossPostDelegate;
	friend class CrossPostSelectionModel;
	class Private;
	Private * const d;
};


class CrossPostDelegate : public Blokkal::Ui::BlogDelegate {
Q_OBJECT
public:
	/**
	 * Constructor
	 */
	CrossPostDelegate( void );
	virtual ~CrossPostDelegate( void );

protected:
	/**
	 * This method is called when the checkstate of @a index
	 * has changed to @a newState.
	 */
	virtual void stateChanged( QAbstractItemModel * model,
	                           const QModelIndex & index,
	                           Qt::CheckState newState );
private:
	friend class CrossPostSelector;
	class Private;
	Private * const d;
};

class CrossPostSelectionModel : public QItemSelectionModel {
Q_OBJECT
public:
	/**
	 * Constructor
	 */
	CrossPostSelectionModel( QAbstractItemModel * model, QObject * parent );
	virtual ~CrossPostSelectionModel( void );

public Q_SLOTS:
	virtual void select( const QModelIndex & index, QItemSelectionModel::SelectionFlags command );
	virtual void select( const QItemSelection & selection, QItemSelectionModel::SelectionFlags command );

private:
	friend class CrossPostSelector;	
	class Private;
	Private * const d;
};

#endif
