/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "dialogs.h"
#include "dialogs.moc"

#include <blokkalaccount.h>
#include <blokkalblog.h>
#include <blokkalio/blogbrowsingdriver.h>
#include <blokkalui/browseentryviewfactory.h>
#include <blokkalui/formattingbackend.h>

#include <klineedit.h>
#include <knuminput.h>
#include <klocale.h>
#include <ktabwidget.h>
#include <khtml_part.h>
#include <kcolorscheme.h>

#include <QLayout>
#include <QGroupBox>
#include <QLabel>
#include <QRadioButton>
#include <QButtonGroup>
#include <QToolButton>

//--------------------
//SetupImageLinkDialog
//--------------------
class SetupImageLinkDialog::Private
{
public:
	Private( void ):
	urlLineEdit( 0 ),
	nameLineEdit( 0 ),
	heightWidthGroupBox( 0 ),
	heightSpinBox( 0 ),
	widthSpinBox( 0 ),
	alignmentGroupBox( 0 ),
	noAlignmentButton( 0 ),
	leftAlignmentButton( 0 ),
	rightAlignmentButton( 0 ),
	centeredAlignmentButton( 0 ) {}

	KLineEdit * urlLineEdit;
	KLineEdit * nameLineEdit;
	
	QGroupBox * heightWidthGroupBox;
	KIntSpinBox * heightSpinBox;
	KIntSpinBox * widthSpinBox;

	QGroupBox * alignmentGroupBox;
	QButtonGroup * alignmentButtonGroup;
	QRadioButton * noAlignmentButton;
	QRadioButton * leftAlignmentButton;
	QRadioButton * rightAlignmentButton;
	QRadioButton * centeredAlignmentButton;
};

SetupImageLinkDialog::SetupImageLinkDialog( QWidget * parent ):
KDialog( parent ),
d( new Private() )
{
	setModal( TRUE );
	setCaption( i18n( "Setup Image" ) );
	setButtons( Ok|Cancel );
	setDefaultButton( Ok );
	
	QWidget * mainWidget = new QWidget( this );

	mainWidget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::MinimumExpanding );
	QVBoxLayout * mainLayout = new QVBoxLayout( mainWidget );
	mainLayout->setMargin( 0 );

	//URL & name
	QHBoxLayout * urlNameLayout = new QHBoxLayout( 0 );
	urlNameLayout->setSpacing( KDialog::spacingHint() );
	mainLayout->addLayout( urlNameLayout );
	
	//the labels
	QVBoxLayout * urlNameLabelLayout = new QVBoxLayout( 0 );
	urlNameLabelLayout->setMargin( 0 );
	urlNameLayout->addLayout( urlNameLabelLayout );
	QLabel * urlLabel = new QLabel( i18n( "&URL" ), mainWidget );
	urlNameLabelLayout->addWidget( urlLabel );
	QLabel * nameLabel = new QLabel( i18n( "&Name" ), mainWidget );
	urlNameLabelLayout->addWidget( nameLabel );

	//the lineedits
	QVBoxLayout * urlNameInputLayout = new QVBoxLayout( 0 );
	urlNameInputLayout->setMargin( 0 );
	urlNameLayout->addLayout( urlNameInputLayout );	
	d->urlLineEdit = new KLineEdit( mainWidget );
	urlLabel->setBuddy( d->urlLineEdit );
	urlNameInputLayout->addWidget( d->urlLineEdit );
	d->nameLineEdit = new KLineEdit( mainWidget );
	nameLabel->setBuddy( d->nameLineEdit );
	urlNameInputLayout->addWidget( d->nameLineEdit );

	//dimensions groupbox
	d->heightWidthGroupBox = new QGroupBox( i18n( "&Include Dimensions" ), mainWidget );
	d->heightWidthGroupBox->setLayout( new QHBoxLayout( d->heightWidthGroupBox ) );
	//d->heightWidthGroupBox->layout()->setMargin( 0 );
	d->heightWidthGroupBox->setCheckable( TRUE );
	d->heightWidthGroupBox->setChecked( FALSE );
	//width
	QLabel * widthLabel = new QLabel( i18n( "&Width" ), d->heightWidthGroupBox );
	d->heightWidthGroupBox->layout()->addWidget( widthLabel );
	d->widthSpinBox = new KIntSpinBox( d->heightWidthGroupBox );
	d->heightWidthGroupBox->layout()->addWidget( d->widthSpinBox );
	widthLabel->setBuddy( d->widthSpinBox );
	d->widthSpinBox->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
	d->widthSpinBox->setMaximum( 32767 );
	d->widthSpinBox->setMinimum( 1 );
	//height
	QLabel * heightLabel = new QLabel( i18n( "&Height"), d->heightWidthGroupBox );
	d->heightWidthGroupBox->layout()->addWidget( heightLabel );
	d->heightSpinBox = new KIntSpinBox( d->heightWidthGroupBox );
	d->heightWidthGroupBox->layout()->addWidget( d->heightSpinBox );
	heightLabel->setBuddy( d->heightSpinBox );
	d->heightSpinBox->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
	d->heightSpinBox->setMaximum( 32767 );
	d->heightSpinBox->setMinimum( 1 );
	mainLayout->addWidget( d->heightWidthGroupBox );

	//alignment groupbox
	d->alignmentGroupBox = new QGroupBox( i18n( "Alignment" ), mainWidget );
	d->alignmentGroupBox->setLayout( new QVBoxLayout( d->alignmentGroupBox ) );
	d->alignmentButtonGroup = new QButtonGroup( d->alignmentGroupBox );
	d->alignmentButtonGroup->setExclusive( TRUE );
	
	d->noAlignmentButton = new QRadioButton( i18n( "None" ), d->alignmentGroupBox );
	d->noAlignmentButton->setChecked( TRUE );
	d->alignmentGroupBox->layout()->addWidget( d->noAlignmentButton );
	d->alignmentButtonGroup->addButton( d->noAlignmentButton );
	
	d->leftAlignmentButton = new QRadioButton( i18n( "Left" ), d->alignmentGroupBox );
	d->alignmentGroupBox->layout()->addWidget( d->leftAlignmentButton );
	d->alignmentButtonGroup->addButton( d->leftAlignmentButton );
	
	d->rightAlignmentButton = new QRadioButton( i18n( "Right" ), d->alignmentGroupBox );
	d->alignmentGroupBox->layout()->addWidget( d->rightAlignmentButton );
	d->alignmentButtonGroup->addButton( d->rightAlignmentButton );
	
	d->centeredAlignmentButton = new QRadioButton( i18n( "Centered" ), d->alignmentGroupBox );
	d->alignmentGroupBox->layout()->addWidget( d->centeredAlignmentButton );
	d->alignmentButtonGroup->addButton( d->centeredAlignmentButton );
	mainLayout->addWidget( d->alignmentGroupBox );
	
	mainLayout->addStretch( 1 );

	setMainWidget( mainWidget );

	restoreDialogSize( KGlobal::config()->group( "SetupImageLinkDialog" ) );
}

SetupImageLinkDialog::~SetupImageLinkDialog( void )
{
	KConfigGroup group = KGlobal::config()->group( "SetupImageLinkDialog" );
	saveDialogSize( group );
	
	delete d;
}

void SetupImageLinkDialog::setURL( const QString & url )
{
	d->urlLineEdit->setText( url );
}

QString SetupImageLinkDialog::link( void ) const
{
	QString linkString = "<img src=\"" +  d->urlLineEdit->text()
		+ "\" alt=\"" + d->nameLineEdit->text()
		+ "\" title=\"" + d->nameLineEdit->text()
		+ "\"";
	if( d->heightWidthGroupBox->isChecked() ) {
		linkString.append( " height=" + QString::number( d->heightSpinBox->value() ) );
		linkString.append( " width=" + QString::number( d->widthSpinBox->value() ) );
	}
	if( d->alignmentButtonGroup->checkedButton() != d->noAlignmentButton ) {
		linkString.append( " style=\"" );
		if( d->alignmentButtonGroup->checkedButton() == d->centeredAlignmentButton ) {
			linkString.append( "display: block; margin-left: auto; margin-right: auto;" );
		}
		else { //left or right
			linkString.append( "display: inline; padding 4px;" );
			if( d->alignmentButtonGroup->checkedButton() == d->leftAlignmentButton ) {
				linkString.append( "float:left; margin: 0 7px 2px 0;" );
			}
			else { //right
				linkString.append( "float:right; margin: 0 0 2px 7px;" );
			}
		}

		linkString.append( "\"");
	}
	linkString.append("/>");
	return linkString;
}

//------------------
//SetupWebLinkDialog
//_-----------------
class SetupWebLinkDialog::Private
{
public:
	Private( Blokkal::Io::BlogBrowsingDriver * driver ):
	browseButton( 0 ),
	urlLineEdit( 0 ),
	titleLineEdit( 0 ),
	driver( driver ) {}

	QToolButton * browseButton;
	
	KLineEdit * urlLineEdit;
	KLineEdit * titleLineEdit;
	Blokkal::Io::BlogBrowsingDriver * const driver;
};

SetupWebLinkDialog::SetupWebLinkDialog( QWidget * parent, Blokkal::Io::BlogBrowsingDriver * driver ):
KDialog( parent ),
d( new Private( driver ) )
{
	setCaption( i18n( "Setup Link" ) );
	setButtons( Ok|Cancel );
	setDefaultButton( Ok );
	setModal( TRUE );
	
	QWidget * mainWidget = new QWidget( this );

	mainWidget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::MinimumExpanding );
	QVBoxLayout * mainLayout = new QVBoxLayout( mainWidget );
	mainLayout->setMargin( 0 );
	QGridLayout * layout = new QGridLayout();
	layout->setMargin( 0 );
	mainLayout->addLayout( layout );

	//the labels
	QLabel * urlLabel = new QLabel( i18n( "&URL" ), mainWidget );
	layout->addWidget( urlLabel, 0, 0 );
	QLabel * titleLabel = new QLabel( i18n( "&Title" ), mainWidget );
	layout->addWidget( titleLabel, 1, 0 );
	
	//the lineedits
	d->urlLineEdit = new KLineEdit( mainWidget );
	urlLabel->setBuddy( d->urlLineEdit );
	layout->addWidget( d->urlLineEdit, 0, 1 );
	d->browseButton = new QToolButton( mainWidget );
	d->browseButton->setToolTip( i18n( "Browse old entries" ) );
	d->browseButton->setIcon( KIcon( "document-open" ) );
	layout->addWidget( d->browseButton, 0, 2 );
	d->titleLineEdit = new KLineEdit( mainWidget );
	titleLabel->setBuddy( d->titleLineEdit );
	layout->addWidget( d->titleLineEdit, 1, 1, -1, -1 );

	mainLayout->addStretch( 1 );

	setMainWidget( mainWidget );

	restoreDialogSize( KGlobal::config()->group( "SetupWebLinkDialog" ) );

	connect( d->browseButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( openEntryBrowser( void ) ) );
	
	if( d->driver ) {
		connect( d->driver->blog()->account(),
		         SIGNAL( connectionStatusChanged( Blokkal::Account* ) ),
		         SLOT(slotAccountStatusChanged( void ) ) );
	}
	
	slotAccountStatusChanged();
}

SetupWebLinkDialog::~SetupWebLinkDialog( void )
{
	KConfigGroup group = KGlobal::config()->group( "SetupWebLinkDialog" );
	saveDialogSize( group );
	
	delete d;
}

void SetupWebLinkDialog::setURL( const QString & url )
{
	d->urlLineEdit->setText( url );
}

QString SetupWebLinkDialog::link( void ) const
{
	return "<a href=\"" +  d->urlLineEdit->text() + "\" title=\"" + d->urlLineEdit->text() + "\">" + d->titleLineEdit->text() + "</a>";
}

void SetupWebLinkDialog::openEntryBrowser( void )
{
	if( d->driver ) {
		Blokkal::Ui::BrowseEntryDialog * const dialog = d->driver->browseBlogDialog( Blokkal::Io::BlogBrowsingDriver::BrowseLink );
		QWidget * const oldDialogParent = dialog->parentWidget();
		const Qt::WindowFlags dialogFlags = dialog->windowFlags();
		dialog->setParent( this, dialogFlags );
		if( dialog->exec() ) {
			d->urlLineEdit->setText( dialog->entryUrl().url() );
		}
		dialog->setParent( oldDialogParent, dialogFlags );
	}
}

void SetupWebLinkDialog::slotAccountStatusChanged( void )
{
	d->browseButton->setEnabled( d->driver
	                             && d->driver->supportedBrowseModes() & Blokkal::Io::BlogBrowsingDriver::BrowseLink
	                             && d->driver->blog()->account()->isConnected() );
}

//-------------
//PreviewDialog
//-------------
class PreviewDialog::Private {
public:
	Private( void ) :
	previewWidget( 0 ) {}

	KTabWidget * previewWidget;
	QString content;
	QMap<const Blokkal::Blog*, QWidget*> blogWidgetMap;
	QList<QWidget*> finishedWidgets;
};


PreviewDialog::PreviewDialog( QWidget * parent ):
KDialog( parent ),
d( new Private() )
{
	setCaption( i18n( "Entry Preview" ) );

	QWidget * const mainWidget = new QWidget( this );
	mainWidget->setLayout( new QVBoxLayout( mainWidget ) );
	setMainWidget( mainWidget );
	
	QLabel * const label = new QLabel( i18n( "Below you can see how the entry will approximately look like.<br>Colors and font styles may differ from the actual appearance in your blog. Images and other remote recources require an Internet-connection to display correctly." ),
	                                   mainWidget );
	label->setWordWrap( TRUE );
	mainWidget->layout()->addWidget( label );

	d->previewWidget = new KTabWidget( mainWidget );
	mainWidget->layout()->addWidget( d->previewWidget );
	connect( d->previewWidget,
	         SIGNAL( currentChanged( int ) ),
	         SLOT( tabChanged( int ) ) );

	restoreDialogSize( KGlobal::config()->group( "PreviewDialog" ) );
}

PreviewDialog::~PreviewDialog( void )
{
	KConfigGroup group = KGlobal::config()->group( "PreviewDialog" );
	saveDialogSize( group );

	delete d;
}

void PreviewDialog::clearPreviews( void )
{
	while( d->previewWidget->count() ) {
		QWidget * const currentWidget = d->previewWidget->widget( 0 );
		d->previewWidget->removeTab( 0 );
		delete currentWidget;
	}

	d->blogWidgetMap.clear();
	d->finishedWidgets.clear();
}

void PreviewDialog::setContent( const QString & content )
{
	d->content = content;
}

void PreviewDialog::addBlog( const Blokkal::Blog * blog, const Blokkal::Ui::FormattingBackend * backend )
{
	if( d->blogWidgetMap.contains( blog ) ) {
		return;
	}

	KHTMLPart * const part = new KHTMLPart( d->previewWidget );
	d->blogWidgetMap.insert( blog, part->widget() );
	const QString label = blog->title().isEmpty() ? blog->id() : blog->title();
	d->previewWidget->setTabTextColor( d->previewWidget->addTab( d->blogWidgetMap[blog], blog->icon(), label ),
	                                   KStatefulBrush( KColorScheme::Button, KColorScheme::NeutralText ).brush( this ).color() );

	connect( part,
	         SIGNAL( completed( void ) ),
	         SLOT( partFinished( void ) ) );
	
	part->begin(); //also clears the widget
	part->write( "<html><head><style type='text/css'>body { font: " );
	part->write( QByteArray::number( KGlobalSettings::generalFont().pointSize() ).constData() );
	part->write( "pt \"");
	part->write( KGlobalSettings::generalFont().family().toAscii().constData() );
	part->write( "\";}</style></head><body>" );
	part->write( backend->formatForPreview( d->content ) );
	part->write("</body></html>" );
	part->end();
}

void PreviewDialog::setCurrentBlog( const Blokkal::Blog * blog )
{
	if( !d->blogWidgetMap.contains( blog ) ) {
		return;
	}
	
	const int index = d->previewWidget->indexOf( d->blogWidgetMap[blog] );
	if( index == -1 ) {
		return;
	}

	d->previewWidget->setCurrentIndex( index );
}

void PreviewDialog::partFinished( void )
{
	if( !sender() ) {
		return;
	}

	KHTMLPart * const part = dynamic_cast<KHTMLPart*>( sender() );
	if( !part ) {
		return;
	}

	const int index = d->previewWidget->indexOf( part->widget() );
	if( index == -1 ) {
		return;
	}
	
	if( index != d->previewWidget->currentIndex() ) {
		d->previewWidget->setTabTextColor( index,
		                                   KStatefulBrush( KColorScheme::Button, KColorScheme::LinkText ).brush( this ).color() );
	}
	else {
		d->previewWidget->setTabTextColor( index,
		                                   KStatefulBrush( KColorScheme::Button, KColorScheme::NormalText ).brush( this ).color() );
	}

	d->finishedWidgets.append( part->widget() );
}

void PreviewDialog::tabChanged( int index )
{
	if( index == -1 ) {
		return;
	}
	
	if( !d->finishedWidgets.contains( d->previewWidget->widget( index ) ) ) {
		return;
	}

	d->previewWidget->setTabTextColor( index,
	                                   KStatefulBrush( KColorScheme::Button, KColorScheme::NormalText ).brush( this ).color() );
}

