/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef DIALOGS_H
#define DIALOGS_H

#include <kdialog.h>

namespace Blokkal {
class Blog;
namespace Io {
class BlogBrowsingDriver;
}
namespace Ui {
class FormattingBackend;
}
}

/**
 * @short A dialog to setup a link to an image.
 *
 * A dialog to setup a link to an image.
 * @author Martin Mueller <orvio@orvio.de>
*/
class SetupImageLinkDialog : public KDialog
{
Q_OBJECT
public:
	/**
	 * Creates a new SetupImageLinkDialog.
	 * @param parent parent widget
	 */
	SetupImageLinkDialog( QWidget * parent = 0 );

	/**
	 * Destructor
	 */
	virtual ~SetupImageLinkDialog( void );

	/**
	 * Sets the url the image link will point to.
	 * @param url
	 */
	void setURL( const QString & url );

	/**
	 * Returns the link tag.
	 * @return the link tag
	 */
	QString link( void ) const;

private:
	class Private;
	Private * const d;
};

/**
 * @short A dialog to setup a link to a website.
 *
 * A dialog to setup a link to a website.
 * @author Martin Mueller <orvio@orvio.de>
*/
class SetupWebLinkDialog : public KDialog
{
Q_OBJECT
public:
	/**
	 * Creates a new SetupWebLinkDialog.
	 * @param parent parent widget
	 * @param driver driver to use to access old entry links
	 */
	SetupWebLinkDialog( QWidget * parent = 0,  Blokkal::Io::BlogBrowsingDriver * driver = 0   );

	/**
	 * Destructor
	 */
	virtual ~SetupWebLinkDialog( void );

	/**
	 * Sets the url the link will point to.
	 * @param url
	 */
	void setURL( const QString & url );

	/**
	 * Returns the link tag.
	 * @return the link tag
	 */
	QString link( void ) const;

private Q_SLOTS:
	void openEntryBrowser( void );
	void slotAccountStatusChanged( void );

private:
	class Private;
	Private * const d;
};

/**
 * @short A dialog that displays entry previews.
 *
 * A dialog that displays entry previews.
 * @author Martin Mueller <orvio@orvio.de>
*/
class PreviewDialog : public KDialog
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param parent parent widget
	 */
	PreviewDialog( QWidget * parent = 0 );

	/**
	 * Destructor
	 */
	virtual ~PreviewDialog( void );

public Q_SLOTS:
	void clearPreviews( void );
	void setContent( const QString & content );
	void addBlog( const Blokkal::Blog * blog, const Blokkal::Ui::FormattingBackend * backend );
	void setCurrentBlog( const Blokkal::Blog * blog );

private Q_SLOTS:
	void partFinished( void );
	void tabChanged( int index );

private:
	class Private;
	Private * const d;
};


#endif
