/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkalmainwindow.h"
#include "blokkalmainwindow.moc"

#include "blokkalapplication.h"
#include "blokkalsystemtray.h"
#include "blokkalblogview.h"
#include "entryqueue/queuestatusmonitor.h"
#include "entryqueue/queuemanagerdialog.h"

#include <blokkalentry.h>
#include <blokkalaccountmanager.h>
#include <blokkalaccount.h>
#include <blokkalblog.h>
#include <blokkalbehavioursettings.h>
#include <blokkalpostentryqueue.h>
#include <blokkalpluginmanager.h>
#include <blokkalplugin.h>
#include <blokkalutils.h>
#include <blokkalui/globalsettings.h>
#include <blokkalui/view.h>
#include <blokkalui/stdaction.h>
#include <blokkalui/blogdelegate.h>
#include <blokkalio/blogbrowsingdriver.h>

#include <klocale.h>
#include <kdialog.h>
#include <kstdaction.h>
#include <kaction.h>
#include <kstatusbar.h>
#include <ksqueezedtextlabel.h>
#include <kstdguiitem.h>
#include <kguiitem.h>
#include <kdeversion.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <kfiledialog.h>
#include <kmessagebox.h>
#include <kactioncollection.h>
#include <kedittoolbar.h>
#include <knotifyconfigwidget.h>
#include <kxmlguifactory.h>

#include <QTimer>
#include <QToolButton>
#include <QLayout>
#include <QCloseEvent>
#include <QDesktopWidget>

class BlokkalMainWindow::Private {
public:
	Private( void ) :
	hideTimer( 0 ),
	queueStatusMonitor( 0 ),
	queueManagerDialog( 0 ),
	systemTray( 0 ),
	blogView( 0 ) {}

	QTimer * hideTimer;

	EntryQueue::QueueStatusMonitor * queueStatusMonitor;
	EntryQueue::QueueManagerDialog * queueManagerDialog;
	BlokkalSystemTray * systemTray;
	BlogView * blogView;
};

BlokkalMainWindow::BlokkalMainWindow( QWidget *parent, Qt::WindowFlags flags  ) :
KXmlGuiWindow( parent, flags ),
d( new Private() )
{
	setAttribute( Qt::WA_DeleteOnClose, FALSE ); //deleted by blokkalapplication
	setAttribute( Qt::WA_QuitOnClose, FALSE );
	
	statusBar()->layout()->setMargin( 0 );
	statusBar()->layout()->setSpacing( 0 );
	d->queueStatusMonitor = new EntryQueue::QueueStatusMonitor( this );
	d->queueStatusMonitor->button()->setIcon( KStandardGuiItem::configure().icon() );
	connect( d->queueStatusMonitor->button(), SIGNAL( clicked( void ) ),
	         this, SLOT( showQueueManager( void ) ) );
	statusBar()->addPermanentWidget( d->queueStatusMonitor, 0 );

	unsigned int viewOptions =
		Blokkal::Ui::BlogDelegate::DisplayAccountIcon
		|Blokkal::Ui::BlogDelegate::DisplayBlogIcon
		|Blokkal::Ui::BlogDelegate::DisplayBlogCount
		|Blokkal::Ui::BlogDelegate::DisplayBlogDescription
		|Blokkal::Ui::BlogDelegate::DisplayAccountStates;
	d->blogView = new BlogView( this, viewOptions );
	setCentralWidget( d->blogView  );

	setupActions();
	
	setAcceptDrops( FALSE );

	connect( Blokkal::BehaviourSettings::self(),
	         SIGNAL( configChanged( void ) ),
	         SLOT( applyConfiguration( void ) ) );
	connect( Blokkal::PluginManager::self(),
	         SIGNAL( pluginLoaded( Blokkal::Plugin * ) ),
	         SLOT( mergePluginGui( Blokkal::Plugin * ) ) );

	//setup GUI is done in BlokkalApplication
	applyConfiguration();
	loadWindowState();
}


BlokkalMainWindow::~BlokkalMainWindow()
{
	if( Blokkal::Ui::GlobalSettings::self()->mainWidget() == this ) {
		Blokkal::Ui::GlobalSettings::self()->setMainWidget( 0 );
	}

	delete d;
}

void BlokkalMainWindow::setupActions( void )
{
	KAction * _action;
	KStandardAction::open( this, SLOT( loadEntry( void ) ), actionCollection() );
	action( KStandardAction::name( KStandardAction::Open ) )->setText( i18n( "Open Local Draft..." ) );
	action( KStandardAction::name( KStandardAction::Open ) )->setStatusTip( i18n( "Open a locally saved entry or draft" ) );
	
	_action = KStandardAction::preferences( this, SLOT( showQueueManager( void ) ), actionCollection() );
	_action->setText( i18n( "Open Queue Manager" ) );
	_action->setStatusTip( i18n( "Opens the queue manager" ) );
	_action->setShortcut( i18n( "Ctrl+M" ) );
	actionCollection()->addAction( "file_show_queue_manager", _action );

	_action = new KAction( KIcon( "network-connect" ), i18n( "Connect All" ), this );
	connect( _action, SIGNAL( triggered( void ) ), SLOT( connectAllAccounts( void ) ) );
	_action->setShortcut( i18n( "Ctrl+C" ) );
	actionCollection()->addAction( "connect_all", _action );

	_action = new KAction( KIcon( "network-disconnect" ), i18n( "Disconnect All" ), this );
	connect( _action, SIGNAL( triggered( void ) ), SLOT( disconnectAllAccounts( void ) ) );
	_action->setShortcut( i18n( "Ctrl+D" ) );
	actionCollection()->addAction( "disconnect_all", _action );

	KStandardAction::quit( BlokkalApplication::self(),
	                       SLOT( quitBlokkal( void ) ),
	                       actionCollection() );
	action( KStandardAction::name( KStandardAction::Quit ) )->setStatusTip( i18n( "Quit Blokkal" ) );

	Blokkal::Ui::StdAction::preferences( actionCollection() );
	action( KStandardAction::name( KStandardAction::Preferences ) )->setStatusTip( i18n( "Opens the configuration dialog" ) );

	setStandardToolBarMenuEnabled( TRUE );
	KStandardAction::configureToolbars( this, SLOT( configureToolbar( void ) ), actionCollection() );
	KStandardAction::configureNotifications( this, SLOT( configureNotifications( void  ) ), actionCollection() );
}

bool BlokkalMainWindow::queryClose( void )
{
	return TRUE;
}

void BlokkalMainWindow::applyConfiguration( void )
{
	//--------------
	//System tray 
	if( Blokkal::BehaviourSettings::showSystemTray() ) {
		if( !d->systemTray ) {
			d->systemTray = new BlokkalSystemTray( this );
			d->systemTray->show();
		}
	}
	else {
		delete d->systemTray;
		d->systemTray = 0;
	}	
	
	//--------------------
	//Auto hiding
	if( Blokkal::BehaviourSettings::self()->autoHide() ) {
		if( isVisible() ) {
			if( !d->hideTimer ) {
				d->hideTimer = new QTimer( this );
				connect( d->hideTimer,
				         SIGNAL( timeout( void ) ),
				         SLOT( maybeAutoHide( void ) ) );
			}
			d->hideTimer->start( Blokkal::BehaviourSettings::autoHideTimeout() * 1000 );
		}
	}
	else {
		delete d->hideTimer;
		d->hideTimer = 0;
	}
}

void BlokkalMainWindow::showEvent( QShowEvent * /*event*/ )
{
	if( Blokkal::BehaviourSettings::self()->autoHide() ) {
		if( !d->hideTimer ) {
			d->hideTimer = new QTimer( this );
			connect( d->hideTimer,
			         SIGNAL( timeout( void ) ),
			         SLOT( maybeAutoHide( void ) ) );
		}
		d->hideTimer->start( Blokkal::BehaviourSettings::autoHideTimeout() * 1000 );
	}
}

void BlokkalMainWindow::showQueueManager( void )
{
	if( !d->queueManagerDialog ) {
		d->queueManagerDialog = new EntryQueue::QueueManagerDialog( this );
	}
	d->queueManagerDialog->show();
}

void BlokkalMainWindow::loadEntry( void )
{
	//kdDebug() << k_funcinfo << endl;
	QString openFileName = KFileDialog::getOpenFileName( KUrl("kfiledialog:///entryfiles"), "application/x-blokkal", this );
	if( !openFileName.isNull() ) {
		bool canceled = FALSE;
		
		Blokkal::Entry * entry = Blokkal::Entry::open( openFileName, &canceled );
		if( entry ) {
			entry->view()->showView();
		}
		else if( !canceled ) {
			KMessageBox::error( this,  i18n( "An error occured while loading the entry!" ) );
		}
	}
	//else means dialog canceled

}

bool BlokkalMainWindow::queryExit( void )
{
	if( d->systemTray
	    && !BlokkalApplication::self()->isShuttingDown()
	    && !kapp->sessionSaving() )
	{
		return FALSE;
	}
	/*
	kDebug() << "*****************************************" << endl;
	kDebug() << "*        Shutting down Blokkal          *" << endl;
	if( kapp->sessionSaving() ) {
		kDebug() << "* Session save in progress              *" << endl;
	}
	if( BlokkalApplication::self()->isShuttingDown() ) {
		kDebug() << "* isShuttingDown() returns TRUE         *" << endl;
	}
	if( isHidden() ) {
		kDebug() << "* Mainwindow is hidden                  *" << endl;
	}
	kDebug() << "*****************************************" << endl;
	*/

	//really shutting down
	//actuall i'm not really sure about this, since
	//the KMainWindow docs state, that the app may not actually
	//exit due to a canceled session shutdown, so this should
	//propably go into a slot connect to QApplication::aboutToQuit()
	//OTOH i'm somehow not really confident that that solution
	//would work

	d->blogView->saveTreeState();
	Blokkal::PostEntryQueue::self()->stopSyncing();
	//need to delete all dialogs before the mainwindow
	//that is neccessary to avoid a crash on session logout
	kDebug() << "killing dialogs" << endl;
	const Blokkal::AccountList accounts = Blokkal::AccountManager::self()->accounts();
	for( int i = 0; i < accounts.size(); ++i ) {
		const QList<Blokkal::Blog*> blogs = accounts[i]->blogs();
		for( int j = 0; j < blogs.size(); ++j ) {
			blogs[j]->blogBrowsingDriver()->killDialogs();
		}
	}
	kDebug() << "done" << endl;
	Blokkal::PluginManager::self()->shutdown();
	saveWindowState();
	
	return TRUE;
}

void BlokkalMainWindow::closeEvent( QCloseEvent * event )
{
	//closing without quiting
	if( d->systemTray
	    && !BlokkalApplication::self()->isShuttingDown()
	    && !kapp->sessionSaving() )
	{
		if( settingsDirty() && autoSaveSettings() ) {
			saveAutoSaveSettings();
		}
		
		if ( queryClose() ) {
			event->accept();
		}
		else {
			event->ignore();
		}
	}
	else { //close and quit
		KXmlGuiWindow::closeEvent( event );
	}
}

void BlokkalMainWindow::saveProperties( KConfigGroup & config )
{
	config.writeEntry( "mainwindow-hidden", isHidden() );
}

void BlokkalMainWindow::readProperties( const KConfigGroup & config )
{
	kDebug() << "keys in group: " << config.keyList() << endl;
	if( config.readEntry( "mainwindow-hidden", Blokkal::BehaviourSettings::startHidden() ) ) {
		hide();
	}
	else {
		show();
	}
}

void BlokkalMainWindow::restoreBlogView( void )
{
	d->blogView->restoreTreeState();
}

void BlokkalMainWindow::hideEvent( QHideEvent * event )
{
	Q_UNUSED( event );
	delete d->hideTimer;
	d->hideTimer = 0;
}

void BlokkalMainWindow::maybeAutoHide( void )
{
	if( !isActiveWindow() ) {
		hide();
	}
}

void BlokkalMainWindow::configureToolbar( void )
{
	KConfigGroup configGroup( KGlobal::config(), "MainWindow" );
	saveMainWindowSettings( configGroup );
 	KEditToolBar * const editDialog = new KEditToolBar( factory() );
	connect( editDialog,
				SIGNAL( newToolbarConfig( void ) ),
	         SLOT( updateToolbar( void ) ) );
	connect( editDialog,
	         SIGNAL( finished( void ) ),
	         editDialog,
	         SLOT( delayedDestruct( void ) ) );
	editDialog->show();
}

void BlokkalMainWindow::updateToolbar( void )
{
	applyMainWindowSettings( KGlobal::config()->group( "MainWindow") );
}

void BlokkalMainWindow::configureNotifications( void )
{
	KNotifyConfigWidget::configure( this );
}

void BlokkalMainWindow::connectAllAccounts( void )
{
	const Blokkal::AccountList accounts = Blokkal::AccountManager::self()->accounts();
	for(int i = 0; i < accounts.size(); ++i ) {
		Blokkal::Account * const account = accounts[i];
		if( account->isDisconnected() ) {
			account->connectAccount();
		}
	}
}

void BlokkalMainWindow::disconnectAllAccounts( void )
{
	const Blokkal::AccountList accounts = Blokkal::AccountManager::self()->accounts();
	for(int i = 0; i < accounts.size(); ++i ) {
		Blokkal::Account * const account = accounts[i];
		if( !account->isDisconnected() ) {
			account->disconnectAccount();
		}
	}	
}

void BlokkalMainWindow::mergePluginGui( Blokkal::Plugin * plugin )
{
	guiFactory()->addClient( plugin );
}

void BlokkalMainWindow::saveWindowState(void )
{
	KConfigGroup windowStateConfig = KConfigGroup( KGlobal::config(), "MainWindow State" );

	const int screenNumber = QApplication::desktop()->screenNumber( parentWidget() );
	QRect desktopSize = QApplication::desktop()->screenGeometry( screenNumber );

	if( QApplication::desktop()->isVirtualDesktop() ) {
		desktopSize = QApplication::desktop()->screenGeometry( QApplication::desktop()->screen() );
	}

	const QString key = QString::fromLatin1( "mainwindow-position %1x%2" ).arg( desktopSize.width() ).arg( desktopSize.height() );
	windowStateConfig.writeEntry( key, pos() );
}

void BlokkalMainWindow::loadWindowState(void )
{
	KConfigGroup windowStateConfig = KConfigGroup( KGlobal::config(), "MainWindow State" );

	const int screenNumber = QApplication::desktop()->screenNumber( parentWidget() );
	QRect desktopSize = QApplication::desktop()->screenGeometry( screenNumber );

	if( QApplication::desktop()->isVirtualDesktop() ) {
		desktopSize = QApplication::desktop()->screenGeometry( QApplication::desktop()->screen() );
	}

	const QString key = QString::fromLatin1( "mainwindow-position %1x%2" ).arg( desktopSize.width() ).arg( desktopSize.height() );
	if( windowStateConfig.hasKey( key ) ) {
		move( windowStateConfig.readEntry( key, QPoint() ) );
	}	
}
