/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALMAINWINDOW_H
#define BLOKKALMAINWINDOW_H

#include <kxmlguiwindow.h>

namespace Blokkal {
class Account;
class Plugin;
}

namespace EntryQueue {
class QueueStatusMonitor;
class QueueManagerDialog;
}

class QAction;
class KSqueezedTextLabel;

class QTimer;

/**
 * @short This class acts as mainwindow for Blokkal.
 *
 * This class acts as mainwindow for Blokkal.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BlokkalMainWindow : public KXmlGuiWindow
{
Q_OBJECT
public:
	/**
	 * Creates a new BlokkalMainWindow.
	 * @param parent the parent widget, 0 makes this window a main window
	 * @param flags flags
	 */
	BlokkalMainWindow( QWidget * parent = 0, Qt::WindowFlags flags = KDE_DEFAULT_WINDOWFLAGS );
	~BlokkalMainWindow( void );

	void restoreBlogView( void );

public slots:
	/**
	 * Shows the queue manager
	 */
	void showQueueManager( void );

protected:
	/**
	 * Reimplemented from KMainWindow. Decides if the window (and the application)
	 * is acutally closed or just hidden
	 */
	virtual bool queryClose( void );

	virtual bool queryExit( void );

	virtual void closeEvent( QCloseEvent * event );

	/**
	 * Reeimplemented for auto hiding.
	 * @param event the event
	 */
	virtual void showEvent( QShowEvent * event );
	
	/**
	 * Reeimplemented for auto hiding.
	 * @param event the event
	 */
	virtual void hideEvent( QHideEvent * event );

	virtual void saveProperties( KConfigGroup & config );
	virtual void readProperties( const KConfigGroup & config );

private slots:
	void applyConfiguration( void );
	void loadEntry( void );
	void maybeAutoHide( void );

	void configureToolbar( void );
	void updateToolbar( void );
	void configureNotifications( void );

	void connectAllAccounts( void );
	void disconnectAllAccounts( void );

	void mergePluginGui( Blokkal::Plugin * plugin );

private:
	void setupActions( void );
	
	void saveWindowState( void );
	void loadWindowState( void );

	class Private;
	Private * const d;
};

#endif
