/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkalsystemtray.h"
#include "blokkalsystemtray.moc"

#include "blokkalapplication.h"
#include "blokkalmainwindow.h"
#include "blokkalactiondispatcher.h"

#include <blokkalaccountmanager.h>
#include <blokkalaccount.h>
#include <blokkalblog.h>
#include <blokkalutils.h>
#include <blokkalactiontargetrelay.h>
#include <blokkalui/stdaction.h>

#include <kaction.h>
#include <kactioncollection.h>
#include <kmenu.h>
#include <klocale.h>
#include <kdebug.h>
#include <kiconeffect.h>
#include <kpassivepopup.h>
#include <kaboutdata.h>
#include <kxmlguifactory.h>

#include <QTimer>
#include <QImage>

class BlokkalSystemTray::Private {
public:
	Private( void ):
	menusConnected( FALSE ),
	hoveredAction( 0 ),
	realCurrentAccount( 0 ),
	realCurrentBlog( 0 ) {}

	bool menusConnected;
	QAction * hoveredAction;
	QMap<QAction*, Blokkal::Account*> actionAccountMap;

	//These two fields are used to remember the real
	//current objects, i.e. the ones selected in the
	//blogview of the mainwindow
	Blokkal::Account * realCurrentAccount;
	Blokkal::Blog * realCurrentBlog;
};

BlokkalSystemTray::BlokkalSystemTray( QWidget * parent ):
KSystemTrayIcon( parent ),
d( new Private() )
{
	setIcon( loadIcon( "blokkal" ) );

	Blokkal::Ui::StdAction::preferences( actionCollection() );
	if( actionCollection()->action( KStandardAction::name( KStandardAction::Quit ) ) ) {
		actionCollection()->removeAction( actionCollection()->action( KStandardAction::name( KStandardAction::Quit ) ) );
	}
	KStandardAction::quit( BlokkalApplication::self(),
	                       SLOT( quitBlokkal( void ) ),
	                       actionCollection() );
	setContextMenu( new KMenu( parent ) );
	
	Blokkal::AccountList accounts = Blokkal::AccountManager::self()->accounts();
	if( !accounts.isEmpty() ) {
		for( Blokkal::AccountList::ConstIterator it = accounts.begin();
			  it != accounts.end();
		     ++it )
		{
			handleNewAccount( *it );
		}
	}

	connect( Blokkal::AccountManager::self(),
	         SIGNAL( accountRegistered( Blokkal::Account * ) ),
	         SLOT( handleNewAccount( Blokkal::Account * ) ) );
	connect( Blokkal::AccountManager::self(),
	         SIGNAL( accountUnregistered( Blokkal::Account * ) ),
	         SLOT( reevaluateAccountStatus( void ) ) );
	connect( contextMenu(),
	         SIGNAL( aboutToShow( void ) ),
	         SLOT( updateMenu( void ) ) );
	connect( contextMenu(),
	         SIGNAL( aboutToHide( void ) ),
	         SLOT( restoreActionTargets( void ) ) );
	
	//done to reconfigure the account menu for
	//account menu actions
	connect( contextMenu(),
	         SIGNAL( hovered( QAction * ) ),
	         SLOT( saveHoverAction( QAction * ) ) );
}

BlokkalSystemTray::~BlokkalSystemTray( void )
{
	delete d;
}

void BlokkalSystemTray::handleNewAccount( Blokkal::Account * account )
{
	if( !account ) {
		return;
	}

	QTimer::singleShot( 0, this, SLOT( reevaluateAccountStatus( void ) ) );

	connect( account,
	         SIGNAL( connectionStatusChanged( Blokkal::Account * ) ),
	         SLOT( reevaluateAccountStatus( void ) ) );
}

void BlokkalSystemTray::reevaluateAccountStatus( void )
{
	QString tooltip = QString::null;
	QString connected = i18n( "Connected" );
	QString disconnected = i18n( "Disconnected" );

	unsigned int accountsOnline = 0;
	Blokkal::AccountList accounts = Blokkal::AccountManager::self()->accounts();
	if( !accounts.isEmpty() ) {
		tooltip += "<p style='white-space:pre'>";
		for( Blokkal::AccountList::Iterator it = accounts.begin();
			  it != accounts.end();
		     ++it )
		{
			if( it != accounts.begin() ) {
				tooltip += "<br>";
			}
			tooltip += i18n( "<img src=\"blokkal:///accounts/%1/stateicon\"/> <b>%2</b> (%3)",
			                 (*it)->id(),
			                 (*it)->id(),
			                 (*it)->isConnected() ? connected : disconnected );
			if( (*it)->isConnected() ) {
				accountsOnline++;
			}
		}
		tooltip += "</p>";
	}
	else {
		tooltip = KGlobal::mainComponent().aboutData()->shortDescription();
	}
	
	setToolTip( tooltip );

	QImage icon = loadIcon( "blokkal" ).pixmap(22,22).toImage();
	if( accountsOnline <= 0 ) {
		KIconEffect::toGray( icon, 0.7 );
	}

	setIcon( KIcon( QPixmap::fromImage( icon ) ) );
}

void BlokkalSystemTray::updateMenu( void )
{
	//save current account/blog to be restored
	//when the menu is closed
	d->realCurrentAccount = Blokkal::ActionTargetRelay::currentAccount();
	d->realCurrentBlog = Blokkal::ActionTargetRelay::currentBlog();
	
	contextMenu()->clear();
	d->actionAccountMap.clear();
	static_cast<KMenu*>(contextMenu())->addTitle( qApp->windowIcon(), KGlobal::caption() );
	if( !d->menusConnected ) {
		connect( BlokkalApplication::self()->actionDispatcher()->configureAccountMenu(),
		         SIGNAL( aboutToHide( void ) ),
		         SLOT( configureDynamicMenues( void ) ) );
		d->menusConnected = TRUE;
	}

	//add account menus to popup
	Blokkal::AccountList accounts = Blokkal::AccountManager::self()->accounts();
	if( !accounts.isEmpty() ) {
		for( Blokkal::AccountList::ConstIterator it = accounts.begin();
				it != accounts.end();
				++it )
		{
			QAction * const menuAction = new KAction( (*it)->id(), contextMenu() );
			contextMenu()->addAction( menuAction );
			menuAction->setMenu( BlokkalApplication::self()->actionDispatcher()->accountMenu() );
			d->actionAccountMap.insert( menuAction, *it );
			menuAction->setIcon( (*it)->isConnected() ? (*it)->icon().pixmap(16,16) : Blokkal::Utils::fadePixmap( (*it)->icon().pixmap(16,16) ) );
		}

		contextMenu()->addSeparator();
	}
	
	contextMenu()->addAction( actionCollection()->action( KStandardAction::name( KStandardAction::Preferences ) ) );
	contextMenu()->addSeparator();
	contextMenu()->addAction( actionCollection()->action( "minimizeRestore" ) );
	contextMenu()->addAction( actionCollection()->action( KStandardAction::name( KStandardAction::Quit ) ) );

	if ( parentWidget() ) {
		QAction * action = actionCollection()->action( "minimizeRestore" );
		if( parentWidget()->isVisible() ) {
			action->setText( i18n("&Minimize") );
		}
		else {
			action->setText( i18n("&Restore") );
		}
	}
}

void BlokkalSystemTray::configureDynamicMenues( void )
{
	if( !d->actionAccountMap.keys().contains( d->hoveredAction ) ) {
		return;
	}

	Blokkal::ActionTargetRelay::setCurrentAccount( d->actionAccountMap[d->hoveredAction] );
	BlokkalApplication::self()->actionDispatcher()->configureAccountMenu( BlokkalActionDispatcher::IncludeBlogMenus );
}

void BlokkalSystemTray::saveHoverAction( QAction * action )
{
	d->hoveredAction = action;
	if( BlokkalApplication::self()->actionDispatcher()->accountMenu()->isVisible() ) {
		return;
	}

	configureDynamicMenues();
}

void BlokkalSystemTray::restoreActionTargets( void )
{
	QTimer::singleShot( 0, this, SLOT( restoreActionTargetsInner( void ) ) );
}

void BlokkalSystemTray::restoreActionTargetsInner( void )
{
	Blokkal::ActionTargetRelay::setCurrentAccount( d->realCurrentAccount );
	Blokkal::ActionTargetRelay::setCurrentBlog( d->realCurrentBlog );
}
