/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALUIACCOUNTCHOOSER_H
#define BLOKKALUIACCOUNTCHOOSER_H

#include <QtGui/QTreeView>

namespace Blokkal {
class Account;
}

class AccountListViewItem;

/**
 * @short Account selection widget.
 *
 * An account selection widget.
 * It fetches the current account list from the account manager,
 * listens for changes and displays them accordingly.
 * @author Martin Mueller <orvio@orvio.de>
 */
class AccountChooser : public QTreeView
{
Q_OBJECT
public:
	/**
	 * Creates a new AccountChooser
	 *
	 * @param parent the parent widget
	 */
	AccountChooser( QWidget * parent = 0 );

	/**
	 * Destroys the widget.
	 */
	~AccountChooser( void );

	/**
	 * This method returns the selected account, or 0 if
	 * no account is selected.
	 *
	 * @return the selected account
	 */
	Blokkal::Account * selectedAccount( void ) const;
	
Q_SIGNALS:
	/**
	 * This signal is emitted, when a new account is selected.
	 * If @a account is 0, the account selection has been cleared.
	 * This does not necessarily mean, that no item is selected at all.
	 * Just no account.
	 *
	 * @param account the selected account, or 0 if thelast account has been unselected
	 */
	void accountSelectionChanged( Blokkal::Account * account );
	
public Q_SLOTS:
	void resort( void );

private Q_SLOTS:
	void emitAccountSelectionChanged( void );
		
private:
	class Private;
	Private * const d;	
};

/**
 * Wraps information provided by Blokkal::AccountManager.
 * @author Martin Mueller <orvio@orvio.de>
 */
class AccountListModel : public QAbstractItemModel {
Q_OBJECT
public:
	enum CustomRoles {
		SortRole = Qt::UserRole + 1
	};

	/**
	 * Constructor
	 */
	AccountListModel( AccountChooser * parent );
	virtual ~AccountListModel( void );

	virtual QModelIndex index( int row,
	                           int column,
	                           const QModelIndex & parent = QModelIndex() ) const;
	virtual QModelIndex parent( const QModelIndex & child ) const;
	virtual int rowCount( const QModelIndex & parent = QModelIndex() ) const;
	virtual int columnCount( const QModelIndex & parent = QModelIndex() ) const;
	virtual QVariant data( const QModelIndex & index,
	                       int role = Qt::DisplayRole) const;

	virtual QVariant headerData( int section,
	                             Qt::Orientation orientation,
	                             int role = Qt::DisplayRole ) const;
	
	Blokkal::Account * accountAt( int row ) const;
			
protected Q_SLOTS:
	virtual void accountWeightChanged( Blokkal::Account * account );
	virtual void connectAccount( Blokkal::Account * account );
	virtual void removeAccount( Blokkal::Account * account );

private:
	class Private;
	Private * const d;
};


#endif
