/***************************************************************************
 *   Copyright (C) 2007- 2009 by Martin Mueller                            *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "accountsconfig.h"
#include "accountsconfig.moc"

#include "accountwizard.h"
#include "accountchooser.h"

#include <blokkalaccountmanager.h>
#include <blokkalaccount.h>
#include <blokkalprotocol.h>
#include <blokkalui/editaccountwidget.h>

#include <kgenericfactory.h>
#include <kdeversion.h>
#include <kpushbutton.h>
#include <klocale.h>
#include <kstdguiitem.h>
#include <kmessagebox.h>

#include <qlayout.h>
#include <qtooltip.h>

K_PLUGIN_FACTORY( BlokkalAccountsConfigFactory, registerPlugin<AccountsConfig>(); )
K_EXPORT_PLUGIN( BlokkalAccountsConfigFactory("kcm_blokkal_accountsconfig") )
;

class AccountsConfig::Private {
public:
	Private( void ) {}

	QHBoxLayout * mainLayout;
	AccountChooser * accountChooser;

	QVBoxLayout * buttonLayout;
	KPushButton * addAccountButton;
	KPushButton * modifyAccountButton;
	KPushButton * removeAccountButton;
	
	KPushButton * accountUpButton;
	KPushButton * accountDownButton;
};

AccountsConfig::AccountsConfig( QWidget * parent, const QVariantList & args) :
KCModule( BlokkalAccountsConfigFactory::componentData(), parent ),
d( new Private() )
{
	Q_UNUSED( args );
	
	d->mainLayout = new QHBoxLayout( this );
	d->mainLayout->setSpacing( KDialog::spacingHint() );

	d->accountChooser = new AccountChooser( this );
	d->mainLayout->addWidget( d->accountChooser );

	d->buttonLayout = new QVBoxLayout( 0 );
	d->buttonLayout->setSpacing( KDialog::spacingHint() );

	d->addAccountButton = new KPushButton( KStandardGuiItem::add(), this );
	d->addAccountButton->setToolTip( i18n( "Add a new account" ) );
	d->addAccountButton->setText( i18n( "Add..." ) );
	d->buttonLayout->addWidget( d->addAccountButton );
	
	d->modifyAccountButton = new KPushButton( KStandardGuiItem::configure(), this );
	d->modifyAccountButton->setToolTip( i18n( "Configure the selected account" ) );
	d->modifyAccountButton->setText( i18n( "Configure..." ) );
	d->buttonLayout->addWidget( d->modifyAccountButton );
	
	d->removeAccountButton = new KPushButton( KStandardGuiItem::remove(), this );
	d->removeAccountButton->setToolTip( i18n( "Remove the selected account" ) );
	d->removeAccountButton->setText( i18n( "Remove" ) );
	d->buttonLayout->addWidget( d->removeAccountButton );
	d->mainLayout->addLayout( d->buttonLayout );

	d->buttonLayout->addStretch();
	
	d->accountUpButton = new KPushButton( KIcon("go-up") ,QString(), this);
	d->buttonLayout->addWidget( d->accountUpButton );
	d->accountDownButton = new KPushButton( KIcon("go-down") ,QString(), this);
	d->buttonLayout->addWidget( d->accountDownButton );
	
	connect( d->accountChooser, SIGNAL( accountSelectionChanged( Blokkal::Account * ) ),
	         this, SLOT( updateButtonStates( Blokkal::Account *) ) );

	connect( d->addAccountButton, SIGNAL( clicked( void ) ),
	         this, SLOT( showAddAccountDialog( void ) ) );

	connect( d->modifyAccountButton, SIGNAL( clicked( void ) ),
	         this, SLOT( showConfigureAccountDialog( void ) ) );
	
	connect( d->removeAccountButton, SIGNAL( clicked( void ) ),
	         this, SLOT( deleteSelectedAccount( void ) ) );
	
	connect( d->accountUpButton, SIGNAL( clicked( void ) ),
		 this, SLOT( moveSelectedAccountUp( void ) ) );
	
	connect( d->accountDownButton, SIGNAL( clicked( void ) ),
		 this, SLOT( moveSelectedAccountDown( void ) ) );
	
	updateButtonStates();

	setButtons( Help );
}

AccountsConfig::~AccountsConfig( void )
{
	delete d;
}

void AccountsConfig::save( )
{
}

void AccountsConfig::load( )
{
}

void AccountsConfig::updateButtonStates( Blokkal::Account * account )
{
	d->modifyAccountButton->setEnabled( account );
	d->removeAccountButton->setEnabled( account );
	
	d->accountUpButton->setEnabled( account );
	d->accountDownButton->setEnabled( account );
}

void AccountsConfig::showAddAccountDialog( void )
{
	BlokkalAccountWizard * wizard = new BlokkalAccountWizard( this );
	
	wizard->exec();

	delete wizard;
}

void AccountsConfig::deleteSelectedAccount( void )
{
	Blokkal::Account * account = d->accountChooser->selectedAccount();
	if( account ) {
		int result = KMessageBox::questionYesNo( this,
		                                         i18n( "Do you really want to delete the account \"%1\"?", account->id() ) );
		if( result == KMessageBox::Yes ) {
			account->removeAccount();
		}
	}	
}

void AccountsConfig::moveSelectedAccountUp( void )
{
	Blokkal::Account * account = d->accountChooser->selectedAccount();
	if( !account ) {
		return;
	}
		
	//Set all account weights first and then switch weight with neighbour here
	//All accounts get their weight set to their index in the list,
	//this makes adjusting the weights correctly easier.
	int weight = 0;
	Blokkal::AccountList accountList = Blokkal::AccountManager::self()->accounts();
	for( Blokkal::AccountList::ConstIterator it = accountList.begin();
	     it != accountList.end();
	     ++it )
	{
		(*it)->setWeight(weight);
		weight++;
	}
	
	int accountIndex = accountList.indexOf( account );
	if( accountIndex <= 0 ) {
		//account is already on top of list, no point in adjusting anything
		return;
	}
	
	accountList.at( accountIndex - 1 )->setWeight( accountIndex );	
	account->setWeight( accountIndex - 1 );
}

void AccountsConfig::moveSelectedAccountDown( void )
{
	Blokkal::Account * account = d->accountChooser->selectedAccount();
	if( !account ) {
		return;
	}
		
	//Set all account weights first and then switch weight with neighbour here
	//All accounts get their weight set to their index in the list,
	//this makes adjusting the weights correctly easier.
	int weight = 0;
	Blokkal::AccountList accountList = Blokkal::AccountManager::self()->accounts();
	for( Blokkal::AccountList::ConstIterator it = accountList.begin();
		    it != accountList.end();
		    ++it )
	{
		(*it)->setWeight(weight);
		weight++;
	}
	
	int accountIndex = accountList.indexOf( account );
	if( accountIndex >= (accountList.size() - 1 ) ) {
		//account is already at the end of the list, no point in adjusting anything
		return;
	}
	
	accountList.at( accountIndex + 1 )->setWeight( accountIndex );
	account->setWeight( accountIndex + 1 );
}

void AccountsConfig::showConfigureAccountDialog( void )
{
	Blokkal::Account * account = d->accountChooser->selectedAccount();
	if( !account ) {
		return;
	}
	
	( new ConfigureAccountDialog( this, account) )->show();	
}

//--------------------------
//ConfigureAccountDialog
//--------------------------
class ConfigureAccountDialog::Private {
public:
	Private( void ) {}

	Blokkal::Ui::EditAccountWidget * editWidget;
};

ConfigureAccountDialog::ConfigureAccountDialog( QWidget * parent, Blokkal::Account * account ) :
KDialog( parent ),
d( new Private() )
{
	setCaption( i18n( "Configure %1", account->id() ) );
	setButtons( Ok|Cancel );
	setDefaultButton( Ok );
	setModal( TRUE );

	d->editWidget = account->protocol()->createEditAccountWidget( account, this );
	setMainWidget( d->editWidget );

	connect( d->editWidget, SIGNAL( valuesChanged( void) ),
	         this, SLOT( validateValues( void ) ) );

	connect( this, SIGNAL( okClicked( void ) ),
	         this, SLOT( applyNewSettings( void ) ) );
	
	connect( this, SIGNAL( hidden( void ) ),
	         this, SLOT( delayedDestruct( void ) ) );


	restoreDialogSize( KGlobal::config()->group( "ConfigureAccountDialog" ) );
}


ConfigureAccountDialog::~ConfigureAccountDialog( void )
{
	KConfigGroup group = KGlobal::config()->group( "ConfigureAccountDialog" );
	saveDialogSize( group );		
	delete d;
}

void ConfigureAccountDialog::validateValues( void )
{
	enableButton( Ok, d->editWidget->isValid() );
}

void ConfigureAccountDialog::applyNewSettings( void )
{
	Blokkal::AccountManager::self()->registerAccount( d->editWidget->apply() );
}
