/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ACCOUNTSCONFIG_H
#define ACCOUNTSCONFIG_H

#include <kcmodule.h>
#include <kdialog.h>

class QWidget;
namespace Blokkal {
class Account;
}

class AccountsConfig : public KCModule
{
Q_OBJECT
public:
	/**
	 * Constructor
	 */
	AccountsConfig( QWidget * parent, const QVariantList & args );

	virtual ~AccountsConfig( void );

	virtual void save();
	virtual void load();

private slots:
	void updateButtonStates( Blokkal::Account * account = 0 );
	void showAddAccountDialog( void );
	void showConfigureAccountDialog( void );
	void deleteSelectedAccount( void );
	
	void moveSelectedAccountUp( void );
	void moveSelectedAccountDown( void );

private:
	class Private;
	Private * const d;
};

/**
 * @short This class provides a dialog to configure existing accounts.
 *
 * This class provides a dialog to configure existing accounts.
 * @author Martin Mller <orvio@orvio.de>
 */
class ConfigureAccountDialog : public KDialog {
Q_OBJECT
public:
	/**
	 * Creates a new ConfigureAccountDialog. The dialog destroys itself when
	 * it is closed, and updates the account when the OK button is clicked.
	 *
	 * @param parent parent widget
	 * @param account the account to configure, may not be 0!
	 */
	ConfigureAccountDialog( QWidget * parent, Blokkal::Account * account );

	virtual ~ConfigureAccountDialog( void );

private slots:
	void validateValues( void );
	void applyNewSettings( void );

private:
	class Private;
	Private * const d;
};
#endif
