/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "accountwizard.h"
#include "accountwizard.moc"

#include "rsddatafetcher.h"

#include <blokkalpluginmanager.h>
#include <blokkalui/editaccountwidget.h>
#include <blokkalprotocol.h>
#include <blokkalaccountmanager.h>
#include <blokkalaccount.h>
#include <blokkalrsdconfigurationdata.h>

#include <kdebug.h>
#include <kdialog.h>
#include <klocale.h>
#include <kiconloader.h>
#include <kpushbutton.h>
#include <klineedit.h>

#include <QLayout>
#include <QLabel>
#include <QCheckBox>
#include <QTreeView>
#include <QSortFilterProxyModel>
#include <QGroupBox>


BlokkalAccountWizard::BlokkalAccountWizard( QWidget *parent ) :
KAssistantDialog( parent ),
m_editAccountPage(),
m_accountName(),
m_fetcher( new RsdDataFetcher( this ) )
{
	setWindowTitle( i18n( "Add Account" ) );
	m_protocolsPage = new SelectProtocolPage( m_fetcher );
	connect( m_protocolsPage, SIGNAL( protocolSelected( const QString & ) ),
	         this, SLOT( prepareAccountWidget( const QString & ) ) );
	addPage( m_protocolsPage );
	setValid( m_protocolsPage, FALSE );
	
	//setup finished page
	m_finishedPageWidget = new QWidget( this );
	QVBoxLayout * finishedLayout = new QVBoxLayout( m_finishedPageWidget );
	finishedLayout->setSpacing( KDialog::spacingHint() );
	QLabel * finishedLabel = new QLabel( i18n( "Account created" ), m_finishedPageWidget );
	finishedLayout->addWidget( finishedLabel );
	m_connectNowCheckBox = new QCheckBox( i18n( "&Connect now" ), m_finishedPageWidget );
	m_connectNowCheckBox->setChecked( TRUE );
	finishedLayout->addWidget( m_connectNowCheckBox );
	finishedLayout->addStretch( 1 );
	m_finishedPage = addPage( m_finishedPageWidget, i18n( "Finished" ) );
	//done with finished page

	connect( this , SIGNAL( currentPageChanged ( KPageWidgetItem *, KPageWidgetItem *) ) ,
	         this, SLOT( slotCurrentPageChanged( KPageWidgetItem *, KPageWidgetItem * ) ) );
	connect( button( KDialog::User1 ), SIGNAL( clicked( void ) ),
	         this, SLOT( slotFinished( void ) ) );

	restoreDialogSize( KGlobal::config()->group( "CreateAccountAssistant" ) );
}

BlokkalAccountWizard::~BlokkalAccountWizard( void )
{
	KConfigGroup group = KGlobal::config()->group( "CreateAccountAssistant" );
	saveDialogSize( group );		
}

void BlokkalAccountWizard::slotCurrentPageChanged( KPageWidgetItem * current, KPageWidgetItem * before )
{
	Q_UNUSED( before );
	if( current == m_editAccountPage ) {
		setValid( m_editAccountPage, m_editAccountPage->isValid() );
	}
}

void BlokkalAccountWizard::validateAccountSettings( void )
{
	setValid( m_editAccountPage,  m_editAccountPage->isValid() );
}

void BlokkalAccountWizard::prepareAccountWidget( const QString & pluginName )
{
	if( !pluginName.isEmpty() ) {
		Blokkal::Protocol * protocol = Blokkal::PluginManager::self()->protocol( pluginName );
		setValid( m_protocolsPage, protocol );
		if( protocol ) {
			if( m_editAccountPage ) {
				disconnect( m_editAccountPage, 
				            SIGNAL( valuesChanged( void ) ),
				            this,
				            SLOT( validateAccountSettings( void ) ) );
				removePage( m_editAccountPage );
				//disabled due to crashes. maybe the object gets
				//deleted in the removePage() call, but that is not stated
				//in the docs...
				//m_editAccountPage->deleteLater();
				m_editAccountPage = 0;
			}
			m_editAccountPage = new SetupAccountPage( protocol );
			connect( m_editAccountPage, 
			         SIGNAL( valuesChanged( void ) ),
			         SLOT( validateAccountSettings( void ) ) );
			insertPage( m_finishedPage, m_editAccountPage );
			
			const KPluginInfo pluginInfo = Blokkal::PluginManager::self()->pluginInfo( pluginName );
			QString rsdProtocolName = pluginInfo.property( QString::fromLatin1( "X-Blokkal-RsdProtocolName" ) ).toString();
	
			//fallback if X-Blokkal-RsdProtocolName is not set
			//use plugin name instead
			if( rsdProtocolName.isEmpty() ) {
				rsdProtocolName = pluginInfo.name();
			}
			const Blokkal::RsdConfigurationData rsdConfig = Blokkal::RsdConfigurationData( rsdProtocolName,
			                                                                               m_fetcher->apiLink( rsdProtocolName ),
			                                                                               m_fetcher->homePageLink(),
			                                                                               m_fetcher->engineName(),
			                                                                               m_fetcher->engineLink(),
			                                                                               m_fetcher->isPreferredApi( rsdProtocolName ) );
			m_editAccountPage->editAccountWidget()->setup( rsdConfig );
		}
		else {
			kError() << "0 pointer for plugin: " << pluginName << endl;
		}
	}

	setValid( m_protocolsPage, !pluginName.isEmpty() && m_editAccountPage );
}

void BlokkalAccountWizard::slotFinished( void )
{
	Blokkal::Account * account = m_editAccountPage->apply();
	if( !account ) {
		return;
	}
	
	m_accountName = account->id();
	Blokkal::AccountManager::self()->registerAccount( account );
	
	//check if the account has been correctly registered
	account = Blokkal::AccountManager::self()->account( m_accountName );

	if( !account ) {
		return;
	}
	
	if( m_connectNowCheckBox->isChecked() ) {
		account->connectAccount();
	}
}

//-------------------------
//ProtocolListModel
//-------------------------
class ProtocolListModel::Private {
public:
	Private( const KPluginInfo::List & protocols ):
	protocols( protocols )
	{}
	
	const KPluginInfo::List protocols;
	QStringList rsdProtocolNames;
};

ProtocolListModel::ProtocolListModel( QWidget * parent ):
QAbstractItemModel( parent ),
d( new Private( Blokkal::PluginManager::self()->availableProtocols() ) )
{}

ProtocolListModel::~ProtocolListModel( void )
{
	delete d;
}

Qt::ItemFlags ProtocolListModel::flags( const QModelIndex & index ) const
{
	if( d->rsdProtocolNames.isEmpty() ) {
		return QAbstractItemModel::flags( index );
	}
	
	const int row = index.row();

	if( row >= d->rsdProtocolNames.size() ) {
		return QAbstractItemModel::flags( index );
	}
	
	const KPluginInfo pluginInfo = d->protocols.at( row );
	QString rsdProtocolName = pluginInfo.property( QString::fromLatin1( "X-Blokkal-RsdProtocolName" ) ).toString();
	
	//fallback if X-Blokkal-RsdProtocolName is not set
	//use plugin name instead
	if( rsdProtocolName.isEmpty() ) {
		rsdProtocolName = pluginInfo.name();
	}

	
	if( d->rsdProtocolNames.contains( rsdProtocolName ) ) {
		return QAbstractItemModel::flags( index );
	}
	
	return Qt::NoItemFlags;
}

QModelIndex ProtocolListModel::index( int row,
                                      int column, 
                                      const QModelIndex & parent ) const
{
	if ( parent.isValid() ) {
		return QModelIndex();
	}

	return createIndex( row, column );
}

QModelIndex ProtocolListModel::parent( const QModelIndex & index ) const
{
	Q_UNUSED( index );
	return QModelIndex();
}

int ProtocolListModel::rowCount( const QModelIndex & parent ) const
{
	if( parent.isValid() ) {
		return 0;
	}
	
	return d->protocols.size();
}

int ProtocolListModel::columnCount( const QModelIndex & parent ) const
{
	Q_UNUSED( parent );
	return 2;
}

QVariant ProtocolListModel::data( const QModelIndex & index, int role ) const
{
	if( index.parent().isValid() ) {
		return QVariant();
	}
	
	const int col = index.column();
	if( col >= columnCount() ) {
		return QVariant();
	}
	
	const int row = index.row();
	
	if( row >= d->protocols.size() ) {
		return QVariant();
	}
	
	const KPluginInfo pluginInfo = d->protocols.at( row );
	
	switch( col ) {
	case 0:
		if( role == Qt::DisplayRole ) {
			return pluginInfo.name();
		}
		else if( role == Qt::DecorationRole ) {
			return SmallIcon( pluginInfo.icon() );
		}
		else if( role == PluginNameRole ) {
			return pluginInfo.pluginName();
		}
		else if( role == RsdNameRole ) {
			QString rsdProtocolName = pluginInfo.property( QString::fromLatin1( "X-Blokkal-RsdProtocolName" ) ).toString();
			if( rsdProtocolName.isEmpty() ) {
				return pluginInfo.name();
			}
			return rsdProtocolName;
		}
		break;
	case 1:
		if( role == Qt::DisplayRole ) {
			return pluginInfo.comment();
		}
		else if( role == PluginNameRole ) {
			return pluginInfo.pluginName();
		}
		else if( role == RsdNameRole ) {
			QString rsdProtocolName = pluginInfo.property( QString::fromLatin1( "X-Blokkal-RsdProtocolName" ) ).toString();
			if( rsdProtocolName.isEmpty() ) {
				return pluginInfo.name();
			}
			return rsdProtocolName;
		}
		break;
	}
	
	return QVariant();
}

QVariant ProtocolListModel::headerData( int section,
                                        Qt::Orientation orientation,
                                        int role ) const
{
	if( orientation != Qt::Horizontal ) {
		return QVariant();
	}
	
	if( section > columnCount() ) {
		return QVariant();
	}
	
	if( role != Qt::DisplayRole ) {
		return QAbstractItemModel::headerData(section, orientation, role);
	}
	
	if( section == 0 ) {
		return i18n( "Protocol" );
	}
	else if( section == 1 ) {
		return i18n( "Description" );
	}
	
	return QVariant();
}


void ProtocolListModel::setRsdProtocolNames( const QStringList & rsdProtocolNames )
{
	d->rsdProtocolNames = rsdProtocolNames;

	if( !d->protocols.isEmpty() ) {
		emit dataChanged( createIndex( 0, 0 ),
		                  createIndex( d->protocols.size() - 1, 1 ) );
	}
}

//-------------------------
//SelectProtocolPage
//-------------------------
SelectProtocolPage::SelectProtocolPage( RsdDataFetcher * fetcher ) :
KPageWidgetItem( createWidget( ), i18n( "Select Protocol" ) ),
m_fetcher( fetcher )
{
	connect( m_protocolsView->selectionModel(),
	         SIGNAL( selectionChanged( const QItemSelection & , const QItemSelection & ) ),
	         SLOT( emitProtocolSelected( void ) ) );
	
	connect( fetcher,
	         SIGNAL( dataReceived( const QString &, bool ) ),
	         SLOT( updateProtocolStates( const QString &, bool ) ) );
	
	connect( m_rsdLoadButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( loadRsdData( void ) ) );
	connect( m_blogAddressLineEdit,
	         SIGNAL( editingFinished( void ) ),
	         SLOT( loadRsdData( void ) ) );
	
	connect( m_blogAddressLineEdit,
	         SIGNAL( textEdited( const QString & ) ),
	         SLOT( validateBlogAddress( const QString & ) ) );
}

void SelectProtocolPage::emitProtocolSelected( void )
{
	const QModelIndexList indexList = m_protocolsView->selectionModel()->selectedRows();
	if( indexList.size() != 1 ) {
		emit protocolSelected( QString() );
		return;
	}
	
	
	emit protocolSelected( m_protocolsView->model()->data( indexList.first(),
	                                                       ProtocolListModel::PluginNameRole ).toString() );
}

QWidget * SelectProtocolPage::createWidget( void )
{
	QWidget * widget = new QWidget();
	QVBoxLayout * widgetLayout = new QVBoxLayout( widget );
	widgetLayout->setSpacing( KDialog::spacingHint() );
	QGroupBox * rsdGroupBox = new QGroupBox( i18n( "Automatic Configuration" ), widget );
	new QVBoxLayout( rsdGroupBox );
	QLabel * rsdExplanationLabel = new QLabel( i18n( "Some sites make the information needed to configure blogging clients automatically available for download. To try this, enter the address of your blog below." ), 
	                                           rsdGroupBox );
	rsdExplanationLabel->setWordWrap( TRUE );
	rsdGroupBox->layout()->addWidget( rsdExplanationLabel );
	
	QHBoxLayout * blogUrlButtonLayout = new QHBoxLayout( 0 );
	blogUrlButtonLayout->setSpacing( KDialog::spacingHint() );
	blogUrlButtonLayout->addWidget( new QLabel( i18n("Blog address"), rsdGroupBox) );
	m_blogAddressLineEdit = new KLineEdit( rsdGroupBox );
	m_blogAddressLineEdit->setValidator( new UrlValidator( m_blogAddressLineEdit ) );
	m_blogAddressLineEdit->setText( QString::fromLatin1( "http://" ) );
	m_blogAddressLineEdit->setTrapReturnKey( TRUE );
	blogUrlButtonLayout->addWidget( m_blogAddressLineEdit );
	m_rsdLoadButton = new KPushButton( i18n( "Load" ), rsdGroupBox );
	m_rsdLoadButton->setDisabled( TRUE );
	blogUrlButtonLayout->addWidget( m_rsdLoadButton );
	rsdGroupBox->layout()->addItem( blogUrlButtonLayout );
 	widgetLayout->addWidget( rsdGroupBox );
	
	QGroupBox * protocolsGroupBox = new QGroupBox( i18n( "Available Protocols" ), widget );
	new QVBoxLayout( protocolsGroupBox );
	m_protocolSelectionExplanationLabel = new QLabel( protocolsGroupBox );
	m_protocolSelectionExplanationLabel->setWordWrap( TRUE );
	m_protocolSelectionExplanationLabel->hide();
	protocolsGroupBox->layout()->addWidget( m_protocolSelectionExplanationLabel );
	
	m_protocolsView = new QTreeView( protocolsGroupBox );
	m_protocolListModel = new ProtocolListModel( m_protocolsView );
	QSortFilterProxyModel * sortModel = new QSortFilterProxyModel( m_protocolsView );
	sortModel->setSourceModel( m_protocolListModel );
	m_protocolsView->setModel( sortModel );
	m_protocolsView->resizeColumnToContents( 0 );
	m_protocolsView->setSelectionBehavior( QAbstractItemView::SelectRows );
	m_protocolsView->setSelectionMode( QAbstractItemView::SingleSelection );
	m_protocolsView->setRootIsDecorated( FALSE );
	
	m_protocolsView->setSortingEnabled( TRUE );
	m_protocolsView->sortByColumn( 0 , Qt::AscendingOrder );
	m_protocolsView->model()->sort(0);
	protocolsGroupBox->layout()->addWidget( m_protocolsView );
	widgetLayout->addWidget( protocolsGroupBox );
	
	return widget;
}

void SelectProtocolPage::updateProtocolStates( const QString & blogUrl, bool success )
{
	m_protocolListModel->setRsdProtocolNames( QStringList() );
	if( success ) {
		const QStringList rsdApis = m_fetcher->apiNames();
		m_protocolSelectionExplanationLabel->setText( i18n( "Protocols not supported by the site you have entered have been disabled. The protocol that appears to be most suitable has been selected." ) );
		m_protocolListModel->setRsdProtocolNames( rsdApis );

		//select best api
		const int rowCount = m_protocolsView->model()->rowCount();
		bool selectedPreferred = FALSE;
		//first try to find the preferred apiName...
		for( int row = 0; row < rowCount; ++row ) {
			QModelIndex index = m_protocolsView->model()->index( row, 0 );
			if( m_protocolsView->model()->data( index, ProtocolListModel::RsdNameRole ).toString() == m_fetcher->preferredApi() ) {
				m_protocolsView->selectionModel()->select( index, QItemSelectionModel::Select|QItemSelectionModel::Rows );
				selectedPreferred = TRUE;
				break;
			}
		}
		
		//then use the first api in the list that we know about...
		if( !selectedPreferred ) {
			//get list of known APIs
			QStringList knownApis;
			for( int row = 0; row < rowCount; ++row ) {
				QModelIndex index = m_protocolsView->model()->index( row, 0 );
				knownApis.append( m_protocolsView->model()->data( index, ProtocolListModel::RsdNameRole ).toString() );
			}
			
			//find best API name
			QString bestApi;
			for( QStringList::ConstIterator it = rsdApis.begin();
			     it != rsdApis.end();
			     ++it )
			{
				if( knownApis.contains( *it ) ) {
					bestApi = *it;
					break;
				}
			}
			
			if( !bestApi.isNull() ) {
				for( int row = 0; row < rowCount; ++row ) {
					QModelIndex index = m_protocolsView->model()->index( row, 0 );
					if( m_protocolsView->model()->data( index, ProtocolListModel::RsdNameRole ).toString() == bestApi ) {
						m_protocolsView->selectionModel()->select( index, QItemSelectionModel::Select|QItemSelectionModel::Rows );
						break;
					}
				}	
			}
		}
	}
	else {
		m_protocolSelectionExplanationLabel->setText( i18n( "The site you have entered does not appear to support automatic configuration. Please select a suitable Protocol." ) );
	}
	
	m_protocolSelectionExplanationLabel->show();
}

void SelectProtocolPage::loadRsdData(void )
{
	m_fetcher->fetchData( m_blogAddressLineEdit->text() );
}

void SelectProtocolPage::validateBlogAddress( const QString & blogUrlString )
{
	const KUrl blogUrl = blogUrlString;
	if( blogUrl.protocol() == QString::fromLatin1( "http" ) 
	    && blogUrl.isValid()
	    && !blogUrl.host().isEmpty() ) 
	{
		m_rsdLoadButton->setEnabled( TRUE );
	}
	else {
		m_rsdLoadButton->setDisabled( TRUE );
	}

	m_protocolSelectionExplanationLabel->hide();
	
	m_protocolListModel->setRsdProtocolNames( QStringList() );
	//TODO deselect any protocols selected in protocol view
}

//-------------------
//SetupAccountPage
//-------------------
SetupAccountPage::SetupAccountPage( Blokkal::Protocol * protocol ):
KPageWidgetItem( createWidget( protocol ), i18n( "Setup Account" ) )
{
	if( !m_editAccountWidget ) {
		kError() << "protocol " << protocol->pluginName() << " returned 0 for createEditAccountWidget()" << endl;
	}
	else {
		connect( m_editAccountWidget, SIGNAL( valuesChanged( void ) ),
		         this, SLOT( emitValuesChanged( void ) ) );
	}	
}

Blokkal::Ui::EditAccountWidget * SetupAccountPage::createWidget( Blokkal::Protocol * protocol )
{
	m_editAccountWidget = protocol->createEditAccountWidget( 0 );

	return m_editAccountWidget;
}

void SetupAccountPage::emitValuesChanged( void )
{
	emit valuesChanged();
}

bool SetupAccountPage::isValid( void ) const
{
	if( !m_editAccountWidget ) {
		return FALSE;
	}
	
	return m_editAccountWidget->isValid();
}

Blokkal::Account * SetupAccountPage::apply( )
{
	if( !m_editAccountWidget ) {
		return 0;
	}

	return m_editAccountWidget->apply();
}

Blokkal::Ui::EditAccountWidget * SetupAccountPage::editAccountWidget(void )
{
	return m_editAccountWidget;
}

//-------------------
//UrlValidator
//-------------------
UrlValidator::UrlValidator( QObject * parent ):
QValidator( parent )
{}

UrlValidator::~UrlValidator(void )
{}

QValidator::State UrlValidator::validate( QString & input, int & pos ) const
{
	Q_UNUSED( pos );
	const KUrl url = input;
	if( url.protocol() == QString::fromLatin1( "http" ) 
	    && url.isValid()
	    && !url.host().isEmpty() )
	{
		return Acceptable;
	}
	
	return Intermediate;
}
