/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ACCOUNTWIZARD_H
#define ACCOUNTWIZARD_H

#include <kassistantdialog.h>

#include <QtGui/QValidator>

class QTreeView;
class QHBoxLayout;
class QVBoxLayout;
class QSpacerItem;
class QCheckBox;
class QLabel;

class KPushButton;
class KLineEdit;

class SelectProtocolPage;
class SetupAccountPage;
class RsdDataFetcher;

namespace Blokkal {
class Protocol;
class Account;
namespace Ui {
class EditAccountWidget;
}
}

class ProtocolListModel;


/**
* @author Martin Mueller <orvio@orvio.de>
*/
class BlokkalAccountWizard : public KAssistantDialog
{
Q_OBJECT
public:
	/**
	 * Creates an new BlokkalAccountWizard.
	 * @param parent parent widget
	 */
	BlokkalAccountWizard( QWidget * parent = 0 );
	~BlokkalAccountWizard( void );

private Q_SLOTS:
	void slotCurrentPageChanged( KPageWidgetItem * current, KPageWidgetItem * before );
	void prepareAccountWidget( const QString & pluginName );
	void validateAccountSettings( void );
	void slotFinished( void );

private:
	KPageWidgetItem * m_protocolsPage;
	SetupAccountPage * m_editAccountPage;
	QString m_accountName;
	QWidget * m_finishedPageWidget;
	KPageWidgetItem * m_finishedPage;
	QCheckBox * m_connectNowCheckBox;
	RsdDataFetcher * m_fetcher;
};

/**
 * @short This class displays available protocols.
 *
 * This class displays available protocols.
 * @author Martin Mueller <orvio@orvio.de>
 */
class SelectProtocolPage : public KPageWidgetItem
{
Q_OBJECT
public:
	/**
	 * Creates a new SelectProtocolPage.
	 */
	SelectProtocolPage( RsdDataFetcher * fetcher );

Q_SIGNALS:
	/**
	 * This signal is emited whenever the protocol selection changes.
	 * If @a pluginName is 0, the selection was cleared.
	 * @param pluginName name of the selected protocol plugin
	 */
	void protocolSelected( const QString & pluginName );

private Q_SLOTS:
	void emitProtocolSelected( void );
	void updateProtocolStates( const QString & blogUrl, bool success );
	void loadRsdData( void );
	void validateBlogAddress( const QString & blogUrl );
	
private:
	QWidget * createWidget( void );
	
	RsdDataFetcher * m_fetcher;
	QTreeView * m_protocolsView;
	KLineEdit * m_blogAddressLineEdit;
	QLabel * m_protocolSelectionExplanationLabel;
	KPushButton * m_rsdLoadButton;
	ProtocolListModel * m_protocolListModel;
};

/**
 * @short Model for displaying Protocol info
 *
 * This can be used with a QTreeView to display information on
 * available protocols
 * @author Martin Mueller <orvio@orvio.de>
 */
class ProtocolListModel : public QAbstractItemModel
{
Q_OBJECT
public:
	ProtocolListModel( QWidget * parent );
	~ProtocolListModel( void );
	virtual Qt::ItemFlags flags( const QModelIndex & index ) const;
	virtual QModelIndex index ( int row, 
	                            int column, 
	                            const QModelIndex & parent = QModelIndex() ) const;
	virtual QModelIndex parent ( const QModelIndex & index ) const;
	virtual int rowCount ( const QModelIndex & parent = QModelIndex() ) const;
	virtual int columnCount ( const QModelIndex & parent = QModelIndex() ) const;
	virtual QVariant data ( const QModelIndex & index, int role = Qt::DisplayRole ) const;
	virtual QVariant headerData ( int section, 
	                              Qt::Orientation orientation,
	                              int role = Qt::DisplayRole ) const;
	
	enum CustomRoles {
		SortRole = Qt::UserRole + 1,
		PluginNameRole = Qt::UserRole + 2,
		RsdNameRole = Qt::UserRole + 3,
	};

	void setRsdProtocolNames( const QStringList & rsdProtocolNames );

private:
	class Private;
	Private * const d;
};

class UrlValidator : public QValidator {
Q_OBJECT
public:
	UrlValidator( QObject * parent );
	~UrlValidator( void );
	
	virtual QValidator::State validate ( QString & input, int & pos ) const;
};

/**
 * @short Small wrapper class around a Blokkal::Ui::EditAccountWidget
 *
 * Small wrapper class around a Blokkal::Ui::EditAccountWidget
 * @author Martin Mueller <orvio@orvio.de>
 */
class SetupAccountPage : public KPageWidgetItem
{
Q_OBJECT
public:
	/**
	 * Creates a new SelectProtocolPage.
	 * @param protocol the protocol for wich the EditAccountWidget will be created
	 */
	SetupAccountPage( Blokkal::Protocol * protocol );

	bool isValid( void ) const;
	Blokkal::Account * apply();
	
	Blokkal::Ui::EditAccountWidget * editAccountWidget( void );

Q_SIGNALS:
	/**
	 * This signal is emitted when the account settings have changed.
	 */
	void valuesChanged( void );

private Q_SLOTS:
	void emitValuesChanged( void );

private:
	Blokkal::Ui::EditAccountWidget * m_editAccountWidget;
	Blokkal::Ui::EditAccountWidget * createWidget( Blokkal::Protocol * protocol );
};

#endif
