/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "appearanceconfig.h"
#include "appearanceconfig.moc"

#include <blokkalappearancesettings.h>

#include <kgenericfactory.h>
#include <kdialog.h>
#include <klocale.h>
#include <kcolorbutton.h>

#include <QLayout>
#include <QLabel>
#include <QCheckBox>

K_PLUGIN_FACTORY( BlokkalAppearanceConfigFactory, registerPlugin<AppearanceConfig>(); )
K_EXPORT_PLUGIN( BlokkalAppearanceConfigFactory("kcm_blokkal_appearanceconfig") )
;

class AppearanceConfig::Private {
public:
	Private( void ) {}

	QVBoxLayout * mainLayout;
	QGroupBox * syntaxGroup;
	HighlightingSetupWidget * tagSetup;
	HighlightingSetupWidget * stringSetup;
	HighlightingSetupWidget * valueSetup;

	QCheckBox * spellingCheckBox;
};

AppearanceConfig::AppearanceConfig( QWidget * parent, const QVariantList & args) :
KCModule( BlokkalAppearanceConfigFactory::componentData(), parent, args ),
d( new Private() )
{
	d->mainLayout = new QVBoxLayout( this );
	d->mainLayout->setSpacing( KDialog::spacingHint() );

	d->syntaxGroup = new QGroupBox( i18n( "Enable Syntax &Highlighting" ),
	                                this );
	QVBoxLayout * syntaxGroupLayout = new QVBoxLayout( d->syntaxGroup );
	d->syntaxGroup->setCheckable( TRUE );
	connect( d->syntaxGroup, SIGNAL( toggled( bool ) ),
	         this, SLOT( emitChanged( void ) ) );

	d->tagSetup = new HighlightingSetupWidget( i18n( "Tags" ), d->syntaxGroup );
	syntaxGroupLayout->addWidget( d->tagSetup );
	connect( d->tagSetup, SIGNAL( valuesChanged( void ) ),
	         this, SLOT( emitChanged( void ) ) );
	d->stringSetup = new HighlightingSetupWidget( i18n( "String Values" ), d->syntaxGroup );
	syntaxGroupLayout->addWidget( d->stringSetup );
	connect( d->stringSetup, SIGNAL( valuesChanged( void ) ),
	         this, SLOT( emitChanged( void ) ) );
	d->valueSetup = new HighlightingSetupWidget( i18n( "Numerical Values" ), d->syntaxGroup );
	syntaxGroupLayout->addWidget( d->valueSetup );
	connect( d->valueSetup, SIGNAL( valuesChanged( void ) ),
	         this, SLOT( emitChanged( void ) ) );
	d->mainLayout->addWidget( d->syntaxGroup );
	
	d->spellingCheckBox = new QCheckBox( i18n( "Enable &Spell Checking" ), this );
	d->mainLayout->addWidget( d->spellingCheckBox );
	connect( d->spellingCheckBox, SIGNAL( toggled( bool ) ),
	         this, SLOT( emitChanged( void ) ) );	
	
	d->mainLayout->addStretch( 1 );

	load();

	setButtons( Help|Apply );
}

AppearanceConfig::~AppearanceConfig( void )
{
	delete d;
}

void AppearanceConfig::save( )
{
	Blokkal::AppearanceSettings * appearanceSettings = Blokkal::AppearanceSettings::self();
	appearanceSettings->setHighlightSyntax( d->syntaxGroup->isChecked() );

	appearanceSettings->setHighlightTagBold( d->tagSetup->bold() );
	appearanceSettings->setHighlightTagItalic( d->tagSetup->italic() );
	appearanceSettings->setHighlightTagColor( d->tagSetup->color() );

	appearanceSettings->setHighlightStringBold( d->stringSetup->bold() );
	appearanceSettings->setHighlightStringItalic( d->stringSetup->italic() );
	appearanceSettings->setHighlightStringColor( d->stringSetup->color() );

	appearanceSettings->setHighlightValueBold( d->valueSetup->bold() );
	appearanceSettings->setHighlightValueItalic( d->valueSetup->italic() );
	appearanceSettings->setHighlightValueColor( d->valueSetup->color() );

	appearanceSettings->setCheckSpelling( d->spellingCheckBox->isChecked() );
	Blokkal::AppearanceSettings::self()->writeConfig();
	emit changed( FALSE );
}

void AppearanceConfig::load( )
{
	Blokkal::AppearanceSettings * appearanceSettings = Blokkal::AppearanceSettings::self();
	d->syntaxGroup->setChecked( appearanceSettings->highlightSyntax() );

	d->tagSetup->setBold( appearanceSettings->highlightTagBold() );
	d->tagSetup->setItalic( appearanceSettings->highlightTagItalic() );
	d->tagSetup->setColor( appearanceSettings->highlightTagColor() );

	d->stringSetup->setBold( appearanceSettings->highlightStringBold() );
	d->stringSetup->setItalic( appearanceSettings->highlightStringItalic() );
	d->stringSetup->setColor( appearanceSettings->highlightStringColor() );

	d->valueSetup->setBold( appearanceSettings->highlightValueBold() );
	d->valueSetup->setItalic( appearanceSettings->highlightValueItalic() );
	d->valueSetup->setColor( appearanceSettings->highlightValueColor() );

	d->spellingCheckBox->setChecked( appearanceSettings->checkSpelling() );
}

void AppearanceConfig::emitChanged( void )
{
	emit changed( TRUE );
}

//------------------------
//HighlightingSetupWidget
//------------------------
class HighlightingSetupWidget::Private {
public:
	Private( void ) {}

	QCheckBox * boldCheckBox;
	QCheckBox * italicCheckBox;
	KColorButton * colorButton;
};

HighlightingSetupWidget::HighlightingSetupWidget( const QString & title, QWidget * parent ):
QGroupBox( title, parent ),
d( new Private() )
{
	QFont font;
	QHBoxLayout * layout = new QHBoxLayout( this );
	
	d->boldCheckBox = new QCheckBox( i18n("Bold"), this );
	font = d->boldCheckBox->font();
	font.setBold( TRUE );
	d->boldCheckBox->setFont( font );
	layout->addWidget( d->boldCheckBox );
	connect( d->boldCheckBox, SIGNAL( toggled( bool ) ),
	         this, SLOT( emitValuesChanged( void ) ) );

	d->italicCheckBox = new QCheckBox( i18n("Italic"), this );
	font = d->italicCheckBox->font();
	font.setItalic( TRUE );
	d->italicCheckBox->setFont( font );
	layout->addWidget( d->italicCheckBox );
	connect( d->italicCheckBox, SIGNAL( toggled( bool ) ),
	         this, SLOT( emitValuesChanged( void ) ) );	

	d->colorButton = new KColorButton( this );
	layout->addWidget( d->colorButton );
	connect( d->colorButton, SIGNAL( changed ( const QColor & ) ),
	         this, SLOT( emitValuesChanged( void ) ) );		
	QLabel * colorLabel = new QLabel( i18n("Text Color"), this );
	colorLabel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout->addWidget( colorLabel );
}

HighlightingSetupWidget::~HighlightingSetupWidget( void )
{
	delete d;
}

bool HighlightingSetupWidget::bold( void ) const
{
	return d->boldCheckBox->isChecked();
}

void HighlightingSetupWidget::setBold( bool bold )
{
	d->boldCheckBox->setChecked( bold );
}

bool HighlightingSetupWidget::italic( void ) const
{
	return d->italicCheckBox->isChecked();
}

void HighlightingSetupWidget::setItalic( bool italic )
{
	d->italicCheckBox->setChecked( italic );
}

QColor HighlightingSetupWidget::color( void ) const
{
	return d->colorButton->color();
}

void HighlightingSetupWidget::setColor( const QColor & color )
{
	d->colorButton->setColor( color );
}

void HighlightingSetupWidget::emitValuesChanged( void )
{
	emit valuesChanged();
}
