/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "behaviourconfig.h"
#include "behaviourconfig.moc"

#include <blokkalbehavioursettings.h>

#include <kgenericfactory.h>
#include <klocale.h>
#include <kcombobox.h>
#include <knuminput.h>
#include <kdialog.h>

#include <QLayout>
#include <QGroupBox>
#include <QCheckBox>
#include <QLabel>

K_PLUGIN_FACTORY( BlokkalBehaviourConfigFactory, registerPlugin<BehaviourConfig>(); )
K_EXPORT_PLUGIN( BlokkalBehaviourConfigFactory("kcm_blokkal_behaviourconfig") )
;

class BehaviourConfig::Private {
public:
	Private( void ) {}

	QVBoxLayout * mainLayout;
	
	QGroupBox * mainWindowGroup;
	QCheckBox * showSystemTrayCheckBox;
	QCheckBox * startHiddenCheckBox;
	QCheckBox * autoHideCheckBox;
	KIntSpinBox * hideTimeoutSpinBox;

	QGroupBox * loadingPolicyGroup;
	KComboBox * webPageLoadingPolicyComboBox;

	QGroupBox * miscGroup;
	QCheckBox * autoConnectCheckBox;
	
};

BehaviourConfig::BehaviourConfig( QWidget * parent, const QVariantList & args) :
KCModule( BlokkalBehaviourConfigFactory::componentData(), parent, args ),
d( new Private() )
{
	d->mainLayout = new QVBoxLayout( this );

	d->mainWindowGroup = new QGroupBox( i18n( "Main Window" ),
	                                    this );
	QVBoxLayout * mainWindowGroupLayout = new QVBoxLayout( d->mainWindowGroup );
	d->showSystemTrayCheckBox = new QCheckBox( i18n( "Show system &tray icon"),
	                                           d->mainWindowGroup );

	mainWindowGroupLayout->addWidget( d->showSystemTrayCheckBox );
	d->startHiddenCheckBox = new QCheckBox( i18n( "&Start with hidden main window" ),
	                                        d->mainWindowGroup );
	mainWindowGroupLayout->addWidget( d->startHiddenCheckBox );
	d->autoHideCheckBox = new QCheckBox( i18n( "Automatically hide &main window" ),
	                                     d->mainWindowGroup );
	mainWindowGroupLayout->addWidget( d->autoHideCheckBox );
	QHBoxLayout * timeoutLayout = new QHBoxLayout();
	timeoutLayout->setMargin( 0 );
	timeoutLayout->addSpacing( KDialog::spacingHint() * 2 );
	timeoutLayout->addWidget( new QLabel( i18n( "Hide timeout" ) ) );
	d->hideTimeoutSpinBox = new KIntSpinBox( d->mainWindowGroup );
	d->hideTimeoutSpinBox->setMinimum( 1 );
	d->hideTimeoutSpinBox->setSuffix( i18n( " s" ) );
	timeoutLayout->addWidget( d->hideTimeoutSpinBox );
	timeoutLayout->addStretch( 1 );
	mainWindowGroupLayout->addLayout( timeoutLayout );

	d->mainLayout->addWidget( d->mainWindowGroup );

	d->loadingPolicyGroup = new QGroupBox( i18n( "Web page loading policy" ),
	                                       this );
	QVBoxLayout * loadingPolicyGroupLayout = new QVBoxLayout( d->loadingPolicyGroup );
	d->webPageLoadingPolicyComboBox = new KComboBox( d->loadingPolicyGroup );
	d->webPageLoadingPolicyComboBox->addItem( i18n( "Use default browser" ) );
	d->webPageLoadingPolicyComboBox->addItem( i18n( "Disable page loading" ) );
	loadingPolicyGroupLayout->addWidget( d->webPageLoadingPolicyComboBox );
	d->mainLayout->addWidget( d->loadingPolicyGroup );
	
	d->miscGroup = new QGroupBox( i18n( "Miscellaneous" ),
	                              this );
	QVBoxLayout * miscGroupLayout = new QVBoxLayout( d->miscGroup );
	d->autoConnectCheckBox = new QCheckBox( i18n( "Connect accounts automatically at startup" ),
	                                        d->miscGroup );
	miscGroupLayout->addWidget( d->autoConnectCheckBox );
	d->mainLayout->addWidget( d->miscGroup );
	d->mainLayout->addStretch( 1 );

	load();

	setButtons( Help|Apply );

	updateTrayCheckboxStatus();

	connect( d->showSystemTrayCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( updateTrayCheckboxStatus( void ) ) );
	connect( d->showSystemTrayCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( emitChanged( void ) ) );
	connect( d->startHiddenCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( emitChanged( void ) ) );
	connect( d->autoHideCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( emitChanged( void ) ) );
	connect( d->autoHideCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( updateTimeoutSpinboxStatus( void ) ) );
	connect( d->hideTimeoutSpinBox,
	         SIGNAL( valueChanged( int ) ),
	         SLOT( emitChanged( void ) ) );
	connect( d->webPageLoadingPolicyComboBox,
	         SIGNAL( activated( int ) ),
	         SLOT( emitChanged( void ) ) );
	connect( d->autoConnectCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( emitChanged( void ) ) );
}

BehaviourConfig::~BehaviourConfig( void )
{
	delete d;
}

void BehaviourConfig::save( )
{
	Blokkal::BehaviourSettings * behaviourSettings = Blokkal::BehaviourSettings::self();

	behaviourSettings->setShowSystemTray( d->showSystemTrayCheckBox->isChecked() );
	behaviourSettings->setStartHidden( d->startHiddenCheckBox->isChecked() );
	behaviourSettings->setAutoHide( d->autoHideCheckBox->isChecked() );
	behaviourSettings->setAutoHideTimeout( d->hideTimeoutSpinBox->value() );

	behaviourSettings->setWebPageLoadingPolicy( d->webPageLoadingPolicyComboBox->currentIndex() );

	behaviourSettings->setAutoConnect( d->autoConnectCheckBox->isChecked() );
	
	behaviourSettings->writeConfig();
	emit changed( FALSE );
}

void BehaviourConfig::load( )
{
	Blokkal::BehaviourSettings * behaviourSettings = Blokkal::BehaviourSettings::self();
	d->showSystemTrayCheckBox->setChecked( behaviourSettings->showSystemTray() );
	d->startHiddenCheckBox->setChecked( behaviourSettings->startHidden() );
	d->autoHideCheckBox->setChecked( behaviourSettings->autoHide() );
	d->hideTimeoutSpinBox->setValue( behaviourSettings->autoHideTimeout() );
	
	d->webPageLoadingPolicyComboBox->setCurrentIndex( behaviourSettings->webPageLoadingPolicy() );
	
	d->autoConnectCheckBox->setChecked( behaviourSettings->autoConnect() );
}

void BehaviourConfig::emitChanged( void )
{
	emit changed( TRUE );
}

void BehaviourConfig::updateTrayCheckboxStatus( void )
{
	d->startHiddenCheckBox->setEnabled( d->showSystemTrayCheckBox->isChecked() );
	d->autoHideCheckBox->setEnabled( d->showSystemTrayCheckBox->isChecked() );
	if( !d->showSystemTrayCheckBox->isChecked() ) {
		d->startHiddenCheckBox->setChecked( FALSE );
		d->autoHideCheckBox->setChecked( FALSE );
	}

	updateTimeoutSpinboxStatus();
}

void BehaviourConfig::updateTimeoutSpinboxStatus( void )
{
	d->hideTimeoutSpinBox->setEnabled( d->showSystemTrayCheckBox->isChecked()
	                                   && d->autoHideCheckBox->isChecked() );
}
