/***************************************************************************
 *   Copyright (C) 2009 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef RSDDATAFETCHER_H
#define RSDDATAFETCHER_H

#include <QtCore/QObject>

class QWidget;

namespace KIO {
	class Job;
};
class KJob;

/**
 * @short Fetches RSD data
 *
 * Fetches RSD data needed to configure new accounts and handles
 * all related operations like displaying the progress dialog.
 * @author Martin Mueller <orvio@orvio.de>
 */
class RsdDataFetcher : public QObject {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param parent parent object for RsdDataFetcher and parent widget for progress dialog
	 */
	RsdDataFetcher( QWidget * parent );
	
	/**
	 * Destructor
	 */
	~RsdDataFetcher( void );
	
	/**
	 * Returns the names of the apis as found in the rsd document. The order in the list is
	 * the same as in the rsd document.
	 * 
	 * @return the names of the apis
	 */
	QStringList apiNames( void ) const;
	
	/**
	 * @return the apiLink for the api
	 */
	QString apiLink( const QString & apiName ) const;
	
	/**
	 * @return TRUE if apiName is the preferred api
	 */
	bool isPreferredApi( const QString & apiName ) const;
	
	/**
	 * @return url passed to the last call to @ref fetchData
	 */
	QString currentBlogUrl( void ) const;
	
	/**
	 * @return apiName of the preferred api
	 */
	QString preferredApi( void ) const;
	
	/**
	 * @return the homePageLink for the blog
	 */
	QString homePageLink( void ) const;
	
	/**
	 * @return the engineName for the blog
	 */
	QString engineName( void ) const;
	
	/**
	 * @return the engineLink for the blog
	 */
	QString engineLink( void ) const;
	
public Q_SLOTS:
	/**
	 * Fetches RSD data asynchronously.
	 * @param url URL of the blog to fetch RSD data from
	 */
	void fetchData( const QString & url );
	
Q_SIGNALS:
	/**
	 * Signals that an attempt to fetch RSD data has finished
	 * @param url URL of the blog
	 * @param success TRUE if RSD data could be obtained
	 */
	void dataReceived( const QString & url, bool success );
	
private Q_SLOTS:
	void slotDataArrived( KIO::Job * job , const QByteArray & data );
	void slotJobFinished( KJob * job );
	void updateProgress( KJob * job );
	void cancelJobs( void );
	
private:
	class Private;
	Private * const d;
};

#endif
