/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "queuestatusview.h"
#include "queuestatusview.moc"

#include <blokkalentry.h>
#include <blokkalpostentryqueue.h>
#include <blokkalblog.h>
#include <blokkalaccount.h>

#include <klocale.h>
#include <kapplication.h>
#include <kiconloader.h>

#include <QMap>
#include <QIcon>
#include <QLinkedList>

//----------------
//QueueStatusLVI
//----------------
class EntryQueue::QueueStatusLVI::Private
{
public:
	Private( void ) {}
	Blokkal::Entry * entry;
};

EntryQueue::QueueStatusLVI::QueueStatusLVI( QueueStatusView * parent, Blokkal::Entry * entry ):
K3ListViewItem( parent ),
d( new Private() )
{
	d->entry = entry;
	update( entry, Blokkal::PostEntryQueue::self()->entryStatus( entry ) );

	connect( Blokkal::PostEntryQueue::self(),
	         SIGNAL( entryStatusChanged( Blokkal::Entry *, const Blokkal::PostEntryStatus * ) ),
	         SLOT( update( Blokkal::Entry *, const Blokkal::PostEntryStatus * ) ) );
}

EntryQueue::QueueStatusLVI::~QueueStatusLVI( void )
{
	delete d;
}

void EntryQueue::QueueStatusLVI::update( Blokkal::Entry * entry, const Blokkal::PostEntryStatus * status )
{
	if( entry == d->entry ) {
		setText( 0, entry->preview() );
		setText( 1, i18n( "%1 (%2)",
		                  entry->blog()->id(),
		                  entry->blog()->account()->id() ) );
		
		switch( status->status() ) {
		case Blokkal::PostEntryStatus::StatusNormal:
			setPixmap( 0 , 0);
			break;
		case Blokkal::PostEntryStatus::StatusPosting:
			setPixmap( 0, QApplication::windowIcon().pixmap( 16, 16 ) );
			break;
		case Blokkal::PostEntryStatus::StatusError:
			setPixmap( 0, SmallIcon( "messagebox_warning" ) );
			break;
		}
	}
}

//----------------
//QueueStatusView
//----------------
class EntryQueue::QueueStatusView::Private
{
public:
	Private( void ) {}

	QMap<Blokkal::Entry*, QueueStatusLVI* >  entryItemMap;
};

EntryQueue::QueueStatusView::QueueStatusView( QWidget * parent ):
K3ListView( parent ),
d( new Private() )
{
	setFullWidth( TRUE );
	setResizeMode( Q3ListView::AllColumns );
	setSelectionMode( Q3ListView::Single );
	setItemsMovable( FALSE );
	setAllColumnsShowFocus( TRUE );
	
	addColumn( i18n( "Entry" ), 300 );
	addColumn( i18n( "Blog" ), 100 );
 	
	if( !Blokkal::PostEntryQueue::self()->isEmpty() ) {
		QLinkedList<Blokkal::Entry*> entries = Blokkal::PostEntryQueue::self()->entries();
		for( QLinkedList<Blokkal::Entry*>::Iterator it = entries.begin();
		     it != entries.end();
		     it++ )
		{
			insertItem( *it );
		}
	}

	connect( Blokkal::PostEntryQueue::self(),
	         SIGNAL( entryAdded( Blokkal::Entry * ) ),
	         SLOT( insertItem( Blokkal::Entry * ) ) );
	connect( Blokkal::PostEntryQueue::self(),
	         SIGNAL( entryRemoved( Blokkal::Entry * ) ),
	         SLOT( removeItem( Blokkal::Entry * ) ) );
}

EntryQueue::QueueStatusView::~QueueStatusView( void )
{
	delete d;
}

void EntryQueue::QueueStatusView::removeItem( Blokkal::Entry * entry )
{
	if( !entry ) {
		return;
	}
	
	if( d->entryItemMap.contains( entry ) ) {
		QueueStatusLVI * item = d->entryItemMap[entry];
		d->entryItemMap.remove( entry );
		item->deleteLater();
	}
}

void EntryQueue::QueueStatusView::insertItem( Blokkal::Entry * entry )
{
	if( !entry ) {
		return;
	}
	
	if( !d->entryItemMap.contains( entry ) ) {
		d->entryItemMap.insert( entry, new QueueStatusLVI( this, entry ) );
	}
}

Blokkal::Entry * EntryQueue::QueueStatusLVI::entry( void ) const
{
	return d->entry;
}
