/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ENTRYQUEUEQUEUESTATUSVIEW_H
#define ENTRYQUEUEQUEUESTATUSVIEW_H

#include <k3listview.h>

namespace Blokkal {
class Entry;
class PostEntryStatus;
}

namespace EntryQueue {

/**
 * @short A listview that displays information about queued entries.
 *
 * A listview that displays information about queued entries.
 * @author Martin Mueller <orvio@orvio.de>
 */
class QueueStatusView : public K3ListView
{
Q_OBJECT
public:
	/**
	 * Creates a new QueueStatusView
	 * @param parent parent widget
	 */
	QueueStatusView( QWidget * parent = 0 );
	virtual ~QueueStatusView( void );

protected slots:
	/**
	 * Removes an item.
	 * @param entry remove item representing this entry
	 */
	void removeItem( Blokkal::Entry * entry );

	/**
	 * Inserts an item.
	 * @param entry insert an item representing this entry
	 */
	void insertItem( Blokkal::Entry * entry );

private:
	class Private;
	Private * d;
};

/**
 * @short This LVI encapsulating status information.
 *
 * This LVI encapsulating status information.
 * @author Martin Mueller <orvio@orvio.de>
 */
class QueueStatusLVI : public QObject, public K3ListViewItem
{
Q_OBJECT
public:
	/**
	 * Creates a new QueueStatusLVI
	 * @param parent parent listview
	 * @param entry the entry this item represents
	 */
	QueueStatusLVI( QueueStatusView * parent, Blokkal::Entry * entry );
	virtual ~QueueStatusLVI( void );

	/**
	 * Returns the entry this item represents in the queue.
	 * @return the entry this item represents in the queue
	 */
	Blokkal::Entry * entry( void ) const;

protected slots:
	/**
	 * Updates this item
	 * @param entry the entry
	 * @param status the new status
	 */
	void update( Blokkal::Entry * entry, const Blokkal::PostEntryStatus * status );

private:
	class Private;
	Private * d;

	friend class QueueStatusView;
};

}

#endif
