/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALACTIONTARGETRELAY_H
#define BLOKKALACTIONTARGETRELAY_H

#include "blokkal_export.h"
#include <QtCore/QObject>

namespace Blokkal {
class ActionTargetRelayPrivate;

class Account;
class Blog;

/**
 * @short Singleton that stores the targets of Account and Blog context menus
 *
 * This global singleton stores the targets of Acconut
 * and Blog context menus, so Plugins can access these in order
 * to perform custom actions.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT ActionTargetRelay : public QObject {
friend class ActionTargetRelayPrivate;
Q_OBJECT
public:
	/**
	 * @return the global ActionTargetRelay
	 */
	static Blokkal::ActionTargetRelay * self( void );

	/**
	 * @return the current account to perform an action, may be 0
	 */
	static Blokkal::Account * currentAccount( void );

	/**
	 * Sets the current account
	 * @param account the current account, may be 0
	 */
	static void setCurrentAccount( Blokkal::Account * account );

	/**
	 * @return the current blog to perform an action, may be 0
	 */
	static Blokkal::Blog * currentBlog( void );

	/**
	 * Sets the current blog
	 * @param blog the current blog, may be 0
	 */	
	static void setCurrentBlog( Blokkal::Blog * blog );

Q_SIGNALS:
	/**
	 * This signal is emitted when a new account has been set to be current
	 */
	void currentAccountChanged( Blokkal::Account * newAccount = 0 );

	/**
	 * This signal is emitted when a new acount has been set to current
	 */
	void currentAccountChanged( bool haveAccount );

	/**
	 * This signal is emitted when a new blog has been set to be current
	 */	
	void currentBlogChanged( Blokkal::Blog * newBlog = 0 );

	/**
	 * This signal is emitted when a new blog has been set to be current
	 */
	void currentBlogChanged( bool haveBlog );
	
private:
	ActionTargetRelay( void );
	~ActionTargetRelay( void );
};

}

#endif
