/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkalblog.h"
#include "blokkalblog.moc"

#include "blokkalpluginmanager.h"
#include "blokkalaccount.h"
#include "blokkalblogconfig.h"
#include "blokkalentry.h"
#include "blokkalbehavioursettings.h"
#include "blokkalui/editentrywidget.h"
#include "blokkalui/browseentryviewfactory.h"
#include "blokkalui/globalsettings.h"
#include "blokkalui/view.h"
#include "blokkalio/blogbrowsingdriver.h"

#include <kdebug.h>
#include <kicon.h>
#include <kmessagebox.h>

class Blokkal::Blog::Private {
public:
	Private( void ) {}

	Account * account;
	BlogConfig * config;
};

Blokkal::Blog::Blog( Blokkal::Account * parentAccount, const QString & id ) :
QObject( parentAccount ),
d( new Private() )
{
	d->account = parentAccount;
	d->config = new BlogConfig( id, parentAccount );
}

Blokkal::Blog::~Blog( void )
{
	emit blogDestroyed( this );
	delete d->config;
	delete d;
}

QString Blokkal::Blog::type( void ) const
{
	return i18n( "Blog" );
}

QString Blokkal::Blog::title( void ) const
{
	return config()->title();
}

void Blokkal::Blog::setTitle( const QString & title )
{
	config()->setTitle( title );
	emit propertiesChanged( this );
}

KIcon Blokkal::Blog::icon( void ) const
{
	return account()->icon();
}

Blokkal::Account * Blokkal::Blog::account( void ) const
{
	return d->account;
}

void Blokkal::Blog::startNewEntry( void )
{
	Blokkal::Entry * const entry = createEntry();
	Ui::View * const view = entry->view();
	if( !view ) {
		KMessageBox::error( Ui::GlobalSettings::self()->mainWidget(), i18n( "Unable to create a view." ) );
		delete entry;
		return;
	}
	
	view->showView();
}

void Blokkal::Blog::browseEntries( void )
{
	
	if( blogBrowsingDriver() ) {
		Ui::BrowseEntryDialog * const browseView = blogBrowsingDriver()->browseBlogDialog( Io::BlogBrowsingDriver::BrowseEdit );
		if( !browseView ) {
			KMessageBox::error( Ui::GlobalSettings::self()->mainWidget(),
			                    i18n( "The component required to browse old entries failed to load." ) );
			return;
		}
		browseView->show();
	}
	else {
		kError() << "no BlogBrowsingDriver" << endl;
	}
}

QString Blokkal::Blog::id( void ) const
{
	return config()->id();
}

Blokkal::BlogConfig * Blokkal::Blog::config( void ) const
{
	return d->config;
}

void Blokkal::Blog::removeBlog( void )
{
	config()->remove();
	deleteLater();
}

void Blokkal::Blog::saveProperties( void )
{
}

Blokkal::Ui::EditEntryWidget * Blokkal::Blog::createEditEntryWidget( Blokkal::Entry * entry, QWidget * parent )
{
	return new Ui::EditEntryWidget( entry, parent );
}

unsigned int Blokkal::Blog::supportedFeatures( void ) const
{
	return FeatureDate|FeatureSubject|FeatureText;
}

QList< QAction * > Blokkal::Blog::actionList( void )
{
	return QList<QAction*>();
}
