/***************************************************************************
 *   Copyright (C) 2007 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_FILEENGINE_H
#define BLOKKAL_FILEENGINE_H

#include <QtCore/QAbstractFileEngine>
#include <QtCore/QAbstractFileEngineHandler>

#include "blokkal_export.h"

namespace Blokkal {
/**
 * @short Factory class for FileEngine
 *
 * Factory class for FileEngine
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT FileEngineHandler : public QAbstractFileEngineHandler {
public:
	/**
	 * Creates a new FileEngine if @a fileName can be handled.
	 */
	virtual QAbstractFileEngine * create( const QString & fileName ) const;
};

/**
 * @short Provides access to icon data of various objects.
 *
 * This class provides access to icon data for tool tips and similar uses.
 * Resources are named in the following fashion:
 * \code
 * "blokkal:///accounts/[accountId]/icon"
 * "blokkal:///accounts/[accountId]/stateicon"
 * "blokkal:///accounts/[accountId]/blogs/[blogId]/icon"
 * "blokkal:///accounts/[accountId]/blogs/[blogId]/stateicon"
 * "blokkal:///protocols/[protocolname]/icon"
 * "blokkal:///smallicons/[iconname]"
 *\endcode
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT FileEngine : public QAbstractFileEngine {
public:
	virtual ~FileEngine( void );

	virtual bool caseSensitive( void ) const;
	virtual bool close( void );
	virtual bool copy( const QString & newName );
	virtual QStringList entryList( QDir::Filters filters, const QStringList & filterNames ) const;
	virtual FileFlags fileFlags( FileFlags type = FileInfoAll ) const;
	virtual QString fileName( FileName file = DefaultName ) const;
	virtual QDateTime fileTime( FileTime time ) const;
	virtual bool isRelativePath( void ) const;
	virtual bool isSequential( void ) const;
	virtual bool mkdir( const QString & dirName, bool createParentDirectories ) const;
	virtual bool open( QIODevice::OpenMode mode );
	virtual QString owner( FileOwner owner ) const;
	virtual unsigned int ownerId( FileOwner owner ) const;
	virtual qint64 pos( void ) const;
	virtual qint64 read( char * data, qint64 maxlen );
	virtual qint64 readLine( char * data, qint64 maxlen );
	virtual bool remove( void );
	virtual bool rename( const QString & newName );
	virtual bool rmdir( const QString & dirName, bool recurseParentDirectories ) const;
	virtual bool seek( qint64 offset );
	virtual void setFileName( const QString & file );
	virtual bool setPermissions( uint perms );
	virtual bool setSize( qint64 size );
	virtual qint64 size( void ) const;
	virtual qint64 write( const char * data, qint64 len );


private:
	FileEngine( void );
	friend class FileEngineHandler;
	
	class Private;
	Private * const d;
};

}

#endif
