/***************************************************************************
 *   Copyright (C) 2006 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkalimagefetcher.h"
#include "blokkalimagefetcher.moc"

#include <kio/job.h>
#include <kdebug.h>

#include <iostream>

class Blokkal::ImageFetcher::Private
{
public:
	Private( void ) :
	autoDelete( TRUE )
	{}

	bool autoDelete;

	KJob * job;
	QByteArray buffer;
	QString url;
};

Blokkal::ImageFetcher::ImageFetcher( const QString & imageUrl, QObject * parent ):
QObject( parent ),
d( new Private() )
{
	d->url = imageUrl;
	d->job = KIO::get( imageUrl,
	                   KIO::Reload, //may take data from cache
	                   KIO::HideProgressInfo //don't show progress info
	                 );

	connect( d->job, SIGNAL( data( KIO::Job *, const QByteArray & ) ), SLOT( bufferData( KIO::Job*, const QByteArray & ) ) );
	connect( d->job, SIGNAL( result( KJob * ) ), SLOT( emitImage( KJob * ) ) );
}

Blokkal::ImageFetcher::~ImageFetcher( void )
{
	if( d->job ) {
		d->job->kill();
	}

	delete d;
}

void Blokkal::ImageFetcher::bufferData( KIO::Job * job, const QByteArray & data )
{
	if( data.size() > 0 && job == d->job ) {
		d->buffer.append( data );
	}
}

void Blokkal::ImageFetcher::emitImage( KJob * job )
{
	if( job == d->job ) {
		QImage image = QImage::fromData( d->buffer );

		emit imageReceived( image, d->url );
	
		d->job = 0;
		if( isAutoDelete( ) ) {
			deleteLater();
		}
	}
}

bool Blokkal::ImageFetcher::isAutoDelete( void )
{
	return d->autoDelete;
}

void Blokkal::ImageFetcher::setAutoDelete( bool enable )
{
	d->autoDelete = enable;
}
