/***************************************************************************
 *   Copyright (C) 2006 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALBLOKKALIMAGEFETCHER_H
#define BLOKKALBLOKKALIMAGEFETCHER_H

#include <QtCore/QObject>
#include <QtGui/QImage>

#include "blokkal_export.h"

namespace KIO {
	class Job;
}
class KJob;

namespace Blokkal {

/**
 * @short A utility class to fetch images via HTTP.
 *
 * A utility class to fetch images via HTTP.
 * <b>Example usage:</b>
 * \code
 * ImageFetcher * fetcher = new ImageFetcher( "some.where.org/image.png" );
 * connect( fetcher, SIGNAL( imageReceived( QImage, const QString & ) ), this, SLOT( processImage( QImage, const QString & ) ) );
 * \endcode
 * @author Martin Mller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT ImageFetcher : public QObject
{
Q_OBJECT
public:
	/**
	 * Creates a new image fetcher instance. You must connect to
	 * @ref imageReceived() imidiately. ImageFetchers are set up
	 * to destroy themself after emitting the @ref imageReceived() signal.
	 * You can disable this with @ref setAutoDelete().
	 *
	 * @param imageUrl url of the image to fetch
	 * @param parent the parent object
	 */
	ImageFetcher( const QString & imageUrl, QObject * parent = 0 );

	/**
	 * Destructor
	 */
	~ImageFetcher( void );

	/**
	 * Sets the auto delete property. When enabled, the ImageFetcher will
	 * destroy itself after emitting @ref imageReceived.
	 * This is enabled by default.
	 * @param enable TRUE to enable auto desruction.
	 */
	void setAutoDelete( bool enable );

	/**
	 * Returns whether this fetcher is set up to destroy
	 * itself after fetching the image.
	 * @return TRUE if the fetcher will destruct itself automatically
	 */
	bool isAutoDelete( void );
	

Q_SIGNALS:
	/**
	 * This signal is emitted when the reqested image becomes availabe.
	 * If <i>image</i> is a null image, an error occured.
	 * @param image the image
	 * @param url the url supplied to the fetcher
	 */
	void imageReceived( QImage image, const QString & url );

private Q_SLOTS:
	void bufferData( KIO::Job * job, const QByteArray & data );
	void emitImage( KJob * job );
	
private:
	class Private;
	Private * d;
};

}

#endif
