/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "categorymanager.h"
#include "categorymanager.moc"

#include "../blokkalaccount.h"

#include <kdebug.h>

#include <QDomElement>
#include <QFile>

#include <QStringList>

//---------------
//CategoryManager
//---------------
class Blokkal::Io::CategoryManager::Private {
public:
	Private( Blokkal::Account * account ) :
	account( account ),
	categoryMap( QMap<QString, CategoryList >() ),
	categoryDocument( QDomDocument( "Categories" ) )
	{
		categoryDocument.appendChild( categoryDocument.createElement( "blogs" ) );
	}

	QDomElement categoryNode( const QString & blogId, const QString & categoryId ) {
		QDomNode currentBlogNode = categoryDocument.firstChild().namedItem( "blog" );
		QDomElement currentBlogElement;
		while( !currentBlogNode.isNull() ) {
			if( currentBlogNode.isElement() && currentBlogNode.nodeName() == "blog" ) {
				currentBlogElement = currentBlogNode.toElement();
				if( currentBlogElement.attribute( "name" ) == blogId ) {
					QDomNode currentCategoryNode = currentBlogNode.namedItem( "category" );
					QDomElement currentCategoryElement;
					while( !currentCategoryNode.isNull() ) {
						if( currentCategoryNode.isElement() && currentCategoryNode.nodeName() == "category" ) {
							currentCategoryElement = currentCategoryNode.toElement();
							if( currentCategoryElement.attribute( "id" ) == categoryId ) {
								return currentCategoryNode.toElement();
							}
						}
						currentCategoryNode = currentCategoryNode.nextSibling();
					}
					//insert category node
					currentCategoryElement = categoryDocument.createElement( "category" );
					currentCategoryElement.setAttribute( "id", categoryId );
					currentBlogNode.appendChild( currentCategoryElement );
					return currentCategoryElement;
				}
				
			}
			currentBlogNode = currentBlogNode.nextSibling();
		}

		//insert blog node
		currentBlogElement = categoryDocument.createElement( "blog" );
		currentBlogElement.setAttribute( "name", blogId );
		currentBlogNode = currentBlogElement;
		categoryDocument.firstChild().appendChild( currentBlogNode );
		
		//insert category node
		QDomElement currentCategoryElement = categoryDocument.createElement( "category" );
		currentCategoryElement.setAttribute( "id", categoryId );
		currentBlogNode.appendChild( currentCategoryElement );
		return currentCategoryElement;
	}

	Blokkal::Account * const account;
	
	QMap<QString, CategoryList > categoryMap;

	QDomDocument categoryDocument;
	QString categoryFileName;
};

Blokkal::Io::CategoryManager::CategoryManager( Blokkal::Account * account ):
QObject( account ),
d( new Private( account ) )
{
	d->categoryFileName = account->dataDirectory() + QString::fromLatin1( "categories.xml" );
	
	//loading and parsing file
	QFile file( d->categoryFileName );
	if( file.open( QIODevice::ReadOnly ) ) {
		if( !d->categoryDocument.setContent( &file ) ) {
			kError() << k_funcinfo << "error parsing category file " << file.fileName() << endl;
		}
		file.close();
	}

	//build map
	QDomNode currentBlogNode = d->categoryDocument.firstChild().namedItem( "blog" );
	while( !currentBlogNode.isNull() ) {
		QDomElement currentBlogElement;
		QString blogName;
		if( currentBlogNode.isElement() && currentBlogNode.nodeName() == "blog" ) {
			currentBlogElement = currentBlogNode.toElement();
			blogName = currentBlogElement.attribute( "name" );
			d->categoryMap.insert( blogName, CategoryList() );
			
			CategoryList & currentCategoryList = d->categoryMap[ blogName ];
			QDomNode currentCategoryNode = currentBlogNode.namedItem( "category" );
			while( !currentCategoryNode.isNull() ) {
				if( currentCategoryNode.isElement() && currentCategoryNode.nodeName() == "category" ) {
					QDomElement currentCategoryElement = currentCategoryNode.toElement();
					Category * category  = new Category( this,
					                                     blogName,
					                                     currentCategoryElement.attribute( "id" ),
					                                     currentCategoryElement.attribute( "name" ),
					                                     currentCategoryElement.attribute( "html" ),
					                                     currentCategoryElement.attribute( "rss" ) );
					currentCategoryList.append( category );
					//kdDebug() << "found  in file: " << currentElement.attribute( "name" ) << endl;
				}
				currentCategoryNode = currentCategoryNode.nextSibling();
			}
			
		}
		currentBlogNode = currentBlogNode.nextSibling();
	}
}

Blokkal::Io::CategoryManager::~CategoryManager( void )
{
	QFile file( d->categoryFileName );
	if( !file.open( QIODevice::WriteOnly ) ) {
		kError() << k_funcinfo << "error opening category file " << file.fileName() << endl;
	}

   QTextStream fileStream( &file );
	d->categoryDocument.save( fileStream, 3 );
 
	file.close();
	
	delete d;
	d = 0;
}

void Blokkal::Io::CategoryManager::setValidCategories( const QString & blogId, QStringList categories )
{
	if( d->categoryMap.contains( blogId ) ) {
		CategoryList & oldCategories = d->categoryMap[blogId];
		for( int i = 0;
		     i < oldCategories.size(); )
		{
			if( !categories.contains( oldCategories.value( i )->id() ) ) {
				Category * removedCategory = oldCategories.takeAt( i );
				emit categoryRemoved( blogId, removedCategory, this );
				removedCategory->destroy();
			}
			else {
				++i;
			}
		}
	}
}

Blokkal::Io::CategoryList Blokkal::Io::CategoryManager::categories( const QString & blogId ) const
{
	if( d->categoryMap.contains( blogId ) ) {
		return d->categoryMap[blogId];
	}

	CategoryList list;
	return list;
}

Blokkal::Io::Category * Blokkal::Io::CategoryManager::category( const QString & blogId, const QString & categoryId )
{
	CategoryList & categoryList = d->categoryMap[ blogId ];
	for( CategoryList::ConstIterator it = categoryList.begin();
	     it != categoryList.end();
	     ++it )
	{
		if( (*it)->id() == categoryId ) {
			return *it;
		}
	}

	Category * category = new Category( this, blogId, categoryId );
	categoryList.append( category );
	emit categoryAdded( blogId, category, this );
	return category;
}

//--------
//Category
//--------
class Blokkal::Io::Category::Private {
public:
	Private( QDomElement node ):
	node( node ){}

	QDomElement node;

};

Blokkal::Io::Category::Category( Blokkal::Io::CategoryManager * manager, const QString & blogId, const QString & id, const QString & name, const QString & htmlUrl, const QString & rssUrl ):
QObject( manager ),
d( new Private( manager->d->categoryNode( blogId, id ) ) )
{
	d->node.setAttribute( "name", name );
	d->node.setAttribute( "html", htmlUrl );
	d->node.setAttribute( "rss", rssUrl );
}

Blokkal::Io::Category::~Category( void )
{
	delete d;
	d = 0;
}

QString Blokkal::Io::Category::id( void ) const
{
	return d->node.attribute( "id" );
}

QString Blokkal::Io::Category::name( void ) const
{
	return d->node.attribute( "name" );
}

QString Blokkal::Io::Category::htmlUrl( void ) const
{
	return d->node.attribute( "html" );
}

QString Blokkal::Io::Category::rssUrl( void ) const
{
	return d->node.attribute( "rss" );
}

void Blokkal::Io::Category::destroy( void )
{
	d->node.parentNode().removeChild( d->node );
	deleteLater();
}

void Blokkal::Io::Category::setName( const QString & name )
{
	d->node.setAttribute( "name", name );
	emit updated( this );
}

void Blokkal::Io::Category::setHtmlUrl( const QString & htmlUrl )
{
	d->node.setAttribute( "html", htmlUrl );
	emit updated( this );
}

void Blokkal::Io::Category::setRssUrl( const QString & rssUrl )
{
	d->node.setAttribute( "rss", rssUrl );
	emit updated( this );
}

Blokkal::Account * Blokkal::Io::CategoryManager::account( void ) const
{
	return d->account;
}
