/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "util.h"

#include <kdebug.h>

QString Blokkal::Io::Util::decodeUtf8Data( const QByteArray & rawData, bool treatAsFormData )
{
	const unsigned long inputLength = rawData.length();
	QString decodedData;
	decodedData.reserve( inputLength );
	unsigned char byte = 0;
	unsigned int unicodeChar = 0;
	char more = 0;
	
	for ( unsigned long i = 0; i < inputLength; i++ ) {
		if( treatAsFormData ) {
			unsigned char character;
			unsigned char highNibble;
			unsigned char lowNibble;
			//kDebug() << rawData.at( i ).cell() << endl;
			switch ( character = rawData.at( i ) ) {
			case '%':
				i++;
				character = rawData.at( i );
				if( character >= '0' && character <= '9' ) {
					highNibble = character - '0';
				}
				else {
					highNibble = ( character - 65 + 10 ) & 0xF;
				}
				i++;
				character = rawData.at( i );
				if( character >= '0' && character <= '9' ) {
					lowNibble = character - '0';
				}
				else {
					lowNibble = ( character - 65 + 10 ) & 0xF;
				}
				byte = ( highNibble << 4 ) | lowNibble ;
				break ;
			case '+':
				byte = ' ';
				break ;
			default:
				byte = character;
			}
		}
		else {
			byte = rawData.at( i );
		}
		
		// 10******* => trailing byte
		// 6 more bits for unicodeChar
		if ( ( byte & 0xc0 ) == 0x80 && more > 0 ) { 
			unicodeChar = ( unicodeChar << 6 ) | ( byte & 0x3f ); 
			more--;
			//was last byte
			if ( more <= 0 ) {
				decodedData.append( QChar( unicodeChar ) );
			}
		}
		// 0******* => US-ASCII character
		else if ( ( byte & 0x80 ) == 0x00 ) {
			decodedData.append( byte ) ;
		}
		// 110***** => 1 byte
		else if ( ( byte & 0xe0 ) == 0xc0 ) {	
			unicodeChar = byte & 0x1f;
			more = 1;
		}
		// 1110**** => 2 bytes
		else if ( ( byte & 0xf0 ) == 0xe0 ) {
			unicodeChar = byte & 0x0f;
			more = 2;
		}
		// 11110*** => 3 bytes
		else if ( ( byte & 0xf8 ) == 0xf0 ) {
			unicodeChar = byte & 0x07;
			more = 3;
		}
		// 111110** => 4 bytes
		else if ( ( byte & 0xfc ) == 0xf8 ) {
			unicodeChar = byte & 0x03;
			more = 4;
		}
		// 1111110* => 5 bytes
		else if ( ( byte & 0xfe ) == 0xfc ) {
			unicodeChar = byte & 0x01;
			more = 5;
		}
		else {
			more = 0;
			kError() << k_funcinfo << "Encountered invalid byte: " << byte << endl;
		}

	}
	return decodedData;
}
