/***************************************************************************
 *   Copyright (C) 2006 - 2007 by Martin Mller                            *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blokkalprotocol.h"
#include "blokkalprotocol.moc"

#include "blokkalaccountmanager.h"
#include "blokkalaccount.h"
#include "blokkalui/formattingbackend.h"

#include <kdebug.h>

Blokkal::Protocol::Protocol( const KComponentData & instance, QObject * parent ):
Plugin( instance, parent )
{}

Blokkal::Protocol::~Protocol( void )
{}

void Blokkal::Protocol::prepareToUnload( void )
{
	AccountList accounts = Blokkal::AccountManager::self()->accounts();
	if( accounts.isEmpty() ) {
		emit readyToUnload( this );
		return;
	}

	bool allAccountsWhereDisconnected = TRUE;
	for( AccountList::Iterator it = accounts.begin();
	     it != accounts.end();
	     ++it )
	{
		if( (*it)->connectionStatus() != Blokkal::Account::Disconnected
		    && (*it)->protocol() == this )
		{
			allAccountsWhereDisconnected = FALSE;
			connect( *it ,
			         SIGNAL( connectionStatusChanged( Blokkal::Account * ) ),
			         SLOT( checkAccountStates( void ) ) );
			(*it)->disconnectAccount();
		}
	}

	if( allAccountsWhereDisconnected ) {
		checkAccountStates();
	}
}

void Blokkal::Protocol::checkAccountStates( void )
{
	//check remaining acccounts
	AccountList accounts = Blokkal::AccountManager::self()->accounts();
	if( accounts.isEmpty() ) {
		emit readyToUnload( this );
		return;
	}

	bool isReadyToUnload = TRUE;
	for( AccountList::ConstIterator it = accounts.begin();
	     it != accounts.end();
	     ++it )
	{
		if( (*it)->connectionStatus() != Blokkal::Account::Disconnected
		    && (*it)->protocol() == this )
		{
			isReadyToUnload = FALSE; //found remaining account that is still not disconnected
			break;

		}
	}

	if( isReadyToUnload ) {
		//kDebug() << pluginName() << " ready to unload" << endl;
		emit readyToUnload( this );
	}
}

Blokkal::Ui::FormattingBackend * Blokkal::Protocol::createFormattingBackend( Blokkal::Ui::EditEntryWidget * widget )
{
	return new Blokkal::Ui::FormattingBackend( widget );
}
