/***************************************************************************
 *   Copyright (C) 2009 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALRSDCONFIGURATIONDATA_H
#define BLOKKALRSDCONFIGURATIONDATA_H

#include "blokkal_export.h"

#include <QtCore/QString>

namespace Blokkal {
	
/**
 * @short Stores RSD data
 * 
 * This object stores RSD data needed to configure an account.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT RsdConfigurationData {
public:
	/**
	 * Constructor
	 */
	RsdConfigurationData( const QString & apiName,
	                      const QString & apiLink, 
	                      const QString & homePageLink,
	                      const QString & engineName,
	                      const QString & engineLink,
	                      bool isPreferred );
	
	/**
	 * Destructor
	 */
	~RsdConfigurationData( void );
	
	/**
	 * @return the apiName
	 */
	QString apiName( void ) const;
	
	/**
	 * @return the apiLink
	 */
	QString apiLink( void ) const;
	
	
	/**
	 * @return the homePageLink
	 */
	QString homePageLink( void ) const;
	
	
	/**
	 * @return the engineName
	 */
	QString engineName( void ) const;
	
	
	/**
	 * @return the engineLink
	 */
	QString engineLink( void ) const;
	
	
	/**
	 * @return TRUE if this is the preferred method for this account
	 */
	bool isPreferred( void ) const;
	
	
private:
	class Private;
	Private * const d;
};
}

#endif