/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALTREEMODEL_H
#define BLOKKALTREEMODEL_H

#include <QtCore/QAbstractItemModel>

#include "blokkal_export.h"

namespace Blokkal {

/**
 * @short Wraps the tree like structures for a view
 *
 * Wraps the tree like structures for a view
 */
class BLOKKAL_EXPORT TreeModel : public QAbstractItemModel
{
Q_OBJECT
protected:
	/**
	 * This class is used as an internal representation of
	 * the underlying data structure
	 */
	class BLOKKAL_EXPORT TreeItemNode {
	public:
		/**
		 * Creates a new TreeItemNode as a child of @a parent
		 * If @a parent != 0, this node will append itself to the
		 * children of @a parent.
		 * @a data
		 * @a parent parent node
		 * @a data data
		 */
		TreeItemNode( TreeItemNode * parent, void * data );

		/**
		 * Destroys the BlogItemNode.
		 * All children will be deleted
		 */
		~TreeItemNode( void );

		///The parent
		TreeItemNode * const parent;

		///Child list
		QList<TreeItemNode*> children;

		///The data, contains the void pointer provided in the constructor
		QVariant data;

		///the checked status of the node
		Qt::CheckState checked;
	};

public:
	/**
	 * Constructor
	 * @a root root node
	 * @a parent parent object
	 */
	TreeModel( Blokkal::TreeModel::TreeItemNode * root = 0, QObject * parent = 0 );
	~TreeModel( void );

	virtual QModelIndex index( int row,
	                           int column,
	                           const QModelIndex & parent = QModelIndex() ) const;
	virtual QModelIndex parent( const QModelIndex & child ) const;
	virtual int rowCount( const QModelIndex & parent = QModelIndex() ) const;
	virtual int columnCount( const QModelIndex & parent = QModelIndex() ) const;
	virtual QVariant data( const QModelIndex & index,
	                       int role = Qt::DisplayRole) const;
	
	virtual Qt::ItemFlags flags( const QModelIndex & index ) const;
	virtual bool setData( const QModelIndex & index,
	                      const QVariant & value,
	                      int role = Qt::EditRole );

protected:
	TreeItemNode * root( void ) const;

private:
	class Private;
	Private * const d;
};

}

#endif
