/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UIBLOGCHOOSER_H
#define BLOKKAL_UIBLOGCHOOSER_H

#include <kdialog.h>

#include "../blokkal_export.h"

class QListBoxItem;
namespace Blokkal {
class Account;
class Blog;
namespace Ui {

/**
 * @short A widget for selecting a blog.
 * 
 * A widget for selecting a blog.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT BlogChooser : public QWidget
{
Q_OBJECT
public:
	/**
	 * Creates a new BlogChooser
	 * @param parent parent widget
	 */
	BlogChooser( QWidget * parent = 0 );

	/**
	 * Destructor
	 */
	virtual ~BlogChooser( void );

	/**
	 * Returns the currently selected account
	 * @return the currently selected account
	 */
	Blokkal::Account * account( void ) const;

	/**
	 * Sets the selected account. Calling this method will also
	 * clear the blog selection if @a account does not equal the currently selected
	 * account.
	 * @param account select this account, 0 to clear selection
	 */
	void setAccount( Blokkal::Account * account );

	/**
	 * Returns the currently selected blog
	 * @return the currently selected blog
	 */
	Blokkal::Blog * blog( void ) const;

	/**
	 * Sets the selected blog. If @a blog does not belong to the currently
	 * selected account or is 0, the selection will be cleared.
	 */
	void setBlog( Blokkal::Blog * blog );

Q_SIGNALS:
	/**
	 * This signal is emitted when a blog selected.
	 * @param blog the selected blog, 0 if the selection has been cleared
	 */
	void blogSelected( Blokkal::Blog * blog );

protected Q_SLOTS:
	/**
	 * An item in the accounts list box has been selected.
	 */
	virtual void slotAccountSelectionChanged( void );

	/**
	 * An item in the blogs list box has been selected.
	 */
	virtual void slotBlogSelectionChanged( void );
	
private:
	class Private;
	Private * d;
};

/**
 * @short A dialog, that displays a BlogChooser.
 *
 * A dialog, that displays a BlogChooser.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT BlogChooserDialog : public KDialog
{
Q_OBJECT
public:
	/**
	 * Creates a new SelectBlogDialog.
	 * @param message the message to display
	 * @param parent parent widget
	 */
	BlogChooserDialog( const QString & message, QWidget * parent = 0 );

	/**
	 * Destructor
	 */
	virtual ~BlogChooserDialog( void );

	/**
	 * Returns the BlogChooser.
	 * @return the BlogChooser
	 */
	Blokkal::Ui::BlogChooser * blogChooser( void ) const;

protected Q_SLOTS:
	/**
	 * Called when a new blog is selected.
	 * The default implementation diasbles the ok button if no
	 * blog is selected.
	 * @param blog the selected blog
	 */
	void slotBlogSelected( Blokkal::Blog * blog );

private:
	class Private;
	Private * d;
};

}

}

#endif
