/***************************************************************************
 *   Copyright (C) 2007 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blogdelegate.h"
#include "blogdelegate.moc"

#include "checkboxview.h"
#include "../blokkalaccount.h"
#include "../blokkalblog.h"
#include "../blokkalutils.h"
#include "../blokkalio/categorymanager.h"

#include <kdebug.h>
#include <klocale.h>
#include <kdialog.h>
#include <kglobalsettings.h>

#include <QPainter>
#include <QModelIndex>
#include <QApplication>

class Blokkal::Ui::BlogDelegate::Private {
public:
	Private( unsigned int displayOptions ):
	displayOptions( displayOptions ){}

	const unsigned int displayOptions;

	static QString blogDescription( const Blokkal::Blog * blog ) {
		return i18n( "%1 (%2)",  blog->id(), blog->type() );
	}

	static QString blogCount( const Blokkal::Account * account ) {
		return i18n( "(%1)", account->blogs().size() );
	}
};

Blokkal::Ui::BlogDelegate::BlogDelegate( unsigned int displayOptions ):
CheckBoxDelegate(),
d( new Private( displayOptions ) )
{
}

Blokkal::Ui::BlogDelegate::~BlogDelegate( void )
{
	delete d;
}

void Blokkal::Ui::BlogDelegate::paint( QPainter * painter,
                                       const QStyleOptionViewItem & option,
                                       const QModelIndex & index ) const
{
	CheckBoxDelegate::paint( painter, option, index );
	painter->save();
	
	Qt::TextElideMode elideMode;
	int alignment = Qt::AlignVCenter;
	if( option.direction == Qt::LeftToRight ) {
		alignment |= Qt::AlignLeft;
		elideMode = Qt::ElideRight;
	}
	else {
		alignment |= Qt::AlignRight;
		elideMode = Qt::ElideLeft;
	}
	

	Blokkal::Account * account = dynamic_cast<Blokkal::Account*>( (QObject*) index.data().toULongLong() );
	if( account ) {
		QRect iconRect;
		QRect textRect;
		QRect countRect;
		layoutAccount( option, account, iconRect, textRect, countRect );
		if( iconRect.isValid() ) {
			if( displayOptions() & DisplayAccountStates ) {
				if( account->connectionStatus() != Account::Connecting ) {
					view()->disconnectIndex( index );
					drawDecoration( painter, option, iconRect, account->isConnected() ? account->icon().pixmap( 16 ) : Utils::fadePixmap( account->icon().pixmap( 16 ) ) );
				}
				else {
					view()->connectIndex( index, 300 );
					drawDecoration( painter,
					                option,
					                iconRect,
					                view()->toogleBlinkState( index ) ? Utils::fadePixmap( account->icon().pixmap( 16 ) ) : account->icon().pixmap( 16 ) );
				}
			}
			else {
				drawDecoration( painter, option, iconRect, account->icon().pixmap( 16 ) );
			}
		}

		QStyleOptionViewItem myOption = option;
		if( displayOptions() & DisplayAccountStates
		    && !(option.state & QStyle::State_Selected)
		    && !account->isConnected() )
		{
			painter->setPen( option.palette.color( QPalette::Disabled, QPalette::Text) );
			
		}
		else if( option.state & QStyle::State_Selected ) {
			painter->setPen( option.palette.color( QPalette::Normal, QPalette::HighlightedText ) );
		}
		
		if( textRect.isValid() ) {
			painter->drawText( textRect, 
			                   alignment,
			                   elidedText( painter->fontMetrics(), 
			                               textRect.width(),
			                               elideMode,
			                               account->id() ) );
		}

		if( countRect.isValid() ) {
			painter->save();
			painter->setFont( KGlobalSettings::smallestReadableFont() );
			painter->drawText( countRect, 
			                   alignment,
			                   elidedText( painter->fontMetrics(),
			                               countRect.width(),
			                               elideMode,
			                               d->blogCount( account ) ) );
			painter->restore();
		}

		painter->restore();
		return;
	}
	Blokkal::Blog * blog = dynamic_cast<Blokkal::Blog*>( (QObject*) index.data().toULongLong() );
	if( blog ) {
		QString title = blog->title().isEmpty() ? blog->id() : blog->title();
		QString idTypeString = d->blogDescription( blog );
		QRect iconRect;
		QRect textRect;
		QRect infoRect;
		layoutBlog( option, index, blog, iconRect, textRect, infoRect );
		if( iconRect.isValid() ) {
			if( displayOptions() & DisplayAccountStates
			    && !blog->account()->isConnected() )
			{
				drawDecoration( painter, option, iconRect, Utils::fadePixmap( blog->icon().pixmap( 16 ) ) );
			}
			else {
				drawDecoration( painter, option, iconRect, blog->icon().pixmap( 16 ) );
			}
		}

		QStyleOptionViewItem myOption = option;
		if( displayOptions() & DisplayAccountStates
		    && !(option.state & QStyle::State_Selected)
		    && !blog->account()->isConnected() )
		{
			painter->setPen( option.palette.color( QPalette::Disabled, QPalette::Text) );	
		}
		else if( option.state & QStyle::State_Selected ) {
			painter->setPen( option.palette.color( QPalette::Normal, QPalette::HighlightedText ) );
		}
		
		if( textRect.isValid() ) {
			painter->drawText( textRect,
			                   alignment,
			                   elidedText( painter->fontMetrics(),
			                               textRect.width(),
			                               elideMode,
			                               title ) );
		}
		
		if( infoRect.isValid() ) {
			painter->save();
			painter->setFont(KGlobalSettings::smallestReadableFont());
			painter->drawText( infoRect,
			                   alignment,
			                   elidedText( painter->fontMetrics(),
			                               infoRect.width(),
			                               elideMode,
			                               idTypeString ) );
			painter->restore();
		}

		painter->restore();
		return;
	}

	Blokkal::Io::Category * category = dynamic_cast<Blokkal::Io::Category*>( (QObject*) index.data().toULongLong() );
	if( category ) {
		QRect textRect;
		layoutPlainItem( option, index, textRect );
		if( textRect.isValid() ) {
			painter->drawText( textRect,
			                   alignment, 
			                   elidedText( painter->fontMetrics(),
			                               textRect.width(),
			                               elideMode,
			                               category->name() ) );
		}

		painter->restore();
		return;
	}

	painter->restore();
}

QSize Blokkal::Ui::BlogDelegate::sizeHint( const QStyleOptionViewItem & option,
                                           const QModelIndex & index ) const
{
	Blokkal::Account * account = dynamic_cast<Blokkal::Account*>( (QObject*) index.data().toULongLong() );
	if( account ) {
		QRect iconRect;
		QRect textRect;
		QRect countRect;
		layoutAccount( option, account, iconRect, textRect, countRect );
		return (iconRect|textRect|countRect).size();
	}
	
	Blokkal::Blog * blog = dynamic_cast<Blokkal::Blog*>( (QObject*) index.data().toULongLong() );
	if( blog ) {
		QRect iconRect;
		QRect textRect;
		QRect infoRect;
		layoutBlog( option, index, blog, iconRect, textRect, infoRect );
		return ( iconRect|textRect|infoRect).size();
	}
	
	Blokkal::Io::Category * category = dynamic_cast<Blokkal::Io::Category*>( (QObject*) index.data().toULongLong() );
	if( category ) {
		QRect textRect;
		layoutPlainItem( option, index, textRect );
		return textRect.size();
	}
	

	kWarning() << "encountered invalid data type" << endl;
	return QSize( 20,20 );
}


unsigned int Blokkal::Ui::BlogDelegate::displayOptions( void ) const
{
	return d->displayOptions;
}

QRect Blokkal::Ui::BlogDelegate::checkBoxRect( const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
	if( displayOptions() & DisplayCheckBoxes ) {
		Blokkal::Account * account = dynamic_cast<Blokkal::Account*>( (QObject*) index.data().toULongLong() );
		if( account ) {
			return QRect();
		}
	}
	else {
		return QRect();
	}

	return CheckBoxDelegate::checkBoxRect( option, index );
}

void Blokkal::Ui::BlogDelegate::layoutAccount( const QStyleOptionViewItem & option,
                                               const Blokkal::Account * account,
                                               QRect & iconRect,
                                               QRect & textRect,
                                               QRect & countRect ) const
{
	QStyle * const style = QApplication::style();
   const int margin = style->pixelMetric( QStyle::PM_FocusFrameHMargin ) + 1;
	const int height = option.fontMetrics.height();
	const int width = option.rect.width();
	const int x = option.rect.x();
	const int y = option.rect.y();

	if( displayOptions() & DisplayAccountIcon
	    || displayOptions() & DisplayAccountStates )
	{
		if( option.direction == Qt::LeftToRight ) {
			//x, y, width, height
			iconRect.setRect( x, y, height, height );
		}
		else {
			iconRect.setRect( x + width - height, y, height, height );
		}
	}
	
	textRect.setRect( x, y, width, height );
	if( option.direction == Qt::LeftToRight ) {
		if( iconRect.isValid() ) {
			textRect.setLeft( iconRect.right() /*+ margin*/ );
		}
		textRect.setWidth( option.fontMetrics.width( account->id() ) + 2*margin );
	}
	else {
		if( iconRect.isValid() ) {
			textRect.setRight( iconRect.left() /*- margin*/ );
		}
		textRect.setLeft( textRect.right() - option.fontMetrics.width( account->id() ) - 2*margin );
	}

	
	if( displayOptions() & DisplayBlogCount ) {
		const int countWidth = QFontMetrics( KGlobalSettings::smallestReadableFont() ).width( d->blogCount( account ) ) + 2*margin;
		if( option.direction == Qt::LeftToRight ) {
			int countX = x + width - countWidth;
			if( textRect.right() + countWidth < x + width ) {
				countX = textRect.right() /*+ margin*/;
			}
			countRect.setRect( countX,
			                   y,
			                   countWidth,
			                   height );
		}
		else {
			int countX = x;
			if( textRect.left() - countWidth > x ) {
				countX = textRect.left() /*- margin*/;
			}
			countRect.setRect( countX, y, countWidth, height );
		}
	}

	if( option.direction == Qt::LeftToRight ) {
		if( countRect.isValid() ) {
			textRect.setRight( countRect.left() /*- margin*/ );
		}
		else {
			textRect.setRight( x + width );
		}
	}
	else {
		if( countRect.isValid() ) {
			textRect.setLeft( countRect.right() /*+ margin */);
		}
		else {
			textRect.setLeft( x );
		}
	}
}

void Blokkal::Ui::BlogDelegate::layoutBlog( const QStyleOptionViewItem & option,
                                            const QModelIndex & index,
                                            const Blokkal::Blog * blog,
                                            QRect & iconRect,
                                            QRect & textRect,
                                            QRect & infoRect ) const
{
	Q_UNUSED( blog );
	//QStyle * const style = QApplication::style();
	//const int margin = style->pixelMetric( QStyle::PM_FocusFrameHMargin ) + 1;
	const int height = option.fontMetrics.height();
	const int width = option.rect.width();
	const int x = option.rect.x();
	const int y = option.rect.y();

	QRect checkBoxRect;
	
	if( displayOptions() & DisplayCheckBoxes ) {
		checkBoxRect = this->checkBoxRect( option, index );
	}
	
	
	if( displayOptions() & DisplayBlogIcon ) {
		const int iconHeight = height;//( displayOptions() & DisplayBlogDescription ) ? height + QFontMetrics( KGlobalSettings::smallestReadableFont() ).height(): height;
		if( option.direction == Qt::LeftToRight ) {
			if( checkBoxRect.isValid() ) {
				iconRect.setRect( checkBoxRect.right(), y, iconHeight, iconHeight );
			}
			else {
				iconRect.setRect( x, y, iconHeight, iconHeight );
			}
		}
		else {
			if( checkBoxRect.isValid() ) {
				iconRect.setRect( checkBoxRect.left() - iconHeight, y, iconHeight, iconHeight );
			}
			else {
				iconRect.setRect( x + width - iconHeight, y, iconHeight, iconHeight );
			}
		}
	}	

	textRect.setRect( x, y, width, height );
	if( option.direction == Qt::LeftToRight ) {
		if( iconRect.isValid() ) {
			textRect.setLeft( iconRect.right() );
		}
		else if( checkBoxRect.isValid() ) {
			textRect.setLeft( checkBoxRect.right() );
		}
		//textRect.setWidth( option.fontMetrics.width( blog->id() ) + 2*margin );
	}
	else {
		if( iconRect.isValid() ) {
			textRect.setRight( iconRect.left() );
		}
		else if( checkBoxRect.isValid() ) {
			textRect.setLeft( checkBoxRect.left() );
		}		
		//textRect.setLeft( textRect.right() - option.fontMetrics.width( blog->id() ) - 2*margin );
	}

	if( displayOptions() & DisplayBlogDescription ) {
		/*if( iconRect.isValid() ) {
			infoRect.setRect( iconRect.right(),
			                  y + height,
			                  width,
			                  QFontMetrics( KGlobalSettings::smallestReadableFont() ).height() );
		}
		else*/ if( checkBoxRect.isValid() ) {
			infoRect.setRect( checkBoxRect.right(),
			                  y + height,
			                  width,
			                  QFontMetrics( KGlobalSettings::smallestReadableFont() ).height() );
		}
		else {
			infoRect.setRect( x,
			                  y + height,
			                  width,
			                  QFontMetrics( KGlobalSettings::smallestReadableFont() ).height() );
		}
	}
}
