/***************************************************************************
 *   Copyright (C) 2007 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALUIBLOGDELEGATE_H
#define BLOKKALUIBLOGDELEGATE_H

#include "checkboxdelegate.h"

#include "../blokkal_export.h"

namespace Blokkal {
class Account;
class Blog;
namespace Ui {
class BlogView;
class BLOKKAL_EXPORT BlogDelegate : public Blokkal::Ui::CheckBoxDelegate
{
Q_OBJECT
public:
	/**
	 * Enumerator for the different options available for use with
	 * this delegate.
	 */
	enum DisplayOptions {
		DisplayAccountIcon = 0x001, ///< Display the icons of accounts
		DisplayBlogIcon = 0x002, ///< Display the icons of blogs
		DisplayBlogDescription = 0x004, ///< Display the blog id and the blog type
		DisplayBlogCount = 0x008, ///< Display the number of blogs belonging to an account
		DisplayCheckBoxes = 0x010, ///< Make blogs and categories checkable
		DisplayAccountStates = 0x020 ///< Displays the account connection states, blinking when connecting
	};
		
	/**
	 * Constructor
	 */
	BlogDelegate( unsigned int displayOptions = 0 );
	~BlogDelegate( void );

	unsigned int displayOptions( void ) const;

	/**
	 * Paints the item
	 */
	virtual void paint( QPainter * painter,
	                    const QStyleOptionViewItem & option,
	                    const QModelIndex &index) const;

	/**
	 * Returns the size hint
	 */
	virtual QSize sizeHint(const QStyleOptionViewItem & option,
	                       const QModelIndex & index) const;
	
	virtual QRect checkBoxRect( const QStyleOptionViewItem & option,
	                            const QModelIndex & index ) const;

protected:
	void layoutAccount( const QStyleOptionViewItem & option,
	                    const Blokkal::Account * account,
	                    QRect & iconRect,
	                    QRect & textRect,
	                    QRect & countRect ) const;

	void layoutBlog( const QStyleOptionViewItem & option,
	                 const QModelIndex & index,
	                 const Blokkal::Blog * blog,
	                 QRect & iconRect,
	                 QRect & textRect,
	                 QRect & infoRect ) const;
private:
	class Private;
	Private * const d;
};
}
}
#endif
