/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UiBROWSEENTRYVIEWFACTORY_H
#define BLOKKAL_UiBROWSEENTRYVIEWFACTORY_H

#include "../blokkalplugin.h"
#include "../blokkalio/blogbrowsingdriver.h"

#include <kdialog.h>
#include <kurl.h>

namespace Blokkal {
namespace Ui {
class BrowseEntryDialog;
/**
 * @short Base class for plugins that provide views to browse entries.
 *
 * Base class for plugins that provide views to browse entries.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT BrowseEntryViewFactory : public Blokkal::Plugin
{
Q_OBJECT
public:
	/**
	 * Creates a new BrowseEntryViewPlugin object.
	 * @param parent owning object
	 * @param instance instance of the plugin
	 * @see Blokkal::Plugin
	 */
	BrowseEntryViewFactory( const KComponentData & instance, QObject * parent = 0 );

	/**
	 * Destructor
	 */
	virtual ~BrowseEntryViewFactory( void );

	/**
	 * Creates a new view to browse entries in @a blog.
	 * @return a new view
	 */
	virtual Blokkal::Ui::BrowseEntryDialog * createView( Blokkal::Blog * blog ) = 0;

};

/**
 * @short Base class for dalogs used to browse old entries
 *
 * Dialogs can be used in different modes to access old entries
 * in a blog. BrowseEntryDialog are not supposed to be destroyed
 * after they have been closed. Blokkal::Blog keeps track of
 * the dialogs and attempts to reuse instances in order to minimise
 * network traffic.
 */
class BLOKKAL_EXPORT BrowseEntryDialog : public KDialog {
public:
	/**
	 * Constructor.
	 * @param parent parent widget
	 * @param flags window flags
	 */
	BrowseEntryDialog( QWidget * parent = 0, Qt::WFlags flags = 0 );

	/**
	 * Destructor
	 */
	virtual ~BrowseEntryDialog( void );

	/**
	 * Sets the mode of this view to @a mode
	 * @param mode the new mode
	 */
	void setMode( Blokkal::Io::BlogBrowsingDriver::BrowseMode mode );

	/**
	 * @return the current mode
	 */
	Blokkal::Io::BlogBrowsingDriver::BrowseMode mode( void ) const;

	/**
	 * This method is primarily intended to be used together with exec()
	 * after the dialog has been closed, but it should always reflect the correct
	 * selection mode.
	 * @return the url of the currently selected entry or KUrl() if no entry has been selected
	 */
	virtual KUrl entryUrl( void ) const = 0;
	
protected:
	/**
	 * This method is called when the mode has been changed by setMode()
	 * Reimplement this method to update the view as appropriate for the
	 * current mode.
	 * The default implementation does nothing.
	 */
	virtual void applyViewMode( void );
	
private:
	class Private;
	Private * const d;
};

}

}

#endif
