/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UIBROWSEOPTIONWIDGET_H
#define BLOKKAL_UIBROWSEOPTIONWIDGET_H

#include "../blokkal_export.h"

#include <QtGui/QWidget>

namespace Blokkal {

namespace Io {
class LoadEntryJob;
class BlogBrowsingDriver;
}

namespace Ui {

/**
 * @short Base class for custom browse options
 *
 * If your blog provides unusual browsing options you can subclass
 * this class and return it in your BlogBrowsingDriver
 * implementation.
 *  @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT BrowseOptionWidget : public QWidget
{
Q_OBJECT
public:
	/**
	 * Constructor.
	 * This widget comes with a default QHBoxLayout so you can just call
	 * layout()->addWidget() to add your child widgets.
	 * @param driver the driver to be used by this widget
	 * @param parent the parent widget
	 */
	BrowseOptionWidget( Blokkal::Io::BlogBrowsingDriver * driver, QWidget * parent = 0 );
	virtual ~BrowseOptionWidget( void );

	/**
	 * Returns the driver used by this widget.
	 * @return the driver used by this widget
	 */
	Blokkal::Io::BlogBrowsingDriver * driver( void ) const;
	
	/**
	 * Creates and returns a LoadEntryJob configured to load
	 * entries according to the current content of the widget.
	 * @return a LoadEntryJob
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( void ) = 0;

Q_SIGNALS:
	/**
	 * If your widget includes a text input or some similar widget
	 * that has a returnPressed() signal, you can emit this signal
	 * after the return key has been pressed to indicate that the entry
	 * list should be reloaded.
	 */
	void reloadRequested( Blokkal::Ui::BrowseOptionWidget * widget );

protected Q_SLOTS:
	/**
	 * This slot emits the reloadRequested() signal
	 */
	void emitReloadRequested( void );

private:
	class Private;
	Private * const d;
};

}

}

#endif
