/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UICATEGORYVIEW_H
#define BLOKKAL_UICATEGORYVIEW_H

#include "../blokkal_export.h"

#include "checkboxview.h"
#include "../blokkaltreemodel.h"

namespace Blokkal {
class Entry;
class Blog;
namespace Io {
class Category;
}
namespace Ui {

/**
 * @short Widget for displaying and selecting categories
 *
 * Widget for displaying and selecting categories.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT CategoryView : public Blokkal::Ui::BlogView
{
Q_OBJECT
public:
	/**
	 * Creates a new CategoryView.
	 * @param entry display the categories of this blog
	 * @param exclusive if TRUE only one category may be selected
	 * @param parent parent widget
	 * @param name object name
	 */
	CategoryView( Blokkal::Entry * entry, bool exclusive, QWidget * parent = 0 );

	/**
	 * Destructor
	 */
	virtual ~CategoryView( void );

	/**
	 * Selects the item that represent a category with a title listed in @a ids
	 */
	void selectCategories( const QStringList & ids );

	/**
	 * Returns a list with the titles of the selected categories.
	 * @return a list with the titles of the selected categories
	 */
	QStringList selectedCategories( void ) const;

Q_SIGNALS:
	/**
	 * This signal is emitted, when a previously checked category has been unchecked ot vice versa
	 */
	void categorySelectionChanged( void );

private:
	class Private;
	Private * const d;
};

/**
 * @a short Model for categories
 *
 * Model for categories
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT CategoryModel : public Blokkal::TreeModel {
Q_OBJECT
public:
	/**
	 * Constructor
	 */
	CategoryModel( Blokkal::Blog * blog, QObject * parent = 0 );
	virtual ~CategoryModel( void );

	/**
	 * Enumeration for custom roles to be used with data()
	 */
	enum CustomRoles {
		SortRole = Qt::UserRole + 1 ///< role to return a data type that can be used with QSortFilterProxModel
	};

	/**
	 * Returns the data for @a index
	 */
	virtual QVariant data( const QModelIndex & index,
	                       int role = Qt::DisplayRole) const;
	
private Q_SLOTS:
	void addCategory( const QString & blogId, Blokkal::Io::Category * category );
	void removeCategory( const QString & blogId, Blokkal::Io::Category * category );
	void emitDataChanged( Blokkal::Io::Category * category );

private:
	class Private;
	Private * const d;
};

}

}

#endif
