/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALUICHECKBOXDELEGATE_H
#define BLOKKALUICHECKBOXDELEGATE_H

#include <QtGui/QItemDelegate>

#include "../blokkal_export.h"

namespace Blokkal {
namespace Ui {
class CheckBoxView;

/**
 * @a short A delegate that paints the checkbox to a custom location
 *
 * A delegate that paints the checkbox to a custom location
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT CheckBoxDelegate : public QItemDelegate
{
Q_OBJECT
public:
	/**
	 * Constructor
	 */
	CheckBoxDelegate( QObject * parent = 0 );
	~CheckBoxDelegate( void );

protected:
	/**
	 * Returns the size of the the checkbox
	 * @return the size of the the checkbox
	 */
	int checkBoxSize( const QFontMetrics & metrics ) const;

	/**
	 * Returns the position of the check box for @a index
	 * The rect is invalid if a checkbox is not appropriate for @a index.
	 * @return the position of the check box for @a index
	 */
	virtual QRect checkBoxRect( const QStyleOptionViewItem & option,
	                            const QModelIndex & index ) const;

	/**
	 * Paints the checkbox in the rect returned by checkBoxRect(), if that
	 * rect is valid.
	 */
	virtual void paint( QPainter * painter,
	                    const QStyleOptionViewItem & option,
	                    const QModelIndex &index) const;
	/**
	 * Returns the size hint. This is the size hint for the checkbox.
	 * If the rect returned by checkBoxRect is invalid, then this method
	 * will return a 0 size.
	 */
	virtual QSize sizeHint(const QStyleOptionViewItem & option,
	                       const QModelIndex & index) const;

	/**
	 * Sets the view for this delegate to @a view
	 */
	void setView( Blokkal::Ui::CheckBoxView * view );

	/**
	 * Returns a pointer to the view.
	 * @return a pointer to the view
	 */
	Blokkal::Ui::CheckBoxView * view( void ) const;

	/**
	 * Reimplemented to use the virtual method checkBoxRect()
	 * instead of the check() as in the QItemDelegate implementation
	 */
	virtual bool editorEvent(QEvent * event,
	                         QAbstractItemModel * model,
	                         const QStyleOptionViewItem & option,
	                         const QModelIndex & index );

	/**
	 * This method is called when the checkstate of @a index
	 * has changed to @a newState.
	 * The default implementation does nothing.
	 */
	virtual void stateChanged( QAbstractItemModel * model,
	                           const QModelIndex & index,
	                           Qt::CheckState newState );

	/**
	 * This method does the layout for a simple item with a
	 * checkbox (if appropriate for @a index ) and a text
	 * that may be drawn into @a textRect
	 */
	void layoutPlainItem( const QStyleOptionViewItem & option,
	                      const QModelIndex & index,
	                      QRect & textRect ) const;
	
private:
	friend class CheckBoxView;
	class Private;
	Private * const d;
};
}
}
#endif
