/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALUICHECKBOXVIEW_H
#define BLOKKALUICHECKBOXVIEW_H

#include <QtGui/QTreeView>

#include "../blokkal_export.h"

namespace Blokkal {
namespace Ui {
class CheckBoxDelegate;

/**
 * @a short Subclass of QTreeView that provides some extra methods for qorking with checkboxes
 *
 * Subclass of QTreeView that provides some extra methods for qorking with checkboxes
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT CheckBoxView : public QTreeView
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param delegate subclass of CheckBoxDelegate to use
	 * @param parent parent widget
	 */
	CheckBoxView( CheckBoxDelegate * delegate, QWidget * parent = 0 );
	virtual ~CheckBoxView( void );

	/**
	 * Returns whether the item at @a index is checked.
	 * @param index index to test
	 * @return TRUE if @a index is checked
	 */
	bool isChecked( const QModelIndex & index ) const;

	/**
	 * Sets the checked status of @a index.
	 * The checkStateHook() will not be called by this method.
	 * @param index index to apply the new state to
	 * @param check the new state
	 */
	void setChecked( const QModelIndex & index, bool check );

protected Q_SLOTS:
	/**
	 * Overridden to remove outdated references stored by this view
	 * @param parent parent index
	 * @param start first row about to be removed
	 * @param end last row about to be remove
	 */
	virtual void rowsAboutToBeRemoved( const QModelIndex & parent, int start, int end );

private Q_SLOTS:
	void updateIndexId( int id );

private:
	friend class CheckBoxDelegate;
	friend class BlogDelegate;
	/**
	 * This method is used by the BlogDelegate to determine
	 * the current blink state of icons.
	 */
	bool toogleBlinkState( const QModelIndex & index );

	/**
	 * This method is used by the BlogDelegate to tell the view
	 * that certain indexes need periodic updating.
	 * If @a index is already connected, nothing happens
	 */
	void connectIndex( const QModelIndex & index, int msecs );

	/**
	 * This method is used by the BlogDelegate to tell the view that
	 * @a index no longer needs periodic updating.
	 * If @a index is not connected, nothing happens
	 */
	void disconnectIndex( const QModelIndex & index );
	
	class Private;
	Private * const d;
};

/**
 * @short A TreeView that works with a BlogDelegate
 *
 * Together with a BlogModel or a BlogCategegoryModel this view
 * displays the tree structure of accounts, blogs
 * and ( with a BlogCategegoryModel) categories.
 * Blogs and Categories may be checkable, depending on the display options
 * set in the constructor. You must set the desired view model to make
 * this view work.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT BlogView : public Blokkal::Ui::CheckBoxView
{
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param delegate subclass of CheckBoxDelegate to use
	 * @param parent parent widget
	 */
	BlogView( QWidget * parent = 0, unsigned int displayOptions = 0 );
	virtual ~BlogView( void );
};
}
}

#endif
