/***************************************************************************
*   Copyright (C) 2006 - 2009 by Martin Mueller                            *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALUiEDITACCOUNTWIDGET_H
#define BLOKKALUiEDITACCOUNTWIDGET_H

#include <qwidget.h>

#include "../blokkal_export.h"

namespace Blokkal {
class Account;
class Protocol;

class RsdConfigurationData;

namespace Ui {
/**
 * @short Base Class for widgets to edit account properties.
 *
 * Base Class for widgets to edit account properties.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT EditAccountWidget : public QWidget
{
Q_OBJECT
public:
	/**
	 * Constructs a new EditAccountWidget.
	 * @param protocol the protocol object
	 * @param account the account object for this widget, may be null
	 * @param parent the parent widget
	 */
	EditAccountWidget( Blokkal::Protocol * protocol,
	                   Blokkal::Account * account,
	                   QWidget * parent = 0 );

	/**
	 * Destructor
	 */
	virtual ~EditAccountWidget( void );

	/**
	 * This method is called to determine if the entered information is valid.
	 * @return TRUE if the entered information is valid
	 */
	virtual bool isValid( void ) const = 0;

	/**
	 * This method returns a pointer to the protocol.
	 * @return the protocol object
	 */
	Blokkal::Protocol * protocol( void ) const;

	/**
	 * This method returns a pointer to the updated account object.
	 * @return the account object
	 */
	virtual Blokkal::Account * apply( void ) = 0;
	
	/**
	 * Sets up the widget according to the information contained in @a data
	 * @param data RSD data to use
	 * @see Blokkal::RsdConfigurationData
	 */
	virtual void setup( const Blokkal::RsdConfigurationData & data ) = 0;

Q_SIGNALS:
	/**
	 * This signal is emitted whenever the values in this widget change.
	 */
	void valuesChanged( void );

protected:
	/**
	 * This method returns a pointer to the account object.
	 * @note this method may return 0, if this widget is used to create a new account and apply() has not been called yet!
	 * @return the account object
	 */
	Blokkal::Account * account( void ) const;

private:
	class Private;
	Private * d;
};
}
}

#endif
