/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "editentrywidget.h"
#include "editentrywidget.moc"

#include "../blokkalprotocol.h"
#include "../blokkalaccount.h"
#include "../blokkalblog.h"
#include "../blokkalentry.h"
#include "../blokkalio/categorymanager.h"
#include "entryextensionwidget.h"
#include "htmlentrytextedit.h"
#include "categoryview.h"
#include "formattingbackend.h"

#include <klineedit.h>
#include <kdatetimewidget.h>
#include <kpushbutton.h>
#include <klocale.h>
#include <kdialog.h>
#include <kxmlguifactory.h>
#include <kmenu.h>
#include <kiconloader.h>
#include <kdebug.h>
#include <kactioncollection.h>
#include <kaction.h>
#include <kpushbutton.h>

#include <QLayout>
#include <QLabel>
#include <QCheckBox>
#include <QTimer>
#include <QSignalMapper>
#include <QSplitter>

class Blokkal::Ui::EditEntryWidget::Private
{
public:
	Private( Blokkal::Entry * entry, unsigned int features ) :
	entry( entry ),
	features( features ),
	entryHadNoDate( FALSE ),
	extension0( 0 ),
	extension1( 0 ),
	extension2( 0 ),
	extension3( 0 ),
	extension4( 0 ),
	addCategoryButton( 0 ),
	addCategoryPopup( 0 ),
	newCategoryLineEdit( 0 ),
	removeCategoryButton( 0 ),
	removeCategoryPopup( 0 ) {}

	Blokkal::Entry * const entry;
	const unsigned int features;

	KDateTimeWidget * dateWidget;
	QCheckBox * useCurrentDateCheckBox;

	QLabel * subjectLabel;
	KLineEdit * subjectLineEdit;

	QSplitter * textCategorySplitter;
	Blokkal::Ui::HtmlEntryTextEdit * entryTextEdit;

	QSplitter * categoryExtensionSplitter;
	QWidget * categoryToolsContainer;
	Blokkal::Ui::CategoryView * categoryView;

	bool entryHadNoDate;
	EntryExtensionWidget * extension0;
	EntryExtensionWidget * extension1;
	EntryExtensionWidget * extension2;
	EntryExtensionWidget * extension3;
	EntryExtensionWidget * extension4;
	FormattingBackend * formattingBackend;
	
	KPushButton * addCategoryButton;
	KMenu * addCategoryPopup;
	KLineEdit * newCategoryLineEdit;

	KPushButton * removeCategoryButton;
	KMenu * removeCategoryPopup;
	QMap<QString, KAction * > removeCategoryActions;
	QSignalMapper * removeCategoryMapper;
	KActionCollection * removeActionCollection;

	QString caption;
};

Blokkal::Ui::EditEntryWidget::EditEntryWidget( Blokkal::Entry * entry, QWidget * parentWidget ) :
QWidget( parentWidget ),
d( new Private( entry, entry->blog()->supportedFeatures() ) )
{
	setLayout( new QVBoxLayout( this ) );
	layout()->setMargin( 0 );

	d->extension0 = entry->createExtensionWidget( EntryExtensionWidget::ExtensionWidget0, this );
	if( d->extension0 ) {
		connect( d->extension0,
		         SIGNAL( contentChanged( Blokkal::Ui::EntryExtensionWidget * ) ),
		         SLOT( emitContentsChanged( void ) ) );
		layout()->addWidget( d->extension0 );
	}
	//date
	QHBoxLayout * dateLayout = new QHBoxLayout( 0 );
	dateLayout->setMargin( 0 );
	dateLayout->setSpacing( KDialog::spacingHint() );
	d->dateWidget = new KDateTimeWidget( this );
	dateLayout->addWidget( d->dateWidget );
	d->useCurrentDateCheckBox = new QCheckBox( i18n( "Use current date" ), this );
	dateLayout->addWidget( d->useCurrentDateCheckBox );
	layout()->addItem( dateLayout );
	connect( d->dateWidget,
	         SIGNAL( valueChanged ( const QDateTime & ) ),
	         SLOT( slotDateChanged( void ) ) );
	d->extension1 = entry->createExtensionWidget( EntryExtensionWidget::ExtensionWidget1, this );
	if( d->extension1 ) {
		connect( d->extension1,
		         SIGNAL( contentChanged( Blokkal::Ui::EntryExtensionWidget * ) ),
		         SLOT( emitContentsChanged( void ) ) );
		dateLayout->addWidget( d->extension1 );
	}

	//subject
	QHBoxLayout * subjectLayout = new QHBoxLayout( 0 );
	subjectLayout->setMargin( 0 );
	subjectLayout->setSpacing( KDialog::spacingHint() );
	d->subjectLabel = new QLabel( i18n( "Subject"), this );
	subjectLayout->addWidget( d->subjectLabel );
	d->subjectLineEdit = new KLineEdit( this );
	d->subjectLabel->setBuddy( d->subjectLineEdit );
	subjectLayout->addWidget( d->subjectLineEdit );
	layout()->addItem( subjectLayout );
	connect( d->subjectLineEdit,
	         SIGNAL( textChanged ( const QString & ) ),
	         SLOT( emitContentsChanged( void ) ) );
	d->extension2 = entry->createExtensionWidget( EntryExtensionWidget::ExtensionWidget2, this );
	if( d->extension2 ) {
		connect( d->extension2,
		         SIGNAL( contentChanged( Blokkal::Ui::EntryExtensionWidget * ) ),
		         SLOT( emitContentsChanged( void ) ) );
		subjectLayout->addWidget( d->extension2 );
	}

	//text edit
	d->textCategorySplitter = new QSplitter( this );
	layout()->addWidget( d->textCategorySplitter );
	QSizePolicy policy( QSizePolicy::Expanding, QSizePolicy::Expanding );
	policy.setVerticalStretch( 2 );
	policy.setHorizontalStretch( 2 );
	d->textCategorySplitter->setSizePolicy( policy );
	d->entryTextEdit = new HtmlEntryTextEdit();
	d->textCategorySplitter->addWidget( d->entryTextEdit );
	d->textCategorySplitter->setStretchFactor( 0, 2 );
	connect( d->entryTextEdit,
	         SIGNAL( plainTextChanged( void ) ),
	         SLOT( emitContentsChanged( void ) ) );
	
	//category view + extension
	d->categoryExtensionSplitter = new QSplitter( Qt::Vertical );
	d->textCategorySplitter->addWidget( d->categoryExtensionSplitter );
	d->categoryExtensionSplitter->setMinimumWidth( 30 );
	d->extension3 = entry->createExtensionWidget( EntryExtensionWidget::ExtensionWidget3, this );
	if( d->extension3 ) {
		d->categoryExtensionSplitter->addWidget( d->extension3 );
		connect( d->extension3,
		         SIGNAL( contentChanged( Blokkal::Ui::EntryExtensionWidget * ) ),
		         SLOT( emitContentsChanged( void ) ) );
		connect( d->extension3,
		         SIGNAL( visibilityChanged( void ) ),
		         SLOT( updateSplitterVisibility( void ) ) );
	}
	d->categoryToolsContainer = new QWidget( );
	d->categoryExtensionSplitter->addWidget( d->categoryToolsContainer );
	QVBoxLayout * categoryToolsContainerLayout = new QVBoxLayout( d->categoryToolsContainer );
	categoryToolsContainerLayout->setMargin( 0 );
	d->categoryView = new CategoryView( entry,
	                                    !( d->features & Blog::FeatureMultipleCategories ),
	                                    d->categoryToolsContainer );
 	categoryToolsContainerLayout->addWidget( d->categoryView );
	d->categoryView->selectCategories( entry->categories() );
	connect( d->categoryView,
	         SIGNAL( categorySelectionChanged( void ) ),
	         SLOT( emitContentsChanged( void ) ) );
	if( ( ( d->features & ( Blog::FeatureAddCategories ) ) & ~Blog::FeatureCategory )
	    || ( ( d->features & ( Blog::FeatureRemoveCategories ) ) & ~Blog::FeatureCategory ) )
	{
		QHBoxLayout * categoryToolsLayout = new QHBoxLayout( 0 );
		categoryToolsLayout->setMargin( 0 );
		if( ( d->features & ( Blog::FeatureAddCategories ) ) & ~Blog::FeatureCategory ) {
			//setup button
			d->addCategoryButton = new KPushButton( d->categoryToolsContainer );
			KGuiItem addItem;
			addItem = KStandardGuiItem::add();
			if( addItem.icon().isNull() ) {
				d->addCategoryButton->setText( addItem.text() );
			}
			else {
				d->addCategoryButton->setIcon( addItem.icon() );
			}
			d->addCategoryButton->setToolTip( i18n( "Add Categories" ) );
			categoryToolsLayout->addWidget( d->addCategoryButton );
			//setup popup
			d->addCategoryPopup = new KMenu( this );
			QWidget * newCategoryItem = new QWidget( d->addCategoryPopup );
			QLayout * newCategoryItemLayout = new QHBoxLayout( newCategoryItem );
			newCategoryItemLayout->setMargin( 1 );
			QLabel * newCategoryLabel = new QLabel( newCategoryItem );
			newCategoryLabel->setPixmap( SmallIcon( "document-new" ) );
			newCategoryItemLayout->addWidget( newCategoryLabel );
			d->newCategoryLineEdit = new KLineEdit( newCategoryItem );
			newCategoryItemLayout->addWidget( d->newCategoryLineEdit );
			d->newCategoryLineEdit->setFocusPolicy( Qt::ClickFocus );
			connect( d->newCategoryLineEdit,
			         SIGNAL( returnPressed( void ) ),
			         SLOT( emitNewCategory( void ) ) );
			newCategoryLabel->setBuddy( d->newCategoryLineEdit );
			KAction * newCategoryAction = new KAction( KIcon("filenew"), i18n("New Category..."), d->addCategoryPopup );
			newCategoryAction->setDefaultWidget( newCategoryItem );
			d->addCategoryPopup->addAction( newCategoryAction );
			
			d->addCategoryButton->setMenu( d->addCategoryPopup );
			//d->addCategoryButton->setMenuDelay( 1 );
		}
		if( ( d->features & ( Blog::FeatureRemoveCategories ) ) & ~Blog::FeatureCategory ) {
			//setup button
			d->removeCategoryButton = new KPushButton( d->categoryToolsContainer );
			KGuiItem removeItem;
			removeItem = KStandardGuiItem::remove();
			if( removeItem.icon().isNull() ) {
				d->removeCategoryButton->setText( removeItem.text() );
			}
			else {
				d->removeCategoryButton->setIcon( removeItem.icon() );
			}
			d->removeCategoryButton->setToolTip( i18n( "Remove Categories" ) );
			categoryToolsLayout->addWidget( d->removeCategoryButton );
			//setup popup
			d->removeCategoryPopup = new KMenu( this );
			d->removeCategoryButton->setMenu( d->removeCategoryPopup );
			//d->removeCategoryButton->setPopupDelay( 1 );
			d->removeActionCollection = new KActionCollection( d->removeCategoryPopup );
			d->removeCategoryMapper = new QSignalMapper( this );
			connect( d->removeCategoryMapper,
			         SIGNAL( mapped( const QString & ) ),
			         SLOT( emitCategoryRemoved( const QString & ) ) );
			Io::CategoryManager * categoryManager = entry->blog()->account()->categoryManager();
			Io::CategoryList categoryList = categoryManager->categories( entry->blog()->id() );
			for( Blokkal::Io::CategoryList::ConstIterator it = categoryList.begin();
			     it != categoryList.end();
			     ++it )
			{
				QString actionText = (*it)->name();
				actionText.replace( '&', "&&" );
				KAction * action = new KAction( actionText,d->removeActionCollection );
				d->removeCategoryActions.insert( (*it)->id(), action );
				connect( action, SIGNAL( triggered (Qt::MouseButtons, Qt::KeyboardModifiers ) ),
				         d->removeCategoryMapper, SLOT( map( void ) ) );
				d->removeCategoryMapper->setMapping( action, (*it)->id() );
				/*action->setShortcut( 0 );
				unsigned index = 0;
				for( ; index < d->removeCategoryPopup->count(); ++index ) {
					if( QString::localeAwareCompare( d->removeCategoryPopup->text( d->removeCategoryPopup->idAt( index ) ),
					                                 actionText ) > 0 )
					{
						break;
					}
				}
				d->removeCategoryActions[(*it)->id()]->plug(d->removeCategoryPopup, index );*/
				d->removeCategoryPopup->addAction( action );
			}
		}
		
		categoryToolsContainerLayout->addLayout( categoryToolsLayout );
	}
	
	//extension 4
	d->extension4 = entry->createExtensionWidget( EntryExtensionWidget::ExtensionWidget4, this );
	if( d->extension4 ) {
		connect( d->extension4,
		         SIGNAL( contentChanged( Blokkal::Ui::EntryExtensionWidget * ) ),
		         SLOT( emitContentsChanged( void ) ) );
		layout()->addWidget( d->extension4 );
	}

	//initialisation done, set widget contents
	rollBack();

	//start update timer
	QTimer * timer = new QTimer( this );
	connect( timer,
	         SIGNAL( timeout( void ) ),
	         SLOT( updateEntryDateTime( void ) ) );
	timer->start( 500 );

	if( d->extension0 ) {
		d->extension0->postInit();
	}
	if( d->extension1 ) {
		d->extension1->postInit();
	}
	if( d->extension2 ) {
		d->extension2->postInit();
	}
	if( d->extension3 ) {
		d->extension3->postInit();
	}
	if( d->extension4 ) {
		d->extension4->postInit();
	}

	d->formattingBackend = entry->blog()->account()->protocol()->createFormattingBackend( this );
	d->formattingBackend->setTextEdit( d->entryTextEdit );

	hideFeature( 0 );

	KConfigGroup configGroup = KGlobal::config()->group( "EditEntryWidget" );
	d->textCategorySplitter->restoreState( configGroup.readEntry( "text-category-splitter-state", QByteArray() ) );
}

Blokkal::Ui::EditEntryWidget::~EditEntryWidget( void )
{
	KConfigGroup configGroup = KGlobal::config()->group( "EditEntryWidget" );
	if( d->features & Blog::FeatureCategory
	    && !d->categoryExtensionSplitter->isHidden() )
	{
		configGroup.writeEntry( "text-category-splitter-state", d->textCategorySplitter->saveState() );
	}

	delete d;
}

Blokkal::Entry * Blokkal::Ui::EditEntryWidget::entry( void ) const
{
	return d->entry;
}

Blokkal::Entry * Blokkal::Ui::EditEntryWidget::apply( void )
{
	entry()->setDate( d->dateWidget->dateTime() );
	entry()->setSubject( d->subjectLineEdit->text() );
	entry()->setText( d->entryTextEdit->toPlainText() );
	entry()->setCategories( d->categoryView->selectedCategories() );

	if( d->extension0 ) {
		d->extension0->apply();
	}
	if( d->extension1 ) {
		d->extension1->apply();
	}
	if( d->extension2 ) {
		d->extension2->apply();
	}
	if( d->extension3 ) {
		d->extension3->apply();
	}
	if( d->extension4 ) {
		d->extension4->apply();
	}
	

	return entry();
}


void Blokkal::Ui::EditEntryWidget::updateEntryDateTime( void )
{
	if( d->useCurrentDateCheckBox->isChecked() ) {
		d->dateWidget->setDateTime( QDateTime::currentDateTime() );
	}
}

void Blokkal::Ui::EditEntryWidget::emitContentsChanged( void )
{
	emit contentsChanged();

	if( !d->subjectLineEdit->hasFocus()
	    && !d->entryTextEdit->hasFocus() )
	{
		return;
	}

	const int maxCaptionLength = 50;
	//the +1 is needed for the "..." replacement below to work
	if( ( d->subjectLineEdit->hasFocus()
	      && d->subjectLineEdit->cursorPosition() > maxCaptionLength + 1 )
	    || ( d->entryTextEdit->hasFocus()
	         && d->entryTextEdit->textCursor().position() > maxCaptionLength + 1) )
	{
		return;
	}


	QString subject = d->subjectLineEdit->text();
	QString newCaption = QString::null;
	if( !subject.isEmpty() ) {
		newCaption = subject;
	}
	else {
		QTextBlock currentBlock = d->entryTextEdit->document()->findBlock( 0 );
		int needed = maxCaptionLength + 1;
		for( ;
		     currentBlock.isValid() && needed > 0;
		     currentBlock = currentBlock.next() )
		{
			QString text = currentBlock.text();
			newCaption.append( text.left( needed ) );
			needed -= text.length();
			needed = needed < 0 ? 0 : needed;
		}
	}

	if( newCaption.length() > maxCaptionLength ) {
		newCaption = newCaption.left( maxCaptionLength );
		const QString elideString = i18n( "..." );
		newCaption.replace( maxCaptionLength - elideString.length(),
		                    elideString.length(),
		                    elideString );
	}	

	if( caption() != newCaption ) {
		setCaption( newCaption );
		emit captionChanged( caption() );
	}
}

void Blokkal::Ui::EditEntryWidget::slotDateChanged( void )
{
	//if the entry was created by this widget
	//the new date is not really a modification
	if( !( d->useCurrentDateCheckBox->isChecked() && d->entryHadNoDate ) ) {
		emitContentsChanged();
	}
}

Blokkal::Ui::EntryExtensionWidget * Blokkal::Ui::EditEntryWidget::extensionWidget( EntryExtensionWidget::ExtensionWidget extension ) const
{
	switch( extension ) {
	case EntryExtensionWidget::ExtensionWidget0:
		return d->extension0;
		break;
	case EntryExtensionWidget::ExtensionWidget1:
		return d->extension1;
		break;
	case EntryExtensionWidget::ExtensionWidget2:
		return d->extension2;
		break;
	case EntryExtensionWidget::ExtensionWidget3:
		return d->extension3;
		break;
	case EntryExtensionWidget::ExtensionWidget4:
		return d->extension4;
		break;
	}
	
	return 0;
}

bool Blokkal::Ui::EditEntryWidget::isContentLegal( void )
{
	return !d->entryTextEdit->document()->isEmpty();
}

void Blokkal::Ui::EditEntryWidget::emitNewCategory( void )
{
	if( !d->newCategoryLineEdit ) {
		kWarning() << k_funcinfo << "has been called with 0 newCategoryLineEdit!" << endl;
		return;
	}

	if( d->addCategoryPopup ) {
		d->addCategoryPopup->close();
	}
	else {
		kWarning() << k_funcinfo << "has been called with 0 addCategoryPopup!" << endl;
	}

	emit newCategory( d->newCategoryLineEdit->text() );
	d->newCategoryLineEdit->clear();
}

void Blokkal::Ui::EditEntryWidget::emitCategoryRemoved( const QString & categoryId )
{
	emit destroyCategory( categoryId );
}

void Blokkal::Ui::EditEntryWidget::rollBack( void )
{
	d->subjectLineEdit->setText( entry()->subject() );
	d->entryTextEdit->setPlainText( entry()->text() );
	d->categoryView->selectCategories( entry()->categories() );

	if( entry()->id().isNull() ) {
		d->dateWidget->setDateTime( QDateTime::currentDateTime() );
		d->useCurrentDateCheckBox->setChecked( TRUE );
		d->entryHadNoDate = TRUE;
	}
	else {
		d->dateWidget->setDateTime( entry()->date() );
		d->useCurrentDateCheckBox->setChecked( FALSE );
		d->entryHadNoDate = FALSE;
	}
	
	if( d->extension0 ) {
		d->extension0->rollBack();
	}
	if( d->extension1 ) {
		d->extension1->rollBack();
	}
	if( d->extension2 ) {
		d->extension2->rollBack();
	}
	if( d->extension3 ) {
		d->extension3->rollBack();
	}
	if( d->extension4 ) {
		d->extension4->rollBack();
	}
}

void Blokkal::Ui::EditEntryWidget::hideFeature( unsigned int hideFeatures )
{
	if( hideFeatures & Blog::FeatureDate ) {
		d->dateWidget->hide();
		d->useCurrentDateCheckBox->hide();
	}
	else if( d->features & Blog::FeatureDate ) {
		d->dateWidget->show();
		d->useCurrentDateCheckBox->show();
	}
	else {
		d->dateWidget->hide();
		d->useCurrentDateCheckBox->hide();		
	}
	
	if( hideFeatures & Blog::FeatureSubject ) {
		d->subjectLabel->hide();
		d->subjectLineEdit->hide();
	}
	else if( d->features & Blog::FeatureSubject ) {
		d->subjectLabel->show();
		d->subjectLineEdit->show();
	}
	else {
		d->subjectLabel->hide();
		d->subjectLineEdit->hide();		
	}
	
	if( hideFeatures & Blog::FeatureText ) {
		d->entryTextEdit->setEnabled( FALSE );
	}
	else if( d->features & Blog::FeatureText ) {
		d->entryTextEdit->setEnabled( TRUE );
	}
	else {
		d->entryTextEdit->setEnabled( FALSE );
	}
	
	if( hideFeatures & Blog::FeatureCategory ) {
		d->categoryToolsContainer->hide();
	}
	else if( d->features & Blog::FeatureCategory ) {
		d->categoryToolsContainer->show();
	}
	else {
		d->categoryToolsContainer->hide();
	}
	
	if( ( hideFeatures & Blog::FeatureAddCategories )
	    && d->addCategoryButton )
	{
		d->addCategoryButton->hide();
	}
	else if( ( ( d->features & ( Blog::FeatureAddCategories ) ) & ~Blog::FeatureCategory )
	         && d->addCategoryButton )
	{
		d->addCategoryButton->show();
	}
	else if( d->addCategoryButton ) {
		d->addCategoryButton->hide();
	}
	
	if( ( hideFeatures & Blog::FeatureRemoveCategories )
	    && d->removeCategoryButton )
	{
		d->removeCategoryButton->hide();
	}
	else if( ( ( d->features & ( Blog::FeatureRemoveCategories ) ) & ~Blog::FeatureCategory )
	         && d->removeCategoryButton )
	{
		d->removeCategoryButton->show();
	}
	else if( d->removeCategoryButton ) {
		d->removeCategoryButton->show();
	}

	updateSplitterVisibility();
}

QString Blokkal::Ui::EditEntryWidget::caption( void ) const
{
	return d->caption;
}

void Blokkal::Ui::EditEntryWidget::setCaption( const QString & caption )
{
	d->caption = caption;
}

void Blokkal::Ui::EditEntryWidget::updateSplitterVisibility( void )
{
	if( d->categoryToolsContainer->isHidden()
	    && ( !d->extension3 || ( d->extension3
	                             && d->extension3->isHidden() ) ) )
	{
		d->categoryExtensionSplitter->hide();
	}
	else if ( !d->categoryToolsContainer->isHidden()
	          || ( d->extension3
	               && !d->extension3->isHidden() ) )
	{
		d->categoryExtensionSplitter->show();
	}
}

Blokkal::Ui::HtmlEntryTextEdit * Blokkal::Ui::EditEntryWidget::entryTextEdit( void ) const
{
	return d->entryTextEdit;
}

Blokkal::Ui::FormattingBackend * Blokkal::Ui::EditEntryWidget::formattingBackend( void ) const
{
	return d->formattingBackend;
}
