/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "formattingbackend.h"
#include "formattingbackend.moc"

#include "editentrywidget.h"
#include "htmlentrytextedit.h"

#include <kdebug.h>

#include <QRegExp>

class Blokkal::Ui::FormattingBackend::Private
{
public:
	Private( Blokkal::Ui::EditEntryWidget * widget ) :
	widget( widget ),
	textEdit( 0 ) {}
	Blokkal::Ui::EditEntryWidget * widget;
	HtmlEntryTextEdit * textEdit;
};

Blokkal::Ui::FormattingBackend::FormattingBackend( Blokkal::Ui::EditEntryWidget * parent ):
QObject( parent ),
KXMLGUIClient(),
d( new Private( parent ) )
{}

Blokkal::Ui::FormattingBackend::~FormattingBackend( void )
{
	delete d;
}

void Blokkal::Ui::FormattingBackend::setTextEdit( HtmlEntryTextEdit * textEdit )
{
	d->textEdit = textEdit;
}

void Blokkal::Ui::FormattingBackend::insertTextAtCursor( const QString & text )
{
	if( !d->textEdit ) {
		return;
	}

	d->textEdit->insertTextAtCursor ( text );
}

void Blokkal::Ui::FormattingBackend::frameSelection( const QString & front, const QString & tail )
{
	if( !d->textEdit ) {
		return;
	}

	d->textEdit->frameSelection( front, tail );
}

QString & Blokkal::Ui::FormattingBackend::formatForPreview( QString & richText ) const
{
	return formatLineBreaks( formatLinks( richText ) );
}

Blokkal::Ui::EditEntryWidget * Blokkal::Ui::FormattingBackend::editEntryWidget( void ) const
{
	return d->widget;
}

QString & Blokkal::Ui::FormattingBackend::formatLinks( QString & richText ) const
{
	QRegExp linkexp( "(^|\\s)(http|https|ftp)://[\\S]+" );
	const QString linkStart = QString::fromLatin1( "<a href=\"" );
	const QString linkEnd = QString::fromLatin1( "\">" );
	const QString linkClose = QString::fromLatin1( "</a>" );
	int offset = 0;
	while( ( offset = linkexp.indexIn( richText, offset ) ) != -1 ) {
		if( offset > 0 && richText.at( offset - 1 ) == '\"' ) {
			offset += linkexp.matchedLength();
			continue;
		}
		
		bool leadingWhiteSpace = linkexp.cap().at( 0 ).isSpace();
		richText.insert( offset + linkexp.matchedLength(), linkEnd );
		richText.insert( offset + ( leadingWhiteSpace ? 1 : 0 ), linkStart );
		richText.insert( offset
		                 + linkexp.matchedLength()
		                 + linkStart.length()
		                 + linkEnd.length(),
		                 ( leadingWhiteSpace ? linkexp.cap().mid( 1 ) : linkexp.cap() ) + linkClose );
		if( leadingWhiteSpace ) {
			offset--;
		}
		offset += 2 * linkexp.matchedLength() + linkStart.length() + linkEnd.length() + linkClose.length();
	}
	return richText;
}

QString & Blokkal::Ui::FormattingBackend::formatLineBreaks( QString & richText ) const
{
	return richText.replace( '\n', "<br/>" );
}
