/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "genericeditaccountwidget.h"
#include "genericeditaccountwidget.moc"

#include "../blokkalaccount.h"
#include "../blokkalaccountmanager.h"
#include "../blokkalprotocol.h"
#include "../blokkalrsdconfigurationdata.h"

#include <klocale.h>
#include <kdialog.h>
#include <klineedit.h>
#include <kdebug.h>

#include <QLayout>
#include <QLabel>
#include <QGroupBox>
#include <QCheckBox>

class Blokkal::Ui::GenericEditAccountWidget::Private {
public:
	Private( void ):
	accountNameModifiedManually( FALSE ) {}

	QVBoxLayout * mainLayout;

	QGroupBox * networkGroupBox;

	QHBoxLayout * serverNameLayout;
	QLabel * serverNameLabel;
	KLineEdit * serverNameLineEdit;
	QCheckBox * useStandardAddressCheckBox;
	KLineEdit * serverAddressLineEdit;
	QCheckBox * noAutoConnectCheckBox;

	QGroupBox * userGroupBox;

	QHBoxLayout * userNameLayout;
	QLabel * userNameLabel;
	KLineEdit * userNameLineEdit;

	QGroupBox * accountGroupBox;

	QHBoxLayout * accountNameLayout;
	QLabel * accountNameLabel;
	KLineEdit * accountNameLineEdit;

	bool accountNameModifiedManually;
};

Blokkal::Ui::GenericEditAccountWidget::GenericEditAccountWidget( Blokkal::Protocol * protocol, Blokkal::Account * account, QWidget *parent ):
EditAccountWidget( protocol, account, parent ),
d( new Private() )
{
	d->mainLayout = new QVBoxLayout( this );
	d->mainLayout->setSpacing( KDialog::spacingHint() );

	//network
	d->networkGroupBox = new QGroupBox( i18n( "Network" ), this );
	new QVBoxLayout( d->networkGroupBox );
	//server name
	d->serverNameLayout = new QHBoxLayout( 0 );
	d->serverNameLayout->setSpacing( KDialog::spacingHint() );
	d->serverNameLabel = new QLabel( i18n( "&Server Name" ), d->networkGroupBox );
	d->serverNameLayout->addWidget( d->serverNameLabel );
	d->serverNameLineEdit = new KLineEdit( d->networkGroupBox );
	d->serverNameLabel->setBuddy( d->serverNameLineEdit );
	d->serverNameLayout->addWidget( d->serverNameLineEdit );
	d->networkGroupBox->layout()->addItem( d->serverNameLayout );
	//use standard address
	d->useStandardAddressCheckBox = new QCheckBox( i18n( "&Use standard address" ), d->networkGroupBox );
	d->useStandardAddressCheckBox->setChecked( TRUE );
	d->networkGroupBox->layout()->addWidget( d->useStandardAddressCheckBox );
	//standard address
	d->serverAddressLineEdit = new KLineEdit( d->networkGroupBox );
	d->serverAddressLineEdit->setEnabled( FALSE );
	QHBoxLayout * serverAddressLayout = new QHBoxLayout( 0 );
	serverAddressLayout->addSpacing( KDialog::spacingHint() );
	serverAddressLayout->addWidget( d->serverAddressLineEdit );
	d->networkGroupBox->layout()->addItem( serverAddressLayout );
	d->noAutoConnectCheckBox = new QCheckBox( i18n( "&Do not connect automatically" ), d->networkGroupBox );
	d->networkGroupBox->layout()->addWidget( d->noAutoConnectCheckBox );
		
	d->mainLayout->addWidget( d->networkGroupBox );

	//user name, etc.
	d->userGroupBox = new QGroupBox( i18n( "User" ), this );
	new QVBoxLayout( d->userGroupBox  );
	//user name
	d->userNameLayout = new QHBoxLayout( 0 );
	d->userNameLayout->setSpacing( KDialog::spacingHint() );
	d->userNameLabel = new QLabel( i18n( "Us&er Name" ), d->userGroupBox );
	d->userNameLayout->addWidget( d->userNameLabel );
	d->userNameLineEdit = new KLineEdit( d->userGroupBox );
	d->userNameLabel->setBuddy( d->userNameLineEdit );
	d->userNameLayout->addWidget( d->userNameLineEdit );
	d->userGroupBox->layout()->addItem( d->userNameLayout );
	
	d->mainLayout->addWidget( d->userGroupBox );

	//account name
	d->accountGroupBox = new QGroupBox( i18n( "Account" ), this );
	new QVBoxLayout( d->accountGroupBox );
	//user name
	d->accountNameLayout = new QHBoxLayout( 0 );
	d->accountNameLayout->setSpacing( KDialog::spacingHint() );
	d->accountNameLabel = new QLabel( i18n( "&Account Name" ), d->accountGroupBox );

	d->accountNameLayout->addWidget( d->accountNameLabel );
	d->accountNameLineEdit = new KLineEdit( d->accountGroupBox );
	d->accountNameLabel->setBuddy( d->accountNameLineEdit );
	d->accountNameLayout->addWidget( d->accountNameLineEdit );
	d->accountGroupBox->layout()->addItem( d->accountNameLayout );

	d->mainLayout->addWidget( d->accountGroupBox );
	
	d->mainLayout->addStretch( 1 );

	if( account ) {
		d->serverNameLineEdit->setText( account->serverName() );
		d->useStandardAddressCheckBox->setChecked( account->usesDefaultConnectAddress() );
		d->serverAddressLineEdit->setDisabled( account->usesDefaultConnectAddress() );
		d->serverAddressLineEdit->setText( account->connectAddress().url() );
		d->noAutoConnectCheckBox->setChecked( !account->autoConnect() );
		
		d->userNameLineEdit->setText( account->userName() );

		d->accountNameModifiedManually = TRUE;
		d->accountNameLineEdit->setDisabled( TRUE );
		d->accountNameLineEdit->setText( account->id() );
	}
	
	connect( d->serverNameLineEdit, SIGNAL( textChanged( const QString & ) ),
	         this, SLOT( setServerAddress( const QString & ) ) );
	connect( d->useStandardAddressCheckBox, SIGNAL( toggled( bool ) ),
	         d->serverAddressLineEdit, SLOT( setDisabled( bool ) ) );
	connect( d->useStandardAddressCheckBox, SIGNAL( toggled( bool ) ),
	         this, SLOT( useStandardAddressToggled( void ) ) );

	connect( d->serverNameLineEdit, SIGNAL( textChanged( const QString & ) ),
	         this, SLOT( emitValuesChanged( void ) ) );
	connect( d->serverAddressLineEdit, SIGNAL( textChanged( const QString & ) ),
	         this, SLOT( emitValuesChanged( void ) ) );
	connect( d->userNameLineEdit, SIGNAL( textChanged( const QString & ) ),
	         this, SLOT( emitValuesChanged( void ) ) );
	connect( d->accountNameLineEdit, SIGNAL( textChanged( const QString & ) ),
	         this, SLOT( emitValuesChanged( void ) ) );
	
}

Blokkal::Ui::GenericEditAccountWidget::~GenericEditAccountWidget( void )
{
	delete d;
}

Blokkal::Account * Blokkal::Ui::GenericEditAccountWidget::apply( void )
{
	Blokkal::Account * account = EditAccountWidget::account();

	if( !account ) {
		account = protocol()->createAccount( d->accountNameLineEdit->text() );
	}

	account->setServerName( d->serverNameLineEdit->text() );
	account->setConnectAddress( d->serverAddressLineEdit->text() );
	account->setUsesDefaultConnectAddress( d->useStandardAddressCheckBox->isChecked() );
	account->setUserName( d->userNameLineEdit->text() );
	account->setAutoConnect( !d->noAutoConnectCheckBox->isChecked() );
	
	return account;
}

bool Blokkal::Ui::GenericEditAccountWidget::isValid( void ) const
{
	//is the relevant information present?
	if( d->serverNameLineEdit->text().isEmpty()
	    || d->serverAddressLineEdit->text().isEmpty()
	    || d->userNameLineEdit->text().isEmpty()
	    || d->accountNameLineEdit->text().isEmpty() )
	{
		return FALSE;
	}

	//check account name
	Blokkal::Account * accountTest = Blokkal::AccountManager::self()->account( d->accountNameLineEdit->text() );
	if( !accountTest ) { //okay
		return TRUE;
	}

	if( accountTest == account() ) { //the registered account is our account => okay
		return TRUE;
	}

	return FALSE; //name used by another account
}

QCheckBox * Blokkal::Ui::GenericEditAccountWidget::useStandardAddressCheckBox( void ) const
{
	return d->useStandardAddressCheckBox;
}

KLineEdit * Blokkal::Ui::GenericEditAccountWidget::connectAddressLineEdit( void ) const
{
	return d->serverAddressLineEdit;
}

void Blokkal::Ui::GenericEditAccountWidget::emitValuesChanged( void )
{
	if( d->accountNameLineEdit->hasFocus() ) {
		d->accountNameModifiedManually = TRUE;
	}

	if( !d->accountNameModifiedManually ) {
		d->accountNameLineEdit->setText( d->userNameLineEdit->text() + "@" + d->serverNameLineEdit->text() );
	}
	
	emit valuesChanged();
}

KLineEdit * Blokkal::Ui::GenericEditAccountWidget::serverNameLineEdit( void ) const
{
	return d->serverNameLineEdit;
}

void Blokkal::Ui::GenericEditAccountWidget::useStandardAddressToggled( void )
{
	setServerAddress( serverNameLineEdit()->text() );
}

QGroupBox * Blokkal::Ui::GenericEditAccountWidget::accountGroupBox( void ) const
{
	return d->accountGroupBox;
}

QGroupBox * Blokkal::Ui::GenericEditAccountWidget::userGroupBox( void ) const
{
	return d->userGroupBox;
}

QGroupBox * Blokkal::Ui::GenericEditAccountWidget::networkGroupBox( void ) const
{
	return d->networkGroupBox;
}

KLineEdit * Blokkal::Ui::GenericEditAccountWidget::userNameLineEdit( void ) const
{
	return d->userNameLineEdit;
}

void Blokkal::Ui::GenericEditAccountWidget::setup( const Blokkal::RsdConfigurationData & data )
{
	const KUrl blogUrl = data.homePageLink();
	serverNameLineEdit()->setText( blogUrl.host() );
	
	connectAddressLineEdit()->setText( data.apiLink() );
}
