/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALUiGENERICEDITACCOUNTWIDGET_H
#define BLOKKALUiGENERICEDITACCOUNTWIDGET_H

#include "editaccountwidget.h"

class QGroupBox;
class QCheckBox;
class KLineEdit;
class QVBoxLayout;

namespace Blokkal {
class Account;
class Protocol;

namespace Ui {
/**
 * @short A widget to edit generic account properties.
 *
 * A widget to edit generic account properties.
 * The account name is composed from user name and server name, as long
 * as it has not been edited manually.
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT GenericEditAccountWidget : public EditAccountWidget
{
Q_OBJECT
public:
	/**
	 * Constructs a new GenericEditAccountWidget
	 *
	 * @param protocol the protocol object
	 * @param account the account object
	 * @param parent the parent widget
	 */
	GenericEditAccountWidget( Blokkal::Protocol * protocol,
	                          Blokkal::Account * account,
	                          QWidget * parent = 0 );

	/**
	 * Destroys the widget
	 */
	virtual ~GenericEditAccountWidget( void );

	/**
	 * This method checks that server name, user name and account name are
	 * set. If the default address is not set, it must be set as well. Also
	 * checks, whether the account manager has an account with the entered account
	 * name.
	 *
	 * @return TRUE if the entered information is valid
	 */
	virtual bool isValid( void ) const;

	/**
	 * This method returns a pointer to the updated account object.
	 * @return the account object
	 */
	virtual Blokkal::Account * apply( void );
	
	/**
	 * Sets up the widget according to the information contained in @a data .
	 * This class will set the server name to the host of data.homaPageUrl() and the
	 * connect address to data.apiLink().
	 * @param data RSD data to use
	 * @see Blokkal::RsdConfigurationData
	 */
	virtual void setup( const Blokkal::RsdConfigurationData & data );
	
protected Q_SLOTS:
	/**
	 * This slot is called whenever the content of the server name
	 * widget changes.
	 * @param serverName the new server name
	 */
	virtual void setServerAddress( const QString & serverName ) = 0;

	/**
	 * This slot is called whenever the contents of the widgets change.
	 */
	virtual void emitValuesChanged( void );

	/**
	 * This slot is called when the "Use stanard address" checkbox
	 * is toggled. The default implementation calls setServerAddress().
	 */
	virtual void useStandardAddressToggled( void );

protected:
	/**
	 * Returns the server name line edit.
	 * @return the server name line edit*/
	KLineEdit * serverNameLineEdit( void ) const;
	
	/**
	 * Returns the "Use stanard address" checkbox
	 * @return the "Use stanard address" checkbox
	 */
	QCheckBox * useStandardAddressCheckBox( void ) const;

	/**
	 * Returns the server address line edit
	 * @return the server address line edit
	 */
	KLineEdit * connectAddressLineEdit( void ) const;

	/**
	 * Returns the network group box.
	 * @return the network group box
	 */
	QGroupBox * networkGroupBox( void ) const;

	/**
	 * Returns the user name line edit.
	 * @return the user name line edit*/
	KLineEdit * userNameLineEdit( void ) const;

	/**
	 * Returns the user group box.
	 * @return the user group box
	 */
	QGroupBox * userGroupBox( void ) const;

	/**
	 * Returns the account group box.
	 * @return the account group box
	 */
	QGroupBox * accountGroupBox( void ) const;

private:
	class Private;
	Private * d;
};

}
}

#endif
