/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "htmlentrytextedit.h"
#include "htmlentrytextedit.moc"

#include "stdaction.h"

#include <blokkalappearancesettings.h>

#include <kmenu.h>
#include <kaction.h>

#include <klocale.h>
#include <kdebug.h>
#include <kglobalsettings.h>

#include <QContextMenuEvent>


//-------------------
//HtmlTextHighlighter
//-------------------
class Blokkal::Ui::HtmlTextHighlighter::Private
{
public:
	Private( const QTextEdit * edit ) :
	edit( edit ){}

	const QTextEdit * const edit;
	
	bool highlightSyntax;
	bool highlightSpelling;

	QTextCharFormat normalFormat;
	QTextCharFormat tagFormat;
	QTextCharFormat stringFormat;
	QTextCharFormat valueFormat;
	
};


Blokkal::Ui::HtmlTextHighlighter::HtmlTextHighlighter( QTextEdit * parent ):
Sonnet::Highlighter( parent ),
d( new Private( parent ) )
{
	slotAppearanceSettingsChanged();
	
	connect( AppearanceSettings::self(),
	         SIGNAL( appearanceSettingsChanged( void ) ),
	         SLOT( slotAppearanceSettingsChanged( void ) ) );
	connect( KGlobalSettings::self(),
	         SIGNAL( kdisplayFontChanged ( void )),
	         SLOT( slotAppearanceSettingsChanged( void ) ) );
}

Blokkal::Ui::HtmlTextHighlighter::~HtmlTextHighlighter( void )
{
	delete d;
}

void Blokkal::Ui::HtmlTextHighlighter::highlightBlock( const QString & text )
{
	if( d->highlightSpelling ) {
		Sonnet::Highlighter::highlightBlock( text );
	}
	else {
 		setFormat( 0,text.length(), d->normalFormat );
 	}

	if( d->highlightSyntax
	    || ( !d->highlightSyntax && d->highlightSpelling ) )
	{
		int tagStart = 0;
		int tagEnd = 0;
		while( ( tagStart = text.indexOf( '<', tagStart ) ) > -1 ) {
			//make sure this is really something like an html tag
			if( ( ( text.length() > tagStart + 1 )
			      && text.at( tagStart + 1 ).isLetter() )
			    || ( ( text.length() > tagStart + 2 )
			         && ( text.at( tagStart + 1 ) == '/' && text.at( tagStart + 2 ).isLetter() ) ) )
			{
				tagEnd = text.indexOf( '>', tagStart );
				if( tagEnd < 0 ) {
					tagEnd = text.length();
				}
		
				if( d->highlightSyntax ) {
					setFormat( tagStart, tagEnd - tagStart + 1 , d->tagFormat );
					bool inString = FALSE;
					bool inValue = FALSE;
					int subStart = tagStart;
					for( int index = tagStart; index < tagEnd; ++index ) {
						switch( text.at( index ).toLower().cell() ) {
						case '"':
							if( inString ) {
								if( text.at( index - 1) != '\\' ) {
									setFormat( subStart, index - subStart + 1, d->stringFormat );
									inString = FALSE;
								}
							}
							else {
								subStart = index;
								inString = TRUE;
							}
							break;
						case '#':
						case '0':
						case '1':
						case '2':
						case '3':
						case '4':
						case '5':
						case '6':
						case '7':
						case '8':
						case '9':
							if( !inString
								&& !inValue
								&& (  text.at( index-1) == ' '
										|| text.at( index-1) == '\t'
										|| text.at( index-1) == '=' ) ) {
								subStart = index;
								inValue = TRUE;
							}
							break;
						case ' ':
						case '\t':
							if( inValue ) {
								setFormat( subStart, index - subStart + 1, d->valueFormat );
								inValue = FALSE;
							}
							break;
						case 'a':
						case 'b':
						case 'c':
						case 'd':
						case 'e':
						case 'f':
							break;
						case '/':
							if( inValue && text.at( index + 1 ) == '>' ) {
								setFormat( subStart, index - subStart, d->valueFormat );
							}
							inValue = FALSE;
							break;
						default:
							if( inValue ) {
								inValue = FALSE;
							}
							break;
						}
					}
					if( inValue ) {
						setFormat( subStart, tagEnd - subStart, d->valueFormat );
					}
	
				}
				else {
					setFormat( 0,text.length(), d->normalFormat );
				}
	
				tagStart = tagEnd; //skip tag body
			}
			else {
				tagStart++; //don't enter endless loop on last found '<'
			}
		}
	}
}

void Blokkal::Ui::HtmlTextHighlighter::slotAppearanceSettingsChanged( void )
{
	d->highlightSyntax = AppearanceSettings::self()->highlightSyntax();
	d->highlightSpelling = AppearanceSettings::self()->checkSpelling();

	d->normalFormat.setFont( KGlobalSettings::generalFont() );
	d->normalFormat.setForeground( d->edit->palette().text() );

	QFont font = KGlobalSettings::generalFont();
	font.setBold( AppearanceSettings::self()->highlightTagBold() );
	font.setItalic( AppearanceSettings::self()->highlightTagItalic() );
	d->tagFormat.setFont( font );
	d->tagFormat.setForeground( AppearanceSettings::self()->highlightTagColor() );

	font = KGlobalSettings::generalFont();
	font.setBold( AppearanceSettings::self()->highlightStringBold() );
	font.setItalic( AppearanceSettings::self()->highlightStringItalic() );
	d->stringFormat.setFont( font );
	d->stringFormat.setForeground( AppearanceSettings::self()->highlightStringColor() );

	font = KGlobalSettings::generalFont();
	font.setBold( AppearanceSettings::self()->highlightValueBold() );
	font.setItalic( AppearanceSettings::self()->highlightValueItalic() );
	d->valueFormat.setFont( font );
	d->valueFormat.setForeground( AppearanceSettings::self()->highlightValueColor() );

	rehighlight();
}

//-----------------
//HtmlEntryTextEdit
//-----------------
class Blokkal::Ui::HtmlEntryTextEdit::Private
{
public:
	Private( void ) :
	highlighter( 0 ),
	spellCheckAction ( 0 ),
	proxy( 0 )
	{}

	HtmlTextHighlighter * highlighter;

	QAction * spellCheckAction;

	DropProxy * proxy;

	QTextBlock currentTextBlock;
	QString blockText;
};

Blokkal::Ui::HtmlEntryTextEdit::HtmlEntryTextEdit( QWidget * parent ):
KTextEdit( parent ),
d( new Private() )
{
	setAcceptRichText( FALSE );
	setTabChangesFocus( TRUE );
	d->highlighter = new HtmlTextHighlighter( this );
	d->currentTextBlock = textCursor().block();
	d->blockText = d->currentTextBlock.text();
	connect( this,
	         SIGNAL( textChanged( void ) ),
	         SLOT( checkCurrentBlock( void ) ) );
}

Blokkal::Ui::HtmlEntryTextEdit::~HtmlEntryTextEdit( void )
{
	delete d;
}

void Blokkal::Ui::HtmlEntryTextEdit::insertTextAtCursor( const QString & text )
{
	QTextCursor cursor( textCursor() );
	const int position = cursor.position();
	cursor.insertText( text );
	cursor.setPosition( position + text.length() );
	setTextCursor( cursor );
}

void Blokkal::Ui::HtmlEntryTextEdit::frameSelection( const QString & front, const QString & tail )
{
	QTextCursor cursor( textCursor() );
	if( cursor.hasSelection() && !cursor.hasComplexSelection() ) {
		const int anchor = cursor.anchor();
		const int selectionEnd = cursor.selectionEnd();
		cursor.clearSelection();
		if( anchor < selectionEnd ) {
			cursor.insertText( tail );
			cursor.setPosition( anchor );
			cursor.insertText( front );
		}
		else {
			cursor.insertText( front );
			cursor.setPosition( selectionEnd + front.length() );
			cursor.insertText( tail );
		}
		
		cursor.setPosition( selectionEnd + front.length() );
		setTextCursor( cursor );
	}
	else if( !cursor.hasSelection() ) {
		cursor.insertText( front );
		int position = cursor.position();
		cursor.insertText( tail );
		cursor.setPosition( position );
		setTextCursor( cursor );
	}
}

void Blokkal::Ui::HtmlEntryTextEdit::dropEvent( QDropEvent * event )
{
	if( isReadOnly() ) {
		KTextEdit::dropEvent( event );
		return;
	}

	if( d->proxy && d->proxy->handleDrop( event, this ) ) {
		return;
	}

	//fall back to KTextEdit drag'n'drop
	KTextEdit::dropEvent( event );
}

void Blokkal::Ui::HtmlEntryTextEdit::checkCurrentBlock( void )
{
	if( d->currentTextBlock == textCursor().block() ) {
		if( d->blockText != d->currentTextBlock.text() ) {
			d->blockText = d->currentTextBlock.text();
			emit plainTextChanged();
		}
	}
	else {
		d->currentTextBlock = textCursor().block();
		d->blockText = d->currentTextBlock.text();
		emit plainTextChanged();
	}
}

void Blokkal::Ui::HtmlEntryTextEdit::contextMenuEvent( QContextMenuEvent * event )
{
	//code borrowed from KTextEdit
	QMenu *popup = createStandardContextMenu();

	connect( popup,
	         SIGNAL( triggered ( QAction* ) ),
	         SLOT( menuActivated( QAction* ) ) ); //own version, just starts the spellcheck dialog

	if( !isReadOnly() ) {
		QList<QAction *> actionList = popup->actions();
		enum { UndoAct, RedoAct, CutAct, CopyAct, PasteAct, ClearAct, SelectAllAct, NCountActs };
		QAction * separatorAction = 0L;
		int idx = actionList.indexOf( actionList[SelectAllAct] ) + 1;
		if ( idx < actionList.count() ) {
			separatorAction = actionList.at( idx );
		}
		if ( separatorAction ) {
			KAction * clearAllAction = KStandardAction::clear( this, SLOT( clear() ), this );
			if ( toPlainText().isEmpty() ) {
				clearAllAction->setEnabled( FALSE );
			}
			popup->insertAction( separatorAction, clearAllAction );
		}
	}
 
	KIconTheme::assignIconsToContextMenu( isReadOnly() ? KIconTheme::ReadOnlyText : KIconTheme::TextEditor,
	                                      popup->actions() );

	if( !isReadOnly() ) {
		popup->addSeparator();
		if( !acceptRichText() ) {
			d->spellCheckAction = popup->addAction( KIcon( "tools-check-spelling" ), i18n( "Check Spelling..." ) );
			if ( document()->isEmpty() ) {
				 d->spellCheckAction->setEnabled( FALSE );
			}
		}
	}
	popup->exec( event->globalPos() );
 
   delete popup;
}

void Blokkal::Ui::HtmlEntryTextEdit::menuActivated( QAction * action )
{
	if( action == d->spellCheckAction ) {
		checkSpelling();
	}
}

void Blokkal::Ui::HtmlEntryTextEdit::setDropProxy( Blokkal::Ui::DropProxy * proxy )
{
	d->proxy = proxy;
}

//---------
//DropProxy
//---------
Blokkal::Ui::DropProxy::~DropProxy( void )
{}
