/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UIHTMLENTRYTEXTEDIT_H
#define BLOKKAL_UIHTMLENTRYTEXTEDIT_H

#include <ktextedit.h>
#include <sonnet/highlighter.h>

#include "../blokkal_export.h"

class QAction;
class KAction;
class KActionCollection;

namespace Blokkal {
namespace Ui {
class HtmlTextHighlighter;
class DropProxy;
/**
 * @short Edits entry texts as HTML.
 *
 * This class provides some utilities to HTML entry texts.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT HtmlEntryTextEdit : public KTextEdit
{
Q_OBJECT
public:
	/**
	 * Creates a new HtmlEntryTextEdit.
	 * @param parent parent widget
	 */
	HtmlEntryTextEdit( QWidget * parent = 0 );

	/**
	 * Destructor
	 */
   virtual ~HtmlEntryTextEdit( void );

	/**
	 * Inserts text at the current cursor position
	 * @param text text to insert
	 */
	void insertTextAtCursor( const QString & text );

	/**
	 * If some text is selected, the selected text is framed
	 * by @a front and  @a tail.
	 * If no text is selected the current cursor position is used instead.
	 * @param front text to place in front of the selection
	 * @param tail text to place after the selection
	 */
	void frameSelection( const QString & front, const QString & tail );

	/**
	 * Sets the DropProxy for this widget.
	 * @param proxy the new drop proxy
	 */
	void setDropProxy( Blokkal::Ui::DropProxy * proxy );

Q_SIGNALS:
	/**
	 * This signal is emitted when the plain text content has
	 * changed. Formatting changes are ignored.
	 * Use this instead of QTextEdit::textChanged()
	 */
	void plainTextChanged( void );
	
protected:
	/**
	 * Displays the drop context menu
	 * @param event the drop event
	 */
	virtual void dropEvent( QDropEvent * event );

	/**
	 * Overridden to hide the autospellcheck item from the popup menu
	 */
	virtual void contextMenuEvent( QContextMenuEvent * event );

private Q_SLOTS:
	void checkCurrentBlock( void );
	void menuActivated( QAction * action );

private:
	class Private;
	Private * const d;

};

/**
 * @short A simple HTML highlighter.
 *
 * This class provides very simple HTML hightlighting and
 * spell checking. Spell checking uses the global spell checking
 * configuration.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT HtmlTextHighlighter : public Sonnet::Highlighter
{
Q_OBJECT
public:
	/**
	 * Creates a new HtmlTextHighlighter.
	 * The settings for this highlighter are fetched
	 * from the Blokkal::AppearanceSettings singleton
	 * @param parent the parent text edit
	 */
	HtmlTextHighlighter( QTextEdit * parent );

	/**
	 * Destructor
	 */
 	virtual ~HtmlTextHighlighter( void );

protected:
	virtual void highlightBlock ( const QString & text );

private Q_SLOTS:
	void slotAppearanceSettingsChanged( void );
	
private:
	class Private;
	Private * const d;
};

/**
 * @short Abstract base class for drop proxys
 *
 * This class defines an interface dor drop proxys.
 * A drop proxy handles drop vents for a widget.
 */
class BLOKKAL_EXPORT DropProxy {
public:
	virtual ~DropProxy( void );
	/**
	 * This method is called when a drop event needs to be handled
	 * @param event the event to handle
	 * @param widget the widget that received the drop event
	 * @return TRUE if the event has been handled by the proxy
	 */
	virtual bool handleDrop( QDropEvent * event, QWidget * widget ) = 0;
};

}
}

#endif
