/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UiMAINWINDOWVIEW_H
#define BLOKKAL_UiMAINWINDOWVIEW_H

#include "view.h"
#include "globalsettings.h"
#include "secondarywindow.h"

namespace Blokkal {
namespace Ui {

/**
 * @short Base class for views that want to display a main window.
 *
 * If you want to create a View that displays a main window
 * use this class instead of subclassing KMainWindow yourself.
 * This class has a custom implementation of closeEvent() that prevents
 * it from quiting the host application if a Blokkal::Ui::SystemTray is present.
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT MainWindowView : public SecondaryWindow, public Blokkal::Ui::View
{
Q_OBJECT
public:
	/**
	 * Creates a new MainWindowView
	 * @param parent parent widget
	 * @param flags widget flags
	 */
	MainWindowView( QWidget * parent = Blokkal::Ui::GlobalSettings::self()->mainWidget(), Qt::WindowFlags flags=KDE_DEFAULT_WINDOWFLAGS );

	/**
	 * Destructor
	 */
	virtual ~MainWindowView( void );

	/**
	 * Makes the window visible
	 */
	virtual void showView( void );

	/**
	 * Hides the window
	 */
	virtual void hideView( void );

	/**
	 * This method is equal to KMainWindow::close().
	 * @return TRUE if the window has been closed
	 */
	virtual bool closeView( void );
	
	/**
	 * Returns the main widget of this view.
	 * @return the main widget
	 */
	virtual QWidget * mainWidget( void );
};

}

}

#endif
