/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mediainfocombobox.h"
#include "mediainfocombobox.moc"

#include <kwindowsystem.h>
#include <kwindowinfo.h>
#include <kicon.h>
#include <kdebug.h>

#include <QDBusInterface>
#include <QDBusConnection>
#include <QDBusReply>

class Blokkal::Ui::MediaInfoComboBox::Private
{
public:
	Private( void ) {}
};

Blokkal::Ui::MediaInfoComboBox::MediaInfoComboBox( QWidget * parent ) :
SemiEditableComboBox( parent ),
d( new Private() )
{
	setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Minimum );
	repopulate();
	connect( this, SIGNAL( activated( const QString & ) ),
	         this, SLOT( repopulate( void ) ) );
}

Blokkal::Ui::MediaInfoComboBox::~MediaInfoComboBox( void )
{
	delete d;
}

void Blokkal::Ui::MediaInfoComboBox::repopulate( void )
{
	const QList<WId> windowList = KWindowSystem::windows();
	QString windowName;
	bool boxWasEmpty = FALSE;
	int oldCount = count();
	int currentListItem = 0;
	
	//first item is for manual editing
	//make sure it exists
	if( count() <= currentListItem ) {
		addItem( QString::null, currentListItem );
		boxWasEmpty = TRUE;
	}
	currentListItem++;

	//find XMMS window
	//then follow xmms windows
	for ( QList<WId>::ConstIterator winListIterator = windowList.begin();
	      winListIterator != windowList.end() && !windowList.isEmpty();
	      ++winListIterator )
	{
		if( !KWindowSystem::hasWId( *winListIterator ) ) {
			continue;
		}
		const KWindowInfo windowInfo = KWindowSystem::windowInfo( *winListIterator, NET::WMVisibleName|NET::WMIconName, NET::WM2StartupId );
		if( windowInfo.visibleName().indexOf( "XMMS - " ) == 0)	{
			int musicInfoStart = windowInfo.visibleName().indexOf( " ", 7 ) + 1;
			int musicInfoLength = windowInfo.visibleName().lastIndexOf( " " ) - musicInfoStart;
			if( count() <= currentListItem ) {
				insertItem( currentListItem,
				            KWindowSystem::icon( *winListIterator,
				                                 fontMetrics().height(),
				                                 fontMetrics().height(),
				                                 TRUE ),
				            windowInfo.visibleName().mid( musicInfoStart, musicInfoLength ) );
			}
			else {
				setItemText( currentListItem,
				             windowInfo.visibleName().mid( musicInfoStart, musicInfoLength ) );
			}
			currentListItem++;
		}
	}
	//done with the windows

	
	//Check amaroK
	QDBusConnection bus = QDBusConnection::sessionBus();
	QDBusInterface *interface = new QDBusInterface( "org.kde.amarok",
							"/Player",
							"org.freedesktop.MediaPlayer",
							bus,
							this );
	QDBusReply< QMap<QString,QVariant> > reply = interface->call( "GetMetadata" );
	if( reply.isValid() ) {
		QString result = QString::fromUtf8("%1 - %2").arg(reply.value()["artist"].toString()).arg(reply.value()["title"].toString());
		if( count() <= currentListItem ) {
			addItem( result, currentListItem );
		}
		else {
			setItemText( currentListItem, result );
		}
		currentListItem++;
	}

	
	//select the last player by default, should be just one most of the time
	if( boxWasEmpty || oldCount < count() ) {
		setCurrentIndex( count()-1 );
	}

	updateEditableState();
}
