/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "providercombobox.h"
#include "providercombobox.moc"

#include <klocale.h>

#include <QIcon>

class Blokkal::Ui::ProviderInformation::Private {
public:
	Private( const QString & id, const QString & name, const QIcon & icon ):
	id( id ),
	name( name ),
	icon( icon ) {}

	QString id;
	QString name;
	QIcon icon;
};

Blokkal::Ui::ProviderInformation::ProviderInformation( const QString & id,	const QString & name ):
d( new Private( id, name, QIcon() ) )
{
}

Blokkal::Ui::ProviderInformation::ProviderInformation( const QString & id, const QString & name, const QIcon & icon ):
d( new Private( id, name, icon ) )
{
}

Blokkal::Ui::ProviderInformation::ProviderInformation( const Blokkal::Ui::ProviderInformation & provider ):
d( new Private( provider.id(), provider.name(), provider.icon() ) )
{
}

Blokkal::Ui::ProviderInformation::~ProviderInformation( void )
{
	delete d;
}

const QString & Blokkal::Ui::ProviderInformation::id( void ) const
{
	return d->id;
}

const QIcon & Blokkal::Ui::ProviderInformation::icon( void ) const
{
	return d->icon;
}

const QString & Blokkal::Ui::ProviderInformation::name( void ) const
{
	return d->name;
}

Blokkal::Ui::ProviderInformation & Blokkal::Ui::ProviderInformation::operator =( const Blokkal::Ui::ProviderInformation & provider )
{
	if( this != &provider ) {
		delete d;
		d = new Private( provider.id(), provider.name(), provider.icon() );
	}
	return *this;
}

class Blokkal::Ui::ProviderComboBox::Private {
public:
	Private( void ) {}

	QMap<unsigned int, ProviderInformation> providerMap;
};

Blokkal::Ui::ProviderComboBox::ProviderComboBox( QWidget * parent ):
KComboBox( parent ),
d( new Private() )
{
	connect( this, SIGNAL( activated( int ) ),
	         this, SLOT( emitProviderSelected( int ) ) );
}

Blokkal::Ui::ProviderComboBox::~ProviderComboBox( void )
{
	delete d;
}

void Blokkal::Ui::ProviderComboBox::setProviders( const ProviderList & providers )
{
	d->providerMap.clear();
	clear();
	
	addProviders( providers );
}

void Blokkal::Ui::ProviderComboBox::addProviders( const ProviderList & providers )
{
	if( count() > 0
	    && d->providerMap.count() > 0
	    && d->providerMap[count()-1].id().isNull() )
	{
		d->providerMap.remove( count() - 1 );
		removeItem( count() -1 );
	}

	for( ProviderList::ConstIterator it = providers.begin();
	     it != providers.end();
	     ++it )
	{
		d->providerMap.insert( count(), *it );
		addItem( (*it).icon(), (*it).name() );
	}
	d->providerMap.insert( count(), ProviderInformation( QString::null, i18n( "Custom" ) ) );
	addItem( i18n( "Custom" ) );
}

QString Blokkal::Ui::ProviderComboBox::currentProvider( void ) const
{
	if( currentIndex() < 0 ) {
		return QString::null;
	}

	return d->providerMap[currentIndex()].id();
}

void Blokkal::Ui::ProviderComboBox::setCurrentProvider( const QString & id )
{
	for( QMap<unsigned int, ProviderInformation>::ConstIterator it = d->providerMap.begin();
	     it != d->providerMap.end();
	     ++it )
	{
		if( it.value().id() == id ) {
			setCurrentIndex( it.key() );
			return;
		}
	}

	if( count() ) {
		setCurrentIndex( count() - 1 );
	}
}

void Blokkal::Ui::ProviderComboBox::emitProviderSelected( int index )
{
	if( d->providerMap.contains( index ) ) {
		emit providerSelected( d->providerMap[index].id() );
	}
}
