/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "semieditablecombobox.h"
#include "semieditablecombobox.moc"

#include <QShowEvent>

class Blokkal::Ui::SemiEditableComboBox::Private
{
public:
	/**
	 * Creates a new object
	 */
	Private( void ) {}
};

Blokkal::Ui::SemiEditableComboBox::SemiEditableComboBox( QWidget * parent ):
KComboBox( parent ),
d( new Private() )
{
	setInsertPolicy( KComboBox::NoInsert );
	connect( this, SIGNAL( activated( int ) ),
	         this, SLOT( updateEditableState( int ) ) );

	connect( this, SIGNAL( editTextChanged ( const QString & ) ),
	         this, SLOT( setCustomInfo( const QString & ) ) );
}

Blokkal::Ui::SemiEditableComboBox::~SemiEditableComboBox( void )
{
	delete d;
}

void Blokkal::Ui::SemiEditableComboBox::setItems( QStringList items )
{
	QString oldText = currentText();
	clear();
	addItem( "" );
	addItems( items );
	setCurrentText( oldText );
}

void Blokkal::Ui::SemiEditableComboBox::setCurrentIndex( int index )
{
	KComboBox::setCurrentIndex( index );
	updateEditableState( );
}

void Blokkal::Ui::SemiEditableComboBox::setCurrentText( const QString & _itemText )
{
	if( !_itemText.isNull() ) {
		int i;
		for( i = 0; i < count(); i++ ) {
			if( itemText( i ) == _itemText ) {
				setCurrentIndex( i );
				break;
			}
		}
		if( i == count() ) {
			if( !count() ) {
				addItem( _itemText );
			}
			setCurrentIndex( 0 );
			setItemText( 0, _itemText );
		}
		updateEditableState( );
	}
}

void Blokkal::Ui::SemiEditableComboBox::showEvent( QShowEvent * event )
{
	if( event && !event->spontaneous() ) {
		updateEditableState( );
	}
}

void Blokkal::Ui::SemiEditableComboBox::updateEditableState( int index )
{
	//box is empty
	if( currentIndex() < 0 ) {
		setEditable( TRUE );
		return;
	}
	
	if( index < 0 ) {
		updateEditableState( currentIndex() );
	}
	else {
		setEditable( index == 0);
	}
}

void Blokkal::Ui::SemiEditableComboBox::setCustomInfo( const QString & text )
{
	if( currentIndex() != 0 ) {
		return;
	}
	
	setItemText( 0, text );
}
