/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UiSEMIEDITABLECOMBOBOX_H
#define BLOKKAL_UiSEMIEDITABLECOMBOBOX_H

#include <kcombobox.h>

#include "../blokkal_export.h"

namespace Blokkal {
namespace Ui {

/**
 * @short A combo box that is editable if the item with index 0
 * is selected and not editable otherwise.
 *
 * A combo box that is editable if the item with index 0
 * is selected and not editable otherwise.
 *
 * This class is used as a base
 * for @ref Blokkal::Ui::MediaInfoComboBox but can be used
 * for other applications as well.
 * 
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT SemiEditableComboBox : public KComboBox
{
Q_OBJECT
public:
	/**
	 * Creates a new SemiEditableComboBox.
	 * @param parent parent widget
	*/
	SemiEditableComboBox( QWidget * parent = 0 );

	/**
	 * Destrcutor
	 */
	virtual ~SemiEditableComboBox( void );

	/**
	 * Clears any previous content and sets the items in the
	 * combo box to those in @a items. Additionally an empty item
	 * is inserted at index 0. If an item was selected before this method was
	 * called, item 0 will be set to the value of the previously
	 * selected item if it does not exist in @a items. If it does it will
	 * be selected.
	 * @param items the items for the combo box
	*/
	void setItems( QStringList items );

	/**
	 * Sets the current item to be the item with index  @a index.
	 * @param index the index of the item to select
	 */
	virtual void setCurrentIndex( int index );

	/**
	* Unlike KComboBox::setCurrentText() this method will search for @a itemText
	* in the combobox items. If it does find it, the appropriate item will be
	* selected. If such an item cannot be found the item with
	* index 0 is set to @a itemText and is selected.
	*/
	virtual void setCurrentText( const QString & itemText );

protected:
	/**
	 * Reeimplemented to make sure the editable state is correct for
	 * the currently selected item.
	 * @param event the show event
	*/
	virtual void showEvent( QShowEvent * event );

protected Q_SLOTS:
	/**
	 * Sets the editable state of the combobox as appropriate
	 * for the currently selected item if @a index < 0 or for
	 * the item with index @a index otherwise.
	 * This slot is called whenever the user selects a new item.
	*/
	virtual void updateEditableState( int index = -1 );

	/**
	 * This slot is connected to the editTextChanged() signal.
	 * If item 0 is selected it will have it's text replaced
	 * with @a text
	 * @param text text of the custom media info
	 */
	virtual void setCustomInfo( const QString & text );

private:
	class Private;
	Private * d;

};
}
}

#endif
