/***************************************************************************
 *   Copyright (C) 2006 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "stdaction.h"

#include "globalsettings.h"

#include <kaction.h>
#include <kactioncollection.h>
#include <klocale.h>
#include <ksettings/dialog.h>

KSettings::Dialog * Blokkal::Ui::StdAction::m_preferencesDialog = 0;

Blokkal::Ui::StdAction::StdAction( void )
{}

Blokkal::Ui::StdAction::~StdAction( void )
{}

KAction * Blokkal::Ui::StdAction::create( Blokkal::Ui::StdAction::StdActions id, const char * name, const QObject * receiver, const char * slot, KActionCollection * parent )
{
	KAction * action = new KAction( parent );

	switch( id ) {
	case Bold:
		action->setText( i18n( "&Bold" ) );
		action->setShortcut( i18n( "Ctrl+B" ) );
		//action->setIcon( KIcon( "blokkal_textbold" ) );
		action->setIcon( KIcon( "format-text-bold" ) );
		break;
	case Italic:
		action->setText( i18n( "&Italic" ) );
		action->setShortcut( i18n( "Ctrl+I" ) );
		//action->setIcon( KIcon( "blokkal_textitalic" ) );
		action->setIcon( KIcon( "format-text-italic" ) );
		break;
	case Underline:
		action->setText( i18n( "&Underline" ) );
		action->setShortcut( i18n( "Ctrl+U" ) );
		//action->setIcon( KIcon( "blokkal_textunderline" ) );
		action->setIcon( KIcon( "format-text-underline" ) );
		break;
	case Strike:
		action->setText( i18n( "Strike &out" ) );
		action->setShortcut( i18n( "Ctrl+O" ) );
		//action->setIcon( KIcon( "blokkal_textstrike" ) );
		action->setIcon( KIcon( "format-text-strikethrough" ) );
		break;
	case Link:
		action->setText( i18n( "Insert &Link" ) );
		action->setShortcut( i18n( "Ctrl+L" ) );
		action->setIcon( KIcon( "insert-link" ) );
		break;
	case Image:
		action->setText( i18n( "Insert Ima&ge" ) );
		action->setShortcut( i18n( "Ctrl+G" ) );
		//action->setIcon( KIcon( "blokkal_insertimage" ) );
		action->setIcon( KIcon( "insert-image" ) );
		break;
	case Post:
		action->setText( i18n( "&Post Entry" ) );
		action->setShortcut( i18n( "Ctrl+P" ) );
		action->setIcon( KIcon( "blokkal" ) );
		break;
	}

	if( receiver && slot ) {
		QObject::connect( action, SIGNAL( triggered (Qt::MouseButtons, Qt::KeyboardModifiers ) ),
		                  receiver, slot );
	}

	if( parent && name ) {
		parent->addAction( name, action );
	}

	return action;
}

KAction * Blokkal::Ui::StdAction::bold( const QObject * receiver, const char * slot, KActionCollection * parent, const char * name )
{
	return create( Bold, name, receiver, slot, parent );
}

KAction * Blokkal::Ui::StdAction::italic( const QObject * receiver, const char * slot, KActionCollection * parent, const char * name )
{
	return create( Italic, name, receiver, slot, parent );
}

KAction * Blokkal::Ui::StdAction::underline( const QObject * receiver, const char * slot, KActionCollection * parent, const char * name )
{
	return create( Underline, name, receiver, slot, parent );
}

KAction * Blokkal::Ui::StdAction::strike( const QObject * receiver, const char * slot, KActionCollection * parent, const char * name )
{
	return create( Strike, name, receiver, slot, parent );
}

KAction * Blokkal::Ui::StdAction::link( const QObject * receiver, const char * slot, KActionCollection * parent, const char * name )
{
	return create( Link, name, receiver, slot, parent );
}

KAction * Blokkal::Ui::StdAction::image( const QObject * receiver, const char * slot, KActionCollection * parent, const char * name )
{
	return create( Image, name, receiver, slot, parent );
}

KAction * Blokkal::Ui::StdAction::post( const QObject * receiver, const char * slot, KActionCollection * parent, const char * name )
{
	return create( Post, name, receiver, slot, parent );
}

KAction * Blokkal::Ui::StdAction::preferences( KActionCollection * parent, const char * name )
{
	if( !m_preferencesDialog ) {
		m_preferencesDialog = new KSettings::Dialog( Blokkal::Ui::GlobalSettings::self()->mainWidget() );
	}

	KAction * action = KStandardAction::preferences( m_preferencesDialog,
	                                                 SLOT( show( void ) ),
	                                                 parent );
	action->setObjectName( name );
	return action;
}
