/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UISTDACTION_H
#define BLOKKAL_UISTDACTION_H

#include "../blokkal_export.h"

#include <kstdaction.h>

class KAction;
class KActionCollection;
class QObject;

namespace KSettings {
class Dialog;
}

namespace Blokkal {
namespace Ui {

/**
 * @short Provides standard actions
 *
 * Provides standard actions
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT StdAction {
public:
	/**
	 * Enumeration for standard actions.
	 */
	enum StdActions {
		Bold, ///< For actions that make a text bold
		Italic, ///< For actions that make a text italic
		Underline, ///< For actions that make a text underlined
		Strike, ///< For actions that make a striked out
		Link, ///< For actions that create links
		Image, ///< For actions that insert images
		Post, ///< For actions that post an entry to the server
	};

	/**
	 * Creates a new Action.
	 * @param id the standard action to create
	 * @param name the action name
	 * @param receiver the receiver of the signal
	 * @param slot the slot to call
	 * @param parent the parent collection
	 * @return the standard action
	 */
	static KAction * create( Blokkal::Ui::StdAction::StdActions id, const char * name, const QObject * receiver, const char * slot, KActionCollection * parent = 0 );

	/**
	 * Creates a new "Bold"-Action
	 * @param receiver the receiver of the signal
	 * @param slot the slot to call
	 * @param parent the parent collection
	 * @param name the action name
	 * @return the action
	 */
	static KAction * bold( const QObject * receiver, const char * slot, KActionCollection * parent = 0, const char * name = "edit_bold" );

	/**
	 * Creates a new "Italic"-Action
	 * @param receiver the receiver of the signal
	 * @param slot the slot to call
	 * @param parent the parent collection
	 * @param name the action name
	 * @return the action
	 */
	static KAction * italic( const QObject * receiver, const char * slot, KActionCollection * parent = 0, const char * name = "edit_italic" );

	/**
	 * Creates a new "Underline"-Action
	 * @param receiver the receiver of the signal
	 * @param slot the slot to call
	 * @param parent the parent collection
	 * @param name the action name
	 * @return the action
	 */
	static KAction * underline( const QObject * receiver, const char * slot, KActionCollection * parent = 0, const char * name = "edit_underline" );

	/**
	 * Creates a new "Strike out"-Action
	 * @param receiver the receiver of the signal
	 * @param slot the slot to call
	 * @param parent the parent collection
	 * @param name the action name
	 * @return the action
	 */
	static KAction * strike( const QObject * receiver, const char * slot, KActionCollection * parent = 0, const char * name = "edit_strike" );

	/**
	 * Creates a new "Insert Link"-Action
	 * @param receiver the receiver of the signal
	 * @param slot the slot to call
	 * @param parent the parent collection
	 * @param name the action name
	 * @return the action
	 */
	static KAction * link( const QObject * receiver, const char * slot, KActionCollection * parent = 0, const char * name = "edit_insertlink" );

	/**
	 * Creates a new "Insert Image"-Action
	 * @param receiver the receiver of the signal
	 * @param slot the slot to call
	 * @param parent the parent collection
	 * @param name the action name
	 * @return the action
	 */
	static KAction * image( const QObject * receiver, const char * slot, KActionCollection * parent = 0, const char * name = "edit_insertimage" );

	/**
	 * Creates a new "Post Entry"-Action
	 * @param receiver the receiver of the signal
	 * @param slot the slot to call
	 * @param parent the parent collection
	 * @param name the action name
	 * @return the action
	 */
	static KAction * post( const QObject * receiver, const char * slot, KActionCollection * parent = 0, const char * name = "file_postentry" );

	 /**
	  * Returns an action to show the Blokkal preferences dialog.
	  * @param parent the parent collection
	  * @param name the action name, if 0 the standard name from KStdAction is used
	  * @return an action to show the Blokkal preferences dialog
	  */
	static KAction * preferences( KActionCollection * parent = 0, const char * name = KStandardAction::name( KStandardAction::Preferences ) );

private:
	static KSettings::Dialog * m_preferencesDialog;
	StdAction( void );
	~StdAction( void );

};

}

}

#endif
