/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UiVIEW_H
#define BLOKKAL_UiVIEW_H

#include "../blokkal_export.h"

class QWidget;

namespace Blokkal {
namespace Ui {
class HtmlEntryTextEdit;
class EditEntryWidget;
/**
 * @short This class is the base for all views.
 *
 * Views are containers used to display widgets like EditEntryWidget or
 * may build all widgets on their own.
 * In most cases implementations of this interface will be subclasses
 * of KMainWindow or KDialogBase.
 * @note Views are expected to destroy themselves!
 * @author Martin Mueller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT View
{
public:
	/**
	 * Constructor.
	 */
	View( void );

	/**
	 * Destructor
	 */
	virtual ~View( void );

	/**
	 * Makes the view visible.
	 */
	virtual void showView( void ) = 0;

	/**
	 * Hides the view
	 */
	virtual void hideView( void ) = 0;

	/**
	 * Closes the view.
	 * If this method returns TRUE the view has been closed.
	 * @note In this case the view must destroy itself.
	 */
	virtual bool closeView( void ) = 0;

	/**
	 * Returns the main widget of this view.
	 * @return the main widget
	 */
	virtual QWidget * mainWidget( void ) = 0;
	
protected:
	/**
	 * @return the HtmlEntryTextEdit of @a widget
	 */
	static Blokkal::Ui::HtmlEntryTextEdit * entryTextEdit( Blokkal::Ui::EditEntryWidget * widget );
};

}

}

#endif
