/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "viewmanager.h"
#include "viewmanager.moc"

#include "../blokkalentry.h"
#include "../blokkalpluginmanager.h"
#include "editentryviewfactory.h"
#include "view.h"

#include <kglobal.h>

#include <QWidget>

class Blokkal::Ui::ViewManagerPrivate {
public:
	ViewManagerPrivate( void ) {}

	ViewManager instance;

	QMap<Blokkal::Entry *, Blokkal::Ui::View *> entryViewMap;
	QMap<Blokkal::Ui::View *, QWidget* > viewWidgetMap;
};

K_GLOBAL_STATIC( Blokkal::Ui::ViewManagerPrivate, _buvmp );

Blokkal::Ui::ViewManager::ViewManager( void )
{
	setObjectName( "ViewManager" );
}

Blokkal::Ui::ViewManager::~ViewManager( void )
{}

Blokkal::Ui::ViewManager * Blokkal::Ui::ViewManager::self( void )
{
	return &_buvmp->instance;
}

Blokkal::Ui::View * Blokkal::Ui::ViewManager::view( Blokkal::Entry * entry )
{
	if( !hasView( entry ) ) {
		entry->ref();
		View * const view = Blokkal::PluginManager::self()->editEntryViewFactory()->createView( entry );
		_buvmp->entryViewMap.insert( entry, view );
		_buvmp->viewWidgetMap.insert( view, view->mainWidget() );
		connect( view->mainWidget(),
		         SIGNAL( destroyed( void ) ),
		         self(),
		         SLOT( slotViewDestroyed( void ) ) );
		connect( entry,
		         SIGNAL( entryDestroyed( Blokkal::Entry * ) ),
		         self(),
		         SLOT( slotEntryDestroyed( Blokkal::Entry * ) ) );
	}

	return _buvmp->entryViewMap[entry];
}

bool Blokkal::Ui::ViewManager::hasView( Blokkal::Entry * entry )
{
	return _buvmp->entryViewMap.contains( entry );
}

void Blokkal::Ui::ViewManager::slotViewDestroyed( void )
{
	for( QMap<Blokkal::Entry *, Blokkal::Ui::View *>::ConstIterator it = _buvmp->entryViewMap.begin();
	     it != _buvmp->entryViewMap.end();
	     ++it )
	{
		if( _buvmp->viewWidgetMap.contains( it.value() ) &&
		    sender() == _buvmp->viewWidgetMap[it.value()] )
		{
			it.key()->deref();
			_buvmp->viewWidgetMap.remove( it.value() );
			_buvmp->entryViewMap.remove( it.key() );
			return;
		}
	}
}

void Blokkal::Ui::ViewManager::slotEntryDestroyed( Blokkal::Entry * entry )
{
	if( !_buvmp->entryViewMap.contains( entry ) ) {
		return;
	}

	_buvmp->viewWidgetMap.remove( _buvmp->entryViewMap[entry] );
	_buvmp->entryViewMap.remove( entry );
}
