/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKAL_UIVIEWMANAGER_H
#define BLOKKAL_UIVIEWMANAGER_H

#include <QtCore/QObject>

#include "../blokkal_export.h"

namespace Blokkal {
class Entry;

namespace Ui {
class View;
class ViewManagerPrivate;

class BLOKKAL_EXPORT ViewManager : public QObject {
friend class ViewManagerPrivate;
Q_OBJECT
public:
	/**
	 * @return the global ViewManager
	 */
	static Blokkal::Ui::ViewManager * self( void );

	/**
	 * @return the view for @a entry
	 */
	static Blokkal::Ui::View * view( Blokkal::Entry * entry );

	/**
	 * @return TRUE if @a entry has a view
	 */
	static bool hasView( Blokkal::Entry * entry );

Q_SIGNALS:
	/**
	 * This signal is emitted, when a @a view is created for @a entry
	 */
	void viewCreated( Blokkal::Entry * entry, Blokkal::Ui::View * view );

private Q_SLOTS:
	void slotViewDestroyed( void );
	void slotEntryDestroyed( Blokkal::Entry * entry );

private:
	ViewManager( void );
	~ViewManager( void );
};

}

}

#endif
