/***************************************************************************
 *   Copyright (C) 2006 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOKKALWALLETMANAGER_H
#define BLOKKALWALLETMANAGER_H

#include <kwindowsystem.h>

#include <QObject>

#include "blokkal_export.h"

namespace KWallet {
class Wallet;
}

namespace Blokkal {
class WalletManagerPrivate;
/**
 * @short Wallet management class.
 *
 * Wallet management class.
 * This class opens the wallet, sets it to the appropriate folder
 * and returns it to the objects that ask for it.
 * 
 * @author Martin Mller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT WalletManager : public QObject
{
friend class WalletManagerPrivate;
Q_OBJECT
public:
	/**
	 * Returns a pointer to the global wallet manager.
	 * If none exists yet, one will be created.
	 *
	 * @return the wallet manager
	 */
	static Blokkal::WalletManager * self( void );

	/**
	 * Opens the wallet and notifies @a receiver by calling the given slot.
	 * If you receive a 0 wallet through the signal, the request was not
	 * successfull.
	 */
	 void openWallet( QObject * receiver, const char * slot );

private Q_SLOTS:
	void slotWalletOpened( void );
	void retryOpen( void );
	
private:
	WalletManager( void );
	~WalletManager( void );
};

/**
 * @short This class is used be Blokkal::WalletManager.
 *
 * This class is used by  Blokkal::WalletManager, so that only
 * objects waiting for the wallet to be opened will be notified.
 * @author Martin Mller <orvio@orvio.de>
 */
class BLOKKAL_EXPORT WalletManagerSignal : public QObject {
Q_OBJECT
public:
	friend class WalletManager;
Q_SIGNALS:
	/**
	 * This signal is send to objects waiting for the wallet to be opened.
	 * @param wallet the wallet, 0 if failed
	 */
	void walletOpened( KWallet::Wallet * wallet );
};

/**
 * @short A small helper class to remove the passwort of an account from the wallet
 *
 * A small helper class to remove the passwort of an account from the wallet.
 * The constructor takes the if of an account and removes the password if it is
 * stored in the waller.
 */
class BLOKKAL_EXPORT PasswordDeleter : public QObject {
Q_OBJECT
public:
	/**
	 * Constructor
	 * Removes the passwort of the account with @a accountId
	 * from the wallet. If no password exists nothing happens.
	 * The object will delete itself after it has finished.
	 * @param accountId id of the account
	 */
	PasswordDeleter( const QString & accountId );
	virtual ~PasswordDeleter( void );

private Q_SLOTS:
	void slotWalletOpened( KWallet::Wallet * wallet );

private:
	class Private;
	Private * const d;
};

}

#endif
