/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "bloggeraccount.h"
#include "bloggeraccount.moc"

#include "bloggerprotocol.h"
#include "bloggerblog.h"
#include "bloggerio/getusersblogsjob.h"

#include <blokkalblogconfig.h>

#include <kdebug.h>
#include <klocale.h>

#include <QTimer>

class Blogger::Account::Private {
public:
	Private( void ) :
	password( QString::null ),
	getUsersBlogsJob( 0 ){}

	QString password;
	Io::GetUsersBlogsJob * getUsersBlogsJob;
	bool passwordFailure;
};

Blogger::Account::Account( Blogger::Protocol * protocol, const QString & accountName ) :
Blokkal::PasswordedAccount( protocol, accountName ),
d( new Private() )
{
}

Blogger::Account::~Account( void )
{
	delete d;
	d = 0;
}

Blokkal::Blog * Blogger::Account::restoreBlog( Blokkal::BlogConfig * config )
{
	if( !config ) {
		return 0;
	}

	return createBlog( config->id(),
	                   config->readEntry( "url" ),
	                   config->title() );
}

void Blogger::Account::disconnectAccount( Blokkal::Account::DisconnectReason reason, const QString & reasonString )
{
	if( d->getUsersBlogsJob ) {
		d->getUsersBlogsJob = 0;
	}
	
	Blokkal::PasswordedAccount::disconnectAccount( reason, reasonString );
}

void Blogger::Account::connectInner( void )
{
	d->password = PasswordedAccount::password();
	d->getUsersBlogsJob = new Io::GetUsersBlogsJob( this );
	connect( d->getUsersBlogsJob,
	         SIGNAL( result( KJob * ) ),
	         SLOT( slotGetUsersBlogsJobReturned( KJob * ) ) );
	d->getUsersBlogsJob->start();
}

const QString & Blogger::Account::password( void ) const
{
	return d->password;
}

void Blogger::Account::slotGetUsersBlogsJobReturned( KJob * job )
{
	if( !job ) {
		return;
	}

	if( job != d->getUsersBlogsJob ) {
		return;
	}

	if( d->getUsersBlogsJob->responseType() != Blokkal::Io::XmlRpcJob::ResponseSuccess ) {
		const QString errorString = d->getUsersBlogsJob->errorString();
		const Blokkal::Io::XmlRpcJob::ResponseType responseType = d->getUsersBlogsJob->responseType();
		d->getUsersBlogsJob = 0; //avoid kill in disconnectAccount()
		if( responseType == Blokkal::Io::XmlRpcJob::ResponseFault ) {
			if( errorString.contains( "pass", Qt::CaseInsensitive )
			    || errorString.contains( "login", Qt::CaseInsensitive ) )
			{
				d->password = QString::null;
				disconnectAccount( Blokkal::Account::AuthenticationDisconnect );
			}
			else {
				disconnectAccount( Blokkal::Account::IOErrorDisconnect );
			}
		}
		else {
			disconnectAccount( Blokkal::Account::IOErrorDisconnect,
			                   i18n( "Failed to fetch list of users blogs." ) );
		}
		
	}
	else {
		QStringList blogIds = d->getUsersBlogsJob->blogIds();
		QList<Blokkal::Blog*> currentBlogs = blogs();
		for( QList<Blokkal::Blog*>::ConstIterator it = currentBlogs.begin();
		     it != currentBlogs.end();
		     ++it )
		{
			if( !blogIds.contains( (*it)->id() ) ) {
				(*it)->removeBlog();
			}
		}
		for( QStringList::ConstIterator it = blogIds.begin();
		     it != blogIds.end();
		     ++it )
		{
			Blokkal::Blog * currentblog = blog( *it );
			if( currentblog ) { //update
				Blogger::Blog * bloggerBlog = dynamic_cast< Blogger::Blog * >( currentblog );
				if( bloggerBlog ) {
					bloggerBlog->setTitle( d->getUsersBlogsJob->blogName( *it ) );
					bloggerBlog->setUrl( d->getUsersBlogsJob->blogUrl( *it ) );
				}
				else {
					kWarning() << "failed to cast to Blogger::Blog" << endl;
				}
			}
			else { //create new blog object
				registerBlog( createBlog( *it,
				                          d->getUsersBlogsJob->blogUrl( *it ),
				                          d->getUsersBlogsJob->blogName( *it ) ) );
			}
		}
		d->getUsersBlogsJob = 0;
		setConnectionStatus( Connected );
	}
}

Blogger::Blog * Blogger::Account::createBlog( const QString & id, const QString & url, const QString & title )
{
	return new Blogger::Blog( this, id, url, title );
}
