/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOGGERBLOGGERACCOUNT_H
#define BLOGGERBLOGGERACCOUNT_H

#include "blogger_export.h"

#include <blokkalaccount.h>

class KJob;

namespace Blogger {
class Protocol;
class Blog;
/**
	@author Martin Mueller <orvio@orvio.de>
*/
class BLOGGER_EXPORT Account : public Blokkal::PasswordedAccount
{
Q_OBJECT
public:
	/**
	 * Creates a new account for protocol @a protocol, with name @a accountName
	 * @param protocol the procotol of this account
	 * @param accountName the name if this account
	 */
	Account( Blogger::Protocol * protocol, const QString & accountName = 0 );

	/**
	 * Destructor
	 */
	virtual ~Account( void );

	/**
	 * Returns the clear text password for this account or
	 * QString::null if none has been provided yet.
	 * @return the clear text password for this account
	 */
	const QString & password( void ) const;

public Q_SLOTS:
	/**
	 * Disconnects the account.
	 * @param reason optional reason id
	 * @param reasonString optional description
	 */
	virtual void disconnectAccount( Blokkal::Account::DisconnectReason reason = UserDisconnect,
	                                const QString & reasonString = 0 );	

	/**
	 * Restores the blog according to the information in @a config.
	 * If you do not want or are unable to restore the blog for some reason
	 * return 0.
	 * @param config restore the blog with this configuration
	 * @return the restored blog or 0 if the blog was not restored
	 */
	virtual Blokkal::Blog * restoreBlog( Blokkal::BlogConfig * config );

protected:
	/**
	 * Creates a new blog object.
	 * Do not register the object in this method.
	 * @param id blog id
	 * @param url the blog url
	 * @param title the blog title
	 */
	virtual Blogger::Blog * createBlog( const QString & id,
	                                    const QString & url,
	                                    const QString & title );
protected Q_SLOTS:
	virtual void connectInner( void );
	
private Q_SLOTS:
	void slotGetUsersBlogsJobReturned( KJob * job );
	
private:
	class Private;
	Private * d;
};

}

#endif
