/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOGGERBLOGGERBLOG_H
#define BLOGGERBLOGGERBLOG_H

#include "blogger_export.h"

#include <blokkalblog.h>

namespace Blogger {
class Account;
/**
	@author Martin Mueller <orvio@orvio.de>
*/
class BLOGGER_EXPORT Blog : public Blokkal::Blog
{
Q_OBJECT
public:
	/**
	 * This creates a new Blog.
	 * @param parentAccount the account this blog belongs to
	 * @param id the blog id
	 * @param url sets the url of this blog
	 */
	Blog( Blogger::Account * parentAccount,
	      const QString & id,
	      const QString & url,
	      const QString & title );

	/**
	 * Destructor
	 */
	virtual ~Blog( void );

	/**
	 * This method returns the url of the blog.
	 * @return the url of the blog.
	 */
	virtual KUrl url( void ) const;

	/**
	 * Sets the url of this blog
	 * @param url the new url
	 */
	void setUrl( const QString & url );

	/**
	 * Creates a new @ref Blokkal::Entry object.
	 * @return a new Blokkal::Entry object.
	 */
	virtual Blokkal::Entry * createEntry( void );

	/**
	 * @return the BlogBrowsingDriver of this blog
	 */
	virtual Blokkal::Io::BlogBrowsingDriver * blogBrowsingDriver( void ) const;

	/**
	 * Saves the properties of this blog, so that it can be restored before
	 * the parent account connects to the server.
	 */
	virtual void saveProperties( void );

	/**
	 * Returns an or'ed list of Blokkal::Blog::EntryFeatures values to indicate
	 * the possible options of entry editing.
	 * Returns FeatureDate|FeatureText.
	 * @return an or'ed list of Blokkal::Blog::EntryFeatures values
	 */
	virtual unsigned int supportedFeatures( void ) const;

private:
	class Private;
	Private * const d;
};

}

#endif
