/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "bloggerentry.h"
#include "bloggerentry.moc"

#include "bloggerblog.h"
#include "bloggerui/entryextensionwidget1.h"
#include "bloggerio/editpostjob.h"
#include "bloggerio/newpostjob.h"
#include "bloggerio/deletepostjob.h"

class Blogger::Entry::Private {
public:
	Private( void ) {}
};

Blogger::Entry::Entry( Blogger::Blog * parentBlog,
                       const QString & id,
                       const KUrl & url ) :
Blokkal::Entry( parentBlog, id, url ),
d( new Private() )
{
}

Blogger::Entry::~Entry( void )
{
	delete d;
	d = 0;
}

bool Blogger::Entry::isPublish( void ) const
{
	return readBoolEntry( "publish" );
}

void Blogger::Entry::setPublish( bool publish )
{
	writeEntry( "publish", publish );
}

Blokkal::Ui::EntryExtensionWidget * Blogger::Entry::createExtensionWidget( Blokkal::Ui::EntryExtensionWidget::ExtensionWidget extension, Blokkal::Ui::EditEntryWidget * parent )
{
	if( extension == Blokkal::Ui::EntryExtensionWidget::ExtensionWidget1 ) {
		return new Ui::EntryExtensionWidget1( this, parent );
	}
	else {
		return 0;
	}
}

Blokkal::Io::EntryJob * Blogger::Entry::createJob( Blokkal::Entry::JobType type )
{
	Blokkal::Io::EntryJob * job = 0;
	switch( type ) {
	case PostJob:
		job = new Blokkal::Io::EntryJob( this );
		if( id().isNull() ) {
			job->addSubjob( new Io::NewPostJob( this ) );
		}
		else {
			job->addSubjob( new Io::EditPostJob( this ) );
		}
		break;
	case DeleteJob:
		job = new Blokkal::Io::EntryJob( this );
		job->addSubjob( new Io::DeletePostJob( this ) );
		break;
	default:
		break;
	}
	
	return job;
}
