/***************************************************************************
 *   Copyright (C) 2007 - 2008  by Martin Mueller                          *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOGGERBLOGGERENTRY_H
#define BLOGGERBLOGGERENTRY_H

#include "blogger_export.h"

#include <blokkalentry.h>

namespace Blogger {
class Blog;
/**
	@author Martin Mueller <orvio@orvio.de>
*/
class BLOGGER_EXPORT Entry : public Blokkal::Entry
{
Q_OBJECT
public:
	/**
	 * Creates a new Entry
	 * @param parentBlog owning blog
	 * @param id the id of the entry
	 * @param url the url of this entry
	 */
	Entry( Blogger::Blog * parentBlog,
	       const QString & id = QString::null,
	       const KUrl & url = KUrl() );

	/**
	 * Destructor
	 */
	virtual ~Entry( void );

	/**
	 * Creates and returns a job of @a type
	 * @return a job of @a type
	 */
	virtual Blokkal::Io::EntryJob * createJob( Blokkal::Entry::JobType type );

	/**
	 * Returns whether the entry is supposed to be public or not.
	 * @return TRUE if the entry is public
	 */
	bool isPublish( void ) const;

	/**
	 * Sets the entry to public if @a publish is TRUE
	 * @param publish TRUE to indicate that this entry is spuusoed to be public
	 */
	void setPublish( bool publish );

	/**
	 * Returns the widget associated with @a extension.
	 * @param extension the identifier for the widget to create
	 * @param parent the parent widget
	 * @return the widget associated with @a extension.
	 */
	virtual Blokkal::Ui::EntryExtensionWidget * createExtensionWidget( Blokkal::Ui::EntryExtensionWidget::ExtensionWidget extension, Blokkal::Ui::EditEntryWidget * parent );

private:
	class Private;
	Private * d;
};

}

#endif
