/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blogbrowsingdriver.h"
#include "blogbrowsingdriver.moc"

#include "../bloggerblog.h"
#include "../bloggerentry.h"

#include "getpostjob.h"
#include "getrecentpostsjob.h"

#include <kdebug.h>

Blogger::Io::BlogBrowsingDriver::BlogBrowsingDriver( Blogger::Blog * parent ):
Blokkal::Io::BlogBrowsingDriver( parent )
{
}

Blogger::Io::BlogBrowsingDriver::~BlogBrowsingDriver( void )
{
}

unsigned int Blogger::Io::BlogBrowsingDriver::supportedOperations( void ) const
{
	return LoadEntryById|LoadRecentEntries;
}

Blokkal::Io::LoadEntryJob * Blogger::Io::BlogBrowsingDriver::createLoadEntryJob( const QStringList & idList )
{
	Blokkal::Io::LoadEntryJob * job = new Blogger::Io::LoadEntryJob( blog() );
	for( QStringList::ConstIterator it = idList.begin();
	     it != idList.end();
	     ++it )
	{
		job->addSubjob( new GetPostJob( static_cast<Blogger::Blog*>( blog() ), *it ) );
	}
	return job;
}

Blokkal::Io::LoadEntryJob * Blogger::Io::BlogBrowsingDriver::createLoadEntryJob( unsigned int n )
{
	Blokkal::Io::LoadEntryJob * job = new Blogger::Io::LoadEntryJob( blog() );
	job->addSubjob( new GetRecentPostsJob( static_cast<Blogger::Blog*>( blog() ), n ) );
	return job;
}

//------------
//LoadEntryJob
//------------
class Blogger::Io::LoadEntryJob::Private {
public:
	Private( void ) {}

	QList<Blokkal::Entry *> entryList;
};
Blogger::Io::LoadEntryJob::LoadEntryJob( Blokkal::Blog * blog ):
Blokkal::Io::LoadEntryJob( blog ),
d( new Private() )
{
}

Blogger::Io::LoadEntryJob::~LoadEntryJob( void )
{
	delete d;
}

void Blogger::Io::LoadEntryJob::subjobFinished( KJob * job )
{
	if( job->error() ) {
		return;
	}

	const Blogger::Io::GetPostJob * const getPostJob = dynamic_cast<Blogger::Io::GetPostJob *>( job );
	if( getPostJob ) {
		Blokkal::Entry * const entry = getPostJob->entry();
		if( entry ) {
			d->entryList.append( entry );
		}
		return;
	}
	
	const Blogger::Io::GetRecentPostsJob * const getRecentPostsJob = dynamic_cast<Blogger::Io::GetRecentPostsJob *>( job );
	if( getRecentPostsJob ) {
		const QList<Blogger::Entry*> entryList = getRecentPostsJob->entries();
		for( QList<Blogger::Entry*>::ConstIterator it = entryList.begin();
		     it != entryList.end();
		     ++it )
		{
			d->entryList.append( *it );
		}
		return;
	}

	kError() << "invalid subjob encountered" << endl;

}

QList< Blokkal::Entry * > Blogger::Io::LoadEntryJob::entries( void ) const
{
	return d->entryList;
}

unsigned int Blogger::Io::BlogBrowsingDriver::supportedBrowseModes( void ) const
{
	return BrowseEdit;
}

