/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOGGER_IOBLOGBROWSINGDRIVER_H
#define BLOGGER_IOBLOGBROWSINGDRIVER_H

#include <blokkalio/blogbrowsingdriver.h>
#include <blokkalio/jobs.h>

#include "../blogger_export.h"

namespace Blogger {
class Blog;
namespace Io {

class BLOGGER_EXPORT BlogBrowsingDriver : public Blokkal::Io::BlogBrowsingDriver {
Q_OBJECT
public:
	using Blokkal::Io::BlogBrowsingDriver::createLoadEntryJob;
	/**
	 * Constructor
	 * @param parent parent blog
	 */
	BlogBrowsingDriver( Blogger::Blog * parent );
	virtual ~BlogBrowsingDriver( void );
	
	/**
	 * This method returns an or'ed list of BlogOperations values.
	 * @return returns an or'ed list of BlogOperations values
	 */
	virtual unsigned int supportedOperations( void ) const;

	/**
	 * Creates and returns a job that loads all entries in @a idList.
	 * @return a job that loads all entries in @a idList
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( const QStringList & idList );

	/**
	 * Creates and returns a job that loads the last @a n entries in the blog.
	 * @return a job that loads the last @a n entries in the blog
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( unsigned int n );

	/**
	 * Returns the browse modes supported by this blog.
	 * @return the browse modes supported by this blog
	 */
	virtual unsigned int supportedBrowseModes( void ) const;
};

class LoadEntryJob : public Blokkal::Io::LoadEntryJob {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param blog parent blog
	 */
	LoadEntryJob( Blokkal::Blog * blog );
	virtual ~LoadEntryJob( void );
	
	/**
	 * @return the loaded entries
	 */
	QList<Blokkal::Entry *> entries( void ) const;
	
protected:
	/**
	 * This method is called when a subjob has finished.
	 * Overridden to get the entry.
	 */
	virtual void subjobFinished( KJob * job );

private:
	class Private;
	Private * const d;
	
};

}

}

#endif
