/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getrecentpostsjob.h"
#include "getrecentpostsjob.moc"

#include "../bloggerentry.h"

#include <QVariant>

class Blogger::Io::GetRecentPostsJob::Private {
public:
	Private( Blogger::Blog * blog ) :
	blog( blog ),
	builtList( FALSE ){}

	Blogger::Blog * blog;
	bool builtList;
	QList< Blogger::Entry * > entries;
};

Blogger::Io::GetRecentPostsJob::GetRecentPostsJob( Blogger::Blog * blog, int count, bool showProgressInfo ) :
AppKeyBlogJob( blog,
               "blogger.getRecentPosts",
               convertToList( count ),
               showProgressInfo ),
d( new Private( blog ) )
{
}

Blogger::Io::GetRecentPostsJob::~GetRecentPostsJob( void )
{
	delete d;
	d = 0;
}

QList< Blogger::Entry * > Blogger::Io::GetRecentPostsJob::entries( void ) const
{
	if( responseType() != ResponseSuccess
	    || d->builtList ) {
		return d->entries;
	}

	QVariant response = this->response();
	
	if( response.type() != QVariant::List ) {
		return d->entries;
	}

	QList<QVariant> variantList = response.toList();
	for( QList<QVariant>::ConstIterator it = variantList.begin();
	     it != variantList.end();
	     ++it )
	{
		if( (*it).type() != QVariant::Map ) {
			return QList< Blogger::Entry * >();
		}

		QMap<QString, QVariant> map = (*it).toMap();

		Entry * entry = new Entry( d->blog, map["postid"].toString() );
		entry->setDate( map["dateCreated"].toDateTime() );
		entry->setText( map["content"].toString() );

		d->entries.append( entry );
	}

	d->builtList = TRUE;
	return d->entries;
}

