/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getusersblogsjob.h"
#include "getusersblogsjob.moc"

#include <QVariant>
#include <QStringList>

class Blogger::Io::GetUsersBlogsJob::Private {
public:
	Private( void ) :
	listsBuilt( FALSE ),
	nameMap(QMap<QString, QString>() ),
	urlMap( QMap<QString, QString>() ) {}

	bool listsBuilt;

	QStringList blogIds;
	QMap<QString, QString> nameMap;
	QMap<QString, QString> urlMap;
};

Blogger::Io::GetUsersBlogsJob::GetUsersBlogsJob(Blogger::Account* account, bool showProgressInfo) :
AppKeyAccountJob( account,
                  "blogger.getUsersBlogs",
                  QList< QVariant >(),
                  showProgressInfo ),
d( new Private() )
{
}


Blogger::Io::GetUsersBlogsJob::~GetUsersBlogsJob()
{
	delete d;
	d = 0;
}

QStringList Blogger::Io::GetUsersBlogsJob::blogIds( void ) const
{
	if( responseType() != ResponseSuccess
	    || d->listsBuilt ) {
		return d->blogIds;
	}

	
	QVariant response = this->response();
	
	if( response.type() != QVariant::List ) {
		return d->blogIds;
	}

	QList<QVariant> variantList = response.toList();
	for( QList<QVariant>::ConstIterator it = variantList.begin();
	     it != variantList.end();
	     ++it )
	{
		if( (*it).type() != QVariant::Map ) {
			return QStringList();
		}

		QMap<QString, QVariant> map = (*it).toMap();

		d->blogIds.append( map["blogid"].toString() );
		d->nameMap.insert( map["blogid"].toString(),map["blogName"].toString() );
		d->urlMap.insert( map["blogid"].toString(),map["url"].toString() );
	}

	d->listsBuilt = TRUE;
	return d->blogIds;
}

QString Blogger::Io::GetUsersBlogsJob::blogName( const QString & id ) const
{
	if( d->nameMap.contains( id ) ){
		return d->nameMap[id];
	}

	return QString::null;
}

QString Blogger::Io::GetUsersBlogsJob::blogUrl( const QString & id ) const
{
	if( d->urlMap.contains( id ) ){
		return d->urlMap[id];
	}

	return QString::null;
}
