/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOGGER_IOGETUSERSBLOGSJOB_H
#define BLOGGER_IOGETUSERSBLOGSJOB_H

#include "appkeyaccountjob.h"

namespace Blogger {

namespace Io {

/**
 * @author Martin Mueller <orvio@orvio.de>
*/
class BLOGGER_EXPORT GetUsersBlogsJob : public Blogger::Io::AppKeyAccountJob
{
Q_OBJECT
public:
	/**
	 * Creates a new GetUsersBlogsJob
	 * @param account use authentication info from this account
	 * @param showProgressInfo TRUE to show progress info
	 */
    GetUsersBlogsJob( Blogger::Account * account, bool showProgressInfo = FALSE );

	/**
	 * Destructor
	 */
	virtual ~GetUsersBlogsJob( void );

	/**
	 * Returns the list of the returned blog ids
	 * @return the list of the returned blog ids
	 */
	QStringList blogIds( void ) const;

	/**
	 * Returns the name of the blog with @a id
	 * You must call blogIds() before calling this method.
	 * @return the name of the blog with @a id
	 */
	QString blogName( const QString & id ) const;

	/**
	 * Returns the url of the blog with @a id
	 * You must call blogIds() before calling this method.
	 * @return the url of the blog with @a id
	 */
	QString blogUrl( const QString & id ) const;

private:
	class Private;
	Private * d;
};

}

}

#endif
