/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef BLOGGER_IOSIMPLEPOSTJOB_H
#define BLOGGER_IOSIMPLEPOSTJOB_H

#include "simpleaccountjob.h"

namespace Blogger {
class Entry;
namespace Io {

/**
	@author Martin Mueller <orvio@orvio.de>
*/
class BLOKKAL_EXPORT SimplePostJob : public Blogger::Io::SimpleAccountJob
{
Q_OBJECT
public:
	/**
	 * Creates a new SimplePostJob
	 * @param entry use information from this entry
	 * @param methodName call this method
	 * @param params parameters
	 * @param workArounds enabled workarounds
	 * @param showProgressInfo TRUE to show progress info
	 */
	SimplePostJob( Blogger::Entry * entry,
	               const QString & methodName,
	               const QList<QVariant> & params,
	               unsigned int workArounds = 0,
	               bool showProgressInfo = FALSE );

	/**
	 * Creates a new SimplePostJob.
	 * Provided for subclasses, that need to add parameters
	 * in front of the standard arguments.
	 * @param entry use information from this entry
	 * @param methodName call this method
	 * @param params parameters
	 * @param leadingParams leading parameters
	 * @param workArounds enabled workarounds
	 * @param showProgressInfo TRUE to show progress info
	 */	
	SimplePostJob( Blogger::Entry * entry,
	               const QString & methodName,
	               const QList<QVariant> & params,
	               const QList<QVariant> & leadingParams,
	               unsigned int workArounds = 0,
	               bool showProgressInfo = FALSE );
	
	/**
	 * Destructor
	 */
	virtual ~SimplePostJob( void );
	
private:
	class Private;
	Private * d;
};

}

}

#endif
