/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gaccount.h"
#include "gaccount.moc"

#include "gprotocol.h"
#include "gblog.h"
#include "gio/jobs.h"

#include <blokkalblogconfig.h>
#include <blokkalaccountconfig.h>

#include <kdebug.h>
#include <klocale.h>
#include <syndication/atom/entry.h>

#include <QTimer>
#include <QRegExp>

class G::Account::Private {
public:
	Private( void ):
	getBlogsJob( 0 ) {}

	G::Io::GetBlogsJob * getBlogsJob;
	QByteArray auth;
};

G::Account::Account( G::Protocol * protocol, const QString & id ) :
Blokkal::PasswordedAccount( protocol, id ),
d( new Private() )
{
}

G::Account::~Account( void )
{
	delete d;
}

void G::Account::connectInner( void )
{
	if( d->getBlogsJob ) {
		d->getBlogsJob->kill();
	}
	
	d->getBlogsJob = new Io::GetBlogsJob( this );
	connect( d->getBlogsJob,
	         SIGNAL( result( KJob * ) ),
	         SLOT( slotgetBlogsJobFinished( KJob * ) ) );
	d->getBlogsJob->start();
}

Blokkal::Blog * G::Account::restoreBlog( Blokkal::BlogConfig * config )
{
	if( !config ) {
		return 0;
	}

	Blog * blog = new Blog( this,
	                        config->id(),
	                        config->readEntry( "url" ),
	                        config->title(),
	                        config->readEntry( "postUrl" ) );
	return blog;
}

void G::Account::slotgetBlogsJobFinished( KJob * job )
{
	if( !job || job != d->getBlogsJob ) {
		return;
	}

	if( d->getBlogsJob->error() ) {
		if( d->getBlogsJob->error() == Blokkal::Io::AuthenticationError ) {
			disconnectAccount( AuthenticationDisconnect, d->getBlogsJob->errorString() );
		}
		else if( d->getBlogsJob->error() > 0 ) {
			disconnectAccount( IOErrorDisconnect );
		}
		else {
			disconnectAccount( OtherDisconnect, d->getBlogsJob->errorString() );
		}
		d->getBlogsJob = 0;
		return;
	}

	//no errors
	QList<Syndication::Atom::Entry> blogInfoList = d->getBlogsJob->blogList();
	d->getBlogsJob = 0;

	//build idlist
	QStringList idList;
	QRegExp idExp( "blog-\\d+" );
	for( int i = 0; i < blogInfoList.size(); ++i ) {
		int idIndex = idExp.indexIn( blogInfoList[i].id() );
		if( idIndex != -1 ) {
			idList.append( blogInfoList[i].id().mid( idIndex + 5, idExp.matchedLength() - 5 ) );
		}
		else {
			kError() << "encoundered malformed id" << endl;
			blogInfoList.removeAt( i );
			i--;
		}
	}
	
	//remove old blogs
	QList<Blokkal::Blog *> blogList = blogs();
	for( QList<Blokkal::Blog *>::ConstIterator it = blogList.begin();
	     it != blogList.end();
	     ++it )
	{
		if( !idList.contains( (*it)->id() ) ) {
			( *it )->removeBlog();
		}
	}

	//create/update blogs
	Blokkal::Blog * blog;
	for( int i = 0; i < blogInfoList.size(); ++i ) {
		blog = PasswordedAccount::blog( idList[i] );
		if( blog ) {
			//update
			G::Blog * const gblog = dynamic_cast<G::Blog*>(blog);
			if( !gblog ) {
				kError() << "found invalid blog!" << endl;
				continue;
			}

			gblog->update( blogInfoList[i] );
		}
		else {
			//create
			blog = new G::Blog( this, idList[i], blogInfoList[i] );
			registerBlog( blog );
		}
	}

	setConnectionStatus( Connected );
}

void G::Account::disconnectAccount( Blokkal::Account::DisconnectReason reason,
                                    const QString & reasonString )
{
	if( d->getBlogsJob ) {
		d->getBlogsJob->kill();
		d->getBlogsJob = 0;
	}
	d->auth = QByteArray();
	Blokkal::PasswordedAccount::disconnectAccount( reason, reasonString );
}

void G::Account::updateAuth( const QByteArray & auth )
{
	d->auth = auth;
}

const QByteArray & G::Account::auth( void ) const
{
	return d->auth;
}

const QString & G::Account::password( void ) const
{
	return Blokkal::PasswordedAccount::password();
}

void G::Account::setProvider( const QString & provider )
{
	config()->writeEntry( "provider", provider );	
}

QString G::Account::provider( void ) const
{
	return config()->readEntry( "provider", QString::null );	
}
