/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GACCOUNT_H
#define GACCOUNT_H

#include <blokkalaccount.h>

class KJob;
namespace G {
class Protocol;
/**
	@author Martin Mueller <orvio@orvio.de>
*/
class Account : public Blokkal::PasswordedAccount
{
Q_OBJECT
public:
	/**
	 * Creates a new account for protocol @a protocol, with id @a accountId
	 * @param protocol the procotol of this account
	 * @param accountId the name if this account
	 */
	Account( G::Protocol * protocol, const QString & accountId = 0 );

	/**
	 * Destructor
	 */
	virtual ~Account( void );

	const QByteArray & auth( void ) const;

	const QString & password( void ) const;

	/**
	 * Sets the provider id
	 * @param provider the new provider id
	 */
	void setProvider( const QString & provider );

	/**
	 * Returns the currently set provider id
	 * @return the provider id
	 */
	QString provider( void ) const;
	
public Q_SLOTS:
	/**
	 * This method is called to close the current session.
	 * Overridden to kill any remaining jobs from last login attempt.
	 * @param reason the reaon for the disconnect
	 * @param reasonString to use instead of a generic error message
	 */
	virtual void disconnectAccount( Blokkal::Account::DisconnectReason reason = UserDisconnect,
	                                const QString & reasonString = 0 );	
	
protected:
	/**
	 * Restores the blog according to the information in @a config.
	 * @param config restore the blog with this configuration
	 * @return the restored blog or 0 if the blog was not restored
	 * @see Blokkal::Blog::saveProperties()
	 */
	virtual Blokkal::Blog * restoreBlog( Blokkal::BlogConfig * config );
	
protected Q_SLOTS:
	/**
	 * Connects to this account
	 */
	virtual void connectInner( void );

private Q_SLOTS:
	void slotgetBlogsJobFinished( KJob * job );
	//called by login jobs
	void updateAuth( const QByteArray & auth );
	
private:
	class Private;
	Private * const d;
};

}

#endif
