/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "gblog.h"
#include "gblog.moc"

#include "gentry.h"
#include "gaccount.h"
#include "gio/blogbrowsingdriver.h"

#include <blokkalblogconfig.h>

#include <syndication/atom/entry.h>
#include <syndication/atom/link.h>
#include <kdebug.h>

class G::Blog::Private {
public:
	Private( void ) :
	driver( 0 ){}

	G::Io::BlogBrowsingDriver * driver;
};

G::Blog::Blog( G::Account * parentAccount,
               const QString & id,
               const Syndication::Atom::Entry & data  ) :
Blokkal::Blog( parentAccount, id ),
d( new Private() )
{
	update( data );
	d->driver = new G::Io::BlogBrowsingDriver( this );
}

G::Blog::Blog( G::Account * parentAccount,
               const QString & id,
               const QString & url,
               const QString & title,
               const QString & postUrl ) :
Blokkal::Blog( parentAccount, id ),
d( new Private() )
{
	config()->writeEntry( "url", url );
	setTitle( title );
	config()->writeEntry( "postUrl", postUrl );
	d->driver = new G::Io::BlogBrowsingDriver( this );
}

G::Blog::~Blog( void )
{}

void G::Blog::update( const Syndication::Atom::Entry & data )
{
	QList< Syndication::Atom::Link > links = data.links();
	for( int i = 0; i < links.size(); ++i ) {
		if( links[i].rel() == "alternate" ) {
			config()->writeEntry( "url", links[i].href() );
		}
		else if( links[i].rel() == "http://schemas.google.com/g/2005#post" ) {
			config()->writeEntry( "postUrl", links[i].href() );
		}
	}
	setTitle( data.title() );
}

Blokkal::Entry * G::Blog::createEntry( void )
{
	return new G::Entry( this );
}

unsigned int G::Blog::supportedFeatures( void ) const
{
	return FeatureDate|FeatureText|FeatureSubject;
}

Blokkal::Io::BlogBrowsingDriver * G::Blog::blogBrowsingDriver( void ) const
{
	return d->driver;
}

KUrl G::Blog::url( void ) const
{
	return config()->readEntry( "url" );
}

KUrl G::Blog::postUrl( void ) const
{
	return config()->readEntry( "postUrl" );
}
