/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GBLOG_H
#define GBLOG_H

#include <blokkalblog.h>

namespace Syndication {
namespace Atom {
class Entry;
}
}

namespace G {
class Account;
/**
 * @short Representation of a MovableType blog
 *
 * Representation of a MovableType blog
 * @author Martin Mueller <orvio@orvio.de>
*/
class Blog : public Blokkal::Blog
{
Q_OBJECT
public:
	/**
	 * This creates a new Blog.
	 * @param parentAccount the account this blog belongs to
	 * @param id the blog id
	 * @param data data to configure this object from
	 */
	Blog( G::Account * parentAccount, const QString & id, const Syndication::Atom::Entry & data );

	/**
	 * Destructor
	 */
	virtual ~Blog( void );

	/**
	 * Creates a new @ref Blokkal::Entry object.
	 * @return a new Blokkal::Entry object.
	 */
	virtual Blokkal::Entry * createEntry( void );

	/**
	 * @return the BlogBrowsingDriver of this blog
	 */
	virtual Blokkal::Io::BlogBrowsingDriver * blogBrowsingDriver( void ) const;

	/**
	 * Returns an or'ed list of Blokkal::Blog::EntryFeatures values to indicate
	 * the possible options of entry editing.
	 * @return an or'ed list of Blokkal::Blog::EntryFeatures values
	 */
	virtual unsigned int supportedFeatures( void ) const;

	/**
	 * This method returns the url of the blog.
	 * @return the url of the blog.
	 */
	virtual KUrl url( void ) const;

	void update( const Syndication::Atom::Entry & data );

	KUrl postUrl( void ) const;

private:
	/**
	 * This constructor is used by G::Account to restore blogs
	 */
	Blog( G::Account * parentAccount,
	      const QString & id,
	      const QString &url,
	      const QString & title,
	      const QString & postUrl );
	
	friend class Account;
	class Private;
	Private * const d;
};

}

#endif
