/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GENTRY_H
#define GENTRY_H

#include <blokkalentry.h>

class QDomElement;

namespace G {
class Blog;
/**
	@author Martin Mueller <orvio@orvio.de>
*/
class Entry : public Blokkal::Entry
{
Q_OBJECT
public:
	/**
	 * Creates a new Entry.
	 * @param parentBlog the blog the entry belongs to
	 */
	Entry( G::Blog * parentBlog );

	/**
	 * Creates a new entry from a XmlRpc Response map.
	 * @param parentBlog the blog the entry belongs to
	 * @param entryElement an atom entry xml element
	 */
	Entry( G::Blog * parentBlog, const QDomElement & entryElement );

	/**
	 * Destructor
	 */
	virtual ~Entry( void );
	
	/**
	 * Returns the widget associated with @a extension.
	 * @param extension the identifier for the widget to create
	 * @param parent the parent widget
	 * @return the widget associated with @a extension.
	 */
	virtual Blokkal::Ui::EntryExtensionWidget * createExtensionWidget( Blokkal::Ui::EntryExtensionWidget::ExtensionWidget extension, Blokkal::Ui::EditEntryWidget * parent );
	
	/**
	 * Creates and returns a job of @a type
	 * @return a job of @a type
	 */
	virtual Blokkal::Io::EntryJob * createJob( Blokkal::Entry::JobType type );

	/**
	 * Returns this entry as atom entry xml
	 * @return this entry as atom entry xml
	 */
	QByteArray toXmlByteArray( void ) const;

	/**
	 * @return the date() of this entry as RFC3339 timestamp
	 */
	QString atomTimeStamp( void ) const;

	/**
	 * Returns the url for editing this entry.
	 * Thie method returns KUrl() if this entry
	 * has not been loaded from the server.
	 * @return the url for editing this entry
	 */
	KUrl editUrl( void ) const;
	
	bool isDraft( void ) const;

	void setDraft( bool draft );

	/**
	 * This method returns the icon of the entry.
	 * @return the entry icon
	 */
	virtual KIcon icon( void ) const;

protected:
	/**
	 * This method is called right before this entry is saved to disk.
	 * Reimplemented to save the atom xml data.
	 */
	virtual void saveProperties( void );

	/**
	 * This method is called right after the entry has been
	 * Reimplemented to restore the atom xml data.
	 */
	virtual void readProperties( void );

private:
	class Private;
	Private * const d;
};

}

#endif
