/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blogbrowsingdriver.h"
#include "blogbrowsingdriver.moc"

#include "jobs.h"
#include "../gblog.h"
#include "../gui/browseoptionwidget.h"

#include <kdebug.h>

class G::Io::BlogBrowsingDriver::Private {
public:
	Private( G::Blog * parent ) :
	blog( parent ) {}

	G::Blog * const blog;
};

G::Io::BlogBrowsingDriver::BlogBrowsingDriver( G::Blog * parent ):
Blokkal::Io::BlogBrowsingDriver( parent ),
d( new Private( parent) )
{
}

G::Io::BlogBrowsingDriver::~BlogBrowsingDriver( void )
{
	delete d;
}

unsigned int G::Io::BlogBrowsingDriver::supportedOperations( void ) const
{
	return LoadEntryById|LoadRecentEntries;
}

Blokkal::Io::LoadEntryJob * G::Io::BlogBrowsingDriver::createLoadEntryJob( const QStringList & idList )
{
	G::Io::LoadEntryJob * job = new G::Io::LoadEntryJob( blog() );
	for( QStringList::ConstIterator it = idList.begin();
	     it != idList.end();
	     ++it )
	{
		job->addSubjob( new GetPostsJob( static_cast<G::Blog*>( blog() ), *it ) );
	}
	return job;	
}

Blokkal::Io::LoadEntryJob * G::Io::BlogBrowsingDriver::createLoadEntryJob( unsigned int n )
{
	G::Io::LoadEntryJob * job = new G::Io::LoadEntryJob( blog() );
	job->addSubjob( new GetPostsJob( static_cast<G::Blog*>( blog() ), n ) );
	return job;
}

Blokkal::Io::LoadEntryJob * G::Io::BlogBrowsingDriver::createLoadEntryJob( const QString & query, G::Io::GetPostsJob::QueryType type )
{
	G::Io::LoadEntryJob * job = new G::Io::LoadEntryJob( blog() );
	job->addSubjob( new GetPostsJob( static_cast<G::Blog*>( blog() ), query, type ) );
	return job;	
}

QList< Blokkal::Ui::BrowseOptionWidget * > G::Io::BlogBrowsingDriver::customBrowseOptionWidgets( QWidget * /*parent*/ )
{
	QList< Blokkal::Ui::BrowseOptionWidget * > list;
	//list.append( new G::Ui::LoadQueryWidget( this, parent ) );
	return list;
}

//------------
//LoadEntryJob
//------------
class G::Io::LoadEntryJob::Private {
public:
	Private( void ) {}

	QList<Blokkal::Entry *> entryList;
};

G::Io::LoadEntryJob::LoadEntryJob( Blokkal::Blog * blog ):
Blokkal::Io::LoadEntryJob( blog ),
d( new Private() )
{
}

G::Io::LoadEntryJob::~LoadEntryJob( void )
{
	delete d;
}

QList< Blokkal::Entry * > G::Io::LoadEntryJob::entries( void ) const
{
	return d->entryList;
}

void G::Io::LoadEntryJob::subjobFinished( KJob * job )
{
	if( job->error() ) {
		return;
	}

	const G::Io::GetPostsJob * const getPostsJob = dynamic_cast<G::Io::GetPostsJob *>( job );
	if( getPostsJob ) {
		d->entryList = getPostsJob->entryList();
		return;
	}

	kError() << "invalid subjob encountered" << endl;	
}
