/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef G_IOBLOGBROWSINGDRIVER_H
#define G_IOBLOGBROWSINGDRIVER_H

#include <blokkalio/blogbrowsingdriver.h>
#include "jobs.h"

namespace G {
class Blog;
namespace Io {
/**
 * @short Driver for GData blogs
 *
 * Driver for GData blogs
 */
class BlogBrowsingDriver : public Blokkal::Io::BlogBrowsingDriver {
Q_OBJECT
public:
	using Blokkal::Io::BlogBrowsingDriver::createLoadEntryJob;
	/**
	 * Constructor
	 * @param parent parent blog
	 */
	BlogBrowsingDriver( G::Blog * parent );
	virtual ~BlogBrowsingDriver( void );

	/**
	 * This method returns an or'ed list of BlogOperations values.
	 * @return returns an or'ed list of BlogOperations values
	 */
	virtual unsigned int supportedOperations( void ) const;

	/**
	 * Creates and returns a job that loads all entries in @a idList.
	 * The default implementation returns 0.
	 * @return a job that loads all entries in @a idList
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( const QStringList & idList );

	/**
	 * Creates and returns a job that loads the last @a n entries in the blog.
	 * The default implementation returns 0.
	 * @see recentLimit()
	 * @return a job that loads the last @a n entries in the blog
	 */
	virtual Blokkal::Io::LoadEntryJob * createLoadEntryJob( unsigned int n );

	/**
	 * Creates and retusn a job that loads the search results for @a query
	 * @return a job that load the query results
	 */
	Blokkal::Io::LoadEntryJob * createLoadEntryJob( const QString & query,
	                                                G::Io::GetPostsJob::QueryType type );

	/**
	 * Returns a widget for searching the blog
	 * @param parent parent widget for the BrowseOptionWidget
	 */
	virtual QList<Blokkal::Ui::BrowseOptionWidget*> customBrowseOptionWidgets( QWidget * parent );	

private:
	class Private;
	Private * const d;
};

class LoadEntryJob : public Blokkal::Io::LoadEntryJob {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param blog parent blog
	 */
	LoadEntryJob( Blokkal::Blog * blog );
	virtual ~LoadEntryJob( void );
	
	/**
	 * @return the loaded entries
	 */
	QList<Blokkal::Entry *> entries( void ) const;
	
protected:
	/**
	 * This method is called when a subjob has finished.
	 * Overridden to get the entry.
	 */
	virtual void subjobFinished( KJob * job );

private:
	class Private;
	Private * const d;
};

}

}
#endif
