/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "loginjob.h"
#include "loginjob.moc"

#include <config.h>

#include "../gaccount.h"

#include <kdebug.h>
#include <kcodecs.h>
#include <kio/job.h>
#include <klocale.h>

#include <QVariant>

class G::Io::LoginJob::Private {
public:
	Private( void ) :
	error( NoError ){}

	ErrorCode error;

	QByteArray responseBuffer;
	QMap<QByteArray,QByteArray> keyValueMap;
};

G::Io::LoginJob::LoginJob( G::Account * account ):
Blokkal::Io::Job( account ),
d( new Private() )
{
	QByteArray postData = "Email=" + QUrl::toPercentEncoding(  account->userName() );
	postData += "&Passwd=" + QUrl::toPercentEncoding( account->password() );
	postData += "&source=KDE-Blokkal-";
	postData += VERSION;
	postData += "&service=blogger";
	KIO::TransferJob * job = KIO::http_post( KUrl( "https://www.google.com/accounts/ClientLogin" ),
	                                         postData,
	                                         KIO::HideProgressInfo );
	job->addMetaData( "UserAgent", QString::fromLatin1( "KDE-Blokkal/%1 GData Plugin").arg( VERSION ) );
	job->addMetaData( "content-type", "Content-Type: application/x-www-form-urlencoded" );
	job->addMetaData( "ConnectTimeout", "300" );
	connect( job,
	         SIGNAL( data( KIO::Job *, const QByteArray & ) ),
	         SLOT( slotDataArrived( KIO::Job *, const QByteArray & ) ) );
	account->connect( this,
	                  SIGNAL( authReceived( const QByteArray & ) ),
	                  SLOT( updateAuth( const QByteArray & ) ) );
	addSubjob( job );
}

G::Io::LoginJob::~LoginJob( void )
{
	delete d;
}

void G::Io::LoginJob::slotDataArrived( KIO::Job * job, const QByteArray & data )
{
	Q_UNUSED( job );
	d->responseBuffer.append( data );
}

void G::Io::LoginJob::jobFinished( void )
{
	//build property map
	int index = 0;
	int oldIndex = 0;
	while( ( index = d->responseBuffer.indexOf( '\n', oldIndex ) ) > -1 )
	{
		const QList<QByteArray> keyValue = d->responseBuffer.mid( oldIndex, index-oldIndex ).split( '=' );
		if( keyValue.size() != 2 ) {
			kDebug() << "invalid data line ecountered: " << d->responseBuffer.mid( oldIndex, index-oldIndex ) << endl;
		}
		else {
			d->keyValueMap[ keyValue[0].trimmed() ] = keyValue[1].trimmed();
		}
		++index;
		oldIndex = index;
	}

	//kDebug() << "valuemap: " << d->keyValueMap << endl;

	const QByteArray errorKey = "Error";
	if( d->keyValueMap.contains( errorKey ) ) {
		setError( Blokkal::Io::AuthenticationError );
		if( d->keyValueMap[errorKey] == "BadAuthentication" ) {
			d->error = BadAuthentication;
			setErrorText( i18n( "The login request used a username or password that is not recognized." ) );
		}
		else if( d->keyValueMap[errorKey] == "NotVerified" ) {
			d->error = NotVerified;
			setErrorText( i18n( "The account email address has not been verified. You will need to access your Google account directly to resolve the issue before logging in using a non-Google application." ) );
		}
		else if( d->keyValueMap[errorKey] == "TermsNotAgreed" ) {
			d->error = TermsNotAgreed;
			setErrorText( i18n( "You have not agreed to terms. You will need to access your Google account directly to resolve the issue before logging in using a non-Google application." ) );
		}
		else if( d->keyValueMap[errorKey] == "CaptchaRequired" ) {
			d->error = CaptchaRequired;
			setErrorText( i18n( "The server requires you to solve a CAPTCHA." ) );
		}
		else if( d->keyValueMap[errorKey] == "Unknown" ) {
			d->error = Unknown;
			setErrorText( i18n( "Unknown Error" ) );
		}
		else if( d->keyValueMap[errorKey] == "AccountDeleted" ) {
			d->error = AccountDeleted;
			setErrorText( i18n( "The account has been deleted." ) );
		}
		else if( d->keyValueMap[errorKey] == "AccountDisabled" ) {
			d->error = AccountDisabled;
			setErrorText( i18n( "The account has been disabled." ) );
		}
		else if( d->keyValueMap[errorKey] == "ServiceDisabled" ) {
			d->error = ServiceDisabled;
			setErrorText( i18n( "Your access to the specified service has been disabled." ) );
		}
		else if( d->keyValueMap[errorKey] == "ServiceUnavailable" ) {
			d->error = ServiceUnavailable;
			setErrorText( i18n( "The service is not available; try again later." ) );
		}
	}
	else if( !d->keyValueMap.contains("Auth") ) {
		d->error = Unknown;
		setError( Blokkal::Io::MalformedReplyError );
		setErrorText( i18n( "Could not read reply received from server" ) );
	}

	emit authReceived( d->keyValueMap["Auth"] );
}

G::Io::LoginJob::ErrorCode G::Io::LoginJob::errorCode( void ) const
{
	return d->error;
}

const QByteArray & G::Io::LoginJob::authToken( void ) const
{
	return d->keyValueMap["Auth"];
}
