/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef G_IOLOGINJOB_H
#define G_IOLOGINJOB_H

#include <blokkalio/jobs.h>

namespace KIO {
class Job;
}

namespace G {
class Account;
namespace Io {

/**
 * @short This job handles the login
 *
 * This job handles the login.
 * @author Martin Mueller <orvio@orvio.de>
*/
class LoginJob : public Blokkal::Io::Job
{
Q_OBJECT
public:
	/**
	 * Creates a new LoginJob.
	 * @param account connect to the address provided by this account
	 * @param password use this password
	 */
	LoginJob( G::Account * account );
	
	/**
	 * Destructor
	 */
	virtual ~LoginJob( void );

	/**
	 * Enumeration for GData authentication error codes.
	 * For more information see
	 * http://code.google.com/apis/accounts/AuthForInstalledApps.html
	 */
	enum ErrorCode {
		NoError = 0, ///< No error
		BadAuthentication = 1, ///< The login request used a username or password that is not recognized
		NotVerified = 2, ///< The account email address has not been verified
		TermsNotAgreed = 3, ///< The user has not agreed to terms
		CaptchaRequired = 4, ///< A CAPTCHA is required
		Unknown = 5, ///< The error is unknown or unspecified
		AccountDeleted = 6, ///< The user account has been deleted
		AccountDisabled = 7, ///< The user account has been disabled
		ServiceDisabled = 8, ///< The user's access to the specified service has been disabled
		ServiceUnavailable = 9 ///< The service is not available; try again later
	};
	
	/**
	 * @return the error code
	 */
	G::Io::LoginJob::ErrorCode errorCode( void ) const;

	/**
	 * The value returned by this method is
	 * only meaningful if no error occured.
	 * @return the auth token
	 */
	const QByteArray & authToken( void ) const;

Q_SIGNALS:
	void authReceived( const QByteArray & auth );
	
protected:
	/**
	 * This method is called when all subjobs have finished
	 * and the result() signal is about to be emitted.
	 * Override this method in subclasses if you need to do special things after
	 * the job has finished.
	 */
	virtual void jobFinished( void );
	
private Q_SLOTS:
	void slotDataArrived( KIO::Job *job, const QByteArray & data );

private:
	class Private;
	Private * const d;
};

}

}

#endif
