/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "browseoptionwidget.h"
#include "browseoptionwidget.moc"

#include "../gio/blogbrowsingdriver.h"

#include <kcombobox.h>
#include <klineedit.h>
#include <klocale.h>

#include <QLabel>
#include <QLayout>

class G::Ui::LoadQueryWidget::Private {
public:
	Private( G::Io::BlogBrowsingDriver * driver ) :
	driver( driver ) {}

	G::Io::BlogBrowsingDriver * const driver;

	KComboBox * comboBox;
	KLineEdit * lineEdit;
};

G::Ui::LoadQueryWidget::LoadQueryWidget( G::Io::BlogBrowsingDriver * driver, QWidget * parent ):
Blokkal::Ui::BrowseOptionWidget( driver, parent ),
d( new Private( driver ) )
{
	QLabel * label = new QLabel( i18n( "Search" ), this );
	layout()->addWidget( label );

	d->comboBox = new KComboBox( this );
	d->comboBox->addItem( i18n( "Full-text" ) );
	d->comboBox->addItem( i18n( "For author" ) );
	layout()->addWidget( d->comboBox );

	d->lineEdit = new KLineEdit( this );
	d->lineEdit->setTrapReturnKey( TRUE );
	connect( d->lineEdit,
	         SIGNAL( returnPressed( void ) ),
	         SLOT( emitReloadRequested( void ) ) );
	layout()->addWidget( d->lineEdit );
}

G::Ui::LoadQueryWidget::~LoadQueryWidget( void )
{
	delete d;
}

Blokkal::Io::LoadEntryJob * G::Ui::LoadQueryWidget::createLoadEntryJob( void )
{
	const QString query = d->lineEdit->text();
	if( query.isEmpty() ) {
		return 0 ;
	}
	
	Io::GetPostsJob::QueryType type = Io::GetPostsJob::QueryText;
	if( d->comboBox->currentIndex() == 1 ) {
		type = Io::GetPostsJob::QueryAuthor;
	}

	return d->driver->createLoadEntryJob( query, type );
}
