/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "editaccountwidget.h"
#include "editaccountwidget.moc"

#include "../gprotocol.h"
#include "../gaccount.h"

#include <blokkalui/providercombobox.h>

#include <klineedit.h>
#include <kdebug.h>
#include <klocale.h>
#include <kicon.h>

#include <QLayout>
#include <QGroupBox>
#include <QCheckBox>

class G::Ui::EditAccountWidget::Private {
public:
	Private( void ) {}

	Blokkal::Ui::ProviderComboBox * providerComboBox;
	QString currentProvider;
};

G::Ui::EditAccountWidget::EditAccountWidget( G::Protocol * protocol,
                                             Blokkal::Account * account,
                                             QWidget * parent ) :
Blokkal::Ui::GenericEditAccountWidget( protocol, account, parent ),
d( new Private() )
{
	G::Account * gaccount = dynamic_cast< G::Account* >( this->account() );
	if( account && !gaccount ) {
		kWarning() << " account is not a gdata account!" << endl;
		return;
	}

	QHBoxLayout * providerLayout = new QHBoxLayout( 0 );
	providerLayout->setMargin( 0 );
	d->providerComboBox = new Blokkal::Ui::ProviderComboBox( this );
	Blokkal::Ui::ProviderList providerList;
	providerList.append( Blokkal::Ui::ProviderInformation( "blogspot.com", i18n( "Blogspot.com" ), KIcon( "blogger" ) ) );
	d->providerComboBox->setProviders( providerList );
	providerLayout->addWidget( d->providerComboBox );
	providerLayout->addStretch( 1 );

	((QBoxLayout*)networkGroupBox()->layout())->insertLayout( 0, providerLayout );

	if( account ) { //use account provider settings if available
		d->providerComboBox->setCurrentProvider( gaccount->provider() );
		configureProvider( gaccount->provider() );
	}
	else { //set provider to blogspots as default
		d->providerComboBox->setCurrentProvider( "blogspot.com" );
		configureProvider( "blogspot.com" );
	}

	connect( d->providerComboBox,
	         SIGNAL( providerSelected( const QString & ) ),
	         SLOT( configureProvider( const QString & ) ) );	
}


G::Ui::EditAccountWidget::~EditAccountWidget( void )
{
	delete d;
}

void G::Ui::EditAccountWidget::setServerAddress( const QString & serverName )
{
	if( useStandardAddressCheckBox()->isChecked() ) {
		if( serverName.isEmpty() ) {
			connectAddressLineEdit()->setText( 0 );
		}
		else if( d->currentProvider == "blogspot.com" ) {
			connectAddressLineEdit()->setText( QString::fromLatin1( "http://www.%1/feeds/default/blogs").arg( serverName ) );
		}
	}
}

void G::Ui::EditAccountWidget::configureProvider( const QString & providerId )
{
	d->currentProvider = providerId;
	if( providerId == "blogspot.com" ) {
		useStandardAddressCheckBox()->setChecked( TRUE );
		useStandardAddressCheckBox()->setEnabled( TRUE );
		serverNameLineEdit()->setEnabled( FALSE );
		serverNameLineEdit()->setText( "blogger.com" );
	}
	else if( providerId.isNull() ) {
		useStandardAddressCheckBox()->setChecked( FALSE );
		useStandardAddressCheckBox()->setEnabled( FALSE );
		serverNameLineEdit()->setEnabled( TRUE );
	}
	else {
		kWarning() << "unknown provider id: " << providerId << endl;
	}	
}

Blokkal::Account * G::Ui::EditAccountWidget::apply( void )
{
	Blokkal::Account * account = GenericEditAccountWidget::apply();
	G::Account * gaccount = dynamic_cast<G::Account*>( account );
	if( !gaccount ) {
		kWarning() << "not editing a GData account!" << endl;
		return account;
	}
		
	gaccount->setProvider( d->currentProvider );
	return account;
}
