/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "livejournalblog.h"
#include "livejournalblog.moc"

#include <blokkalblogconfig.h>

#include "livejournalaccount.h"
#include "ljentry.h"
#include "ljio/userpicturemanager.h"
#include "ljio/getusertagsjob.h"
#include "ljio/blogbrowsingdriver.h"

#include <klocale.h>
#include <kicon.h>

class LiveJournalBlog::Private
{
public:
	Private( void ) :
	getTagsJob( 0 ) {}

	KIcon icon;
	LiveJournalBlog::JournalType type;

	KUrl url;

	LJ::IO::GetUserTagsJob * getTagsJob;
	LJ::IO::BlogBrowsingDriver * driver;
};

LiveJournalBlog::LiveJournalBlog( LiveJournalAccount * parentAccount,
                                  const QString & internalName,
                                  LiveJournalBlog::JournalType type,
                                  const QString & blogName ) :
Blokkal::Blog( parentAccount, internalName ),
d( new Private() )
{
	d->type = type;
	d->driver = new LJ::IO::BlogBrowsingDriver( this );
	switch( d->type ) {
	case UserJournal:
		setTitle( blogName );
		if( parentAccount->userPictureManager()->defaultPicture() ) {
			updateUserPicture( parentAccount->userPictureManager()->defaultPicture() );
		}
		else {
			removeUserPicture();
		}
		connect( parentAccount->userPictureManager(),
		         SIGNAL( pictureAdded( LJ::IO::UserPicture* ) ),
		         SLOT( updateUserPicture( LJ::IO::UserPicture* ) ) );
		connect( parentAccount->userPictureManager(),
		         SIGNAL( pictureRemoved( LJ::IO::UserPicture* ) ),
		         SLOT( removeUserPicture( void ) ) );
		break;
	case CommunityJournal:
		setTitle( QString::null );
		d->icon = KIcon( "blokkal_lj_community" );
		break;
	}

	d->url = "http://" + internalName + '.' + parentAccount->serverName();

	connect( parentAccount,
	         SIGNAL( connectionStatusChanged( Blokkal::Account * ) ),
	         SLOT( slotAccountStatusChanged( void ) ) );
	slotAccountStatusChanged();
}

LiveJournalBlog::~LiveJournalBlog( void )
{
	delete d;
}

KIcon LiveJournalBlog::icon( void ) const
{
	return d->icon;
}

QString LiveJournalBlog::type( void ) const
{
	switch( d->type ) {
	case UserJournal:
		return i18n( "Journal" );
		break;
	case CommunityJournal:
		return i18n( "Community" );
		break;
	}

	return 0;
}

void LiveJournalBlog::setBlogName( const QString & blogName )
{
	if( d->type == UserJournal ) {
		setTitle( blogName );
	}
}

Blokkal::Entry * LiveJournalBlog::createEntry( void )
{
	return new LJ::Entry( this );
}

KUrl LiveJournalBlog::url( void ) const
{
	return d->url;
}

void LiveJournalBlog::slotAccountStatusChanged( void )
{
	if( account()->isConnected() ) {
		d->getTagsJob = new LJ::IO::GetUserTagsJob( this, this );
		connect( d->getTagsJob,
		         SIGNAL( result( KJob * ) ),
		         SLOT( slotGetTagsJobFinished( KJob * ) ) );		
		d->getTagsJob->start();
	}
	else {
		if( d->getTagsJob ) {
			d->getTagsJob->kill();
			d->getTagsJob = 0;
		}
	}
}

void LiveJournalBlog::slotGetTagsJobFinished( KJob * job )
{
	if( job && job == d->getTagsJob ) {
		if( job->error() > 0 ) {
			account()->disconnectAccount( Blokkal::Account::IOErrorDisconnect );
		}
		d->getTagsJob = 0;
	}
}

void LiveJournalBlog::saveProperties( void )
{
	Blokkal::Blog::saveProperties();
	config()->writeEntry( "type", d->type );
}

unsigned int LiveJournalBlog::supportedFeatures( void ) const
{
	return FeatureDate|FeatureSubject|FeatureText|FeatureMultipleCategories|FeatureAddCategories;
}

void LiveJournalBlog::updateUserPicture( LJ::IO::UserPicture * picture )
{
	if( !picture ) {
		return;
	}

	if( !picture->keyword().isNull() ) {
		return;
	}


	if( !picture->picture().isNull() ) {
		//d->icon = QPixmap::fromImage( picture->picture().scaled( 16, 16, Qt::KeepAspectRatio ) );
		d->icon = KIcon( QPixmap::fromImage( picture->picture() ) );
		connect( picture,
		         SIGNAL( pictureChanged( LJ::IO::UserPicture* ) ),
		         SLOT( updateUserPicture( LJ::IO::UserPicture* ) ) );
	}
	else {
		d->icon = KIcon( "blokkal_lj_user" );
	}
	
	emit propertiesChanged( this );
}

void LiveJournalBlog::removeUserPicture( void )
{
	d->icon = KIcon( "blokkal_lj_user" );
	emit propertiesChanged( this );
}

Blokkal::Io::BlogBrowsingDriver * LiveJournalBlog::blogBrowsingDriver( void ) const
{
	return d->driver;
}
