/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIVEJOURNALBLOG_H
#define LIVEJOURNALBLOG_H

#include <blokkalblog.h>

class LiveJournalAccount;
class KJob;

namespace LJ {
namespace IO {
class Job;
class UserPicture;
}
}
/**
 * @short This class represents a LiveJournal blog.
 *
 * This class represents a LiveJournal blog.
 * This can either be a
 * @author Martin Mller <orvio@orvio.de>
*/
class LiveJournalBlog : public Blokkal::Blog
{
Q_OBJECT
public:
	/**
	 * This enum is used to distinguish the journal types
	 */
	enum JournalType {
		UserJournal, ///< A normal user journal
		CommunityJournal  ///< A community journal
	};
	
	/**
	 * This creates a new LiveJournalBlog.
	 * @param parentAccount the account this blog belongs to
	 * @param internalName the user or community name
	 * @param type sets the type of this blog
	 * @param blogName the full name, only relevant for user journals
	 */
	LiveJournalBlog( LiveJournalAccount * parentAccount,
	                 const QString & internalName,
	                 LiveJournalBlog::JournalType type,
	                 const QString & blogName = 0 );
	virtual ~LiveJournalBlog( void );

	/**
	 * This method returns the url of the blog.
	 * @return the url of the blog.
	 */
	virtual KUrl url( void ) const;

	/**
	 * This method sets the blog name, if this object represents a user journal.
	 * @param blogName the new blog name
	 */
	void setBlogName( const QString & blogName );

	/**
	 * This method returns an icon, representing the type of the blog.
	 * @return an icon representing the type of the blog
	 */
	virtual KIcon icon( void ) const;

	/**
	 * This method returns either "Journal" or "Community", depending
	 * on the blog type
	 * @return a very short description of the blog type
	 */
	virtual QString type( void ) const;

	/**
	 * Creates a new @ref LJ::Entry object.
	 * @return a new LiveJournalEntry object.
	 */
	virtual Blokkal::Entry * createEntry( void );

	/**
	 * @return the BlogBrowsingDriver of this blog
	 */
	virtual Blokkal::Io::BlogBrowsingDriver * blogBrowsingDriver( void ) const;

	/**
	 * Saves the journal type.
	 */
	virtual void saveProperties( void );
	
	/**
	 * Returns an or'ed list of Blokkal::Blog::EntryFeatures values to indicate
	 * the possible options of entry editing.
	 * @return an or'ed list of Blokkal::Blog::EntryFeatures values
	 */
	virtual unsigned int supportedFeatures( void ) const;

private Q_SLOTS:
	void slotAccountStatusChanged( void );
	void slotGetTagsJobFinished( KJob * job );

	//these two slots are used for user journals to set
	//the icon correctly
	void updateUserPicture( LJ::IO::UserPicture * picture );
	void removeUserPicture( void );
	
private:
	class Private;
	Private * const d;
};

#endif
