/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mller                            *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LIVEJOURNALPROTOCOL_H
#define LIVEJOURNALPROTOCOL_H

#include <blokkalprotocol.h>

#include <QtCore/QVariant>

namespace Blokkal {
class Account;
}

/**
 * @short Protocol class for communication with LiveJournal servers.
 *
 * This class implements the protocol plugin for communication with LiveJournal.com
 * ans similar servers.
 *
 * @author Martin Mller <orvio@orvio.de>
 */
class LiveJournalProtocol : public Blokkal::Protocol {
Q_OBJECT
public:
	/**
	 * Creates a new LiveJournalProtocol object.
	 * @param parent the owning object
	 * @param args arguments
	 */
	LiveJournalProtocol( QObject * parent, const QVariantList & args );
	virtual ~LiveJournalProtocol( void );

	/**
	 * Creates and returns an new LJ::UI::EditAccountWidget.
	 * @param account the account this widget should edit. If 0, this widget is used to create a new account.
	 * @param parent The designated parent widget.
	 * @return a widget for editing the properties of an account.
	 */
	virtual Blokkal::Ui::EditAccountWidget * createEditAccountWidget( Blokkal::Account * account = 0, QWidget * parent = 0 );

	/**
	 * Creates a LiveJournalAccount object.
	 * @param accountName the name of the new account
	 * @return a new account object
	 */
	virtual Blokkal::Account * createAccount( const QString & accountName = 0 );
	
	/**
	 * Creates a LJ::UI::FormattingBackend.
	 * @param widget parent EditEntryWidget.
	 * @return a LJ::UI::FormattingBackend
	 */
	virtual Blokkal::Ui::FormattingBackend * createFormattingBackend( Blokkal::Ui::EditEntryWidget * widget );

};

#endif
