/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljentry.h"
#include "ljentry.moc"

#include "livejournalblog.h"
#include "livejournalaccount.h"
#include "ljio/moodmanager.h"
#include "ljio/postentryjob.h"
#include "ui/ljentryextensionwidget1.h"
#include "ui/ljentryextensionwidget2.h"
#include "ui/ljentryextensionwidget3.h"
#include "ui/ljentryextensionwidget4.h"

#include "blokkalio/jobs.h"

#include <kdebug.h>
#include <kicon.h>

LJ::Entry::Entry( LiveJournalBlog * parentBlog,
                  const QString & poster,
                  unsigned int id,
                  const KUrl & url ):
Blokkal::Entry( parentBlog, id ? QString::number( id ) : QString::null, url )
{
	setPoster( poster );
}

LJ::Entry::~Entry( void )
{
}

QString LJ::Entry::userIcon( void ) const
{
	return readEntry( "picture_keyword" );
}

void LJ::Entry::setUserIcon( const QString & keyword )
{
	writeEntry( "picture_keyword", keyword );
}

QString LJ::Entry::currentMusic( void ) const
{
	return readEntry( "current_music" );
}

void LJ::Entry::setCurrentMusic( const QString & currentMusic )
{
	writeEntry( "current_music", currentMusic );
}

QString LJ::Entry::currentMood( void ) const
{
	return readEntry( "current_mood" );
}

void LJ::Entry::setCurrentMood( const QString & mood )
{
	writeEntry( "current_mood", mood );
}

unsigned int LJ::Entry::currentMoodID( void ) const
{
	return readUIntEntry( "current_moodid" );
}

void LJ::Entry::setCurrentMoodID( unsigned int id )
{
	writeEntry( "current_moodid", id );
}

QString LJ::Entry::currentLocation( void ) const
{
	return readEntry( "current_location" );
}

void LJ::Entry::setCurrentLocation( const QString & location )
{
	writeEntry( "current_location", location );
}

LJ::Entry::SecurityLevel LJ::Entry::security( void ) const
{
	switch( readIntEntry( "security" ) ) {
	case PublicLevel:
		return PublicLevel;
		break;
	case FriendsOnlyLevel:
		return FriendsOnlyLevel;
		break;
	case PrivateLevel:
		return PrivateLevel;
		break;
	case CustomLevel:
		return CustomLevel;
		break;
	}

	kWarning() << "LJ::Entry::security() : encountered invalid security level, using PrivateLevel" << endl;
	return PrivateLevel;
}

void LJ::Entry::setSecurity( LJ::Entry::SecurityLevel security )
{
	writeEntry( "security", security );
}

unsigned int LJ::Entry::allowMask( void ) const
{
	return readUIntEntry( "allowmask" );
}

void LJ::Entry::setAllowMask( unsigned int allowMask )
{
	writeEntry( "allowmask", allowMask );
}

QString LJ::Entry::poster( void ) const
{
	return readEntry( "poster" );
}

void LJ::Entry::setPoster( const QString & poster )
{
	writeEntry( "poster", poster );
}

unsigned int LJ::Entry::uintId( void ) const
{
	return id().toUInt();
}

bool LJ::Entry::preformatted( void ) const
{
	return readBoolEntry( "opt_preformatted" );
}

void LJ::Entry::setPreformatted( bool preformatted )
{
	writeEntry( "opt_preformatted", preformatted );
}

bool LJ::Entry::backdated( void ) const
{
	return readBoolEntry( "opt_backdated" );
}

void LJ::Entry::setBackdated( bool backdated )
{
	writeEntry( "opt_backdated", backdated );
}

LJ::Entry::CommentSettings LJ::Entry::commentSettings( void ) const
{
	switch( readIntEntry( "comment_settings" ) ) {
	case CommentsDefault:
		return CommentsDefault;
		break;
	case CommentsDisabled:
		return CommentsDisabled;
		break;
	case CommentsEnabled:
		return CommentsEnabled;
		break;
	case CommentsNoEmail:
		return CommentsNoEmail;
		break;
	}

	kWarning() << "encountered invalid comment settings, using CommentsDefault" << endl;
	return CommentsDefault;
}

void LJ::Entry::setCommentSettings( LJ::Entry::CommentSettings commentSettings )
{
	writeEntry( "comment_settings", commentSettings );
}

LJ::Entry::ScreeningSettings LJ::Entry::screening( void ) const
{
	switch( readIntEntry( "opt_screening" ) ) {
	case ScreeningDefault:
		return ScreeningDefault;
		break;
	case ScreenAll:
		return ScreenAll;
		break;
	case ScreenAnonymous:
		return ScreenAnonymous;
		break;
	case ScreenNonFriends:
		return ScreenNonFriends;
		break;
	case ScreenNone:
		return ScreenNone;
		break;
	}

	kWarning() << "encountered invalid screening settings, using ScreeningDefault" << endl;
	return ScreeningDefault;
}

void LJ::Entry::setScreening( LJ::Entry::ScreeningSettings screeningSettings )
{
	writeEntry( "opt_screening", screeningSettings );
}

unsigned long LJ::Entry::revnum( void ) const
{
	return readULongEntry( "revnum" );
}

void LJ::Entry::setRevnum( unsigned long revnum )
{
	writeEntry( "revnum", revnum );
}

unsigned long LJ::Entry::revtime( void ) const
{
	return readULongEntry( "revtime" );
}

void LJ::Entry::setRevtime( unsigned long revtime )
{
	writeEntry( "revtime", revtime );
}

unsigned int LJ::Entry::anum( void ) const
{
	return readUIntEntry( "anum" );
}

void LJ::Entry::setAnum( unsigned int anum )
{
	writeEntry( "anum", anum );
}

KIcon LJ::Entry::icon( void ) const
{
	switch( security() ) {
	case PublicLevel:
		return KIcon( "blokkal_global" );
		break;
	case FriendsOnlyLevel:
		return KIcon( "blokkal_lj_friendsonly" );
		break;
	case CustomLevel:
		return KIcon( "blokkal_lj_customsecurity" );
		break;
	case PrivateLevel:
		return KIcon( "blokkal_lj_private" );
		break;
	}

	return Blokkal::Entry::icon();
}

Blokkal::Ui::EntryExtensionWidget * LJ::Entry::createExtensionWidget( Blokkal::Ui::EntryExtensionWidget::ExtensionWidget extension, Blokkal::Ui::EditEntryWidget * parent )
{
	switch ( extension ) {
	case Blokkal::Ui::EntryExtensionWidget::ExtensionWidget1:
		return new UI::EntryExtensionWidget1( this, parent );
		break;
	case Blokkal::Ui::EntryExtensionWidget::ExtensionWidget2:
		return new UI::EntryExtensionWidget2( this, parent );
		break;
	case Blokkal::Ui::EntryExtensionWidget::ExtensionWidget3:
		return new UI::EntryExtensionWidget3( this, parent );
		break;
	case Blokkal::Ui::EntryExtensionWidget::ExtensionWidget4:
		return new UI::EntryExtensionWidget4( this, parent );
		break;
	default:
		break;
	}

	return 0;
}

void LJ::Entry::merge( const Blokkal::Entry * entry )
{
	Blokkal::Entry::merge( entry );

	const LJ::Entry * ljentry = dynamic_cast<const LJ::Entry *>( entry );
	
	if( !ljentry ) {
		//this time a failed cast is actually legal
		return;
	}

	setCurrentMusic( ljentry->currentMusic() );
	setCurrentLocation( ljentry->currentLocation() );
	setPreformatted( ljentry->preformatted() );
	setCommentSettings( ljentry->commentSettings() );
	setScreening( ljentry->screening() );
	setCurrentMood( ljentry->currentMood() );
	if( blog() == ljentry->blog() ) {
		setCurrentMoodID( ljentry->currentMoodID() );
		setUserIcon( ljentry->userIcon() );
	}

	//no revtime/revnum/anum/allowmask/security
}

Blokkal::Io::EntryJob * LJ::Entry::createJob( Blokkal::Entry::JobType type )
{
	Blokkal::Io::EntryJob * job = 0;
	switch( type ) {
	case PostJob:
		job = new Blokkal::Io::EntryJob( this );
		job->addSubjob( new IO::PostEntryJob( this, static_cast<LiveJournalAccount*>( blog()->account() ) ) );
		break;
	case DeleteJob:
		job = new Blokkal::Io::EntryJob( this );
		{
			Entry * fakeEntry = new LJ::Entry(static_cast<LiveJournalBlog*>(blog()), poster(), uintId() );
			IO::PostEntryJob * deleteEntryJob = new IO::PostEntryJob( fakeEntry, static_cast<LiveJournalAccount*>( blog()->account() ) );
			connect( deleteEntryJob, SIGNAL( destroyed( void ) ),
			         fakeEntry, SLOT( deleteLater( void ) ) );
			job->addSubjob( deleteEntryJob );
		}
		break;
	default:
		break;
	}
	
	return job;
}

LJ::Entry::AdultContent LJ::Entry::adultContent( void ) const
{
	switch( readIntEntry( "adult_content" ) ) {
	case AdultDefault:
		return AdultDefault;
		break;
	case AdultNone:
		return AdultNone;
		break;
	case AdultConcepts:
		return AdultConcepts;
		break;
	case AdultExplicit:
		return AdultExplicit;
		break;
	}

	kWarning() << "encountered invalid adult content settings, using AdultDefault" << endl;
	return AdultDefault;
}

void LJ::Entry::setAdultContent( LJ::Entry::AdultContent content )
{
	writeEntry( "adult_content", content );
}
