/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blogbrowsingdriver.h"
#include "blogbrowsingdriver.moc"

#include "../livejournalblog.h"
#include "../ljentry.h"

#include "geteventsjob.h"
#include "getdaycountsjob.h"

#include <kdebug.h>

LJ::IO::BlogBrowsingDriver::BlogBrowsingDriver( LiveJournalBlog * parent ):
Blokkal::Io::BlogBrowsingDriver( parent )
{
}

LJ::IO::BlogBrowsingDriver::~BlogBrowsingDriver( void )
{
}

unsigned int LJ::IO::BlogBrowsingDriver::recentLimit( void ) const
{
	return 50;
}

unsigned int LJ::IO::BlogBrowsingDriver::supportedOperations( void ) const
{
	return LoadEntryById|LoadIndex|LoadEntriesByDate|LoadRecentEntries;
}

Blokkal::Io::LoadEntryJob * LJ::IO::BlogBrowsingDriver::createLoadEntryJob( const QStringList & idList )
{
	Blokkal::Io::LoadEntryJob * job = new LJ::IO::LoadEntryJob( blog() );
	for( QStringList::ConstIterator it = idList.begin();
	     it != idList.end();
	     ++it )
	{
		job->addSubjob( new GetEventsJob( static_cast<LiveJournalBlog*>( blog() ), (*it) ) );
	}
	return job;
}

Blokkal::Io::LoadEntryJob * LJ::IO::BlogBrowsingDriver::createLoadEntryJob( const QDate & date )
{
	Blokkal::Io::LoadEntryJob * job = new LJ::IO::LoadEntryJob( blog() );
	job->addSubjob( new GetEventsJob( static_cast<LiveJournalBlog*>( blog() ), date ) );
	return job;
}

Blokkal::Io::LoadEntryJob * LJ::IO::BlogBrowsingDriver::createLoadEntryJob( unsigned int n )
{
	Blokkal::Io::LoadEntryJob * job = new LJ::IO::LoadEntryJob( blog() );
	job->addSubjob( new GetEventsJob( static_cast<LiveJournalBlog*>( blog() ), n ) );
	return job;
}

Blokkal::Io::LoadIndexJob * LJ::IO::BlogBrowsingDriver::createLoadIndexJob( const QDate & from, const QDate & to )
{
	Q_UNUSED( from );
	Q_UNUSED( to );
	Blokkal::Io::LoadIndexJob * job = new LJ::IO::LoadIndexJob( blog() );
	job->addSubjob( new GetDayCountsJob( static_cast<LiveJournalBlog*>( blog() ) ) );
	return job;
}

//------------
//LoadEntryJob
//------------
class LJ::IO::LoadEntryJob::Private {
public:
	Private( void ) {}

	QList<Blokkal::Entry *> entryList;
};
LJ::IO::LoadEntryJob::LoadEntryJob( Blokkal::Blog * blog ):
Blokkal::Io::LoadEntryJob( blog ),
d( new Private() )
{
}

LJ::IO::LoadEntryJob::~LoadEntryJob( void )
{
	delete d;
}

void LJ::IO::LoadEntryJob::subjobFinished( KJob * job )
{
	if( job->error() ) {
		return;
	}

	const GetEventsJob * const getEventsJob = dynamic_cast<GetEventsJob *>( job );
	if( getEventsJob ) {
		const QList<LJ::Entry*> entryList = getEventsJob->entries();
		for( QList<LJ::Entry*>::ConstIterator it = entryList.begin();
		     it != entryList.end();
		     ++it )
		{
			d->entryList.append( *it );
		}
		return;
	}

	kError() << "invalid subjob encountered" << endl;

}

QList< Blokkal::Entry * > LJ::IO::LoadEntryJob::entries( void ) const
{
	return d->entryList;
}

//------------
//LoadIndexJob
//------------
class LJ::IO::LoadIndexJob::Private {
public:
	Private( void ) {}

	QMap<QDate, QStringList> index;
};

LJ::IO::LoadIndexJob::LoadIndexJob( Blokkal::Blog * blog ):
Blokkal::Io::LoadIndexJob( blog ),
d( new Private() )
{
}

LJ::IO::LoadIndexJob::~LoadIndexJob( void )
{
	delete d;
}

void LJ::IO::LoadIndexJob::subjobFinished( KJob * job )
{
	if( job->error() ) {
		return;
	}

	const GetDayCountsJob * const getDayCountsJob = dynamic_cast<GetDayCountsJob *>( job );
	if( getDayCountsJob ) {
		d->index = getDayCountsJob->dayCounts();
		return;
	}

	kError() << "invalid subjob encountered" << endl;

}

QMap< QDate, QStringList > LJ::IO::LoadIndexJob::index( void ) const
{
	return d->index;
}
