/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOCHECKFRIENDSJOB_H
#define LJ_IOCHECKFRIENDSJOB_H

#include "job.h"

namespace LJ {

namespace IO {

/**
 * @short This job checks for updates on the friends page.
 *
 * @note On livejournal.com this feature is available for paid accounts only.
 * @author Martin Mueller <orvio@orvio.de>
*/
class CheckFriendsJob : public LJ::IO::Job
{
Q_OBJECT
public:
	/**
	 * Constructs a new CheckFriendsJob.
	 * @param parent parent object
	 * @param lastUpdate a string identifying the last update, use 0 if you don't have any
	 * @param groupMask identifier for the friend groups to be considered, use 1 for all groups
	 * @param account use the user information of this account
	 */
	CheckFriendsJob( const LiveJournalAccount * account,
	                 const QString & lastUpdate = 0,
	                 unsigned long groupMask = 1,
	                 QObject * parent = 0 );
	virtual ~CheckFriendsJob( void );

	/**
	 * The time of the last update to the friends page.
	 * The protocol documentation says:
	 * <i>Don't try to infer anything from this time.
	 * It's currently of the form "yyyy-mm-dd hh:mm:ss", in PST.
	 * However, in the future, it may not even be a date...
	 * just blindly store and return this value back later, ignoring its format.</i>
	 * @return the time of the last update
	 */
	QString lastUpdate( void ) const;

	/**
	 * Returns TRUE if the friends page has updates.
	 * This includes new comments
	 * @return TRUE if the friends page has updates
	 */
	bool hasUpdates( void ) const;

	/**
	 * Returns the number of seconds to wait until the next request.
	 * The server may block you if do not honor this value.
	 * @return seconds to wait until next request
	 */
	long interval( void ) const;

protected:
	/**
	* @return the postdata needed for this job.
	*/
	virtual QByteArray postData( void ) const;

private:
	class Private;
	Private * d;

};

}

}

#endif
