/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "editfriendsjob.h"
#include "editfriendsjob.moc"

#include <kdebug.h>

#include <iostream>

class AddUser
{
public:
	AddUser( void ):
	user(),
	fg( QColor() ),
	bg( QColor() ),
	groupmask() {}
	
	AddUser( const QString & user,
	         const QColor & fg,
	         const QColor & bg,
	         unsigned long groupmask ) :
	user( user ),
	fg( fg ),
	bg( bg ),
	groupmask( groupmask ) {}

	QString user;
	QColor fg;
	QColor bg;
	unsigned long groupmask;
};

class LJ::IO::EditFriendsJob::Private
{
public:
	Private( void ) :
	usersToDelete( QStringList() ),
	usersToAdd( QList<AddUser>() ){}

	QStringList usersToDelete;
	QList<AddUser> usersToAdd;
};

LJ::IO::EditFriendsJob::EditFriendsJob( const LiveJournalAccount * account, QObject * parent ):
Job( account, parent ),
d( new Private() )
{
}

LJ::IO::EditFriendsJob::~EditFriendsJob( void )
{
	delete d;
}

QByteArray LJ::IO::EditFriendsJob::postData( void ) const
{
	QByteArray postData = "mode=editfriends";

	for( QStringList::ConstIterator it = d->usersToDelete.begin();
	     it != d->usersToDelete.end();
	     ++it )
	{
		postData.append( QString::fromLatin1( "&editfriend_delete_%1=1" ).arg( *it ).toAscii() );
	}

	int i = 1;
	for( QList<AddUser>::ConstIterator it = d->usersToAdd.begin();
	     it != d->usersToAdd.end();
	     ++it, ++i )
	{
		postData.append( QString::fromLatin1( "&editfriend_add_%1_user=%2" ).arg( i ).arg( (*it).user ).toAscii() );
		if( (*it).fg.isValid() ) {
			postData.append( QString::fromLatin1( "&editfriend_add_%1_fg=%2" ).arg( i ).arg( (*it).fg.name() ).toAscii() );
		}
		if( (*it).bg.isValid() ) {
			postData.append( QString::fromLatin1( "&editfriend_add_%1_bg=%2" ).arg( i ).arg( (*it).bg.name() ).toAscii() );
		}
		postData.append( QString::fromLatin1( "&editfriend_add_%1_groupmask=%2" ).arg( i ).arg( (*it).groupmask ? (*it).groupmask : 1 ).toAscii() );
	}

	//kdDebug() << postData << endl;
	return postData;
}

void LJ::IO::EditFriendsJob::addUserToDelete( const QString & deleteUser )
{
	if( d->usersToDelete.contains( deleteUser ) ) {
		return;
	}

	d->usersToDelete.append( deleteUser );
}

void LJ::IO::EditFriendsJob::addUserToAdd( const QString & user, const QColor & fg, const QColor & bg, unsigned long groupmask )
{
	d->usersToAdd.append( AddUser( user, fg, bg, groupmask ) );
}

QStringList LJ::IO::EditFriendsJob::addedFriends( void )
{
	QStringList userNames;
	unsigned int addedFriendCount = getValueOfKey( "friends_added" ).toUInt();

	for( unsigned int i = 1; i <= addedFriendCount; ++i ) {
		userNames.append( getValueOfKey( QString::fromLatin1( "friend_%1_user" ).arg( i ) ) );
	}

	return userNames;
}

QString LJ::IO::EditFriendsJob::realName( const QString & user )
{
	QString fullName = QString::null;
	unsigned int addedFriendCount = getValueOfKey( "friends_added" ).toUInt();

	for( unsigned int i = 1; i <= addedFriendCount; ++i ) {
		if( user == getValueOfKey( QString::fromLatin1( "friend_%1_user" ).arg( i ) ) ) {
			fullName = getValueOfKey( QString::fromLatin1("friend_%1_name").arg( i ) );
			break;
		}
	}	
			
	return fullName;
}
