/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOEDITFRIENDSJOB_H
#define LJ_IOEDITFRIENDSJOB_H

#include "job.h"

#include <QColor>

namespace LJ {

namespace IO {

/**
 * @short Base class for jobs performing editfriends operations
 *
 * This class provides methods to get the added friends as returned by the server.
 * It also handles the creation of the postdata.
 * @author Martin Mueller <orvio@orvio.de>
*/
class EditFriendsJob : public LJ::IO::Job
{
Q_OBJECT
public:
	/**
	 * Creates a new EditFriendsJob.
	 * @param account use the authentication information from this account
	 * @param parent the owning object
	 */
	EditFriendsJob( const LiveJournalAccount * account, QObject * parent = 0 );
	virtual ~EditFriendsJob( void );

	/**
	 * Returns the usernames of added friends as returned by the server.
	* @return the list of added friends as returned by the server
	*/
	QStringList addedFriends( void );

	/**
	 * Returns the real name of the user @a user.
	 * @param user the user name
	 * @return the real name of @a user if it was received from the server
	 */
	QString realName( const QString & user );

	/**
	 * Add a user to delete
	 * @param deleteUser add this user to the delete list
	 */
	void addUserToDelete( const QString & deleteUser );

	/**
	 * Add a user to the add list
	 * @param user username
	 * @param fg text color
	 * @param bg background color
	 * @param groupmask the group mask
	 */
	void addUserToAdd( const QString & user, const QColor & fg, const QColor & bg, unsigned long groupmask );
	
protected:
	/**
	 * Returns the postdata for this job
	 * @return postdata
	 */
	virtual QByteArray postData( void ) const;

private:
	class Private;
	Private * d;
};

}

}

#endif
