/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getdaycountsjob.h"
#include "getdaycountsjob.moc"

#include "../livejournalblog.h"
#include "../livejournalaccount.h"

#include <QDateTime>
#include <QRegExp>
#include <QStringList>

class LJ::IO::GetDayCountsJob::Private
{
public:
	Private( void ) :
	blog( 0 ) {}

	const LiveJournalBlog * blog;
};

LJ::IO::GetDayCountsJob::GetDayCountsJob( const LiveJournalBlog * blog, QObject * parent ) :
Job( dynamic_cast<LiveJournalAccount*>( blog->account() ), parent ),
d( new Private() )
{
	d->blog = blog;
}

LJ::IO::GetDayCountsJob::~GetDayCountsJob( void )
{
	delete d;
}

QByteArray LJ::IO::GetDayCountsJob::postData( void ) const
{
	return "mode=getdaycounts&usejournal=" + d->blog->id().toAscii();
}

QMap< QDate, QStringList > LJ::IO::GetDayCountsJob::dayCounts( void ) const
{
	QMap<QDate, QStringList> dayCounts;
	int offset = 0;
	int keyStart = 0;
	const int keyLength = 10;
	int valueStart = 0;
	int valueLength = 0;
	QString key;
	int entryCount;
	QStringList entryKeys;
	//search keys like yyyy-mm-dd
	QString buffer = QString::fromAscii( responseBuffer() );
	while( (keyStart = /*responseB*/buffer/*()*/.indexOf(QRegExp("(\n|^)[0-9]{4,4}-[0-9]{2,2}-[0-9]{2,2}[\r]?\n"), offset) ) >= 0 ) {
		//if this key is in the first line there is no \n in front of it
		if( keyStart > 0 )
			keyStart++;
		
		//cannot use fixed length as there might still be a \r
		valueStart = responseBuffer().indexOf('\n', keyStart + keyLength) + 1;
		
		//\r is a white space so it won't hurt later conversion
		valueLength = responseBuffer().indexOf('\n', valueStart ) - valueStart;
		key = responseBuffer().mid(keyStart, keyLength);
		entryCount = responseBuffer().mid( valueStart, valueLength ).toUInt();
		entryKeys = QStringList();
		for( int i = 0; i < entryCount; ++i ) {
			entryKeys.append( QString::null );
		}
		
		dayCounts.insert( QDate( key.mid( 0, 4 ).toUInt(),  key.mid( 5, 2 ).toUInt(), key.mid( 8, 2 ).toUInt() ),
		                  entryKeys );
		
		//kdDebug() << key << " : " << value << endl;
		offset = valueStart + valueLength;
	}
	return dayCounts;
}
