/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getfriendgroupsjob.h"
#include "getfriendgroupsjob.moc"

#include "friendgroupsmanager.h"
#include "../livejournalaccount.h"

#include <kdebug.h>

#include <iostream>

LJ::IO::GetFriendGroupsJob::GetFriendGroupsJob( const LiveJournalAccount * account, QObject * parent ) :
Job( account, parent )
{
}

LJ::IO::GetFriendGroupsJob::~GetFriendGroupsJob( void )
{
}

QByteArray LJ::IO::GetFriendGroupsJob::postData( void ) const
{
	return "mode=getfriendgroups";
}

void LJ::IO::GetFriendGroupsJob::ioJobFinished()
{

	LJ::IO::FriendGroupsManager * groupManager = account()->friendGroupsManager();
	LJ::IO::FriendGroup * currentGroup;

	QList<unsigned short> validIDs = QList<unsigned short>();
	unsigned short maxGroupNumber = getValueOfKey( "frgrp_maxnum" ).toUShort();

	QString groupName;
	QString sortorderString;
	QString publicString;

	//if maxGroupNumber == 0, no friendgroups where returned
	for( unsigned short currentGroup = 1;
	     currentGroup <= maxGroupNumber;
	     ++currentGroup )
	{
		groupName = getValueOfKey( "frgrp_" + QString::number( currentGroup ) + "_name" );
		if( !groupName.isEmpty() ) {
			validIDs.append( currentGroup );
		}
	}

	//remove old groups
	groupManager->setValidGroupIDs( validIDs );

	//set current values
	for( QList<unsigned short>::ConstIterator it = validIDs.begin();
	     it != validIDs.end();
	     ++it )
	{
		currentGroup = groupManager->group( *it );
		currentGroup->setGroupName( getValueOfKey( "frgrp_" + QString::number( *it ) + "_name" ) );
		currentGroup->setSortorder( getValueOfKey( "frgrp_" + QString::number( *it ) + "_sortorder" ).toUShort() );
		currentGroup->setPublic( getValueOfKey( "frgrp_" + QString::number( *it ) + "_public" ).toInt() == 1 );
	}


}
