/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getfriendinfojob.h"
#include "getfriendinfojob.moc"

#include "friendinfomanager.h"
#include "../livejournalaccount.h"

#include <kdebug.h>

#include <iostream>

class LJ::IO::GetFriendInfoJob::Private
{
public:
	Private( void ) :
	include( 0 ) {}

	int include;
};

LJ::IO::GetFriendInfoJob::GetFriendInfoJob( const LiveJournalAccount * account,
                                            QObject * parent,
                                            int include ) :
GetFriendGroupsJob( account, parent),
d( new Private() )
{
	d->include = include;
}

LJ::IO::GetFriendInfoJob::~GetFriendInfoJob( void )
{
	delete d;
}

QByteArray LJ::IO::GetFriendInfoJob::postData( void ) const
{
	QByteArray postData = "mode=getfriends";
	if( d->include & IncludeFriendOf) {
		postData.append( "&includefriendof=1" );
	}
	if( d->include & IncludeFriendGroups ) {
		postData.append( "&includegroups=1" );
	}
	return postData;
}

void LJ::IO::GetFriendInfoJob::ioJobFinished( void )
{
	//kdDebug() << k_funcinfo << endl;
	//kdDebug() << responseBuffer() << endl;

	//--------------------------
	//friend
	FriendInfoManager * manager = account()->friendInfoManager();

	const unsigned int friendsCount = getValueOfKey( "friend_count" ).toUInt();
	QStringList friends;
	for( unsigned int i = 1; i <= friendsCount; ++i ) {
		friends.append( getValueOfKey( QString::fromLatin1( "friend_%1_user").arg( i ) ) );
	}
	manager->setValidFriends( friends );

	unsigned int i = 1;
	for( QStringList::ConstIterator it = friends.begin();
	     it != friends.end();
	     ++it, ++i )
	{
		FriendInfo * friendInfo = manager->friendInfo( *it );
		updateFriendInfoBase( friendInfo, i, "friend" );
		friendInfo->setGroupmask( getValueOfKey( QString::fromLatin1( "friend_%1_groupmask").arg( i ) ).toULong() );
		//TODO birthday
	}

	//--------------------------
	//friendof
	if( d->include & IncludeFriendOf ) {
		const unsigned int friendOfCount = getValueOfKey( "friendof_count" ).toUInt();
		QStringList friendOfs;
		for( unsigned int i = 1; i <= friendOfCount; ++i ) {
			friendOfs.append( getValueOfKey( QString::fromLatin1( "friendof_%1_user").arg( i ) ) );
		}
		manager->setValidFriendOfs( friendOfs );

		unsigned int n = 1;
		for( QStringList::ConstIterator it = friendOfs.begin();
		     it != friendOfs.end();
		     ++it, ++n )
		{
			FriendOfInfo * friendOfInfo = manager->friendOfInfo( *it );
			updateFriendInfoBase( friendOfInfo, n, "friendof" );
		}
	}

	
	if( d->include & IncludeFriendGroups ) {
		GetFriendGroupsJob::ioJobFinished( );
	}
}

void LJ::IO::GetFriendInfoJob::updateFriendInfoBase( FriendInfoBase * info, unsigned int index, const QString & prefix )
{
	info->setRealName( getValueOfKey( QString::fromLatin1( "%1_%2_name").arg( prefix ).arg( index ) ) );
	info->setBackground( getValueOfKey( QString::fromLatin1( "%1_%2_bg").arg( prefix ).arg( index ) ) );
	info->setForeground( getValueOfKey( QString::fromLatin1( "%1_%2_fg").arg( prefix ).arg( index ) ) );
			
	QString value = getValueOfKey( QString::fromLatin1( "%1_%2_type").arg( prefix ).arg( index ) );
	FriendInfoBase::FriendType type;
	if( value == "community" ) {
		type = FriendInfoBase::TypeCommunity;
	}
	else if( value == "syndicated" ) {
		type = FriendInfoBase::TypeSyndicated;
	}
	else {
		type = FriendInfoBase::TypeUser;
	}
	info->setType( type );
	
	value = getValueOfKey( QString::fromLatin1( "%1_%2_status").arg( prefix ).arg( index ) );
	FriendInfoBase::FriendStatus status;
	if( value == "deleted") {
		status = FriendInfoBase::StatusDeleted;
	}
	else if( value == "suspended") {
		status = FriendInfoBase::StatusSuspended;
	}
	else if( value == "purged") {
		status = FriendInfoBase::StatusPurged;
	}
	else {
		status = FriendInfoBase::StatusNormal;
	}
	info->setStatus( status );
}
