/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOGETFRIENDINFOJOB_H
#define LJ_IOGETFRIENDINFOJOB_H

#include "getfriendgroupsjob.h"

namespace LJ {

namespace IO {
class FriendInfoBase;
/**
 * @short Fetches friend information.
 *
 * Fetches friend information.
 * @author Martin Mueller <orvio@orvio.de>
*/
class GetFriendInfoJob : public GetFriendGroupsJob
{
Q_OBJECT
public:
	enum IncludeInfo {
		IncludeFriendOf = 0x01,
		IncludeFriendGroups = 0x02,
	};
	
	/**
	 * Constructs a new GetFriendInfoJob
	 * @param account use the user information of this account
	 * @param parent parent object
	 * @param include fetch friend groups and friendof information according to these flags
	 */
	GetFriendInfoJob( const LiveJournalAccount * account,
	                  QObject * parent = 0,
	                  int include = IncludeFriendOf|IncludeFriendGroups );
	virtual ~GetFriendInfoJob( void );

protected:
	/**
	* @return the postdata needed for this job.
	*/
	virtual QByteArray postData( void ) const;

	/**
	 * Updates @a info with the information for the item with index @a index in the response buffer.
	 * @a prefix is used to select the information to use.
	 * @param info update this item
	 * @param index index of the data in the response buffer
	 * @param prefix prefix to use
	 */
	void updateFriendInfoBase( FriendInfoBase * info, unsigned int index, const QString & prefix );
	
	/**
	 * This method is called when the io job has finished without io
	 * error and the success value indicates a successfull query
	 */
	virtual void ioJobFinished( void );

private:
	class Private;
	Private * d;
};

}

}

#endif
