/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getusertagsjob.h"
#include "getusertagsjob.moc"

#include <blokkalio/categorymanager.h>

#include "../livejournalaccount.h"
#include "../livejournalblog.h"

#include <kdebug.h>

#include <QStringList>

class LJ::IO::GetUserTagsJob::Private
{
public:
	Private( const LiveJournalBlog * blog ) :
	blog( blog ) {}
	
	const LiveJournalBlog * const blog;
};

LJ::IO::GetUserTagsJob::GetUserTagsJob( const LiveJournalBlog * blog, QObject * parent ) :
Job( dynamic_cast<LiveJournalAccount *>( blog->account() ), parent ),
d( new Private( blog ) )
{
}

LJ::IO::GetUserTagsJob::~GetUserTagsJob( void )
{
	delete d;
}

QByteArray LJ::IO::GetUserTagsJob::postData( void ) const
{
	return QString::fromLatin1( "&mode=getusertags&usejournal=%1" ).arg( d->blog->id() ).toAscii();
}

void LJ::IO::GetUserTagsJob::ioJobFinished( void )
{
	Blokkal::Io::CategoryManager * categoryManager = account()->categoryManager();
	unsigned int tagCount = getValueOfKey( "tag_count" ).toUInt();
	QStringList tags;

	for( unsigned int i = 1; i <= tagCount; ++i ) {
		QString currentTag = getValueOfKey( "tag_" + QString::number(i) + "_name" );
		if( !currentTag.isEmpty() ) {
			//kdDebug() << k_funcinfo << currentTag << endl;
			tags.append( currentTag );
		}
	}

	categoryManager->setValidCategories( d->blog->id(), tags );

	for( QStringList::ConstIterator it = tags.begin();
	     it != tags.end();
	     ++it )
	{
		Blokkal::Io::Category * category = categoryManager->category( d->blog->id(), *it );
		category->setName( *it );
	}
}
