/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "loginjob.h"
#include "loginjob.moc"

#include "../livejournalaccount.h"
#include "moodmanager.h"
#include "userpicturemanager.h"

#include <kdebug.h>

#include <QStringList>

LJ::IO::LoginJob::LoginJob( LiveJournalAccount * account, QObject * parent ) :
GetFriendGroupsJob( account, parent )
{
}

LJ::IO::LoginJob::~LoginJob( void )
{
}

QByteArray LJ::IO::LoginJob::postData( void ) const
{
	return "mode=login&getpickws=1&getpickwurls=1&getmoods="
		+ QByteArray::number( account()->moodManager()->maxMoodID() );/*&getmenus=1";*/
}

QString LJ::IO::LoginJob::message( void ) const
{
	return getValueOfKey( "message" );
}

QString LJ::IO::LoginJob::name( void ) const
{
	return getValueOfKey( "name" );
}

bool LJ::IO::LoginJob::mayAccessFastServer( void ) const
{
	const QString fastServer = getValueOfKey( "fastserver" );
	return ( !fastServer.isEmpty() && fastServer.toUInt() == 1 );
}

QStringList LJ::IO::LoginJob::accessList( void ) const
{
	QStringList accessList;
	accessList.append( account()->userName() );
	QString accessCount = getValueOfKey( "access_count" );
	
	if( !accessCount.isEmpty() ) {
		QString currentAccess;
		for( unsigned int i = 1; i <= accessCount.toUInt(); i++ ) {
			currentAccess = getValueOfKey( "access_" + QString::number( i ) );
			if( !currentAccess.isEmpty() ) {
				accessList.append( currentAccess );
			}
		}
	}
	
	return accessList;
}

void LJ::IO::LoginJob::ioJobFinished( void )
{
	//kdDebug() << responseBuffer() << endl;

	//update moods
	MoodManager * moodManager = account()->moodManager();
	unsigned int newMoodCount = getValueOfKey( "mood_count" ).toUInt();

	unsigned int currentMoodID;
	unsigned int currentMoodParent;
	QString currentMoodName;

	//1 is first valid mood id
	for( unsigned int i = 1; i <= newMoodCount; ++i ) {
		currentMoodName = getValueOfKey( "mood_" + QString::number(i) +  "_name");
		currentMoodID = getValueOfKey( "mood_" + QString::number(i) +  "_id").toUInt();
		currentMoodParent = getValueOfKey( "mood_" + QString::number(i) +  "_parent").toUInt();
		moodManager->addMood( currentMoodID, currentMoodParent, currentMoodName );

		//kdDebug() << "Current Mood: " << currentMoodName << " <=> " << currentMoodID << " p: " << currentMoodParent << endl;
	}

	//update user pictures
	UserPictureManager * pictureManager = account()->userPictureManager();
	pictureManager->setDefaultPictureURL( getValueOfKey( "defaultpicurl" ) );

	unsigned int keywordCount = getValueOfKey( "pickw_count" ).toUInt();
	unsigned int pictureCount = getValueOfKey( "pickwurl_count" ).toUInt();

	QList<QString> validKeywords;
	QString currentKeyword;

	if( keywordCount == pictureCount ) {
		for( unsigned int i = 1; i <= keywordCount; ++i ) {
			currentKeyword = getValueOfKey( "pickw_" + QString::number( i ) );
			validKeywords.append( currentKeyword );
			pictureManager->picture( currentKeyword )->setPictureURL( getValueOfKey( "pickwurl_" + QString::number( i ) ) );
		}
	}

	pictureManager->setValidKeywords( validKeywords );
}



