/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mller                            *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_IOLOGINJOB_H
#define LJ_IOLOGINJOB_H

#include "getfriendgroupsjob.h"

namespace LJ {
namespace IO {

/**
 * @short This job connects to the server with mode = login
 * @author Martin Mller <orvio@orvio.de>
*/
class LoginJob : public LJ::IO::GetFriendGroupsJob
{
Q_OBJECT
public:
	/**
	 * Creates a new login job.
	 * @param account the context account. Authentication information is retrieved from this object
	 * @param parent the owning object
	 */
	LoginJob( LiveJournalAccount * account, QObject *parent = 0 );

	virtual ~LoginJob( void );

	/**
	 * The server may send a message required to be shown to the user.
	 * Use this method to access it.
	 * If the returned string is empty, no message was sent.
	 *
	 * @return a message the server wants the user to see
	*/
	QString message( void ) const;

	/**
	 * This method returns the real name of the user.
	 * Users may set this to whatever they want, so do not
	 * expect this to be really a real name. (Though it might be one.)
	 *
	 * @return the name of the user.
	*/
	QString name( void ) const;

	/**
	 * This method tells whether the server allows this user to access
	 * the fast server.
	 *
	 * @return TRUE if the server allows this user to access the fast servers
	 */
	bool mayAccessFastServer( void ) const;
	
	/**
	 * This method returns a list of journals the user has access to.
	 * The first element in the list is the user's own journal.
	 * @see name()
	 *
	 * @return list of journal names the user may post to
	 */
	QStringList accessList( void ) const;

protected:
	/**
	 * Returns the required postdata for a login job
	 * @return login job postdata
	 */
	virtual QByteArray postData( void ) const;
	
	/**
	 * This method is called when the io job has finished without io
	 * error and the success value indicates a successfull query
	 */
	virtual void ioJobFinished( void );
};

}
}
#endif
