/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "postentryjob.h"
#include "postentryjob.moc"

#include "../ljentry.h"
#include "../livejournalaccount.h"
#include "../livejournalblog.h"

#include <kurl.h>

class LJ::IO::PostEntryJob::Private
{
public:
	Private( const LJ::Entry * entry ):
	entry( entry ) {}

	const LJ::Entry * entry;
};

LJ::IO::PostEntryJob::PostEntryJob( const LJ::Entry * entry, const LiveJournalAccount * account, QObject * parent ) :
Job( account, parent ),
d( new Private( entry ) )
{
}

LJ::IO::PostEntryJob::~PostEntryJob( void )
{
	delete d;
}

QByteArray LJ::IO::PostEntryJob::postData( void ) const
{
	QByteArray postData = "&mode=";
	if( d->entry->uintId() > 0) {
		postData.append(  "editevent&itemid=" + d->entry->id().toUtf8()  );
	}
	else {
		postData.append( "postevent" );
	}

	postData.append( "&usejournal="
	                 + QUrl::toPercentEncoding( d->entry->blog()->id() ) );

	postData.append("&event="
	                + QUrl::toPercentEncoding( d->entry->text() )
	                + "&lineendings=unix" );

	switch( d->entry->security() ) {
	case LJ::Entry::PublicLevel:
		postData.append( "&security=public" );
		break;
	case LJ::Entry::PrivateLevel:
		postData.append( "&security=private" );
		break;
	case LJ::Entry::FriendsOnlyLevel:
		postData.append( "&security=usemask" );
		postData.append( "&allowmask=1" );
		break;
	case LJ::Entry::CustomLevel:
		postData.append( "&security=usemask" );
		postData.append( "&allowmask="
		                 + QByteArray::number( d->entry->allowMask() ) );
		break;
	}

	QDateTime dateTime = d->entry->date();
	if( dateTime.isValid() ) {
		QDate date = dateTime.date();
		QTime time = dateTime.time();
		postData.append( "&year="
		                 + QByteArray::number( date.year() )
		                 + "&mon=" + QByteArray::number( date.month() )
		                 + "&day=" + QByteArray::number( date.day() )
		                 + "&min=" + QByteArray::number( time.minute() )
		                 + "&hour=" + QByteArray::number( time.hour() ) );
	}

	if( !d->entry->subject().isEmpty() ) {
		postData.append("&subject=" + QUrl::toPercentEncoding( d->entry->subject() ) );
	}
	
	if( !d->entry->currentMusic().isEmpty() ) {
		postData.append("&prop_current_music="
		                + QUrl::toPercentEncoding( d->entry->currentMusic() ) );
	}
	
	if( !d->entry->currentMood().isEmpty() ) {
		postData.append("&prop_current_mood="
		                + QUrl::toPercentEncoding( d->entry->currentMood() ) );
	}
	if( d->entry->currentMoodID() > 0 ) {
		postData.append("&prop_current_moodid="
		                + QByteArray::number( d->entry->currentMoodID() ) );
	}

	if( !d->entry->userIcon().isEmpty() ) {
		postData.append( "&prop_picture_keyword="
		                 + QUrl::toPercentEncoding( d->entry->userIcon() ) );
	}
	
	if( !d->entry->currentLocation().isEmpty() ) {
		postData.append( "&prop_current_location="
		                 + QUrl::toPercentEncoding( d->entry->currentLocation() ) );
	}
	
	if( d->entry->backdated() ) {
		postData.append( "&prop_opt_backdated=1" );
	}
	
	if( d->entry->preformatted() ) {
		postData.append( "&prop_opt_preformatted=1" );
	}
	
	if( d->entry->revnum() > 0 ) {
		postData.append( "&prop_revnum="
		                 + QByteArray::number( (qulonglong) d->entry->revnum() ) );
	}
	if( d->entry->revtime() > 0 ) {
		postData.append( "&prop_revtime="
		                 + QByteArray::number( (qulonglong) d->entry->revtime() ) );
	}
	
	QStringList tags = d->entry->categories();
	if( !tags.isEmpty() ) {
		postData.append( "&prop_taglist="
		                 + QUrl::toPercentEncoding( tags.join(",") ) );
	}

	switch( d->entry->commentSettings() ) {
	case LJ::Entry::CommentsDefault:
		break;
	case LJ::Entry::CommentsEnabled:
		postData.append( "&prop_opt_nocomments=0" );
		break;
	case LJ::Entry::CommentsDisabled:
		postData.append( "&prop_opt_nocomments=1" );
		break;
	case LJ::Entry::CommentsNoEmail:
		postData.append( "&prop_opt_nocomments=0&prop_opt_noemail=1" );
		break;
	}

	switch( d->entry->screening() ) {
	case LJ::Entry::ScreeningDefault:
		break;
	case LJ::Entry::ScreenNone:
		postData.append( "&prop_opt_screening=N" );
		break;
	case LJ::Entry::ScreenNonFriends:
		postData.append( "&prop_opt_screening=F" );
		break;
	case LJ::Entry::ScreenAnonymous:
		postData.append( "&prop_opt_screening=R" );
		break;
	case LJ::Entry::ScreenAll:
		postData.append( "&prop_opt_screening=A" );
		break;
	}

	switch( d->entry->adultContent() ) {
	case Entry::AdultDefault:
		break;
	case Entry::AdultNone:
		postData.append( "&prop_adult_content=none" );
		break;
	case Entry::AdultConcepts:
		postData.append( "&prop_adult_content=concepts" );
		break;
	case Entry::AdultExplicit:
		postData.append( "&prop_adult_content=explicit" );
		break;
	}
	
	return postData;
}
