/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljcommentsettingsselector.h"
#include "ljcommentsettingsselector.moc"

#include <kcombobox.h>
#include <klocale.h>
#include <kdialog.h>

#include <QLayout>

class LJ::UI::CommentSettingsSelector::Private
{
public:
	Private( void ) {}

	KComboBox * screeningSettingsComboBox;
	KComboBox * commentSettingsComboBox;
};

LJ::UI::CommentSettingsSelector::CommentSettingsSelector( QWidget * parent ):
QWidget( parent ),
d( new Private() )
{
	QHBoxLayout * layout = new QHBoxLayout( this );
	layout->setMargin( 0 );
	d->commentSettingsComboBox = new KComboBox( this ) ;
	d->commentSettingsComboBox->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
	layout->addWidget( d->commentSettingsComboBox );
	d->screeningSettingsComboBox = new KComboBox( this ) ;
	d->screeningSettingsComboBox->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
	layout->addWidget( d->screeningSettingsComboBox );

	//setFocusPolicy( QWidget::StrongFocus );

	d->commentSettingsComboBox->addItem( i18n( "Default" ) );
	d->commentSettingsComboBox->addItem( i18n( "Enabled" ) );
	d->commentSettingsComboBox->addItem( i18n( "No Emails" ) );
	d->commentSettingsComboBox->addItem( i18n( "Disabled" ) );

	d->screeningSettingsComboBox->addItem( i18n( "Default" ) );
	d->screeningSettingsComboBox->addItem( i18n( "Screen None" ) );
	d->screeningSettingsComboBox->addItem( i18n( "Screen Anonymous" ) );
	d->screeningSettingsComboBox->addItem( i18n( "Screen Non-Friends" ) );
	d->screeningSettingsComboBox->addItem( i18n( "Screen All" ) );

	updateScreeningSettingsComboBoxEnabledState();

	connect( d->commentSettingsComboBox, SIGNAL( activated( int ) ),
	         this, SLOT( updateScreeningSettingsComboBoxEnabledState( int ) ) );

	connect( d->commentSettingsComboBox, SIGNAL( activated( int ) ),
	         this, SLOT( emitSettingsChanged( void ) ) );
	connect( d->screeningSettingsComboBox, SIGNAL( activated( int ) ),
	         this, SLOT( emitSettingsChanged( void ) ) );
}

LJ::UI::CommentSettingsSelector::~CommentSettingsSelector( void )
{
	delete d;
}

void LJ::UI::CommentSettingsSelector::updateScreeningSettingsComboBoxEnabledState( int index )
{
	if( index  < 0 ) {
		updateScreeningSettingsComboBoxEnabledState( d->commentSettingsComboBox->currentIndex() );
	}
	else {
		d->screeningSettingsComboBox->setEnabled( index != 3 );
	}
}

void LJ::UI::CommentSettingsSelector::emitSettingsChanged( void )
{
	emit settingsChanged();
}

void LJ::UI::CommentSettingsSelector::setCommentSettings( LJ::Entry::CommentSettings commentSettings )
{
	switch( commentSettings ) {
	case LJ::Entry::CommentsDefault:
		d->commentSettingsComboBox->setCurrentIndex( 0 );
		break;
	case LJ::Entry::CommentsEnabled:
		d->commentSettingsComboBox->setCurrentIndex( 1 );
		break;
	case LJ::Entry::CommentsNoEmail:
		d->commentSettingsComboBox->setCurrentIndex( 2 );
		break;
	case LJ::Entry::CommentsDisabled:
		d->commentSettingsComboBox->setCurrentIndex( 3 );
		break;
	}
}

LJ::Entry::CommentSettings LJ::UI::CommentSettingsSelector::commentSettings( void ) const
{
	switch( d->commentSettingsComboBox->currentIndex() ) {
	case 0:
		return LJ::Entry::CommentsDefault;
		break;
	case 1:
		return LJ::Entry::CommentsEnabled;
		break;
	case 2:
		return LJ::Entry::CommentsNoEmail;
		break;
	case 3:
		return LJ::Entry::CommentsDisabled;
		break;
	}

	return LJ::Entry::CommentsDefault;
}

void LJ::UI::CommentSettingsSelector::setScreeningSettings( LJ::Entry::ScreeningSettings screeningSettings )
{
	switch( screeningSettings ) {
	case LJ::Entry::ScreeningDefault:
		d->screeningSettingsComboBox->setCurrentIndex( 0 );
		break;
	case LJ::Entry::ScreenNone:
		d->screeningSettingsComboBox->setCurrentIndex( 1 );
		break;
	case LJ::Entry::ScreenAnonymous:
		d->screeningSettingsComboBox->setCurrentIndex( 2 );
		break;
	case LJ::Entry::ScreenNonFriends:
		d->screeningSettingsComboBox->setCurrentIndex( 3 );
		break;
	case LJ::Entry::ScreenAll:
		d->screeningSettingsComboBox->setCurrentIndex( 4 );
		break;
	}
}

LJ::Entry::ScreeningSettings LJ::UI::CommentSettingsSelector::screeningSettings( void ) const
{
	switch( d->screeningSettingsComboBox->currentIndex() ) {
	case 0:
		return LJ::Entry::ScreeningDefault;
		break;
	case 1:
		return LJ::Entry::ScreenNone;
		break;
	case 2:
		return LJ::Entry::ScreenAnonymous;
		break;
	case 3:
		return LJ::Entry::ScreenNonFriends;
		break;
	case 4:
		return LJ::Entry::ScreenAll;
		break;
	}
	
	return LJ::Entry::ScreeningDefault;
}


