/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_UILJCOMMENTSETTINGSSELECTOR_H
#define LJ_UILJCOMMENTSETTINGSSELECTOR_H

#include "../ljentry.h"

#include <QWidget>

namespace LJ {
namespace UI {

/**
 * @short An input widget for comment settings on a livejournal entry.
 *
 * An input widget for comment settings on a livejournal entry.
 * This includes screening settings.
 * @author Martin Mueller <orvio@orvio.de>
*/
class CommentSettingsSelector : public QWidget
{
Q_OBJECT
public:
	/**
	 * Creates a new CommentSettingsSelector.
	 * @param parent parent widget
	 */
	CommentSettingsSelector( QWidget * parent = 0 );
   virtual ~CommentSettingsSelector( void );

	/**
	 * Returns the currently selected comment setting
	 * @return the current comment setting
	 */
	LJ::Entry::CommentSettings commentSettings( void ) const;

	/**
	 * Sets the selected comment setting
	 * @param commentSettings the new comment setting
	 */
	void setCommentSettings( LJ::Entry::CommentSettings commentSettings );

	/**
	 * Returns the currently selected screening setting
	 * @return the current screening setting
	 */
	LJ::Entry::ScreeningSettings screeningSettings( void ) const;
	
	/**
	 * Sets the selected screening setting
	 * @param screeningSettings the new screening setting
	 */
	void setScreeningSettings( LJ::Entry::ScreeningSettings screeningSettings );

signals:
	/**
	 * This signal is emitted when the user selects other
	 * comment or screening settings
	 */
	void settingsChanged( void );

protected slots:
	/**
	 * Enables or disabled the screning settings combobox.
	 * @param index used for the decision
	 */
	virtual void updateScreeningSettingsComboBoxEnabledState( int index = -1 );

	/**
	 * This slot is emits the settingsChanged() signal
	 */
	virtual void emitSettingsChanged( void );
	
private:
	class Private;
	Private * d;

};

}

}

#endif
