/***************************************************************************
 *   Copyright (C) 2006 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljeditaccountwidget.h"
#include "ljeditaccountwidget.moc"

#include "../livejournalprotocol.h"

#include "../livejournalaccount.h"

#include <klineedit.h>
#include <kcombobox.h>
#include <klocale.h>
#include <kdialog.h>
#include <knuminput.h>
#include <kdebug.h>

#include <QCheckBox>
#include <QLayout>
#include <QLabel>
#include <QGroupBox>

#include <iostream>

class LJ::UI::EditAccountWidget::Private
{
public:
	Private( void ) {};

	KComboBox * providerComboBox;

	QCheckBox * monitorFriendsPageCheckBox;
	QLabel * timeoutLabel;
	KIntSpinBox * timeoutSpinBox;
};


LJ::UI::EditAccountWidget::EditAccountWidget( LiveJournalProtocol * protocol, Blokkal::Account * account, QWidget * parent ) :
GenericEditAccountWidget( protocol, account, parent ),
d( new Private() )
{
	LiveJournalAccount * ljaccount = dynamic_cast< LiveJournalAccount* >( this->account() );
	if( account && !ljaccount ) {
		kWarning() << k_funcinfo << " account is not a lj account!" << endl;
		return;
	}
	
	//Provider setup widget
	QHBoxLayout * providerLayout = new QHBoxLayout( 0 );
	providerLayout->setSpacing( KDialog::spacingHint() );
	d->providerComboBox = new KComboBox( networkGroupBox() );
	d->providerComboBox->addItem( i18n( "About My Life" ) );
	d->providerComboBox->addItem( i18n( "DeadJournal" ) );
	d->providerComboBox->addItem( i18n( "GreatestJournal" ) );
	d->providerComboBox->addItem( protocol->icon(), i18n( "LiveJournal" ) );
	d->providerComboBox->addItem( i18n( "Custom" ) );
	providerLayout->addWidget( d->providerComboBox );
	providerLayout->addStretch( 1 );

	((QBoxLayout*)networkGroupBox()->layout())->insertLayout( 0, providerLayout );

	//friends page monitor configuration
	d->monitorFriendsPageCheckBox = new QCheckBox( i18n( "&Monitor friends page for updates" ),
	                                               accountGroupBox() );
	accountGroupBox()->layout()->addWidget( d->monitorFriendsPageCheckBox );
	QHBoxLayout * timeoutLayout = new QHBoxLayout( 0 );
	timeoutLayout->setSpacing( KDialog::spacingHint() );
	timeoutLayout->addItem( new QSpacerItem( KDialog::spacingHint(), 0, QSizePolicy::Fixed, QSizePolicy::Minimum ) );
	d->timeoutLabel = new QLabel( i18n( "Check interval" ), accountGroupBox() );
	timeoutLayout->addWidget( d->timeoutLabel );
	d->timeoutSpinBox = new KIntSpinBox( accountGroupBox() );
	d->timeoutSpinBox->setMinimum( 1 );
	d->timeoutSpinBox->setSuffix( i18n( " min" ) );
	timeoutLayout->addWidget( d->timeoutSpinBox );
	timeoutLayout->addStretch( 1 );
	accountGroupBox()->layout()->addItem( timeoutLayout );

	if( !account ) { //new account
		d->providerComboBox->setCurrentIndex( 3 );
		setupProvider();
		d->monitorFriendsPageCheckBox->setChecked( FALSE );
		d->timeoutSpinBox->setEnabled( FALSE );
		d->timeoutSpinBox->setValue( 10 );
	}
	else {
		if( account->serverName().contains( "aboutmylife.net", Qt::CaseInsensitive ) ) {
			d->providerComboBox->setCurrentIndex( 0 );
			serverNameLineEdit()->setEnabled( FALSE );
		}
		else if( account->serverName().contains( "deadjournal.com", Qt::CaseInsensitive ) ) {
			d->providerComboBox->setCurrentIndex( 1 );
			serverNameLineEdit()->setEnabled( FALSE );
		}
		else if( account->serverName().contains( "greatestjournal.com", Qt::CaseInsensitive ) ) {
			d->providerComboBox->setCurrentIndex( 2 );
			serverNameLineEdit()->setEnabled( FALSE );
		}
		else if( account->serverName().contains( "livejournal.com", Qt::CaseInsensitive ) ) {
			d->providerComboBox->setCurrentIndex( 3 );
			serverNameLineEdit()->setEnabled( FALSE );
		}
		else {
			d->providerComboBox->setCurrentIndex( 4 );
			serverNameLineEdit()->setEnabled( TRUE );
		}

		d->monitorFriendsPageCheckBox->setChecked( ljaccount->isCheckFriends() );
		d->timeoutSpinBox->setEnabled( ljaccount->isCheckFriends() );
		d->timeoutSpinBox->setValue( ljaccount->checkFriendsInterval() );
	}

	connect( d->providerComboBox, SIGNAL( activated( int ) ),
	         this, SLOT( setupProvider( void ) ) );
	connect( d->monitorFriendsPageCheckBox, SIGNAL( toggled( bool ) ),
	         d->timeoutSpinBox, SLOT( setEnabled( bool ) ) );
}

LJ::UI::EditAccountWidget::~EditAccountWidget( void )
{
	delete d;
}


void LJ::UI::EditAccountWidget::setServerAddress( const QString & serverName )
{
	if( useStandardAddressCheckBox()->isChecked() ) {
		if( serverName.isEmpty() ) {
			connectAddressLineEdit()->setText( 0 );
		}
		else {
			connectAddressLineEdit()->setText( "http://www." + serverName + "/cgi-bin/log.cgi" );
		}
	}
}

void LJ::UI::EditAccountWidget::setupProvider( void )
{
	switch( d->providerComboBox->currentIndex() ) {
	case 0:
		serverNameLineEdit()->setText( "aboutmylife.net" );
		break;
	case 1:
		serverNameLineEdit()->setText( "deadjournal.com" );
		break;
	case 2:
		serverNameLineEdit()->setText( "greatestjournal.com" );
		break;
	case 3:
		serverNameLineEdit()->setText( "livejournal.com" );
		break;
	}

	serverNameLineEdit()->setEnabled( d->providerComboBox->currentIndex() == 4 );
	
	if( d->providerComboBox->currentIndex() != 4 ) {
		setServerAddress( serverNameLineEdit()->text() );
	}
}

Blokkal::Account * LJ::UI::EditAccountWidget::apply( void )
{
	Blokkal::Account * account = GenericEditAccountWidget::apply();
	LiveJournalAccount * ljaccount = dynamic_cast<LiveJournalAccount*>( account );
	if( !ljaccount ) {
		kWarning() << k_funcinfo << "not editing a lj account!" << endl;
	}
	else {
		ljaccount->setCheckFriends( d->monitorFriendsPageCheckBox->isChecked() );
		ljaccount->setCheckFriendsInterval( d->timeoutSpinBox->value() );
	}
	return account;
}
