/***************************************************************************
 *   Copyright (C) 2006 by Martin Mller                                   *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_UIEDITACCOUNTWIDGET_H
#define LJ_UIEDITACCOUNTWIDGET_H

#include <blokkalui/genericeditaccountwidget.h>

class LiveJournalProtocol;

namespace LJ {
namespace UI {

/**
 * @short A widget for editing livejournal accounts.
 *
 * A widget for editing livejournal accounts.
 * @author Martin Mueller <orvio@orvio.de>
 */
class EditAccountWidget : public Blokkal::Ui::GenericEditAccountWidget
{
Q_OBJECT
public:
	/**
	 * Creates a new widget for editing @ref LiveJournalAccount objects.
	 *
	 * @param protocol owning protocol
	 * @param account account to edit, 0 to create a new account
	 * @param parent parent widget
	 */
	EditAccountWidget( LiveJournalProtocol * protocol,
	                              Blokkal::Account * account,
	                              QWidget *parent = 0 );
	virtual ~EditAccountWidget( void );

	/**
	 * Updates the account object.
	 * @return the updated account object
	 */
	virtual Blokkal::Account * apply( void );

protected slots:
	/**
	 * This slot is called whenever the content of the server name
	 * widget changes.
	 *
	 * @param serverName the new server name
	 */
	virtual void setServerAddress( const QString & serverName );

	/**
	 * Sets up the server name and address accordig to the currently selected
	 * provider.
	 */
	void setupProvider( void );

private:
	class Private;
	Private * d;
	
};
}
}
#endif
