/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljentryextensionwidget1.h"
#include "ljentryextensionwidget1.moc"

#include "../ljentry.h"
#include "../livejournalaccount.h"

#include <blokkalaccount.h>
#include <blokkalblog.h>
#include <blokkalui/editentrywidget.h>
#include <blokkalio/categorymanager.h>

#include <kdialog.h>
#include <kdebug.h>
#include <klocale.h>

#include <QCheckBox>
#include <QLayout>

class LJ::UI::EntryExtensionWidget1::Private {
public:
	Private( void ) :
	entry( 0 ){}

	LJ::Entry * entry;

	QHBoxLayout * layout;
	QCheckBox * backdatedCheckBox;
};

LJ::UI::EntryExtensionWidget1::EntryExtensionWidget1( Blokkal::Entry * entry, Blokkal::Ui::EditEntryWidget * parent ):
EntryExtensionWidget( entry, parent ),
d( new Private() )
{
	d->entry = dynamic_cast< LJ::Entry * >( entry );
	if( !d->entry) {
		kError() << "entry is not a LJ::Entry!" << endl;
		return;
	}

	LiveJournalAccount * ljaccount = dynamic_cast<LiveJournalAccount*>( d->entry->blog()->account() );
	if( !ljaccount ) {
		kError() << "ljentry does not belong to a ljaccount!" << endl;
		return;
	}	

	d->layout = new QHBoxLayout( this );
	d->layout->setMargin( 0 );
	
	d->backdatedCheckBox = new QCheckBox( i18n( "Backdated" ), this );
	d->layout->addWidget( d->backdatedCheckBox );
	if( d->entry->blog()->id() != ljaccount->userName() ) {
		d->backdatedCheckBox->hide();
	}

	d->layout->addStretch();
	
	rollBack();

	connect( d->backdatedCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( emitContentChanged( void ) ) );
}

LJ::UI::EntryExtensionWidget1::~EntryExtensionWidget1( void )
{
	delete d;
}

void LJ::UI::EntryExtensionWidget1::rollBack( void )
{
	d->backdatedCheckBox->setChecked( d->entry->backdated() );
}

void LJ::UI::EntryExtensionWidget1::apply( void )
{
	d->entry->setBackdated( d->backdatedCheckBox->isChecked() );
	
	if( d->entry->id() > 0 ) {
		d->entry->setRevnum( d->entry->revnum() + 1 );
		d->entry->setRevtime( QDateTime::currentDateTime().toTime_t() );
	}
}

void LJ::UI::EntryExtensionWidget1::postInit( void )
{
	connect( editEntryWidget(),
	         SIGNAL( newCategory( const QString& ) ),
	         SLOT( addNewTags(const QString & ) ) );
}

void LJ::UI::EntryExtensionWidget1::addNewTags( const QString & newCategoryString )
{
	QStringList newTags = newCategoryString.split( ',', QString::SkipEmptyParts );
	Blokkal::Io::CategoryManager * manager = entry()->blog()->account()->categoryManager();
	const QString blogId = entry()->blog()->id();
	for( QStringList::ConstIterator it = newTags.begin();
	     it != newTags.end();
	     ++it )
	{
		manager->category( blogId, (*it).trimmed() )->setName( (*it).trimmed() );
   }	
}
