/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljentryextensionwidget2.h"
#include "ljentryextensionwidget2.moc"

#include "../ljentry.h"

#include "ljsecurityselector.h"

#include <blokkalui/editentrywidget.h>

#include <kdebug.h>
#include <klocale.h>
#include <kdialog.h>

#include <qlayout.h>

#include <iostream>

class LJ::UI::EntryExtensionWidget2::Private {
public:
	Private( void ) :
	entry( 0 ){}

	LJ::Entry * entry;

	QHBoxLayout * layout;
	SecuritySelector * securitySelector;
};

LJ::UI::EntryExtensionWidget2::EntryExtensionWidget2( Blokkal::Entry * entry, Blokkal::Ui::EditEntryWidget * parent ):
EntryExtensionWidget( entry, parent ),
d( new Private() )
{
	d->entry = dynamic_cast< LJ::Entry * >( entry );
	if( !d->entry) {
		kWarning() << "LJ::UI::EntryExtensionWidget2::EntryExtensionWidget2() : entry is not a LJ::Entry!" << endl;
		return;
	}

	d->layout = new QHBoxLayout( this );
	d->layout->setMargin( 0 );
	d->securitySelector = new SecuritySelector( d->entry, this );
	d->layout->addWidget( d->securitySelector );

	//cannot set group list visibility here, because the widget may not have been constructed yet
	d->securitySelector->setCurrentLevel( d->entry->security() );
	
	connect( d->securitySelector, SIGNAL( activated( int ) ),
	         this, SLOT( emitContentChanged( void ) ) );
	connect( d->securitySelector, SIGNAL( activated( int ) ),
	         this, SLOT( updateGroupListVisibility( void ) ) );
}

LJ::UI::EntryExtensionWidget2::~EntryExtensionWidget2( void )
{
	delete d;
}

void LJ::UI::EntryExtensionWidget2::rollBack( void )
{
	d->securitySelector->setCurrentLevel( d->entry->security() );
	updateGroupListVisibility();
}

void LJ::UI::EntryExtensionWidget2::apply( void )
{
	d->entry->setSecurity( d->securitySelector->currentLevel() );
}

void LJ::UI::EntryExtensionWidget2::updateGroupListVisibility( void )
{
	editEntryWidget()->extensionWidget( Blokkal::Ui::EntryExtensionWidget::ExtensionWidget3 )->setHidden( d->securitySelector->currentLevel() != LJ::Entry::CustomLevel );
}

void LJ::UI::EntryExtensionWidget2::postInit( void )
{
	updateGroupListVisibility();
}

