/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljentryextensionwidget3.h"
#include "ljentryextensionwidget3.moc"

#include "ljfriendgroupselector.h"

#include "../ljentry.h"
#include "../livejournalaccount.h"

#include <blokkalblog.h>

#include <kdebug.h>
#include <klocale.h>
#include <kdialog.h>

#include <qlayout.h>

#include <iostream>

class LJ::UI::EntryExtensionWidget3::Private {
public:
	Private( void ) :
	entry( 0 ){}

	LJ::Entry * entry;

	QVBoxLayout * layout;
	FriendGroupSelector * friendGroupSelector;
};

LJ::UI::EntryExtensionWidget3::EntryExtensionWidget3( Blokkal::Entry * entry, Blokkal::Ui::EditEntryWidget * parent ):
EntryExtensionWidget( entry, parent ),
d( new Private() )
{
	d->entry = dynamic_cast< LJ::Entry * >( entry );
	if( !d->entry) {
		kWarning() << "LJ::UI::EntryExtensionWidget3::EntryExtensionWidget3() : entry is not a LJ::Entry!" << endl;
		return;
	}

	LiveJournalAccount * ljaccount = dynamic_cast<LiveJournalAccount*>( d->entry->blog()->account() );
	if( !ljaccount ) {
		kError() << "LJ::UI::EntryExtensionWidget3::EntryExtensionWidget3() : entry does not belong to a ljaccount!" << endl;
		return;
	}

	d->layout = new QVBoxLayout( this );
	d->layout->setMargin( 0 );
	d->friendGroupSelector = new FriendGroupSelector( ljaccount, this );
	d->friendGroupSelector->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Expanding );
	d->layout->addWidget( d->friendGroupSelector );

	rollBack();
	
	connect( d->friendGroupSelector,
	         SIGNAL( groupSelectionChanged( void ) ),
	         SLOT( emitContentChanged( void ) ) );
}

LJ::UI::EntryExtensionWidget3::~EntryExtensionWidget3( void )
{
	delete d;
}

void LJ::UI::EntryExtensionWidget3::rollBack( void )
{
	d->friendGroupSelector->setGroupSelection( d->entry->allowMask() );
}

void LJ::UI::EntryExtensionWidget3::apply( void )
{
	d->entry->setAllowMask( d->friendGroupSelector->groupSelection() );
}

