/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljentryextensionwidget4.h"
#include "ljentryextensionwidget4.moc"

#include "../ljentry.h"
#include "../livejournalaccount.h"
#include "../livejournalblog.h"
#include "ljusericonselector.h"
#include "ljmoodselector.h"
#include "ljcommentsettingsselector.h"

#include <blokkalui/mediainfocombobox.h>
#include <blokkalui/htmlentrytextedit.h>

#include <klocale.h>
#include <kdebug.h>
#include <kdialog.h>
#include <klineedit.h>

#include <QLayout>
#include <QLabel>
#include <QCheckBox>

class LJ::UI::EntryExtensionWidget4::Private {
public:
	Private( void ) :
	entry( 0 ){}

	LJ::Entry * entry;

	QGridLayout * layout;

	UserIconSelector * userIconSelector;
	Blokkal::Ui::MediaInfoComboBox * musicComboBox;
	KLineEdit * locationLineEdit;
	
	MoodSelector * moodSelector;
	CommentSettingsSelector * commentSettingsSelector;
	QCheckBox * noAutoFormatCheckBox;
	AdultSettingsSelector * adultSettingsSelector;
};

LJ::UI::EntryExtensionWidget4::EntryExtensionWidget4( Blokkal::Entry * entry, Blokkal::Ui::EditEntryWidget * parent ):
EntryExtensionWidget( entry, parent ),
d( new Private() )
{
	d->entry = dynamic_cast< LJ::Entry * >( entry );
	if( !d->entry) {
		kWarning() << "entry is not a LJ::Entry!" << endl;
		return;
	}

	LiveJournalBlog * ljblog = dynamic_cast<LiveJournalBlog*>( d->entry->blog() );
	if( !ljblog ) {
		kError() << "entry does not belong to a ljblog!" << endl;
		return;
	}
	
	LiveJournalAccount * ljaccount = dynamic_cast<LiveJournalAccount*>( ljblog->account() );
	if( !ljaccount ) {
		kError() << "entry does not belong to a ljaccount!" << endl;
		return;
	}

	d->layout = new QGridLayout( this );
	d->layout->setMargin( 0 );

	//user icons
	QLabel * userIconLabel = new QLabel( i18n( "User Icon" ), this );
	d->layout->addWidget( userIconLabel, 0, 0 );
	d->userIconSelector = new UserIconSelector( ljaccount, this );
	userIconLabel->setBuddy( d->userIconSelector );
	d->layout->addWidget( d->userIconSelector, 0, 1 );
	
	//media info
	QLabel * musicLabel = new QLabel( i18n( "Music" ), this );
	d->layout->addWidget( musicLabel, 1, 0 );
	d->musicComboBox = new Blokkal::Ui::MediaInfoComboBox( this );
	d->layout->addWidget( d->musicComboBox, 1, 1 );

	QHBoxLayout * bottomLayout = new QHBoxLayout;
	d->layout->addLayout( bottomLayout, 2, 1, -1, -1 );
	//location
	QLabel * locationLabel = new QLabel( i18n( "Location" ), this );
	d->layout->addWidget( locationLabel, 2, 0 );
	d->locationLineEdit = new KLineEdit( this );
	locationLabel->setBuddy( d->locationLineEdit );
	bottomLayout->addWidget( d->locationLineEdit );

	//moods
	QLabel * moodsLabel = new QLabel( i18n( "Mood" ), this );
	d->layout->addWidget( moodsLabel, 0, 2 );
	d->moodSelector = new MoodSelector( ljaccount, this );
	moodsLabel->setBuddy( d->moodSelector );
	d->layout->addWidget( d->moodSelector, 0, 3 );
	
	//comment settings
	QLabel * commentSettingsLabel = new QLabel( i18n( "Comments" ), this );
	d->layout->addWidget( commentSettingsLabel, 1, 2 );
	d->commentSettingsSelector = new CommentSettingsSelector( this );
	commentSettingsLabel->setBuddy( d->commentSettingsSelector );
	d->layout->addWidget( d->commentSettingsSelector, 1, 3 );

	//adult content
	QLabel * adultSettingsLabel = new QLabel( i18n( "Adult Content" ), this );
	bottomLayout->addWidget( adultSettingsLabel );
	d->adultSettingsSelector = new AdultSettingsSelector( this );
	adultSettingsLabel->setBuddy( d->adultSettingsSelector );
	bottomLayout->addWidget( d->adultSettingsSelector );
	
	d->noAutoFormatCheckBox = new QCheckBox( i18n( "Don't Auto Format" ), this );
	bottomLayout->addWidget( d->noAutoFormatCheckBox );

	d->layout->setColumnStretch( 1, 1 );

	rollBack();

	connect( d->userIconSelector,
	         SIGNAL( activated( int ) ),
	         SLOT( emitContentChanged( void ) ) );
	connect( d->moodSelector,
	         SIGNAL( moodChanged( void ) ),
	         SLOT( emitContentChanged( void ) ) );
	connect( d->commentSettingsSelector,
	         SIGNAL( settingsChanged( void ) ),
	         SLOT( emitContentChanged( void ) ) );
	connect( d->musicComboBox,
	         SIGNAL( editTextChanged ( const QString & ) ),
	         SLOT( emitContentChanged( void ) ) );
	connect( d->musicComboBox,
	         SIGNAL( activated( int ) ),
	         SLOT( emitContentChanged( void ) ) );
	connect( d->locationLineEdit,
	         SIGNAL( textChanged ( const QString & ) ),
	         SLOT( emitContentChanged( void ) ) );
	connect( d->noAutoFormatCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( emitContentChanged( void ) ) );
	connect( d->adultSettingsSelector,
	         SIGNAL( activated( int ) ),
	         SLOT( emitContentChanged( void ) ) );
}

LJ::UI::EntryExtensionWidget4::~EntryExtensionWidget4( void )
{
	delete d;
}

void LJ::UI::EntryExtensionWidget4::rollBack( void )
{
	if( !d->entry->userIcon().isEmpty() ) {
		d->userIconSelector->setCurrentItem( d->entry->userIcon() );
	}
	d->moodSelector->setMoodID( d->entry->currentMoodID() );
	d->moodSelector->setMoodName( d->entry->currentMood() );
	d->commentSettingsSelector->setCommentSettings( d->entry->commentSettings() );
	d->commentSettingsSelector->setScreeningSettings( d->entry->screening() );
	//if this entry has some text for current music, set it here
	//this will obviously not be the case, if this entry is a newly
	//created one.
	//If this entry has an id > 0 and has therefore been loaded from the server
	//set the content of the music combo box to the current music value
	//regardless of it's content.
	//Now what is the difference? An entry may have an id == 0 and still
	//some content in the current music field if some crossposting entry editing
	//is going on in the/some view.
	if( !d->entry->currentMusic().isNull() || d->entry->uintId() > 0 ) {
		d->musicComboBox->setCurrentIndex( 0 );
		d->musicComboBox->setItemText( 0, d->entry->currentMusic() );
	}
	d->locationLineEdit->setText( d->entry->currentLocation() );
	d->adultSettingsSelector->setAdultContent( d->entry->adultContent() );
	d->noAutoFormatCheckBox->setChecked( d->entry->preformatted() );
}

void LJ::UI::EntryExtensionWidget4::apply( void )
{
	d->entry->setUserIcon( d->userIconSelector->keyword() );
	d->entry->setCurrentMood( d->moodSelector->moodName() );
	d->entry->setCurrentMoodID( d->moodSelector->moodID() );
	d->entry->setCommentSettings( d->commentSettingsSelector->commentSettings() );
	d->entry->setScreening( d->commentSettingsSelector->screeningSettings() );
	d->entry->setCurrentMusic( d->musicComboBox->currentText() );
	d->entry->setCurrentLocation( d->locationLineEdit->text() );
	d->entry->setAdultContent( d->adultSettingsSelector->adultContent() );
	d->entry->setPreformatted( d->noAutoFormatCheckBox->isChecked() );
}

//---------------------
//AdultSettingsSelector
//---------------------
LJ::UI::AdultSettingsSelector::AdultSettingsSelector( QWidget * parent ):
KComboBox( parent )
{
	addItem( i18n( "Default" ) );
	addItem( i18n( "No Adult Content" ) );
	addItem( i18n( "Adult Concepts" ) );
	addItem( i18n( "Explicit Adult Content" ) );
	setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
}

LJ::UI::AdultSettingsSelector::~AdultSettingsSelector( void )
{
}

LJ::Entry::AdultContent LJ::UI::AdultSettingsSelector::adultContent( void ) const
{
	switch( currentIndex() ) {
	case Entry::AdultDefault:
		return Entry::AdultDefault;
		break;
	case Entry::AdultNone:
		return Entry::AdultNone;
		break;
	case Entry::AdultConcepts:
		return Entry::AdultConcepts;
		break;
	case Entry::AdultExplicit:
		return Entry::AdultExplicit;
		break;
	};

	return Entry::AdultDefault;
}

void LJ::UI::AdultSettingsSelector::setAdultContent( LJ::Entry::AdultContent content )
{
	switch( content ) {
	case Entry::AdultDefault:
		setCurrentIndex( content );
		break;
	case Entry::AdultNone:
		setCurrentIndex( content );
		break;
	case Entry::AdultConcepts:
		setCurrentIndex( content );
		break;
	case Entry::AdultExplicit:
		setCurrentIndex( content );
		break;
	}
}

