/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljformattingbackend.h"
#include "ljformattingbackend.moc"

#include "../livejournalprotocol.h"
#include <blokkalui/editentrywidget.h>

#include <klocale.h>
#include <kaction.h>
#include <kinputdialog.h>
#include <kactionmenu.h>
#include <kactioncollection.h>
#include <kdebug.h>

#include <QValidator>
#include <QRegExp>

LJ::UI::FormattingBackend::FormattingBackend( LiveJournalProtocol * protocol,
                                              Blokkal::Ui::EditEntryWidget * widget ):
Blokkal::Ui::FormattingBackend( widget )
{
	setComponentData( protocol->componentData() );
	setXMLFile( "blokkal_ljeditentry.rc" );

	KActionMenu * ljTagsMenu = new KActionMenu( KIcon("blokkal_lj_tags" ),
	                                            i18n( "LiveJournal &Tags" ),
	                                            actionCollection() );
	actionCollection()->addAction( "edit_ljtags", ljTagsMenu );

	//LJ Cut
	KAction * action =  new KAction( KIcon("blokkal_lj_cut"),
	                                 i18n( "Live&Journal Cut" ),
	                                 actionCollection() );
	action->setShortcut( i18n( "Ctrl+J" ) );
	connect( action,
	         SIGNAL( triggered (Qt::MouseButtons, Qt::KeyboardModifiers ) ),
	         SLOT( insertLjCut( void ) ) );
	ljTagsMenu->addAction( action );
	actionCollection()->addAction( "edit_ljcut", action );

	//Lj User
	action =  new KAction( KIcon("blokkal_lj_user"),
	                       i18n( "Link To User" ),
	                       actionCollection() );
	connect( action,
	         SIGNAL( triggered (Qt::MouseButtons, Qt::KeyboardModifiers ) ),
	         SLOT( insertLjUserLink( void ) ) );
	ljTagsMenu->addAction( action );
	actionCollection()->addAction( "edit_ljuser", action );

	//LJ Raw
	action =  new KAction( KIcon("blokkal_lj_raw"),
	                       i18n( "Raw Text" ),
	                       actionCollection() );
	connect( action,
	         SIGNAL( triggered (Qt::MouseButtons, Qt::KeyboardModifiers ) ),
	         SLOT( insertLjRawTags( void ) ) );
	ljTagsMenu->addAction( action );
	actionCollection()->addAction( "edit_ljraw", action );
}

LJ::UI::FormattingBackend::~FormattingBackend( void )
{
}

QString & LJ::UI::FormattingBackend::formatForPreview( QString & richText ) const
{
	QRegExp rawStartExp( "<[\\s]*(lj-raw)[\\s]*>" );
	QRegExp rawEndExp( "<[\\s]*(/lj-raw)[\\s]*>" );
	int rawStartOffset = 0;
	int rawEndOffset = 0;
	QStringList segments;
	while( ( rawStartOffset = rawStartExp.indexIn( richText, rawEndOffset ) ) != -1 ) {
		segments.append( formatIgnoreRaw( richText.mid( rawEndOffset, rawStartOffset - rawEndOffset ) ) );
		rawEndOffset = rawEndExp.indexIn( richText, rawStartOffset + rawStartExp.matchedLength() );
		if( rawEndOffset != -1 ) {
			segments.append( richText.mid( rawStartOffset,
			                               rawEndOffset + rawEndExp.matchedLength() - rawStartOffset ) );
			rawEndOffset += rawEndExp.matchedLength();
		}
		else {
			segments.append( richText.mid( rawStartOffset ) );
			rawEndOffset = richText.length();
			break; //raw area coninues to the end of the entry
		}
	}

	if( rawEndOffset < richText.length() ) {
		segments.append( formatIgnoreRaw( richText.mid( rawEndOffset ) ) );
	}

	//stitch everything together again
	richText.clear();
	for( int i = 0; i < segments.size(); ++i ) {
		richText.append( segments[i] );
	}

	return richText;
}

void LJ::UI::FormattingBackend::insertLjCut( void )
{
	QString cutName = KInputDialog::getText( i18n("Create LJ Cut"),
	                                         i18n("Enter cut name:"),
	                                         i18n("Read More..."),
	                                         0,
	                                         editEntryWidget() );
	QString cutEnd = "</lj-cut>";
	if( !cutName.isNull() ) {
		if( cutName.isEmpty() ) {
			frameSelection( "<lj-cut>", cutEnd );
		}
		else {
			frameSelection( "<lj-cut text=\"" + cutName + "\">", cutEnd );
		} 

	}
	//else means aborted, don't do anything	
}

void LJ::UI::FormattingBackend::insertLjUserLink( void )
{
	QRegExpValidator * validator = new QRegExpValidator( QRegExp( ".+" ), this );
	QString userName = KInputDialog::getText( i18n( "Link to other user" ),
	                                          i18n( "Enter user name:" ),
	                                          QString::null,
	                                          0,
	                                          editEntryWidget(),
	                                          validator );
	if( !userName.isEmpty() ) {
		insertTextAtCursor( "<lj user=\"" + userName + "\">" );
	}	
}

void LJ::UI::FormattingBackend::insertLjRawTags( void )
{
	frameSelection( QString::fromLatin1( "<lj-raw>" ),
	                QString::fromLatin1( "</lj-raw>" ) );
}


QString LJ::UI::FormattingBackend::formatIgnoreRaw( QString richText ) const
{
	QRegExp cutStartExp( "<[\\s]*(lj-cut)(([\\s]*)|([\\s]+text=\\\"[\\d\\w\\s.,!\\-]*\\\"[\\s]*))>" );
	QRegExp cutEndExp( "<[\\s]*(/lj-cut)[\\s]*>" );
	int cutStartOffset = 0;
	int cutEndOffset = 0;
	while( ( cutStartOffset = cutStartExp.indexIn( richText, cutEndOffset ) ) != -1 ) {
		cutEndOffset = cutEndExp.indexIn( richText, cutStartOffset + cutStartExp.matchedLength() );
		if( cutEndOffset != -1 ) {
			richText.replace( cutEndOffset,
			                  cutEndExp.matchedLength(),
			                  QString::fromLatin1( "</div>") );
		}

		richText.replace( cutStartOffset,
		                  cutStartExp.matchedLength(),
		                  QString::fromLatin1( "<div style=\"color : %1\">").arg( editEntryWidget()->palette().color( QPalette::Disabled,QPalette::Text).name() ) );
		
		if( cutEndOffset == -1 ) {
			break; //cut area coninues to the end of the entry
		}
	}

	if( cutEndOffset == 0 ) {
		cutEndOffset = cutEndExp.indexIn( richText );
		if( cutEndOffset != -1 ) {
			richText.replace( cutEndOffset,
			                  cutEndExp.matchedLength(),
			                  QString::fromLatin1( "</div>") );
		}
	}

	QRegExp userLinkExp( "<[\\s]*(lj)[\\s]*(user=\\\"[_\\d\\w]+\\\")[\\s]*>" );
	QRegExp userNameExp( "\\\"[_\\d\\w]+\\\"" );
	int userOffset = 0;
	while( ( userOffset = userLinkExp.indexIn( richText, userOffset ) ) != -1 ) {
		if( userNameExp.indexIn( userLinkExp.cap() ) != -1 ) {
			QString userName = userNameExp.cap();
			userName.remove( 0, 1 );
			userName.remove( userName.length() - 1, 1 );
			richText.replace( userOffset,
			                  userLinkExp.matchedLength(),
			                  QString::fromLatin1( "<a href=\"\"><b>%1</b></a>" ).arg( userName ) );
		}
		userOffset += userLinkExp.matchedLength();
	}
	
	return Blokkal::Ui::FormattingBackend::formatForPreview( richText );
}
