/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_UILJFORMATTINGBACKEND_H
#define LJ_UILJFORMATTINGBACKEND_H

#include <blokkalui/formattingbackend.h>

class LiveJournalProtocol;

namespace LJ {

namespace UI {

/**
 * This class provides actions for the livejournal
 * specific tags and emulates the autoformatting feature
 * for entries in formatForPreview()
 * @author Martin Mueller <orvio@orvio.de>
 */
class FormattingBackend : public Blokkal::Ui::FormattingBackend
{
Q_OBJECT
public:
	/**
	 * Constructor.
	 * @param widget owning EditEntryWidget
	 */
	FormattingBackend( LiveJournalProtocol * protocol,
	                   Blokkal::Ui::EditEntryWidget * widget );

	/**
	 * Destructor
	 */
	virtual ~FormattingBackend( void );

	/**
	 * Applys formatting for some special livejournal tags.
	 * @param richText rich text string you may apply special formatting to
	 * @return a reference to @a richText
	 */
	virtual QString & formatForPreview( QString & richText ) const;

public slots:
	/**
	 * Puts a LJ-cut around the current selection, or inserts the tag at
	 * the current cursor position if nothing is selected.
	 * This slot will pop up a dialog asking for the name of the cut.
	 */
	void insertLjCut( void );

	/**
	 * Inserts a link to another LJ-user at the current cursor position.
	 * This slot will pop up a dialog asking for the user's name.
	 */
	void insertLjUserLink( void );

	/**
	 * Puts "<lj-raw>" tags around the current selection, or inserts the tags at
	 * the current cursor position if nothing is selected.
	 */
	void insertLjRawTags( void );

	/**
	 * Formats @a richText for preview ignoring any "<lj-raw>" tags
	 */
	QString formatIgnoreRaw( QString richText ) const;

};

}

}

#endif
