/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljfriendgroupselector.h"
#include "ljfriendgroupselector.moc"

#include "../ljio/friendgroupsmanager.h"
#include "../livejournalaccount.h"

#include <kdebug.h>
#include <klocale.h>
#include <kdialog.h>

#include <qpainter.h>

#include <iostream>
#include <math.h>

//-------------------
//FriendGroupDelegate
//-------------------
class LJ::UI::FriendGroupDelegate::Private {
public:
	Private( void ) :
	signalProxy( 0 ){}

	FriendGroupSelector * signalProxy;
};

LJ::UI::FriendGroupDelegate::FriendGroupDelegate( void ):
Blokkal::Ui::CheckBoxDelegate( ),
d( new Private() )
{
}

LJ::UI::FriendGroupDelegate::~FriendGroupDelegate( void )
{
	delete d;
}

void LJ::UI::FriendGroupDelegate::paint( QPainter * painter, const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
	CheckBoxDelegate::paint( painter, option, index );
	painter->save();

	LJ::IO::FriendGroup * friendGroup = dynamic_cast<LJ::IO::FriendGroup*>( (QObject*) index.data().toULongLong() );
	if( friendGroup ) {
		QString name = friendGroup->isPublic() ?  i18n( "%1 (public)", friendGroup->groupName() ) : friendGroup->groupName();
		QRect textRect;
		layoutPlainItem( option, index, textRect );
		if( textRect.isValid() ) {
			drawDisplay( painter, option, textRect, name );
		}
	}
	
	painter->restore();
}

QSize LJ::UI::FriendGroupDelegate::sizeHint( const QStyleOptionViewItem & option, const QModelIndex & index ) const
{
	LJ::IO::FriendGroup * friendGroup = dynamic_cast<LJ::IO::FriendGroup*>( (QObject*) index.data().toULongLong() );
	if( friendGroup ) {
		QRect textRect;
		layoutPlainItem( option, index, textRect );
		return textRect.size();
	}
		
	return QSize();
}

void LJ::UI::FriendGroupDelegate::stateChanged( QAbstractItemModel * model,
                                                const QModelIndex & index,
                                                Qt::CheckState newState )
{
	Q_UNUSED( model );
	Q_UNUSED( index );
	Q_UNUSED( newState );
	
	if( d->signalProxy ) {
		emit d->signalProxy->groupSelectionChanged();
	}
}

//-------------------
//FriendGroupSelector
//--------------------
class LJ::UI::FriendGroupSelector::Private{
public:
	Private( void ) {}
};

LJ::UI::FriendGroupSelector::FriendGroupSelector( LiveJournalAccount * account, QWidget * parent ):
Blokkal::Ui::CheckBoxView( new FriendGroupDelegate(),parent ),
d( new Private() )
{
	setSelectionMode( NoSelection );
	dynamic_cast<FriendGroupDelegate*>(itemDelegate())->d->signalProxy = this;
	setModel( new FriendGroupModel( account, this ) );
	setRootIsDecorated( FALSE );
}

LJ::UI::FriendGroupSelector::~FriendGroupSelector( void )
{
	delete d;
}

void LJ::UI::FriendGroupSelector::setGroupSelection( unsigned long allowMask )
{
	
	//the groupmask is a 32bit Integer with each bit indicating
	//one friend group. Bit 0 means friends is general, Bit 31
	//is reserved.

	for( int row = 0; row < model()->rowCount(); ++row ) {
		QModelIndex index = model()->index( row, 0 );
		LJ::IO::FriendGroup * currentGroup = dynamic_cast<LJ::IO::FriendGroup*>((QObject*)index.data().toULongLong());
		if( !currentGroup ) {
			kError() << "invalid data type encountered!" << endl;
			continue;
		}

		setChecked( index, allowMask & (unsigned long) pow( 2, currentGroup->groupID() ) );
	}
}

unsigned long LJ::UI::FriendGroupSelector::groupSelection( void )
{
	unsigned long allowmask = 0;
	for( int row = 0; row < model()->rowCount(); ++row ) {
		QModelIndex index = model()->index( row, 0 );
		LJ::IO::FriendGroup * currentGroup = dynamic_cast<LJ::IO::FriendGroup*>((QObject*)index.data().toULongLong());
		if( !currentGroup ) {
			kError() << "invalid data type encountered!" << endl;
			continue;
		}

		if( isChecked( index ) ) {
			allowmask += ( unsigned long ) pow( 2, currentGroup->groupID() );
		}
	}

	return allowmask;
}

//----------------
//FriendGroupModel
//----------------
LJ::UI::FriendGroupModel::FriendGroupModel( LiveJournalAccount * account, QObject * parent ):
TreeModel( new TreeModel::TreeItemNode( 0, 0 ), parent )
{
	QList< LJ::IO::FriendGroup * > groupList = account->friendGroupsManager()->groups();

	if( groupList.isEmpty() ) {
		return;
	}

	for( QList< LJ::IO::FriendGroup *>::ConstIterator it = groupList.begin();
	     it != groupList.end();
	     ++it )
	{
		addGroup( *it );
	}

	connect( account->friendGroupsManager(),
	         SIGNAL( groupAdded( LJ::IO::FriendGroup * ) ),
	         SLOT( addGroup( LJ::IO::FriendGroup * ) ) );
	connect( account->friendGroupsManager(),
	         SIGNAL( groupRemoved( LJ::IO::FriendGroup * ) ),
	         SLOT( removeGroup( LJ::IO::FriendGroup * ) ) );
}

LJ::UI::FriendGroupModel::~FriendGroupModel( void )
{}

void LJ::UI::FriendGroupModel::addGroup( LJ::IO::FriendGroup * friendGroup )
{
	if( !friendGroup ) {
		return;
	}

	int index = 0;
	for( ; index < root()->children.size(); ++index ) {
		TreeItemNode * groupNode = root()->children.at( index );
		LJ::IO::FriendGroup * currentGroup = dynamic_cast<LJ::IO::FriendGroup*>((QObject*)groupNode->data.toULongLong());
		if( !currentGroup ) {
			kError() << "invalid data type encountered!" << endl;
			continue;
		}
		if( currentGroup->sortorder() > friendGroup->sortorder() ) {
			beginInsertRows( QModelIndex(), index, index );
			TreeItemNode * newNode = new TreeItemNode( root(), friendGroup );
			//children are appended to the list
			//but we need to heed the sortorder
			//so the item last is removed, which will always
			//be newNode and then inserted again at index
			root()->children.removeLast();
			root()->children.insert( index, newNode );
			endInsertRows();
			connect( friendGroup,
			         SIGNAL( groupModified( LJ::IO::FriendGroup* ) ),
			         SLOT( emitDataChanged( LJ::IO::FriendGroup* ) ) );
			return;
		}
	}

	beginInsertRows( QModelIndex(),
	                 root()->children.size(),
	                 root()->children.size() );
	new TreeItemNode( root(), friendGroup );
	endInsertRows();

	connect( friendGroup,
	         SIGNAL( groupModified( LJ::IO::FriendGroup* ) ),
	         SLOT( emitDataChanged( LJ::IO::FriendGroup* ) ) );
}

void LJ::UI::FriendGroupModel::removeGroup( LJ::IO::FriendGroup * friendGroup )
{
	TreeItemNode * groupNode;
	for( int i = 0; i != root()->children.size(); ++i ) {
		groupNode = root()->children.at(i);
		if( ((LJ::IO::FriendGroup *)groupNode->data.toULongLong() ) == friendGroup ) {
			break;
		}
		groupNode = 0;
	}

	if( !groupNode ) {
		return;
	}

	int index = root()->children.indexOf( groupNode );
	beginRemoveRows( QModelIndex(),
	                 index,
	                 index );
	delete root()->children.takeAt( index );
	endRemoveRows();
}

void LJ::UI::FriendGroupModel::emitDataChanged( LJ::IO::FriendGroup * friendGroup )
{
	TreeItemNode * node;
	for( int i = 0; i != root()->children.size(); ++i ) {
		node = root()->children.at(i);
		if( ((LJ::IO::FriendGroup *)node->data.toULongLong() ) == friendGroup ) {
			QModelIndex index = createIndex( node->parent->children.indexOf( node ),
			                                 0,
			                                 node );
			emit dataChanged( index, index );
			break;
		}

		node = 0;
	}

	if( !node ) {
		return;
	}

	//maybe the sortorder has changed
	int index = 0;
	for( ; index < root()->children.size(); ++index ) {
		TreeItemNode * groupNode = root()->children.at( index );
		if( groupNode == node ) {
			continue;
		}
		
		LJ::IO::FriendGroup * currentGroup = dynamic_cast<LJ::IO::FriendGroup*>((QObject*)groupNode->data.toULongLong());
		if( !currentGroup ) {
			kError() << "invalid data type encountered!" << endl;
			continue;
		}
		if( currentGroup->sortorder() > friendGroup->sortorder() ) {
			break;
		}
	}

	int oldIndex = root()->children.indexOf( node );
	if( ( !index && oldIndex != index )
	    || oldIndex != ( index - 1 ) )
	{
		emit layoutAboutToBeChanged();
		QModelIndexList oldIndexList;
		QModelIndexList newIndexList;
		if( oldIndex < index ) {
			for( int i = oldIndex; i < index; ++i ) {
				oldIndexList.append( this->index( i, 0 ) );
			}
			root()->children.takeAt( oldIndex );
			root()->children.insert( index - 1 , node );
			newIndexList.append( createIndex( index - 1, 0, node ) );
			for( int i = oldIndex; i < index - 1; ++i ) {
				newIndexList.append( createIndex( i, 0, root()->children.at( i ) ) );
			}
		}
		else {
			for( int i = index; i <= oldIndex; ++i ) {
				oldIndexList.append( this->index( i, 0 ) );
			}
			root()->children.takeAt( oldIndex );
			root()->children.insert( index , node );
			for( int i = index + 1; i <= oldIndex; ++i ) {
				newIndexList.append( createIndex( i, 0, root()->children.at( i ) ) );
			}
			newIndexList.append( createIndex( index, 0, node) );
		}
		changePersistentIndexList( oldIndexList, newIndexList );
		emit layoutChanged();
	}
}
