/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_UIFRIENDGROUPSELECTOR_H
#define LJ_UIFRIENDGROUPSELECTOR_H

#include <blokkaltreemodel.h>
#include <blokkalui/checkboxview.h>
#include <blokkalui/checkboxdelegate.h>

class LiveJournalAccount;

namespace LJ {
namespace IO {
class FriendGroup;
}

namespace UI {
/**
 * @short A widget for selecting friend groups.
 *
 * A widget for selecting friend groups.
 * @author Martin Mueller <orvio@orvio.de>
*/
class FriendGroupSelector : public Blokkal::Ui::CheckBoxView
{
Q_OBJECT
public:
	/**
	 * Creates a new FriendGroupSelector.
	 * @param account use the data of this account
	 * @param parent parent widget
	 */
	FriendGroupSelector( LiveJournalAccount * account, QWidget * parent = 0 );
	virtual ~FriendGroupSelector( void );

	/**
	 * Sets the selected groups.
	 * @param allowMask select the groups according to this allow mask
	 */
	void setGroupSelection( unsigned long allowMask );

	/**
	 * Returns the allowmask representing the current group selection.
	 * This value is calculated as documented in the livejournal protocol
	 * documentation and can be fed into LJ::Entry::setAllowMask().
	 * @return the allow mask representing the current group selection.
	 */
	unsigned long groupSelection( void );

Q_SIGNALS:
	void groupSelectionChanged( void );

private:
	friend class FriendGroupDelegate;
	class Private;
	Private * d;
};

class FriendGroupDelegate : public Blokkal::Ui::CheckBoxDelegate {
Q_OBJECT
public:
	FriendGroupDelegate( void );
	virtual ~FriendGroupDelegate( void );
	/**
	 * Paints the item
	 */
	virtual void paint( QPainter * painter,
	                    const QStyleOptionViewItem & option,
	                    const QModelIndex &index) const;

	/**
	 * Returns the size hint
	 */
	virtual QSize sizeHint(const QStyleOptionViewItem & option,
	                       const QModelIndex & index) const;
	
protected:
	/**
	 * This method is called when the checkstate of @a index
	 * has changed to @a newState.
	 */
	virtual void stateChanged( QAbstractItemModel * model,
	                           const QModelIndex & index,
	                           Qt::CheckState newState );

private:
	friend class FriendGroupSelector;
	class Private;
	Private * const d;
};

class FriendGroupModel : public Blokkal::TreeModel {
Q_OBJECT
public:
	/**
	 * Constructor
	 * @param account use the data of this account
	 * @param parent parent widget
	 */
	FriendGroupModel( LiveJournalAccount * account, QObject * parent = 0 );
	virtual ~FriendGroupModel( void );
	
private Q_SLOTS:
	void addGroup( LJ::IO::FriendGroup * friendGroup );
	void removeGroup( LJ::IO::FriendGroup * friendGroup );
	void emitDataChanged( LJ::IO::FriendGroup * friendGroup );	
};

}

}

#endif
