/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljfriendslistview.h"
#include "ljfriendslistview.moc"

#include "../livejournalaccount.h"

#include "../ljio/friendinfomanager.h"
#include "../ljio/friendgroupsmanager.h"

#include <klocale.h>
#include <kiconloader.h>
#include <kdebug.h>

#include <QMap>
#include <QSortFilterProxyModel>
#include <QHeaderView>

//----------------
//FriendsListModel
//----------------
class LJ::UI::FriendsListModel::Private {
public:
	Private( const LiveJournalAccount * account, FriendsListView * view ) :
	account( account ),
	view( view ),
	useColors( FALSE ),
	userIcon( SmallIcon( "blokkal_lj_user" ) ),
	communityIcon( SmallIcon( "blokkal_lj_community" ) ),
	syndicatedIcon( SmallIcon( "application-rss+xml" ) ),
	friendIcon( SmallIcon( "blokkal_lj_friend" ) ),
	friendOfIcon( SmallIcon( "blokkal_lj_friendof" ) ),
	mutualIcon( SmallIcon( "blokkal_lj_mutualfriend" ) ) {}

	const LiveJournalAccount * const account;
	const FriendsListView * const view;	
	bool useColors;
	
	QPixmap userIcon;
	QPixmap communityIcon;
	QPixmap syndicatedIcon;

	QPixmap friendIcon;
	QPixmap friendOfIcon;
	QPixmap mutualIcon;
	
	QMap<QString, QPair<LJ::IO::FriendInfo*, LJ::IO::FriendOfInfo*> > userInfoMap;
	QStringList userList; //used to ensure consistent order
	
	QString groupsList( unsigned long groupmask ) const
	{
		//if groupmask == 1 -> usual friend, no group
		//if groupmask == 0 -> should not happen, but no group anyway
		if( groupmask < 2 ) {
			return 0;
		}
 
		QString groupsString;
		unsigned int groupId = 1;
		LJ::IO::FriendGroupsManager * manager = account->friendGroupsManager();
 
		for( groupmask /= 2;//Skip Bit 0
		     groupmask > 0;
		     groupmask /= 2, ++groupId )
		{
			if( groupmask % 2 == 1 ) { //current group ID bit is set
			LJ::IO::FriendGroup * group = manager->group( groupId );
				if( groupId ) {
					groupsString.append( "<li>" +  group->groupName() + "</li>" );
				}
			}
		}
		return groupsString;
	}
};

LJ::UI::FriendsListModel::FriendsListModel( const LiveJournalAccount * account,
                                            FriendsListView * parent ) :
QAbstractItemModel( parent ),
d( new Private( account, parent ) )
{
	LJ::IO::FriendInfoManager * const manager = account->friendInfoManager();
	QList< LJ::IO::FriendInfo * > friends = manager->friends();
	for( QList< LJ::IO::FriendInfo * >::ConstIterator it = friends.begin();
	     it != friends.end();
	     ++it )
	{
		addFriend( *it );
	}

	QList< LJ::IO::FriendOfInfo * > friendOfs = manager->friendOfs();
	for( QList< LJ::IO::FriendOfInfo * >::ConstIterator it = friendOfs.begin();
	     it != friendOfs.end();
	     ++it )
	{
		addFriendOf( *it );
	}
	
	connect( manager,
	         SIGNAL( friendAdded( LJ::IO::FriendInfo* ) ),
	         SLOT( addFriend( LJ::IO::FriendInfo * ) ) );
	connect( manager,
	         SIGNAL( friendOfAdded( LJ::IO::FriendOfInfo * ) ),
	         SLOT( addFriendOf( LJ::IO::FriendOfInfo * ) ) );

	connect( manager,
	         SIGNAL( friendRemoved( LJ::IO::FriendInfo* ) ),
	         SLOT( removeFriend( LJ::IO::FriendInfo* ) ) );
	connect( manager,
	         SIGNAL( friendOfRemoved( LJ::IO::FriendOfInfo* ) ),
	         SLOT( removeFriendOf( LJ::IO::FriendOfInfo* ) ) );
}

LJ::UI::FriendsListModel::~FriendsListModel( void )
{
	delete d;
}

QModelIndex LJ::UI::FriendsListModel::index( int row,
                                             int column,
                                             const QModelIndex & parent ) const
{
	if ( !hasIndex( row, column, parent ) ) {
		return QModelIndex();
	}

	if ( parent.isValid() ) {
		return QModelIndex();
	}

	return createIndex( row, column );
}

QModelIndex LJ::UI::FriendsListModel::parent( const QModelIndex & child ) const
{
	Q_UNUSED( child );
	return QModelIndex();
}

int LJ::UI::FriendsListModel::rowCount( const QModelIndex & parent ) const
{
	if( parent.isValid() ) {
		return 0;
	}

	return d->userList.size();
}

int LJ::UI::FriendsListModel::columnCount( const QModelIndex & parent ) const
{
	Q_UNUSED( parent );
	return 4;
}

QVariant LJ::UI::FriendsListModel::data( const QModelIndex & index, int role ) const
{
	if( !index.isValid() ) {
		return QVariant();
	}

	const int row = index.row();
	if(  row >= d->userList.size() ) {
		return QVariant();
	}

	const QString user = d->userList.at( row );
	const int column = index.column();
	const QPair<LJ::IO::FriendInfo*, LJ::IO::FriendOfInfo*> & data = d->userInfoMap[user];
	const LJ::IO::FriendInfo * const friendInfo = data.first;
	const LJ::IO::FriendOfInfo * const friendOfInfo = data.second;
	
	if( role == Qt::BackgroundRole
	    && column == 2
	    && friendInfo
	    && d->useColors )
	{
		return friendInfo->background();
	}

	if( role == Qt::ForegroundRole
	    && column == 2
	    && friendInfo
	    && d->useColors )
	{
		return friendInfo->foreground();
	}

	if( role == Qt::FontRole && column == 2 ) {
		if( ( friendInfo && friendInfo->status() != IO::FriendInfoBase::StatusNormal ) ||
		    ( friendOfInfo && friendOfInfo->status() != IO::FriendInfoBase::StatusNormal ) )
		{
			QFont font = d->view->viewOptions().font;
			font.setStrikeOut( TRUE );
			return font; 
		}
	}

	if( role == Qt::ToolTipRole ) {
		QString relationString = QString::null;
		QString groupsString = QString::null;
		QString userNameFormat = "<img src=\"blokkal:///smallicons/%2\"/><b>%1</b>";
		QString userName = QString::null;
		if( friendInfo ) {
			userName = userNameFormat.arg( friendInfo->userName() );
			switch( friendInfo->type() ) {
			case LJ::IO::FriendInfoBase::TypeUser:
				userName = userName.arg( "blokkal_lj_user" );
				if( friendOfInfo ) {
					relationString = i18n( "You and %1 are mutual friends.",
					                       userName );
				}
				else {
					relationString = i18n( "%1 is one of your friends.",
					                       userName );
				}
				
				if( friendInfo->groupmask() ) {
					groupsString = i18n( "%1 belongs to the following friend groups:<ul>%2</ul>",
					                     userName,
					                     d->groupsList( friendInfo->groupmask() ) );
				}
				else {
					groupsString = i18n( "%1 does not belong to any friend groups.",
					                     userName );
				}
			break;
			case LJ::IO::FriendInfoBase::TypeCommunity:
				userName = userName.arg( "blokkal_lj_community" );
				if( friendOfInfo ) {
					relationString = i18n( "You are a member of the %1 community.",
					                       userName );
				}
				else {
					relationString = i18n( "You are monitoring the %1 community.",
					                       userName );
				}

				if( friendInfo->groupmask() ) {
					groupsString = i18n( "%1 belongs to the following friend groups:<ul>%2</ul>",
					                     userName,
					                     d->groupsList( friendInfo->groupmask() ) );
				}
				
				break;
			case LJ::IO::FriendInfoBase::TypeSyndicated:
				userName = userName.arg( "application-rss+xml" );
				relationString = i18n( "Your are monitoring the %1 feed.", userName );
				if( friendInfo->groupmask() ) {
					groupsString = i18n( "%1 belongs to the following friend groups:<ul>%2</ul>",
					                     userName,
					                     d->groupsList( friendInfo->groupmask() ) );
				}
				break;
			}
		}
		else if( friendOfInfo ) {
			userName = userNameFormat.arg( friendOfInfo->userName() );
			switch( friendOfInfo->type() ) {
			case LJ::IO::FriendInfoBase::TypeUser:
				userName = userName.arg( "blokkal_lj_user" );
				relationString = i18n( "You are a friend of %1.", userName );
				break;
			case LJ::IO::FriendInfoBase::TypeCommunity:
				userName = userName.arg( "blokkal_lj_community" );
				relationString = i18n( "You are a member of the %1 community, but you are not monitoring it.",
				                       userName );
				break;
			case LJ::IO::FriendInfoBase::TypeSyndicated:
				//feeds dont have friends list
				break;
			}
		}
		QString statusString = QString::null;
		LJ::IO::FriendInfoBase::FriendStatus status = LJ::IO::FriendInfoBase::StatusNormal;
		if( friendInfo ) {
			status = friendInfo->status();
		}
		else if( friendOfInfo ) {
			status = friendOfInfo->status();
		}

		switch( status ) {
		case LJ::IO::FriendInfoBase::StatusNormal:
			break;
		case LJ::IO::FriendInfoBase::StatusDeleted:
			statusString = i18n( "The account has been deleted." );
			break;
		case LJ::IO::FriendInfoBase::StatusPurged:
			statusString = i18n( "The account has been purged." );
			break;
		case LJ::IO::FriendInfoBase::StatusSuspended:
			statusString = i18n( "The account has been suspended." );
			break;
		}

		QString tipString = "<p style='white-space:pre'>" + relationString;
 
		if( !statusString.isEmpty() ) {
			tipString.append( "<br>" + statusString );
		}
	
		if( !groupsString.isEmpty() ) {
			tipString.append( "<br>" + groupsString );
		}

		tipString += "</p>";
		return tipString;
	}
	
	switch( column ) {
	case 0:
		if( data.first && data.second ) {
			if( role == Qt::DecorationRole ) {
				return QVariant::fromValue( d->mutualIcon );
			}
			else if( role == SortRole ) {
				return 1;
			}
		}
		else if( data.first ) {
			if( role == Qt::DecorationRole ) {
				return d->friendIcon;
			}
			else if( role == SortRole ) {
				return 2;
			}
		}
		else if( data.second ) {
			if( role == Qt::DecorationRole ) {
				return d->friendOfIcon;
			}
			else if( role == SortRole ) {
				return 3;
			}
		}
		break;
	case 1:
		{
			IO::FriendInfoBase * info = 0;
			if( data.first ) {
				info = data.first;
			}
			else if( data.second ) {
				info = data.second;
			}
			if( info ) {
				if( role == Qt::DecorationRole ) {
					switch( info->type() ) {
					case IO::FriendInfoBase::TypeUser:
						return d->userIcon;
						break;
					case IO::FriendInfoBase::TypeCommunity:
						return d->communityIcon;
						break;
					case IO::FriendInfoBase::TypeSyndicated:
						return d->syndicatedIcon;
						break;
					}
				}
				else if( role == SortRole ) {
					return info->type();
				}
			}
		}
		break;
	case 2:
		if( role == Qt::DisplayRole || role == SortRole ) {
			if( data.first ) {
				return data.first->userName();
			}
			else if( data.second ) {
				return data.second->userName();
			}
			kWarning() << "empty pair in userInfoMap" << endl;
		}
		break;
	case 3:
		if( role == Qt::DisplayRole || role == SortRole ) {
			if( data.first ) {
				return data.first->realName();
			}
			else if( data.second ) {
				return data.second->realName();
			}
			kWarning() << "empty pair in userInfoMap" << endl;
		}		
		break;
	default:
		return QVariant();
		break;
	}

	return QVariant();
}

void LJ::UI::FriendsListModel::addFriend( LJ::IO::FriendInfo * info )
{
	if( !info ) {
		return;
	}
	
	if( d->userInfoMap.contains( info->userName() ) ) {
		d->userInfoMap[info->userName()].first = info;
		emit dataChanged( createIndex( d->userList.indexOf( info->userName() ), 0 ),
		                  createIndex( d->userList.indexOf( info->userName() ), columnCount() - 1 ) );
	}
	else {
		beginInsertRows( QModelIndex(),
		                 d->userInfoMap.size(),
		                 d->userInfoMap.size() );
		QPair<LJ::IO::FriendInfo*, LJ::IO::FriendOfInfo*> data;
		data.first = info;
		data.second = 0;
		d->userInfoMap.insert( info->userName(), data );
		d->userList.append( info->userName() );
		endInsertRows();
	}
}

void LJ::UI::FriendsListModel::addFriendOf( LJ::IO::FriendOfInfo * info )
{
	if( !info ) {
		return;
	}

	if( d->userInfoMap.contains( info->userName() ) ) {
		d->userInfoMap[info->userName()].second = info;
		emit dataChanged( createIndex( d->userList.indexOf( info->userName() ), 0 ),
		                  createIndex( d->userList.indexOf( info->userName() ), columnCount() - 1 ) );
	}
	else {
		beginInsertRows( QModelIndex(),
		                 d->userInfoMap.size(),
		                 d->userInfoMap.size() );
		QPair<LJ::IO::FriendInfo*, LJ::IO::FriendOfInfo*> data;
		data.first = 0;
		data.second = info;
		d->userInfoMap.insert( info->userName(), data );
		d->userList.append( info->userName() );
		endInsertRows();
	}
}

void LJ::UI::FriendsListModel::removeFriend( LJ::IO::FriendInfo * info )
{
	if( !info ) {
		return;
	}

	if( !d->userList.contains( info->userName() ) ) {
		return;
	}
	const int row = d->userList.indexOf( info->userName() );
	
	QPair<LJ::IO::FriendInfo*, LJ::IO::FriendOfInfo*> & data = d->userInfoMap[info->userName()];
	if( data.second ) {
		data.first = 0;
		emit dataChanged( createIndex( row, 0 ),
		                  createIndex( row, columnCount() - 1 ) );
	}
	else {
		beginRemoveRows( QModelIndex(), row, row );
		d->userList.removeAll( info->userName() );
		d->userInfoMap.remove( info->userName() );
		endRemoveRows();
	}
}

void LJ::UI::FriendsListModel::removeFriendOf( LJ::IO::FriendOfInfo * info )
{
	if( !info ) {
		return;
	}

	if( !d->userList.contains( info->userName() ) ) {
		return;
	}
	const int row = d->userList.indexOf( info->userName() );
	
	QPair<LJ::IO::FriendInfo*, LJ::IO::FriendOfInfo*> & data = d->userInfoMap[info->userName()];
	if( data.first ) {
		data.second = 0;
		emit dataChanged( createIndex( row, 0 ),
		                  createIndex( row, columnCount() - 1 ) );
	}
	else {
		beginRemoveRows( QModelIndex(), row, row );
		d->userList.removeAll( info->userName() );
		d->userInfoMap.remove( info->userName() );
		endRemoveRows();
	}	
}

QVariant LJ::UI::FriendsListModel::headerData( int section,
                                               Qt::Orientation orientation,
                                               int role ) const
{
	Q_UNUSED( orientation );
	if( role != Qt::DisplayRole ) {
		return QVariant();
	}

	switch( section ) {
	case 0:
		return QString();
		break;
	case 1:
		return QString();
		break;
	case 2:
		return i18n("User Name");
		break;
	case 3:
		return i18n("Full Name");
		break;
	default:
		break;
	}

	return QVariant();
}

void LJ::UI::FriendsListModel::setUseColors( bool useColors )
{
	if( d->useColors == useColors ) {
		return;
	}
	
	d->useColors = useColors;
	emit dataChanged( createIndex( 0, 2 ),
	                  createIndex( rowCount() - 1, 2 ) );
}

//---------------
//FriendsListView
//---------------
class LJ::UI::FriendsListView::Private
{
public:
	Private ( const LiveJournalAccount * account,
	          FriendsListModel * friendsModel,
	          QSortFilterProxyModel * sortModel ):
	account( account ),
	friendsModel( friendsModel ),
	sortModel( sortModel ){}

	const LiveJournalAccount * account;
	FriendsListModel * const friendsModel;
	QSortFilterProxyModel * const sortModel;
};

LJ::UI::FriendsListView::FriendsListView( const LiveJournalAccount * account,
                                          QWidget * parent ) :
QTreeView( parent ),
d( new Private( account,
                new FriendsListModel( account, this ),
                new QSortFilterProxyModel(this) ) )
{
	d->sortModel->setSourceModel( d->friendsModel );
	d->sortModel->setSortRole( FriendsListModel::SortRole );
	setModel( d->sortModel );

	setSortingEnabled( TRUE );
	setRootIsDecorated( FALSE );
	setAllColumnsShowFocus( TRUE );
	setColumnWidth( 0, 20 );
	setColumnWidth( 1, 20 );

	model()->sort( 1 );
	header()->setSortIndicator( 1, Qt::AscendingOrder );
}

LJ::UI::FriendsListView::~FriendsListView( void )
{
	delete d;
}

const LiveJournalAccount * LJ::UI::FriendsListView::account( void ) const
{
	return d->account;
}

void LJ::UI::FriendsListView::setUseColors( bool useColors )
{
	d->friendsModel->setUseColors( useColors );
}

const LJ::IO::FriendInfo * LJ::UI::FriendsListView::friendInfo( const QModelIndex & _row ) const
{
	const QModelIndex friendIndex = d->sortModel->mapToSource(_row);
	if( !friendIndex.isValid() ) {
		return 0;
	}
	const int row = friendIndex.row();
	if( row >= d->friendsModel->d->userList.size() ) {
		return 0;
	}
	
	return d->friendsModel->d->userInfoMap[d->friendsModel->d->userList.at(row)].first;
}

const LJ::IO::FriendOfInfo * LJ::UI::FriendsListView::friendOfInfo( const QModelIndex & _row ) const
{
	const QModelIndex friendIndex = d->sortModel->mapToSource(_row);
	if( !friendIndex.isValid() ) {
		return 0;
	}
	const int row = friendIndex.row();
	if( row >= d->friendsModel->d->userList.size() ) {
		return 0;
	}
	
	return d->friendsModel->d->userInfoMap[d->friendsModel->d->userList.at(row)].second;
}
