/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_UILJFRIENDSLISTVIEW_H
#define LJ_UILJFRIENDSLISTVIEW_H

#include <QtCore/QAbstractItemModel>
#include <QtGui/QTreeView>
#include <QtGui/QItemDelegate>

class LiveJournalAccount;

namespace LJ {
namespace IO {
class FriendInfo;
class FriendOfInfo;
}
namespace UI {

/**
 * @short A listview for displaying friend relations.
 *
 * A listview for displaying friend relations.
 * @author Martin Mueller <orvio@orvio.de>
*/
class FriendsListView : public QTreeView
{
Q_OBJECT
public:
	/**
	 * Creates a new FriendsListView.
	 * @param account use the information of this account
	 * @param parent widget
	 */
	FriendsListView( const LiveJournalAccount * account, QWidget * parent = 0 );
	virtual ~FriendsListView( void );

	const LiveJournalAccount * account( void ) const;

	const LJ::IO::FriendInfo * friendInfo( const QModelIndex & row ) const;
	const LJ::IO::FriendOfInfo * friendOfInfo( const QModelIndex & row ) const;

public Q_SLOTS:
	void setUseColors( bool useColors );

private:
	friend class FriendsListModel;
	class Private;
	Private * d;
	
};


class FriendsListModel : public QAbstractItemModel {
Q_OBJECT
public:
	enum CustomRoles {
		SortRole = Qt::UserRole + 1
	};

	/**
	 * Constructor
	 */
	FriendsListModel( const LiveJournalAccount * account, FriendsListView * parent );
	virtual ~FriendsListModel( void );

	virtual QModelIndex index( int row,
	                           int column,
	                           const QModelIndex & parent = QModelIndex() ) const;
	virtual QModelIndex parent( const QModelIndex & child ) const;
	virtual int rowCount( const QModelIndex & parent = QModelIndex() ) const;
	virtual int columnCount( const QModelIndex & parent = QModelIndex() ) const;
	virtual QVariant data( const QModelIndex & index,
	                       int role = Qt::DisplayRole) const;

	virtual QVariant headerData( int section,
	                             Qt::Orientation orientation,
	                             int role = Qt::DisplayRole ) const;
public Q_SLOTS:
	void addFriend( LJ::IO::FriendInfo * info );
	void addFriendOf( LJ::IO::FriendOfInfo * info );

	void removeFriend( LJ::IO::FriendInfo * info );
	void removeFriendOf( LJ::IO::FriendOfInfo * info );

	void setUseColors( bool useColors );

private:
	friend class FriendsListView;
	class Private;
	Private * const d;
};

}

}

#endif
