/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "ljfriendsmanager.h"
#include "ljfriendsmanager.moc"

#include "../ljio/friendinfomanager.h"
#include "../ljio/getfriendinfojob.h"
#include "../ljio/editfriendsjob.h"

#include "../livejournalaccount.h"

#include "ljfriendslistview.h"
#include "ljfriendgroupselector.h"

#include <blokkalprotocol.h>
#include <blokkalutils.h>
#include <blokkalbehavioursettings.h>

#include <klocale.h>
#include <kguiitem.h>
#include <kpushbutton.h>
#include <kprogressdialog.h>
#include <kseparator.h>
#include <kmessagebox.h>
#include <kstdguiitem.h>
#include <kdeversion.h>
#include <kcolorbutton.h>
#include <klineedit.h>
#include <kdebug.h>
#include <kmessagebox.h>


#include <QItemSelection>
#include <QLayout>
#include <QLabel>
#include <QGroupBox>
#include <QTimer>
#include <QCheckBox>

#include <iostream>

//-------------------
//ManageFriendsDialog
//-------------------
class LJ::UI::ManageFriendsDialog::Private
{
public:
	Private( LiveJournalAccount * account ) :
	account( account ) {}

	LiveJournalAccount * account;
	FriendsManager * friendsManager;
};

LJ::UI::ManageFriendsDialog::ManageFriendsDialog( LiveJournalAccount * account,
                                                  QWidget * parent ):
KDialog( parent ),
d( new Private( account ) )
{
	setModal( FALSE );
	setCaption( i18n( "Manage friends of %1", account->id() ) );
	setButtons( Close );
	setDefaultButton( Close );
	
	connect( account,
	         SIGNAL( accountDestroyed( Blokkal::Account* ) ),
	         SLOT( close( void ) ) );
	connect( this,
	         SIGNAL( finished( void ) ),
	         SLOT( deleteLater( void ) ) );

	d->friendsManager = new FriendsManager( account, this );
	setMainWidget( d->friendsManager  );

	restoreDialogSize( KGlobal::config()->group( "LJManageFriendsDialog" ) );
}

LJ::UI::ManageFriendsDialog::~ManageFriendsDialog( void )
{
	KConfigGroup group = KGlobal::config()->group( "LJManageFriendsDialog" );
	saveDialogSize( group );	
	delete d;
}

//---------------
//AddFriendDialog
//---------------
class LJ::UI::AddFriendDialog::Private {
public:
	Private(  LiveJournalAccount * account,
	          FriendsManager * manager ) :
	account( account ),
	manager( manager ){}

	const LiveJournalAccount * const account;
	const FriendsManager * const manager;

	QGroupBox * mainWidget;
	QLabel * userLabel;
	KLineEdit * userLineEdit;
	KSeparator * userSeparator;

	KColorButton * textColorButton;
	KColorButton * backgroundColorButton;

	FriendGroupSelector * friendGroupSelector;
};


LJ::UI::AddFriendDialog::AddFriendDialog( LiveJournalAccount * account,
                                          FriendsManager * parent ) :
KDialog( parent ),
d( new Private( account, parent ) )
{
	setButtons( Ok|Cancel );
	setDefaultButton( Cancel );
	setModal( TRUE );
	
	d->mainWidget = new QGroupBox( this );
	d->mainWidget->setLayout( new QVBoxLayout( d->mainWidget ) );

	//user
	QHBoxLayout * userLayout = new QHBoxLayout( 0 );
	d->mainWidget->layout()->addItem( userLayout );
	userLayout->setMargin( 0 );
	d->userLabel = new QLabel( i18n( "&User" ), d->mainWidget );
	userLayout->addWidget( d->userLabel );
	d->userLineEdit = new KLineEdit( d->mainWidget );
	d->userLabel->setBuddy( d->userLineEdit );
	userLayout->addWidget( d->userLineEdit );
	d->userSeparator = new KSeparator( Qt::Horizontal, d->mainWidget );
	d->mainWidget->layout()->addWidget( d->userSeparator );
	
	QHBoxLayout * labelInputLayout = new QHBoxLayout( 0 );
	labelInputLayout->setMargin( 0 );
	d->mainWidget->layout()->addItem( labelInputLayout );
	//labels
	QVBoxLayout * labelLayout = new QVBoxLayout( 0 );
	labelLayout->setSpacing( KDialog::spacingHint() );
	//text color label
	QLabel * textColorLabel = new QLabel( i18n( "&Text Color" ), d->mainWidget );
	labelLayout->addWidget( textColorLabel );
	//background color label
	QLabel * backgroundColorLabel = new QLabel( i18n( "&Background Color" ), d->mainWidget );
	labelLayout->addWidget( backgroundColorLabel );
	labelInputLayout->addLayout( labelLayout );
	//inputs
	QVBoxLayout * inputLayout = new QVBoxLayout( 0 );
	inputLayout->setSpacing( KDialog::spacingHint() );
	//text color button
	d->textColorButton = new KColorButton( QColor( "black" ), QColor( "black"),  d->mainWidget );
	d->textColorButton->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Minimum );
	textColorLabel->setBuddy( d->textColorButton );
	inputLayout->addWidget( d->textColorButton );
	//background color button
	d->backgroundColorButton = new KColorButton( QColor( "white" ), QColor( "white" ), d->mainWidget );
	backgroundColorLabel->setBuddy( d->backgroundColorButton );
	inputLayout->addWidget( d->backgroundColorButton );
	labelInputLayout->addLayout( inputLayout );

   //friend groups
	d->mainWidget->layout()->addWidget( new KSeparator( Qt::Horizontal, d->mainWidget ) );
	QLabel * friendGroupsLabel = new QLabel( i18n( "&Select Friend Groups:" ), d->mainWidget );
	d->mainWidget->layout()->addWidget( friendGroupsLabel );
	d->friendGroupSelector = new FriendGroupSelector( account, d->mainWidget );
	friendGroupsLabel->setBuddy( d->friendGroupSelector );
	d->mainWidget->layout()->addWidget( d->friendGroupSelector );

	setMainWidget( d->mainWidget );

	restoreDialogSize( KGlobal::config()->group( "LJAddFriendDialog" ) );	
}

LJ::UI::AddFriendDialog::~AddFriendDialog( void )
{
	KConfigGroup group = KGlobal::config()->group( "LJAddFriendDialog" );
	saveDialogSize( group );		
	delete d;
}

void LJ::UI::AddFriendDialog::setup( const QModelIndex & item )
{
	d->userLabel->show();
	d->userLineEdit->show();
	d->userSeparator->show();
	
	if( !item.isValid() ) {
		setCaption( i18n( "Add Friend" ) );
		d->mainWidget->setTitle( i18n( "Add Friend" ) );
		d->userLineEdit->clear();
		d->friendGroupSelector->setGroupSelection( 0 );
		return;
	}

	if( d->manager->listView()->friendOfInfo( item )
	    && !d->manager->listView()->friendInfo( item ) )
	{
		setCaption( i18n( "Add Friend" ) );
		d->mainWidget->setTitle( i18n( "Add Friend" ) );
		d->userLineEdit->setText( d->manager->listView()->friendOfInfo( item )->userName() );
		d->friendGroupSelector->setGroupSelection( 0 );
	}
	else if( d->manager->listView()->friendInfo( item ) ) {
		setCaption( i18n( "Edit Friend" ) );
		d->mainWidget->setTitle( i18n( "Modify Friend: %1",
		                               d->manager->listView()->friendInfo( item )->userName() ) );
		d->userLineEdit->setText( d->manager->listView()->friendInfo( item )->userName() );
		d->friendGroupSelector->setGroupSelection( d->manager->listView()->friendInfo( item )->groupmask() );
		d->textColorButton->setColor( d->manager->listView()->friendInfo( item )->foreground() );
		d->backgroundColorButton->setColor( d->manager->listView()->friendInfo( item )->background() );
		d->userLabel->hide();
		d->userLineEdit->hide();
		d->userSeparator->hide();
	}
}

QString LJ::UI::AddFriendDialog::user( void ) const
{
	return d->userLineEdit->text();
}

QColor LJ::UI::AddFriendDialog::fg( void ) const
{
	return d->textColorButton->color();
}

QColor LJ::UI::AddFriendDialog::bg( void ) const
{
	return d->backgroundColorButton->color();
}

unsigned long LJ::UI::AddFriendDialog::groupmask( void ) const
{
	return d->friendGroupSelector->groupSelection();
}

//--------------
//FriendsManager
//--------------
class LJ::UI::FriendsManager::Private
{
public:
	Private( LiveJournalAccount * account ) :
	account( account ),
	progressDialog( 0 ),
	friendInfoJob( 0 ),
	editFriendsJob( 0 ),
	addDialog( 0 ){}

	LiveJournalAccount * account;
	KProgressDialog * progressDialog;
	LJ::IO::GetFriendInfoJob * friendInfoJob;
	LJ::IO::EditFriendsJob * editFriendsJob;
	AddFriendDialog * addDialog;

	FriendsListView * listView;
	QCheckBox * useColorsCheckBox;
	QLabel * friendsStatLabel;
	QLabel * communityStatLabel;
	QLabel * feedStatLabel;
	KPushButton * addButton;
	KPushButton * removeButton;
	KPushButton * editButton;
	KPushButton * refreshButton;

	KPushButton * showBlogButton;
};

LJ::UI::FriendsManager::FriendsManager( LiveJournalAccount * account, QWidget * parent ) :
QWidget( parent ),
d( new Private( account ) )
{
	QHBoxLayout * mainLayout = new QHBoxLayout( this );
	mainLayout->setSpacing( KDialog::spacingHint() );

	QVBoxLayout * listViewLayout = new QVBoxLayout( 0 );
	listViewLayout->setSpacing( KDialog::spacingHint() );
	d->listView = new FriendsListView( account, this );
	listViewLayout->addWidget( d->listView );
	d->useColorsCheckBox = new QCheckBox( i18n( "&Draw user names with their assigned colors" ), this );
	listViewLayout->addWidget( d->useColorsCheckBox );
	mainLayout->addLayout( listViewLayout );

	QVBoxLayout * buttonLayout = new QVBoxLayout( 0 );
	buttonLayout->setSpacing( KDialog::spacingHint() );
	d->addButton = new KPushButton( KStandardGuiItem::add(), this );
	d->addButton->setText( i18n( "Add Friend..." ) );
	buttonLayout->addWidget( d->addButton );

	d->removeButton = new KPushButton( KStandardGuiItem::remove(), this );
	d->removeButton->setText( i18n( "Remove Friend" ) );
	buttonLayout->addWidget( d->removeButton );

	d->editButton = new KPushButton( KStandardGuiItem::configure(), this );
	d->editButton->setText( i18n( "Modify Friend..." ) );
	buttonLayout->addWidget( d->editButton );

	d->showBlogButton = new KPushButton( KStandardGuiItem::open(), this );
	d->showBlogButton->setText( i18n( "&Open Blog" ) );
	buttonLayout->addWidget( d->showBlogButton );
	
	d->friendsStatLabel = new QLabel( this );
	d->friendsStatLabel->setAlignment( Qt::AlignLeft | Qt::AlignTop );
	d->friendsStatLabel->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
	buttonLayout->addWidget( d->friendsStatLabel );
	buttonLayout->addWidget( new KSeparator( Qt::Horizontal, this ) );
	
	d->communityStatLabel = new QLabel( this );
	d->communityStatLabel->setAlignment( Qt::AlignLeft | Qt::AlignTop );
	d->communityStatLabel->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::Minimum );
	buttonLayout->addWidget( d->communityStatLabel );
	buttonLayout->addWidget( new KSeparator( Qt::Horizontal, this ) );
	
	d->feedStatLabel = new QLabel( this );
	d->feedStatLabel->setAlignment( Qt::AlignLeft | Qt::AlignTop );
	d->feedStatLabel->setSizePolicy( QSizePolicy::Minimum, QSizePolicy::MinimumExpanding );
	buttonLayout->addWidget( d->feedStatLabel );
	d->refreshButton = new KPushButton( KGuiItem( i18n( "Reload"), "view-refresh" ), this );
	buttonLayout->addWidget( d->refreshButton );
	mainLayout->addLayout( buttonLayout );

	refreshStatistics();
	slotSelectionChanged();

	connect( account->friendInfoManager(),
	         SIGNAL( friendAdded( LJ::IO::FriendInfo* ) ),
	         SLOT(refreshStatistics( void ) ) );
	connect( account->friendInfoManager(),
	         SIGNAL( friendRemoved( LJ::IO::FriendInfo* ) ),
	         SLOT(refreshStatistics( void ) ) );
	connect( account->friendInfoManager(),
	         SIGNAL( friendOfAdded( LJ::IO::FriendOfInfo* ) ),
	         SLOT(refreshStatistics( void ) ) );
	connect( account->friendInfoManager(),
	         SIGNAL( friendOfRemoved( LJ::IO::FriendOfInfo* ) ),
	         SLOT(refreshStatistics( void ) ) );

	connect( d->listView->selectionModel(),
	         SIGNAL( selectionChanged ( const QItemSelection & , const QItemSelection &  ) ),
	         SLOT( slotSelectionChanged( void ) ) );
	
	connect( d->refreshButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( refresh( void ) ) );
	connect( d->addButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( addFriend( void ) ) );
	connect( d->editButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( editFriend( void ) ) );
	connect( d->removeButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( deleteFriends( void ) ) );

	connect( d->showBlogButton,
	         SIGNAL( clicked( void ) ),
	         SLOT( showFriendBlog( void ) ) );

	connect( d->useColorsCheckBox, SIGNAL( toggled( bool ) ),
	         d->listView, SLOT( setUseColors( bool ) ) );

	QTimer::singleShot( 0, this, SLOT( refresh( void ) ) );
}

LJ::UI::FriendsManager::~FriendsManager( void )
{
	delete d;
}

void LJ::UI::FriendsManager::refreshStatistics( void )
{
	QStringList friendUsers;
	QList< LJ::IO::FriendInfo * > friends = d->account->friendInfoManager()->friends();
	QList< LJ::IO::FriendOfInfo * > friendOfs = d->account->friendInfoManager()->friendOfs();

	unsigned int mutualFriendCount = 0;
	unsigned int memberCommunityCount = 0;
	unsigned int watchedCommunityCount = 0;
	unsigned int watchedFeedCount = 0;
	
	for( QList< LJ::IO::FriendInfo * >::ConstIterator it = friends.begin();
	     it != friends.end();
	     ++it )
	{
		switch( (*it)->type() ) {
		case LJ::IO::FriendInfoBase::TypeUser:
			friendUsers.append( (*it)->userName() );
			break;
		case LJ::IO::FriendInfoBase::TypeCommunity:
			watchedCommunityCount++;
			break;
		case LJ::IO::FriendInfoBase::TypeSyndicated:
			watchedFeedCount++;
			break;			
		}
		
	}

	for( QList< LJ::IO::FriendOfInfo * >::ConstIterator it = friendOfs.begin();
	     it != friendOfs.end();
	     ++it )
	{
		switch( (*it)->type() ) {
		case LJ::IO::FriendInfoBase::TypeUser:
			if( friendUsers.contains( (*it)->userName() ) ) {
				mutualFriendCount++;
			}
			break;
		case LJ::IO::FriendInfoBase::TypeCommunity:
			memberCommunityCount++;
			break;
		case LJ::IO::FriendInfoBase::TypeSyndicated:
			//feeds never add users to their contact list....
			break;	
		}		
	}

	d->friendsStatLabel->setText( i18n( "<b>Friends:</b> %1<br><b>Friend of:</b> %2<br><b>Mututal friends:</b> %3",
	                                    friends.count() - watchedFeedCount - watchedCommunityCount,
	                                    friendOfs.count() - memberCommunityCount,
	                                    mutualFriendCount)  );
	d->communityStatLabel->setText( i18n( "<b>Communities:</b> %1<br><b>Watched:</b> %2",
	                                      memberCommunityCount,
	                                      watchedCommunityCount ) );

	d->feedStatLabel->setText( i18n( "<b>Watched Feeds</b>: %1",
	                                 watchedFeedCount ) );
}

void LJ::UI::FriendsManager::refresh( void )
{
	if( d->friendInfoJob ) {
		return;
	}

	initProgressDialog( i18n( "Loading friends..." ) );
	connect( d->progressDialog,
	         SIGNAL( cancelClicked( void ) ),
	         SLOT( cancelFriendInfoJob( void ) ) );
	d->friendInfoJob = new LJ::IO::GetFriendInfoJob( d->account, this );
	connect( d->friendInfoJob,
	         SIGNAL( result( KJob * ) ),
	         SLOT( cancelFriendInfoJob( void ) ) );
	connect( d->friendInfoJob,
	         SIGNAL( percent( KJob*, unsigned long ) ),
	         SLOT( setProgress( KJob*, unsigned long ) ) );
	
	d->friendInfoJob->start();
}

void LJ::UI::FriendsManager::initProgressDialog( const QString & message )
{
	destroyProgressDialog();
	d->progressDialog = new KProgressDialog( this );
	d->progressDialog->setModal( TRUE );
	d->progressDialog->progressBar()->setRange( 0, 100 );
	d->progressDialog->progressBar()->setValue( 0 );
	//d->progressDialog->setAutoClose( FALSE );
	//d->progressDialog->setAllowCancel( FALSE );
	d->progressDialog->setMinimumDuration( 100 );
	d->progressDialog->setLabelText( message );
}

void LJ::UI::FriendsManager::destroyProgressDialog( void )
{
	if( d->progressDialog ) {
		d->progressDialog->hide();
		d->progressDialog->deleteLater();
		d->progressDialog = 0;
	}
}

void LJ::UI::FriendsManager::cancelFriendInfoJob( void )
{
	if( !d->friendInfoJob ) {
		return;
	}

	if( d->friendInfoJob->error() ) {
		KMessageBox::error( this, i18n( "An error occured while fetching friends." ) );
		if( d->friendInfoJob->error() > 0 ) {
			d->account->disconnectAccount( Blokkal::Account::IOErrorDisconnect );
		}		
	}
	
	d->friendInfoJob->deleteLater();
	d->friendInfoJob = 0;

	destroyProgressDialog();
	d->listView->selectionModel()->reset();
	slotSelectionChanged();
}

void LJ::UI::FriendsManager::setProgress( KJob * job, unsigned long percent )
{
	Q_UNUSED( job );
	
	if( !d->progressDialog ) {
		return;
	}

	d->progressDialog->progressBar()->setValue( percent );
}

void LJ::UI::FriendsManager::slotSelectionChanged( void )
{
	QModelIndexList indexes = d->listView->selectionModel()->selectedRows();
	bool mayAdd = TRUE;
	bool mayRemove = FALSE;
	bool mayEdit = FALSE;
	bool mayOpen = FALSE;
	if( indexes.size() == 1 ) {
		if( listView()->friendInfo( indexes.first() ) ) {
				mayRemove = TRUE;
				mayEdit = TRUE;
		}
		mayOpen = TRUE;
	}

	d->addButton->setEnabled( mayAdd );
	d->removeButton->setEnabled( mayRemove );
	d->editButton->setEnabled( mayEdit );
	d->showBlogButton->setEnabled( mayOpen );
}

void LJ::UI::FriendsManager::addFriend( void )
{
	if( d->editFriendsJob ) {
		return;
	}
	
	if( !d->addDialog ) {
		d->addDialog = new AddFriendDialog( d->account, this );
	}

	QModelIndexList indexes = d->listView->selectionModel()->selectedRows();
	if( indexes.size() == 1 && !listView()->friendInfo( indexes.first() ) ) {
		d->addDialog->setup( indexes.first() );
	}
	else {
		d->addDialog->setup( QModelIndex() );
	}

	if( d->addDialog->exec() ) {
		initProgressDialog( i18n( "Updating friends..." ) );
		setupEditFriendsJob();
		d->editFriendsJob->addUserToAdd( d->addDialog->user(), d->addDialog->fg(), d->addDialog->bg(), d->addDialog->groupmask() );
		d->editFriendsJob->start();
	}
}

void LJ::UI::FriendsManager::editFriend( void )
{
	if( d->editFriendsJob ) {
		return;
	}
	
	if( !d->addDialog ) {
		d->addDialog = new AddFriendDialog( d->account, this );
	}
	
	QModelIndexList indexes = d->listView->selectionModel()->selectedRows();
	if( indexes.size() == 1 && listView()->friendInfo( indexes.first() ) ) {
		d->addDialog->setup( indexes.first() );
	}
	else {
		return;
	}

	if( d->addDialog->exec() ) {
		initProgressDialog( i18n( "Updating friends..." ) );
		setupEditFriendsJob();
		d->editFriendsJob->addUserToAdd( d->addDialog->user(), d->addDialog->fg(), d->addDialog->bg(), d->addDialog->groupmask() );
		d->editFriendsJob->start();
	}	
}

void LJ::UI::FriendsManager::cancelEditFriendsJob( void )
{
	if( !d->editFriendsJob ) {
		return;
	}

	if( d->editFriendsJob->error() ) {
		KMessageBox::error( this, i18n( "An error occured while updating the friend list." ) );
		if( d->editFriendsJob->error() > 0 ) {
			d->account->disconnectAccount( Blokkal::Account::IOErrorDisconnect );
		}		
	}
	
	d->editFriendsJob->deleteLater();
	d->editFriendsJob = 0;

	destroyProgressDialog();

	refresh();
}

void LJ::UI::FriendsManager::setupEditFriendsJob( void )
{
	d->editFriendsJob = new LJ::IO::EditFriendsJob( d->account, this );

	connect( d->progressDialog,
	         SIGNAL( cancelClicked( void ) ),
	         SLOT( cancelEditFriendsJob( void ) ) );

	connect( d->editFriendsJob,
	         SIGNAL( result( KJob * ) ),
	         SLOT( cancelEditFriendsJob( void ) ) );
	connect( d->editFriendsJob,
	         SIGNAL( percent( KJob*, unsigned long ) ),
	         SLOT( setProgress( KJob*, unsigned long ) ) );
}

void LJ::UI::FriendsManager::deleteFriends( void )
{
	if( d->editFriendsJob ) {
		return;
	}

	QModelIndexList indexes = d->listView->selectionModel()->selectedRows();
	if( indexes.size() != 1 ) {
		return;
	}

	QString deleteUser;
	if( listView()->friendInfo( indexes.first() ) ) {
		deleteUser = listView()->friendInfo( indexes.first() )->userName();
	}

	if( deleteUser.isEmpty() ) {
		return;
	}

	if( KMessageBox::Yes != KMessageBox::warningYesNo( this, i18n( "Are you sure you want to remove the user \"%1\" from your friends list?", deleteUser ) ) )
	{
		return;
	}

	initProgressDialog( i18n( "Deleting friend %1...", deleteUser ) );
	setupEditFriendsJob();
	d->editFriendsJob->addUserToDelete( deleteUser );

	d->editFriendsJob->start();
}

void LJ::UI::FriendsManager::showFriendBlog( void )
{
	QModelIndexList indexes = d->listView->selectionModel()->selectedRows();
	if( indexes.size() != 1 ) {
		return;
	}
	
	QString userName = QString::null;
	
	if( listView()->friendInfo( indexes.first() ) ) {
		userName = listView()->friendInfo( indexes.first() )->userName();
	}
	else if( listView()->friendOfInfo( indexes.first() ) ) {
		userName = listView()->friendOfInfo( indexes.first() )->userName();
	}

	if( userName.isEmpty() ) {
		return;
	}

	Blokkal::Utils::openWebpage( QString::fromLatin1( "http://%1.%2" ).arg( userName).arg( d->account->serverName() ) );
}

LJ::UI::FriendsListView * LJ::UI::FriendsManager::listView( void ) const
{
	return d->listView;
}
