/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_UILJFRIENDSMANAGER_H
#define LJ_UILJFRIENDSMANAGER_H

#include <kdialog.h>

#include <QtCore/QModelIndex>

class LiveJournalAccount;
class KJob;

namespace LJ {
namespace UI {
class FriendLvi;
class FriendsListView;
/**
 * @short Widget for managing friends.
 *
 * Widget for managing friends.
 * @author Martin Mueller <orvio@orvio.de>
*/
class FriendsManager : public QWidget
{
Q_OBJECT
public:
	/**
	 * Creates a new FriendsManager.
	 * @param account manage the friends of this account
	 * @param parent parent widget
	 */
	FriendsManager( LiveJournalAccount * account,
	                QWidget * parent = 0 );

	virtual ~FriendsManager( void );

	/**
	 * Returns the internal FriendsListView.
	 * @return the internal FriendsListView
	 */
	LJ::UI::FriendsListView * listView( void ) const;

public slots:
	void refresh( void );
	void refreshStatistics( void );

private slots:
	void cancelFriendInfoJob( void );
	void destroyProgressDialog( void );
	void setProgress( KJob* job, unsigned long percent );
	void slotSelectionChanged( void );
	void addFriend( void );
	void editFriend( void );
	void cancelEditFriendsJob( void );
	void setupEditFriendsJob( void );
	void deleteFriends( void );
	void showFriendBlog( void );
	
private:
	void initProgressDialog( const QString & message );

	class Private;
	Private * d;

};

/**
 * @short Displays a FriendsManager widget.
 *
 * Displays a FriendsManager widget.
 * @author Martin Mueller <orvio@orvio.de>
 */
class ManageFriendsDialog : public KDialog
{
Q_OBJECT
public:
	/**
	 * Creates a new ManageFriendsDialog
	 * @param account the account
	 * @param parent parent widget
	 */
	ManageFriendsDialog( LiveJournalAccount * account,
	                     QWidget * parent );
	virtual ~ManageFriendsDialog( void );

private:
	class Private;
	Private * d;
};


/**
 * @short Dialog for adding/editing friends.
 * 
* This class provides a dialog to ask the user for the required information
* to add a friend or modify an existing friend.
 * @author Martin Mueller <orvio@orvio.de>
*/
class AddFriendDialog : public KDialog
{
Q_OBJECT
public:
	AddFriendDialog( LiveJournalAccount * account,
	                 FriendsManager * parent = 0 );

	virtual ~AddFriendDialog( void );

	/**
	* Sets up the dialog for @a item
	*/
	void setup( const QModelIndex &  item );

	QString user( void ) const;
	QColor fg( void ) const;
	QColor bg( void ) const;
	unsigned long groupmask( void ) const;

private:
	class Private;
	Private * d;
};


}

}

#endif
