/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_UILJMOODSELECTOR_H
#define LJ_UILJMOODSELECTOR_H

#include <qwidget.h>

class LiveJournalAccount;

namespace LJ {
namespace IO {
class Mood;
}

namespace UI {

/**
 * @short A widget to select moods.
 *
 * A widget to select moods.
 * @author Martin Mueller <orvio@orvio.de>
*/
class MoodSelector : public QWidget
{
Q_OBJECT
public:
	/**
	 * Creates a new MoodSelector.
	 * @param account use the information of this account
	 * @param parent parent widget
	 */
	MoodSelector( LiveJournalAccount * account, QWidget * parent = 0 );
	~MoodSelector( void );

	/**
	 * This method return the currently selected mood id.
	 * @return the selected mood id, 0 means invalid
	 */
	unsigned int moodID( void ) const;

	/**
	 * This method select the mood with the given id.
	 * The data in the line edit is not changed.
	 * @param id the id to select
	 */
	void setMoodID( unsigned int id );

	/**
	 * This method returns the data in the line edit.
	 * @return the mood name entered into the line edit
	 */
	QString moodName( void ) const;

	/**
	 * This method sets the current mood name. The current mood
	 * name is the text in the line edit. The data in the combobox
	 * is not affected by this.
	 * @param moodName the new mood name.
	 */
	void setMoodName( const QString moodName );
	
signals:
	/**
	 * This signal is emitted when a new mood is entered.
	 */
	void moodChanged( void );
	
protected slots:
	/**
	 * This slot is called when a new mood is reported by the
	 * mood manager.
	 * @param newMood the new mood
	 */
	virtual void addMood( LJ::IO::Mood * newMood );

	/**
	 * This slot is called when new mood data is entered/selected
	 * by the user
	 */
	virtual void emitMoodChanged( void );
	
private:
	class Private;
	Private * d;
};

}

}

#endif
