/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_UILJSECURITYSELECTOR_H
#define LJ_UILJSECURITYSELECTOR_H

#include <kcombobox.h>

#include "../ljentry.h"

namespace LJ {
namespace UI {

/**
 * @short This class provides a selector for the available
 * security levels of a livejournal entry.
 *
 * This class provides a selector for the available
 * security levels of a livejournal entry.
 * It will setup itself according to the type of journal the
 * entry belongs to.
 * @author Martin Mueller <orvio@orvio.de>
*/
class SecuritySelector : public KComboBox
{
Q_OBJECT
public:
	/**
	 * Creates a new SecuritySelector. The selector will display only
	 * security levels valid for @a entry.
	 * @param entry used to figure out how to set up this selector, not stored
	 * @param parent parent widget
	 */
	SecuritySelector( LJ::Entry * entry, QWidget * parent = 0 );
	~SecuritySelector( void );

	/**
	 * This method returns the currently elected security level.
	 * The result of this method is indefined, if an items is selected
	 * that was not inserted by the constructor.
	 * @return the currently selected security level
	 */
	LJ::Entry::SecurityLevel currentLevel( void );

public slots:
	/**
	 * This method sets the selected security level.
	 * @param newLevel the security level to select
	 */
	void setCurrentLevel( LJ::Entry::SecurityLevel newLevel );

};

}

}

#endif
