/***************************************************************************
 *   Copyright (C) 2006 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef LJ_UIUSERICONSELECTOR_H
#define LJ_UIUSERICONSELECTOR_H

#include <kcombobox.h>

class LiveJournalAccount;

namespace LJ {
namespace IO {
class UserPictureManager;
class UserPicture;
}
namespace UI {
class IconDelegate;

/**
 * @short This class provides a selector for user icon.
 *
 * This class provides a selector for user icon.
 * The widget will disable itself if no icons are available.
 * @author Martin Mueller <orvio@orvio.de>
*/
class UserIconSelector : public KComboBox
{
Q_OBJECT
public:
	/**
	 * Creates a new UserIconSelector.
	 * @param account take the usr picture of this account
	 * @param parent parent widget
	 */
	UserIconSelector( LiveJournalAccount * account, QWidget * parent = 0 );
	~UserIconSelector( void );

	/**
	 * This method returns the keyword of the selected icon
	 * or QString::null if no icon or the default icon is selected.
	 * @return keyword of the currently selected icon
	 */
	QString keyword( void );

protected Q_SLOTS:
	/**
	 * Disables the selector if only the deault picture is available
	 */
	void setState( void );

private:
	class Private;
	Private * const d;
};

/**
 * @short Model for the popup list of UserIconSelector
 *
 * Model for the popup list of UserIconSelector
 * @author Martin Mueller <orvio@orvio.de>
*/
class IconListModel : public QAbstractListModel
{
Q_OBJECT
public:
	IconListModel( const LJ::IO::UserPictureManager * manager, QObject * parent = 0 );
	virtual ~IconListModel( void );
	virtual int rowCount( const QModelIndex & parent = QModelIndex() ) const;
	virtual QVariant data( const QModelIndex & index, int role = Qt::DisplayRole ) const;

	enum {
		ImageRole = Qt::UserRole,
	};

private Q_SLOTS:
	void updateUserPicture( LJ::IO::UserPicture * picture );
	void addUserPicture( LJ::IO::UserPicture * picture );
	void removeUserPicture( LJ::IO::UserPicture * picture );

private:
	QList< LJ::IO::UserPicture * > m_icons;
};

}

}

#endif
