/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MWAMWAACCOUNT_H
#define MWAMWAACCOUNT_H

#include "mwa_export.h"

#include <bloggeraccount.h>

namespace Mwa {
class Protocol;
/**
	@author Martin Mueller <orvio@orvio.de>
*/
class MWA_EXPORT Account : public Blogger::Account
{
Q_OBJECT
public:
	/**
	 * Creates a new account for protocol @a protocol, with name @a accountName
	 * @param protocol the procotol of this account
	 * @param accountName the name if this account
	 */
	Account( Mwa::Protocol * protocol, const QString & accountName = 0 );

	/**
	 * Destructor
	 */
	virtual ~Account( void );

	/**
	 * Sets the provider id
	 * @param provider the new provider id
	 */
	void setProvider( const QString & provider );

	/**
	 * Returns the currently set provider id
	 * @return the provider id
	 */
	QString provider( void ) const;

	/**
	 * Returns the or'ed values for the workarounds active for this account.
	 * @return workarounds active for this account
	 * @see Blokkal::IO::XmlRpcJob::WorkArounds
	 */
	unsigned int activeWorkArounds( void ) const;

	/**
	 * Sets the active workarounds.
	 * @param workArounds the workarounds to use for this account
	 */
	void setActiveWorkArounds( unsigned int workArounds );
	
protected:
	/**
	 * Creates a new blog object.
	 * Do not register the object in this method.
	 * @param id blog id
	 * @param url the blog url
	 * @param title the blog title
	 */
	virtual Blogger::Blog * createBlog( const QString & id,
	                                    const QString & url,
	                                    const QString & title );

private:
	class Private;
	Private * d;	
};

}

#endif
