/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mwablog.h"
#include "mwablog.moc"

#include "mwaaccount.h"
#include "mwaentry.h"
#include "mwaio/getcategoriesjob.h"
#include "mwaio/blogbrowsingdriver.h"

class Mwa::Blog::Private {
public:
	Private( Io::BlogBrowsingDriver * browsingDriver) :
	categoriesJob( 0 ),
	browsingDriver( browsingDriver ){}

	Io::GetCategoriesJob * categoriesJob;
	Io::BlogBrowsingDriver * const browsingDriver;
};

Mwa::Blog::Blog( Mwa::Account * parentAccount,
                 const QString & id,
                 const QString & url,
                 const QString & title ):
Blogger::Blog( parentAccount, id, url, title ),
d( new Private( new Io::BlogBrowsingDriver( this ) ) )
{
	connect( parentAccount,
	         SIGNAL( connectionStatusChanged( Blokkal::Account* ) ),
	         SLOT( fetchCategories( void ) ) );
	fetchCategories();
}

Mwa::Blog::~Blog( void )
{
	delete d;
}

Mwa::Io::GetCategoriesJob * Mwa::Blog::createGetCategoriesJob( void )
{
	return new Io::GetCategoriesJob( this );
}

void Mwa::Blog::fetchCategories( void )
{
	if( account()->isConnected() && !d->categoriesJob ) {
		d->categoriesJob = createGetCategoriesJob();
		connect( d->categoriesJob,
		         SIGNAL( result( KJob * ) ),
		         SLOT( slotCategoriesFetched( void ) ) );
	}
}

void Mwa::Blog::slotCategoriesFetched( void )
{
	d->categoriesJob = 0;
}

Blokkal::Entry * Mwa::Blog::createEntry( void )
{
	return new Mwa::Entry( this );
}

unsigned int Mwa::Blog::supportedFeatures( void ) const
{
	return FeatureSubject|FeatureText|FeatureDate|FeatureMultipleCategories;
}

Blokkal::Io::BlogBrowsingDriver * Mwa::Blog::blogBrowsingDriver( void ) const
{
	return d->browsingDriver;
}

Mwa::Entry * Mwa::Blog::createEntry( QMap< QString, QVariant > map )
{
	return new Mwa::Entry( this, map );
}
