/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MWAMWABLOG_H
#define MWAMWABLOG_H

#include "mwa_export.h"

#include <bloggerblog.h>

template <class T1,class T2> class QMap;

namespace Mwa {
class Entry;
namespace Io {
class GetCategoriesJob;
}

class Account;
/**
 * @short Representation of a MetaWeblog blog
 *
 * Representation of a MetaWeblog blog
 * @author Martin Mueller <orvio@orvio.de>
*/
class MWA_EXPORT Blog : public Blogger::Blog
{
Q_OBJECT
public:
	/**
	 * This creates a new Blog.
	 * @param parentAccount the account this blog belongs to
	 * @param id the blog id
	 * @param url sets the url of this blog
	 */
	Blog( Mwa::Account * parentAccount,
	      const QString & id,
	      const QString & url,
	      const QString & title);

	/**
	 * Destructor
	 */
	virtual ~Blog( void );

	/**
	 * Creates a new @ref Blokkal::Entry object.
	 * @return a new Blokkal::Entry object.
	 */
	virtual Blokkal::Entry * createEntry( void );

	/**
	 * Creates a new Mwa::Entry object from @a map
	 * @param map property map to use to configure the entry
	 * @return a new Mwa::Entry object
	 */
	virtual Mwa::Entry * createEntry( QMap<QString, QVariant> map );

	/**
	 * @return the BlogBrowsingDriver of this blog
	 */
	virtual Blokkal::Io::BlogBrowsingDriver * blogBrowsingDriver( void ) const;

	/**
	 * Returns an or'ed list of Blokkal::Blog::EntryFeatures values to indicate
	 * the possible options of entry editing.
	 * The default implementation returns FeatureDate|FeatureSubject|FeatureText.
	 * @return an or'ed list of Blokkal::Blog::EntryFeatures values
	 */
	virtual unsigned int supportedFeatures( void ) const;

protected:
	/**
	 * Creates an XmlRpcJob that fetches and updates the categories for this blog.
	 * @return an XmlRpcJob that fetches and updates the categories for this blog
	 */
	virtual Mwa::Io::GetCategoriesJob * createGetCategoriesJob( void );

protected slots:
	virtual void slotCategoriesFetched( void );
	
private slots:
	virtual void fetchCategories( void );

private:
	class Private;
	Private * const d;
};

}

#endif
