/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mwaentry.h"
#include "mwaentry.moc"

#include "mwablog.h"
#include "mwaio/editpostjob.h"
#include "mwaio/newpostjob.h"

#include <blokkalutils.h>
#include <blokkalio/xmlrpcjob.h>

#include <kdebug.h>
#include <kicon.h>

#include <QVariant>
#include <QDomElement>

class Mwa::Entry::Private {
public:
	Private( void ) {}

	QMap< QString, QVariant > propertyMap;
};

Mwa::Entry::Entry( Mwa::Blog * parentBlog ) :
Blogger::Entry( parentBlog, QString::null),
d( new Private() )
{
}

Mwa::Entry::Entry( Mwa::Blog * parentBlog, QMap<QString, QVariant> map ):
Blogger::Entry( parentBlog, map["postid"].toString(), map["permaLink"].toString() ),
d( new Private() )
{
	d->propertyMap = map;
	setDate( map["dateCreated"].toDateTime() );
	writeEntry( "user", map["userid"].toString() );
	setText( map["description"].toString() );
	setSubject( map["title"].toString() );
	writeEntry( "link", map["link"].toString() );
	writeEntry( "permaLink", map["permaLink"].toString() );
	if( map["categories"].type() == QVariant::List ) {
		QList<QVariant> categoryList = map["categories"].toList();
		QStringList categoryTitles;
		for( QList<QVariant>::ConstIterator it = categoryList.begin();
		     it != categoryList.end();
		     ++it )
		{
			categoryTitles.append( (*it).toString() );
		}
		setCategories( categoryTitles );
	}
	else {
		kWarning() << "categories entry has wrong type" << endl;
	}
	
	if( map.contains( "post_status" ) ) {
		setPublish( map["post_status"].toString() != "draft" );
	}
	//TODO enclosure
}

Mwa::Entry::~Entry( void )
{
	delete d;
}

const QMap< QString, QVariant > & Mwa::Entry::propertyMap( void ) const
{
	updatePropertyMap( d->propertyMap );
	return d->propertyMap;
}

void Mwa::Entry::updatePropertyMap( QMap< QString, QVariant > & map ) const
{
	map["dateCreated"] = date();
	map["userid"] = user();
	map["postid"] = id();
	map["description"] = text();
	map["title"] = subject();
	map["link"] = link();
	map["permaLink"] = permaLink();
	map["categories"] = categories();
	//TODO enclosure
}

QString Mwa::Entry::user( void ) const
{
	return readEntry( "user" );
}

QString Mwa::Entry::link( void ) const
{
	return readEntry( "link" );
}

QString Mwa::Entry::permaLink( void ) const
{
	return readEntry( "permaLink" );
}

void Mwa::Entry::merge( const Blokkal::Entry * entry )
{
	Blokkal::Entry::merge( entry );

	/*const Mwa::Entry * mwaentry = dynamic_cast<const Mwa::Entry *>( entry );

	if( !mwaentry ) {
		return;
	}

	writeEntry( "link", mwaentry->link() );
	writeEntry( "permaLink", mwaentry->permaLink() );*/
}

Blokkal::Io::EntryJob * Mwa::Entry::createJob( Blokkal::Entry::JobType type )
{
	Blokkal::Io::EntryJob * job = 0;
	switch( type ) {
	case PostJob:
		job = new Blokkal::Io::EntryJob( this );
		if( id().isNull() ) {
			job->addSubjob( new Io::NewPostJob( this ) );
		}
		else {
			job->addSubjob( new Io::EditPostJob( this ) );
		}
		break;
	default:
		job = Blogger::Entry::createJob( type );
		break;
	}
	
	return job;	
}

void Mwa::Entry::saveProperties( void )
{
	QDomDocument doc;
	QDomElement rpcContainer = doc.createElement( "rpcContainer" );
	doc.appendChild( rpcContainer );
	Blokkal::Io::XmlRpcJob::encodeData( rpcContainer, d->propertyMap );
	writeComplex( "xml-rpc", doc.toString() );
}

void Mwa::Entry::readProperties( void )
{
	QDomDocument doc;
	doc.setContent( readComplex( "xml-rpc" ) );
	QDomNode valueNode = doc.firstChild().firstChild();
	if( valueNode.nodeName() != QString::fromUtf8( "value" )
		|| !valueNode.isElement() )
	{
		kError() << "invalid data in xml: " << valueNode.nodeName() << endl;
		return;
	}
	QVariant data = Blokkal::Io::XmlRpcJob::decodeData( valueNode.toElement() );
	d->propertyMap = data.toMap();
}

KIcon Mwa::Entry::icon( void ) const
{
	if( isPublish() ) {
		return Blogger::Entry::icon();
	}

	return KIcon( Blokkal::Utils::fadePixmap( Blogger::Entry::icon().pixmap( 32, 32 ) ) );
}
