/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MWAMWAENTRY_H
#define MWAMWAENTRY_H

#include "mwa_export.h"

#include <bloggerentry.h>

namespace Mwa {
class Blog;
/**
 * @short Represents a MetaWeblog entry
 *
 * Represents a MetaWeblog entry.
 * @author Martin Mueller <orvio@orvio.de>
 */
class MWA_EXPORT Entry : public Blogger::Entry
{
Q_OBJECT
public:
	/**
	 * Creates a new Entry.
	 * @param parentBlog the blog the entry belongs to
	 */
	Entry( Mwa::Blog * parentBlog );

	/**
	 * Creates a new entry from a XmlRpc Response map.
	 * @param parentBlog the blog the entry belongs to
	 * @param map a map containing the struct elements of the xmlrpc response
	 */
	Entry( Mwa::Blog * parentBlog, QMap<QString, QVariant> map );

	/**
	 * Destructor
	 */
	virtual ~Entry( void );

	/**
	 * Returns a map of the entry properties.
	 * @return a map of the entry properties
	 */
	const QMap<QString, QVariant> & propertyMap( void ) const;

	/**
	 * Returns the id of the user who created this entry.
	 * @return the id of the user who created this entry
	 */
	QString user( void ) const;

	/**
	 * Returns the link to this entry.
	 * @return the link to this entry
	 */
	QString link( void ) const;

	/**
	 * Returns the permalink to this entry.
	 * @return the permalink to this entry
	 */
	QString permaLink( void ) const;

	virtual void merge( const Blokkal::Entry * entry );

	/**
	 * Creates and returns a job of @a type
	 * @return a job of @a type
	 */
	virtual Blokkal::Io::EntryJob * createJob( Blokkal::Entry::JobType type );

	/**
	 * This method returns the icon of the entry.
	 * @return the entry icon
	 */
	virtual KIcon icon( void ) const;

protected:
	/**
	 * Reimplement this method if you need to update additional
	 * elements of the property map.
	 * This method is called by propertyMap() before the reference to
	 * the property map is returned.
	 * @param map reference to the property map
	 */
	virtual void updatePropertyMap( QMap<QString, QVariant> & map ) const;
	
	/**
	 * This method is called right before this entry is saved to disk.
	 * Reimplemented to save the Xml-Rpc data.
	 */
	virtual void saveProperties( void );

	/**
	 * This method is called right after the entry has been
	 * Reimplemented to restore the Xml-Rpc data.
	 */
	virtual void readProperties( void );

private:
	class Private;
	Private * const d;
};

}

#endif
