/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "blogbrowsingdriver.h"
#include "blogbrowsingdriver.moc"

#include "../mwablog.h"
#include "../mwaentry.h"

#include "getpostjob.h"
#include "getrecentpostsjob.h"

#include <kdebug.h>

Mwa::Io::BlogBrowsingDriver::BlogBrowsingDriver( Mwa::Blog * parent ):
Blokkal::Io::BlogBrowsingDriver( parent )
{
}

Mwa::Io::BlogBrowsingDriver::~BlogBrowsingDriver( void )
{
}

unsigned int Mwa::Io::BlogBrowsingDriver::supportedOperations( void ) const
{
	return LoadEntryById|LoadRecentEntries;
}

Blokkal::Io::LoadEntryJob * Mwa::Io::BlogBrowsingDriver::createLoadEntryJob( const QStringList & idList )
{
	Blokkal::Io::LoadEntryJob * job = new Mwa::Io::LoadEntryJob( blog() );
	for( QStringList::ConstIterator it = idList.begin();
	     it != idList.end();
	     ++it )
	{
		job->addSubjob( new GetPostJob( static_cast<Mwa::Blog*>( blog() ), *it ) );
	}
	return job;
}

Blokkal::Io::LoadEntryJob * Mwa::Io::BlogBrowsingDriver::createLoadEntryJob( unsigned int n )
{
	Blokkal::Io::LoadEntryJob * job = new Mwa::Io::LoadEntryJob( blog() );
	job->addSubjob( new GetRecentPostsJob( static_cast<Mwa::Blog*>( blog() ), n ) );
	return job;
}

//------------
//LoadEntryJob
//------------
class Mwa::Io::LoadEntryJob::Private {
public:
	Private( void ) {}

	QList<Blokkal::Entry *> entryList;
};
Mwa::Io::LoadEntryJob::LoadEntryJob( Blokkal::Blog * blog ):
Blokkal::Io::LoadEntryJob( blog ),
d( new Private() )
{
}

Mwa::Io::LoadEntryJob::~LoadEntryJob( void )
{
	delete d;
}

void Mwa::Io::LoadEntryJob::subjobFinished( KJob * job )
{
	if( job->error() ) {
		return;
	}

	const Mwa::Io::GetPostJob * const getPostJob = dynamic_cast<Mwa::Io::GetPostJob *>( job );
	if( getPostJob ) {
		Blokkal::Entry * const entry = getPostJob->entry();
		if( entry ) {
			d->entryList.append( entry );
		}
		return;
	}
	
	const Mwa::Io::GetRecentPostsJob * const getRecentPostsJob = dynamic_cast<Mwa::Io::GetRecentPostsJob *>( job );
	if( getRecentPostsJob ) {
		const QList<Mwa::Entry*> entryList = getRecentPostsJob->entries();
		for( QList<Mwa::Entry*>::ConstIterator it = entryList.begin();
		     it != entryList.end();
		     ++it )
		{
			d->entryList.append( *it );
		}
		return;
	}

	kError() << "invalid subjob encountered" << endl;

}

QList< Blokkal::Entry * > Mwa::Io::LoadEntryJob::entries( void ) const
{
	return d->entryList;
}

unsigned int Mwa::Io::BlogBrowsingDriver::supportedBrowseModes( void ) const
{
	return Blokkal::Io::BlogBrowsingDriver::supportedBrowseModes();
}

