/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getcategoriesjob.h"
#include "getcategoriesjob.moc"

#include <blokkalio/categorymanager.h>

#include "../mwablog.h"
#include "../mwaaccount.h"

#include <kdebug.h>

#include <QVariant>
#include <QStringList>

class Mwa::Io::GetCategoriesJob::Private {
public:
	Private( Account * account, const QString & blogId ) :
	account( account ),
	blogId( blogId ){}

	Account * account;
	QString blogId;
	
	static QList<QVariant> arguments( const QString & id ) {
		QList<QVariant> args;
		args.append( id );
		return args;
	}

	void updateCategory( const QString & id, const QMap<QString, QVariant> & map ) {
		QString html = map["htmlUrl"].toString();
		QString rss = map["rssUrl"].toString();

		Blokkal::Io::Category * const category = account->categoryManager()->category( blogId, id );
		category->setHtmlUrl( html );
		category->setRssUrl( rss );
		category->setName( id );
	}
};

Mwa::Io::GetCategoriesJob::GetCategoriesJob( Mwa::Blog * blog ):
SimpleBlogJob( blog,
               "metaWeblog.getCategories",
               Private::arguments( blog->id() ) ),
d( new Private( dynamic_cast<Mwa::Account*>( blog->account() ),
                blog->id() ) )
{
}

Mwa::Io::GetCategoriesJob::~GetCategoriesJob( void )
{
	delete d;
	d = 0;
}

void Mwa::Io::GetCategoriesJob::jobFinished( void )
{
	SimpleBlogJob::jobFinished();

	if( responseType() != ResponseSuccess ) {
		return;
	}

	QVariant response = this->response();
	//kDebug() << "response: " << response << endl;
	
	if( response.type() != QVariant::List ) {
		return;
	}

	QStringList validCategoryIds;
	Blokkal::Io::CategoryManager * const manager = d->account->categoryManager();
	//The MWA-Reference talks about a struct being returned.
	//However, Wordpress comes up with an array.
	//structs translate to QMaps, and arrays to QList
	//Test the response.typ() to figure out which one to take
	if( response.type() == QVariant::Map ) {
		const QMap<QString, QVariant> variantMap = response.toMap();
		for( QMap<QString, QVariant>::ConstIterator it  = variantMap.begin();
		     it != variantMap.end();
		     ++it )
		{
			if( it.value().type() != QVariant::Map ) {
				manager->setValidCategories( d->blogId, QStringList() );
				return;
			}
			
			const QMap<QString, QVariant> map = it.value().toMap();
			const QString id = it.key();
			validCategoryIds.append( id );
			d->updateCategory( id, map );
		}
	}
	else if( response.type() == QVariant::List ) {
		const QList<QVariant> variantList = response.toList();
		for( QList<QVariant>::ConstIterator it = variantList.begin();
		     it != variantList.end();
		     ++it )
		{
			if( (*it).type() != QVariant::Map ) {
				manager->setValidCategories( d->blogId, QStringList() );
				return;
			}

			const QMap<QString, QVariant> map = (*it).toMap();
			//QString id = map["description"].toString();
			const QString id = map["categoryName"].toString();
			validCategoryIds.append( id );
			d->updateCategory( id, map );
		}
	}
	else {
		kError() << "encountered invalid response type:" << response.type() << endl;
	}

	manager->setValidCategories( d->blogId, validCategoryIds );
}
