/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getrecentpostsjob.h"
#include "getrecentpostsjob.moc"

#include "../mwablog.h"
#include "../mwaentry.h"

#include <QVariant>

class Mwa::Io::GetRecentPostsJob::Private {
public:
	Private( Mwa::Blog * blog ) :
	blog( blog ),
	listBuilt( FALSE ){}

	Mwa::Blog * blog;

	bool listBuilt;
	QList< Mwa::Entry * > entries;
};

Mwa::Io::GetRecentPostsJob::GetRecentPostsJob( Mwa::Blog * blog, int n, bool showProgressInfo ):
Blogger::Io::SimpleBlogJob( blog,
                            "metaWeblog.getRecentPosts",
                            convertToList( n ),
                            0,
                            showProgressInfo ),
d( new Private( blog ) )
{
}

Mwa::Io::GetRecentPostsJob::~GetRecentPostsJob( void )
{
	delete d;
}

QList< Mwa::Entry * > Mwa::Io::GetRecentPostsJob::entries( void ) const
{
	if( responseType() != ResponseSuccess
	    || d->listBuilt ) {
		return d->entries;
	}

	QVariant response = this->response();

	if( response.type() != QVariant::List ) {
		QList< Mwa::Entry * >();
	}
	
	QList<QVariant> variantList = response.toList();
	for( QList<QVariant>::ConstIterator it = variantList.begin();
	     it != variantList.end();
	     ++it )
	{
		if( (*it).type() != QVariant::Map ) {
			return QList< Mwa::Entry * >();
		}

		QMap<QString, QVariant> map = (*it).toMap();

		Entry * entry =  d->blog->createEntry( map );
		d->entries.append( entry );
	}

	d->listBuilt = TRUE;
	return d->entries;
}
