/***************************************************************************
 *   Copyright (C) 2007 - 2009 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "editaccountwidget.h"
#include "editaccountwidget.moc"

#include "../mwaprotocol.h"
#include "../mwaaccount.h"

#include <blokkalrsdconfigurationdata.h>
#include <blokkalui/providercombobox.h>
#include <blokkalio/xmlrpcjob.h>

#include <klineedit.h>
#include <kdebug.h>
#include <kdialog.h>
#include <klocale.h>
#include <kicon.h>

#include <QCheckBox>
#include <QLayout>
#include <QGroupBox>

class Mwa::Ui::EditAccountWidget::Private {
public:
	Private( void ) :
	providerComboBox( 0 ),
	useWpDateWorkAroundCheckBox( 0 ){}

	Blokkal::Ui::ProviderComboBox * providerComboBox;
	QCheckBox * useWpDateWorkAroundCheckBox;
	QString currentProvider;
};

Mwa::Ui::EditAccountWidget::EditAccountWidget( Mwa::Protocol * protocol,
                                               Blokkal::Account * account,
                                               QWidget * parent ) :
Blogger::Ui::EditAccountWidget( protocol, account, parent ),
d( new Private() )
{
	Mwa::Account * mwaaccount = dynamic_cast< Mwa::Account* >( this->account() );
	if( account && !mwaaccount ) {
		kWarning() << " account is not a MWA account!" << endl;
		return;
	}
	
	QHBoxLayout * providerLayout = new QHBoxLayout( 0 );
	providerLayout->setMargin( 0 );
	d->providerComboBox = new Blokkal::Ui::ProviderComboBox( this );;
	Blokkal::Ui::ProviderList providerList;
	providerList.append( Blokkal::Ui::ProviderInformation( "wordpress.com", i18n( "WordPress.com" ), KIcon( "wordpress" ) ) );
	providerList.append( Blokkal::Ui::ProviderInformation( "wordpress", i18n( "Generic WordPress server" ), KIcon( "wordpress" ) ) );
	providerList.append( Blokkal::Ui::ProviderInformation( "typepad.com", i18n( "TypePad.com" ) ) );
	d->providerComboBox->setProviders( providerList );
	providerLayout->addWidget( d->providerComboBox );
	providerLayout->addStretch( 1 );

	((QBoxLayout*)networkGroupBox()->layout())->insertLayout( 0, providerLayout );

	d->useWpDateWorkAroundCheckBox = new QCheckBox( i18n( "Use &WordPress date workaround" ), networkGroupBox() );
	networkGroupBox()->layout()->addWidget( d->useWpDateWorkAroundCheckBox );

	if( account ) { //use account provider settings if available
		d->providerComboBox->setCurrentProvider( mwaaccount->provider() );
		configureProvider( mwaaccount->provider() );
		d->useWpDateWorkAroundCheckBox->setChecked( mwaaccount->activeWorkArounds() & Blokkal::Io::XmlRpcJob::WordPressDateWorkAround );
	}
	else { //set provider to custom as default
		d->providerComboBox->setCurrentProvider( QString::null );
		configureProvider( QString::null );
	}

	connect( userNameLineEdit(),
	         SIGNAL( textChanged( const QString & ) ),
	         SLOT( setServerAddressUser( const QString & ) ) );
	connect( d->providerComboBox,
	         SIGNAL( providerSelected( const QString & ) ),
	         SLOT( configureProvider( const QString & ) ) );
}


Mwa::Ui::EditAccountWidget::~EditAccountWidget( void )
{
	delete d;
}

void Mwa::Ui::EditAccountWidget::setServerAddress( const QString & serverName )
{
	if( useStandardAddressCheckBox()->isChecked() ) {
		if( serverName.isEmpty() ) {
			connectAddressLineEdit()->setText( 0 );
		}
		else if( d->currentProvider == "wordpress.com" ) {
			connectAddressLineEdit()->setText( QString::fromLatin1( "http://%1.%2/xmlrpc.php").arg( userNameLineEdit()->text() ).arg( serverName ) );
		}
		else if( d->currentProvider == "wordpress" ) {
			connectAddressLineEdit()->setText( QString::fromLatin1( "http://%1/wordpress/xmlrpc.php").arg( serverName ) );
		}
		else if( d->currentProvider == "typepad.com" ) {
			connectAddressLineEdit()->setText( QString::fromLatin1( "http://www.%1/t/api" ).arg( serverName ) );
		}
	}	
}

void Mwa::Ui::EditAccountWidget::setServerAddressUser( const QString & userName )
{
	if( useStandardAddressCheckBox()->isChecked() && d->currentProvider == "wordpress.com" ) {
		if( serverNameLineEdit()->text().isEmpty() ) {
			connectAddressLineEdit()->setText( 0 );
		}
		else {
			connectAddressLineEdit()->setText( QString::fromLatin1( "http://%1.%2/xmlrpc.php").arg( userName ).arg( serverNameLineEdit()->text() ) );
		}
	}	
}

void Mwa::Ui::EditAccountWidget::configureProvider( const QString & id )
{
	d->currentProvider = id;
	if( id == "wordpress.com" ) {
		useStandardAddressCheckBox()->setChecked( TRUE );
		useStandardAddressCheckBox()->setEnabled( TRUE );
		serverNameLineEdit()->setEnabled( FALSE );
		serverNameLineEdit()->setText( "wordpress.com" );
		d->useWpDateWorkAroundCheckBox->setChecked( TRUE );
	}
	else if( id == "wordpress") {
		useStandardAddressCheckBox()->setChecked( TRUE );
		useStandardAddressCheckBox()->setEnabled( TRUE );
		serverNameLineEdit()->setEnabled( TRUE );
		serverNameLineEdit()->setText( "yourhost.domain" );
		d->useWpDateWorkAroundCheckBox->setChecked( TRUE );
	}
	else if( id == "typepad.com") {
		useStandardAddressCheckBox()->setChecked( TRUE );
		useStandardAddressCheckBox()->setEnabled( TRUE );
		serverNameLineEdit()->setEnabled( FALSE );
		serverNameLineEdit()->setText( "typepad.com" );
		d->useWpDateWorkAroundCheckBox->setChecked( TRUE );
	}	
	else if( id.isNull() ) {
		useStandardAddressCheckBox()->setChecked( FALSE );
		useStandardAddressCheckBox()->setEnabled( FALSE );
		serverNameLineEdit()->setEnabled( TRUE );
		d->useWpDateWorkAroundCheckBox->setChecked( FALSE );
	}
	else {
		kWarning() << "unknown provider id: " << id << endl;
	}
}

Blokkal::Account * Mwa::Ui::EditAccountWidget::apply( void )
{
	Blokkal::Account * account = GenericEditAccountWidget::apply();
	Mwa::Account * mwaaccount = dynamic_cast<Mwa::Account*>( account );
	if( !mwaaccount ) {
		kWarning() << "not editing a MWA account!" << endl;
	}
	else {
		mwaaccount->setProvider( d->currentProvider );
		if( d->useWpDateWorkAroundCheckBox->isChecked() ) {
			mwaaccount->setActiveWorkArounds( mwaaccount->activeWorkArounds() | Blokkal::Io::XmlRpcJob::WordPressDateWorkAround );
		}
		else {
			mwaaccount->setActiveWorkArounds( mwaaccount->activeWorkArounds() & !Blokkal::Io::XmlRpcJob::WordPressDateWorkAround );
		}
		
	}
	return account;
}

void Mwa::Ui::EditAccountWidget::setup(const Blokkal::RsdConfigurationData & data)
{
	Blokkal::Ui::GenericEditAccountWidget::setup( data );
	useWpDateWorkAroundCheckBox()->setChecked( data.engineName().toLower() == QString( "wordpress" ) );
}

QCheckBox * Mwa::Ui::EditAccountWidget::useWpDateWorkAroundCheckBox() const
{
	return d->useWpDateWorkAroundCheckBox;
}
