/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mtaccount.h"
#include "mtaccount.moc"

#include "mtprotocol.h"
#include "mtblog.h"
#include "mtio/getsupportedmethodsjob.h"

#include <kdebug.h>
#include <klocale.h>

#include <QTimer>

class Mt::Account::Private {
public:
	Private( void ) :
	supportedMethods( QStringList() ),
	supportedMethodsJob( 0 ) {}

	QStringList supportedMethods;
	Mt::Io::GetSupportedMethodsJob * supportedMethodsJob;
};

Mt::Account::Account( Mt::Protocol * protocol, const QString & accountName ) :
Mwa::Account( protocol, accountName ),
d( new Private() )
{
}

Mt::Account::~Account( void )
{
	delete d;
}

void Mt::Account::connectAccount( void )
{
	setConnectionStatus( Connecting );
	d->supportedMethodsJob = new Mt::Io::GetSupportedMethodsJob( this );
	connect( d->supportedMethodsJob,
	         SIGNAL( result( KJob * ) ),
	         SLOT( slotLoginJobReturned( KJob * ) ) );
	d->supportedMethodsJob->start();
}

void Mt::Account::slotLoginJobReturned( KJob * job )
{
	if( !job ) {
		kWarning() << "job == 0!" << endl;
	}

	if( job == d->supportedMethodsJob ) {
		if( d->supportedMethodsJob->responseType() != Blokkal::Io::XmlRpcJob::ResponseSuccess ) {
			d->supportedMethodsJob = 0; //avoid kill in disconnectAccount()
			disconnectAccount( Blokkal::Account::IOErrorDisconnect,
			                   i18n( "Failed to fetch list of supported methods." ) );
		}
		else {
			d->supportedMethods = d->supportedMethodsJob->supportedMethods();
			d->supportedMethodsJob = 0; //avoid kill in disconnectAccount()
			if( d->supportedMethods.contains( "blogger.getUsersBlogs" )
			    && entryHandlingModes() )
			{
				disconnectAccount(); //clean start for Blogger code
				Mwa::Account::connectAccount();
			}
			else {
				disconnectAccount( Blokkal::Account::IOErrorDisconnect,
				                   i18n( "No entry handling modes available." ) );
			}
		}
		d->supportedMethodsJob = 0;
	}
}

void Mt::Account::disconnectAccount( Blokkal::Account::DisconnectReason reason,
                                     const QString & reasonString )
{
	if( d->supportedMethodsJob ) {
		d->supportedMethodsJob = 0;
	}
	Mwa::Account::disconnectAccount( reason, reasonString );
}

unsigned int Mt::Account::entryHandlingModes( void ) const
{
	unsigned int modes = 0;
	if( d->supportedMethods.contains( "blogger.deletePost" )
	    && d->supportedMethods.contains( "blogger.getPost" )
	    && d->supportedMethods.contains( "blogger.getRecentPosts" )
	    && d->supportedMethods.contains( "blogger.newPost" )
	    && d->supportedMethods.contains( "blogger.editPost" ) )
	{
		modes |= BloggerHandling;
	}
	else 	if( d->supportedMethods.contains( "blogger.deletePost" )
	     	    && d->supportedMethods.contains( "metaWeblog.getPost" )
	     	    && d->supportedMethods.contains( "metaWeblog.getRecentPosts" )
	     	    && d->supportedMethods.contains( "metaWeblog.newPost" )
	     	    && d->supportedMethods.contains( "metaWeblog.editPost" ) )
	{
		modes |= MwaHandling;
	}
	
	return modes;
}

bool Mt::Account::isMethodAvailable( const QString & methodName ) const
{
	return d->supportedMethods.contains( methodName );
}

Blogger::Blog * Mt::Account::createBlog( const QString & id, const QString & url, const QString & title )
{
	return new Mt::Blog( this, id, url, title );
}
