/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MTACCOUNT_H
#define MTACCOUNT_H

#include <mwaaccount.h>

class KJob;
namespace Mt {
class Protocol;
/**
	@author Martin Mueller <orvio@orvio.de>
*/
class Account : public Mwa::Account
{
Q_OBJECT
public:
	/**
	 * Creates a new account for protocol @a protocol, with name @a accountName
	 * @param protocol the procotol of this account
	 * @param accountName the name if this account
	 */
	Account( Mt::Protocol * protocol, const QString & accountName = 0 );

	/**
	 * Destructor
	 */
	virtual ~Account( void );

	/**
	 * Enumerator to identify the protocol modes available
	 * for creating/modifying entries*/
	enum EntryHandling {
		BloggerHandling = 0x01, ///< Blogger methods available
		MwaHandling = 0x02 ///< MetaWeblog methods available
	};

	/**
	 * Returns the EntryHandling modes available for this account.
	 * @return the EntryHandling modes available for this account
	 */
	unsigned int entryHandlingModes( void ) const;

	/**
	 * Test if a method with name @a methodName is available.
	 * @return TRUE if @a methodName is available
	 */
	bool isMethodAvailable( const QString & methodName ) const;
	
public Q_SLOTS:
	/**
	 * This method is called to prepare any succeeding actions.
	 * You are required to gather any required information
	 * (e.g. blogs accessible for the user ) and emit
	 * connectionStatusChanged(). Do not forget to set the new
	 * connection status with setConnectionStatus()
	 */
	virtual void connectAccount( void );

	/**
	 * Disconnects the account.
	 * @param reason optional reason id
	 * @param reasonString optional description
	 */
	virtual void disconnectAccount( Blokkal::Account::DisconnectReason reason = UserDisconnect,
	                                const QString & reasonString = 0 );	

protected Q_SLOTS:
	/**
	 * This slot checks which job returned and proceeds with
	 * the login sequence according to the result
	 * @param job the returned job
	 */
	virtual void slotLoginJobReturned( KJob * job );
	
protected:
	/**
	 * Creates a new blog object.
	 * Do not register the object in this method.
	 * @param id blog id
	 * @param url the blog url
	 * @param title the blog title
	 */
	virtual Blogger::Blog * createBlog( const QString & id,
	                                    const QString & url,
	                                    const QString & title );
	
private:
	class Private;
	Private * const d;
};

}

#endif
