/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "mtentry.h"
#include "mtentry.moc"

#include "mtblog.h"
#include "mtaccount.h"
#include "mtui/entryextensionwidget4.h"

#include <kdebug.h>

#include <QVariant>

Mt::Entry::Entry( Mt::Blog * parentBlog ):
Mwa::Entry( parentBlog )
{
}

Mt::Entry::Entry( Mt::Blog * parentBlog, QMap<QString, QVariant> map ):
Mwa::Entry( parentBlog, map )
{
	//kDebug() << map << endl;
	setAllowComments( map["mt_allow_comments"].toBool() );
	setAllowPings( map["mt_allow_pings"].toBool() );
}

Mt::Entry::~Entry( void )
{
}

Blokkal::Io::EntryJob * Mt::Entry::createJob( Blokkal::Entry::JobType type )
{
	Mt::Account * account = dynamic_cast<Mt::Account*>( blog()->account() );
	if( !account ) {
		kError() << "owning account is not a Mt::Account!" << endl;
		return 0;
	}
		
	if( account->entryHandlingModes() | Mt::Account::MwaHandling ) {
		return Mwa::Entry::createJob( type );
	}
	else if( account->entryHandlingModes() | Mt::Account::BloggerHandling ) {
		return Blogger::Entry::createJob( type );
	}
	
	kError() << "no valid entryHandlingModes available" << endl;
	return 0;
}

bool Mt::Entry::isAllowPings( void ) const
{
	return readBoolEntry( QString::fromUtf8( "allowPings" ), TRUE );
}

void Mt::Entry::setAllowPings( bool allow )
{
	writeEntry( QString::fromUtf8( "allowPings" ), allow );
}

bool Mt::Entry::isAllowComments( void ) const
{
	return readBoolEntry( QString::fromUtf8( "allowComments" ), TRUE );
}

void Mt::Entry::setAllowComments( bool allow )
{
	writeEntry( QString::fromUtf8( "allowComments" ), allow );
}

void Mt::Entry::updatePropertyMap( QMap< QString, QVariant > & map ) const
{
	Mwa::Entry::updatePropertyMap( map );
	map["mt_allow_comments"] = (int) isAllowComments();
	map["mt_allow_pings"] = (int) isAllowPings();
	//kDebug() << "updated map: " << map << endl;
}

Blokkal::Ui::EntryExtensionWidget * Mt::Entry::createExtensionWidget( Blokkal::Ui::EntryExtensionWidget::ExtensionWidget extension, Blokkal::Ui::EditEntryWidget * parent )
{
	if( extension != Blokkal::Ui::EntryExtensionWidget::ExtensionWidget4 ){
		return Mwa::Entry::createExtensionWidget( extension, parent );
	}

	return new Ui::EntryExtensionWidget4( this, parent );
}
