/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MTMTENTRY_H
#define MTMTENTRY_H

#include <mwaentry.h>

namespace Mt {
class Blog;
/**
	@author Martin Mueller <orvio@orvio.de>
*/
class Entry : public Mwa::Entry
{
Q_OBJECT
public:
	/**
	 * Creates a new Entry.
	 * @param parentBlog the blog the entry belongs to
	 */
	Entry( Mt::Blog * parentBlog );

	/**
	 * Creates a new entry from a XmlRpc Response map.
	 * @param parentBlog the blog the entry belongs to
	 * @param map a map containing the struct elements of the xmlrpc response
	 */
	Entry( Mt::Blog * parentBlog, QMap<QString, QVariant> map );

	/**
	 * Destructor
	 */
	virtual ~Entry( void );

	/**
	 * Creates and returns a job of @a type
	 * @return a job of @a type
	 */
	virtual Blokkal::Io::EntryJob * createJob( Blokkal::Entry::JobType type );

	/**
	 * @return the status of the mt_allow_pings flag
	 */
	bool isAllowPings( void ) const;

	/**
	 * Sets the mt_allow_pings flag.
	 * @param allow the new status
	 */
	void setAllowPings( bool allow );

	/**
	 * @return the status of the mt_allow_comments flag
	 */
	bool isAllowComments( void ) const;

	/**
	 * Sets the mt_allow_comments flag.
	 * @param allow the new status
	 */	
	void setAllowComments( bool allow );

	/**
	 * Returns the widget associated with @a extension.
	 * @param extension the identifier for the widget to create
	 * @param parent the parent widget
	 * @return the widget associated with @a extension.
	 */
	virtual Blokkal::Ui::EntryExtensionWidget * createExtensionWidget( Blokkal::Ui::EntryExtensionWidget::ExtensionWidget extension, Blokkal::Ui::EditEntryWidget * parent );
	

protected:
	/**
	 * This method is called by propertyMap() before the reference to
	 * the property map is returned.
	 * Reimplemented to update the mt_allow_comments and mt_allow_pings flags.
	 * @param map reference to the property map
	 */
	virtual void updatePropertyMap( QMap<QString, QVariant> & map ) const;	

};

}

#endif
