/***************************************************************************
 *   Copyright (C) 2007 - 2008 by Martin Mueller                           *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "getsupportedmethodsjob.h"
#include "getsupportedmethodsjob.moc"

#include "../mtaccount.h"

#include <kurl.h>

#include <QVariant>
#include <QStringList>

class Mt::Io::GetSupportedMethodsJob::Private {
public:
	Private( void ) :
	supportedMethods( QStringList() ),
	methodListBuilt( FALSE ) {}

	QStringList supportedMethods;
	bool methodListBuilt;
};

Mt::Io::GetSupportedMethodsJob::GetSupportedMethodsJob( Mt::Account * account, bool showProgressInfo ):
Blokkal::Io::XmlRpcJob( account->connectAddress(),
                        QString::fromLatin1( "mt.supportedMethods" ),
                        QList<QVariant>(),
                        showProgressInfo ),
d( new Private() )
{
}

Mt::Io::GetSupportedMethodsJob::~GetSupportedMethodsJob( void )
{
	delete d;
}

QStringList Mt::Io::GetSupportedMethodsJob::supportedMethods( void ) const
{
	if( responseType() != ResponseSuccess
	    || d->methodListBuilt ) {
		return d->supportedMethods;
	}
	
	QVariant response = this->response();
	
	if( response.type() != QVariant::List ) {
		return d->supportedMethods;
	}

	QList<QVariant> variantList = response.toList();
	for( QList<QVariant>::ConstIterator it = variantList.begin();
	     it != variantList.end();
	     ++it )
	{
		if( (*it).type() != QVariant::String ) {
			return QStringList();
		}

		d->supportedMethods.append( (*it).toString() );
	}

	d->methodListBuilt = TRUE;
	return d->supportedMethods;
}
