/***************************************************************************
 *   Copyright (C) 2006 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef MT_IOGETSUPPORTEDMETHODSJOB_H
#define MT_IOGETSUPPORTEDMETHODSJOB_H

#include <blokkalio/xmlrpcjob.h>

namespace Mt {
class Account;
namespace Io {

/**
 * @short This Job calls the mt.supportedMethods Method.
 *
 * This Job calls the mt.supportedMethods Method.
 * @author Martin Mueller <orvio@orvio.de>
*/
class GetSupportedMethodsJob : public Blokkal::Io::XmlRpcJob
{
Q_OBJECT
public:
	/**
	 * Creates a new GetSupportedMethodsJob.
	 * @param account connect to the address provided by this account
	 * @param showProgressInfo TRUE to show progress info
	 */
	GetSupportedMethodsJob( Mt::Account * account, bool showProgressInfo = FALSE );
	
	/**
	 * Destructor
	 */
	virtual ~GetSupportedMethodsJob( void );

	/**
	 * Returns the list of the supported methods.
	 * Calling this method before the finished() signal
	 * has been emitted yields an undefined result.
	 * @return the list of the supported methods
	 */
	QStringList supportedMethods( void ) const;

private:
	class Private;
	Private * d;
};

}

}

#endif
