/***************************************************************************
 *   Copyright (C) 2008 by Martin Mueller                                  *
 *   orvio@orvio.de                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "entryextensionwidget4.h"
#include "entryextensionwidget4.moc"

#include "../mtentry.h"

#include <kdialog.h>
#include <klocale.h>

#include <QCheckBox>
#include <QLayout>

class Mt::Ui::EntryExtensionWidget4::Private {
public:
	Private( Mt::Entry * entry ) :
	entry( entry ) {}

	Mt::Entry * const entry;

	QCheckBox * pingsCheckBox;
	QCheckBox * commentsCheckBox;
};

Mt::Ui::EntryExtensionWidget4::EntryExtensionWidget4( Mt::Entry * entry, Blokkal::Ui::EditEntryWidget * parent ):
Blokkal::Ui::EntryExtensionWidget( entry, parent ),
d( new Private( entry ) )
{
	QHBoxLayout * const layout = new QHBoxLayout( this );
	setLayout( layout );
	layout->setSpacing( 0 );
	layout->setMargin( KDialog::spacingHint() );

	d->pingsCheckBox = new QCheckBox( i18n( "Allow Pings" ), this );
	layout->addWidget( d->pingsCheckBox );
	d->commentsCheckBox = new QCheckBox( i18n( "Allow Comments" ), this );
	layout->addWidget(d->commentsCheckBox );
	layout->addStretch( 1 );

	rollBack();

	connect( d->pingsCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( emitContentChanged( void ) ) );
	connect( d->commentsCheckBox,
	         SIGNAL( toggled( bool ) ),
	         SLOT( emitContentChanged( void ) ) );	
}

Mt::Ui::EntryExtensionWidget4::~EntryExtensionWidget4( void )
{
	delete d;
}

void Mt::Ui::EntryExtensionWidget4::rollBack( void )
{
	d->pingsCheckBox->setChecked( d->entry->isAllowPings() );
	d->commentsCheckBox->setChecked( d->entry->isAllowComments() );
}

void Mt::Ui::EntryExtensionWidget4::apply( void )
{
	d->entry->setAllowPings( d->pingsCheckBox->isChecked() );
	d->entry->setAllowComments( d->commentsCheckBox->isChecked() );
}
